//---------------------------------------------------------------------------
// TestAnisotropicDisperse.cpp
//---------------------------------------------------------------------------
#include "TestAnisotropicDisperse.h"
#include "BehaviorBase.h"
#include "GridBase.h"
#include "SimManager.h"
#include <stdio.h>
#include <fstream.h>
#include <math.h>

/////////////////////////////////////////////////////////////////////////////
//  TestDispersal1()
/////////////////////////////////////////////////////////////////////////////
void clTestAnisotropicDisperse::TestIsotropicDispersal1() {

   clSimManager *p_oSimManager = new clSimManager(3, 3, "");

   //Feed our file to the sim manager
   p_oSimManager->ReadFile(WriteXMLFile1());

   //Run one timestep - can't just use RunSim() because TimestepCleanup will
   //erase the results we want to check
   int i;
   for (i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++) {
     clBehaviorBase *p_oBase = p_oSimManager->GetBehaviorObject(i);
     p_oBase->Action();
   }

   //Verify file 1
   VerifyXMLFile1(p_oSimManager->GetGridObject("dispersed_seeds"));

   //Clean up
   delete p_oSimManager;
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  TestDispersal2()
/////////////////////////////////////////////////////////////////////////////
void clTestAnisotropicDisperse::TestIsotropicDispersal2() {

   clSimManager *p_oSimManager = new clSimManager(3, 3, "");

   //Feed our file to the sim manager
   p_oSimManager->ReadFile(WriteXMLFile2());

   //Run one timestep - can't just use RunSim() because TimestepCleanup will
   //erase the results we want to check
   int i;
   for (i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++) {
     clBehaviorBase *p_oBase = p_oSimManager->GetBehaviorObject(i);
     p_oBase->Action();
   }

   //Verify file 1
   VerifyXMLFile2(p_oSimManager->GetGridObject("dispersed_seeds"));

   //Clean up
   delete p_oSimManager;
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  VerifyXMLFile1
/////////////////////////////////////////////////////////////////////////////
void clTestAnisotropicDisperse::VerifyXMLFile1(clGridBase *p_oSeedGrid) {
  float fExpectedSeeds, fActualSeeds;
  int iNumXCells, iNumYCells,
      iX, iY,
      iNumSpecies = 4,
      *p_iCodes = new int[iNumSpecies];
  char *cCode = new char[10];
  bool bIsGap;

  for (iX = 0; iX < iNumSpecies; iX++) {
    sprintf(cCode, "%s%d", "seeds_", iX);
    p_iCodes[iX] = p_oSeedGrid->GetFloatDataCode(cCode);
    //Make sure we got the codes
    CPPUNIT_ASSERT(p_iCodes[iX] > -1);
  }

  //Explicitly check the seed levels in a few different grid cells
  //The number of seeds will be different depending on the random rounding -
  //make sure the expected and actual values are within 5 percent

  //Cell 3, 5
  iX = 3;
  iY = 5;
  //Species 1
  fExpectedSeeds = 403.6759164;
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[0], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds)/fExpectedSeeds < 0.05);
  //Species 2
  fExpectedSeeds = 0;
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[1], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds) < 0.01);
  //Species 3
  fExpectedSeeds = 58.06226421;
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[2], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds)/fExpectedSeeds < 0.05);

  //Cell 4, 5
  iX = 4;
  iY = 5;
  //Species 1
  fExpectedSeeds = 1077.595114;
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[0], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds)/fExpectedSeeds < 0.05);
  //Species 2
  fExpectedSeeds = 184.1257034;
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[1], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds)/fExpectedSeeds < 0.05);
  //Species 3
  fExpectedSeeds = 69.9990589;
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[2], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds)/fExpectedSeeds < 0.05);

  //Cell 5, 5
  iX = 5;
  iY = 5;
  //Species 1
  fExpectedSeeds = 1204.204581;
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[0], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds)/fExpectedSeeds < 0.05);
  //Species 2
  fExpectedSeeds = 606.9596143;
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[1], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds)/fExpectedSeeds < 0.05);
  //Species 3
  fExpectedSeeds = 70.74357896;
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[2], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds)/fExpectedSeeds < 0.05);

  //Cell 6, 5
  iX = 6;
  iY = 5;
  //Species 1
  fExpectedSeeds = 921.8744;
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[0], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds)/fExpectedSeeds < 0.05);
  //Species 2
  fExpectedSeeds = 619.3314874;
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[1], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds)/fExpectedSeeds < 0.05);
  //Species 3
  fExpectedSeeds = 63.41038967;
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[2], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds)/fExpectedSeeds < 0.05);

  //Cell 7, 5
  iX = 7;
  iY = 5;
  //Species 1
  fExpectedSeeds = 0;
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[0], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds) < 0.01);
  //Species 2
  fExpectedSeeds = 285.6340571;
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[1], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds)/fExpectedSeeds < 0.05);
  //Species 3
  fExpectedSeeds = 8.181394472;
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[2], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds)/fExpectedSeeds < 0.05);

  //Cell 9, 9
  iX = 9;
  iY = 9;
  fExpectedSeeds = 0;
  //Species 1
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[0], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds) < 0.01);
  //Species 2
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[1], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds) < 0.01);
  //Species 3
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[2], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds) < 0.01);

  //Cell 2, 2
  iX = 2;
  iY = 2;
  fExpectedSeeds = 0;
  //Species 1
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[0], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds) < 0.01);
  //Species 2
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[1], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds) < 0.01);
  //Species 3
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[2], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds) < 0.01);

  //Make sure there are no seeds from species 4
  fExpectedSeeds = 0;
  for (iX = 0; iX < iNumXCells; iX++) {
    for (iY = 0; iY < iNumYCells; iY++) {
      p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[3], &fActualSeeds);
      CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds) < 0.01);
    }
  }
  delete[] p_iCodes;
  delete[] cCode;
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  VerifyXMLFile2
/////////////////////////////////////////////////////////////////////////////
void clTestAnisotropicDisperse::VerifyXMLFile2(clGridBase *p_oSeedGrid) {
  float fExpectedSeeds, fActualSeeds;
  int iNumXCells, iNumYCells,
      iX, iY,
      iNumSpecies = 4,
      *p_iCodes = new int[iNumSpecies];
  char *cCode = new char[10];
  bool bIsGap;

  for (iX = 0; iX < iNumSpecies; iX++) {
    sprintf(cCode, "%s%d", "seeds_", iX);
    p_iCodes[iX] = p_oSeedGrid->GetFloatDataCode(cCode);
    //Make sure we got the codes
    CPPUNIT_ASSERT(p_iCodes[iX] > -1);
  }

  //Explicitly check the seed levels in a few different grid cells
  //The number of seeds will be different depending on the random rounding -
  //make sure the expected and actual values are within 5 percent

  //Cell 3, 5
  iX = 3;
  iY = 5;
  //Species 1
  fExpectedSeeds = 1211.027749;
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[0], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds)/fExpectedSeeds < 0.05);
  //Species 2
  fExpectedSeeds = 0;
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[1], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds) < 0.01);
  //Species 3
  fExpectedSeeds = 174.1867926;
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[2], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds)/fExpectedSeeds < 0.05);

  //Cell 4, 5
  iX = 4;
  iY = 5;
  //Species 1
  fExpectedSeeds = 3232.785341;
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[0], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds)/fExpectedSeeds < 0.05);
  //Species 2
  fExpectedSeeds = 552.3771103;
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[1], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds)/fExpectedSeeds < 0.05);
  //Species 3
  fExpectedSeeds = 209.9971767;
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[2], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds)/fExpectedSeeds < 0.05);

  //Cell 5, 5
  iX = 5;
  iY = 5;
  //Species 1
  fExpectedSeeds = 3612.613743;
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[0], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds)/fExpectedSeeds < 0.05);
  //Species 2
  fExpectedSeeds = 1820.878843;
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[1], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds)/fExpectedSeeds < 0.05);
  //Species 3
  fExpectedSeeds = 212.2307369;
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[2], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds)/fExpectedSeeds < 0.05);

  //Cell 6, 5
  iX = 6;
  iY = 5;
  //Species 1
  fExpectedSeeds = 2765.6232;
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[0], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds)/fExpectedSeeds < 0.05);
  //Species 2
  fExpectedSeeds = 1857.994462;
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[1], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds)/fExpectedSeeds < 0.05);
  //Species 3
  fExpectedSeeds = 190.231169;
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[2], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds)/fExpectedSeeds < 0.05);

  //Cell 7, 5
  iX = 7;
  iY = 5;
  //Species 1
  fExpectedSeeds = 0;
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[0], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds) < 0.01);
  //Species 2
  fExpectedSeeds = 856.9021712;
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[1], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds)/fExpectedSeeds < 0.05);
  //Species 3
  fExpectedSeeds = 24.54418342;
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[2], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds)/fExpectedSeeds < 0.05);

  //Cell 9, 9
  iX = 9;
  iY = 9;
  fExpectedSeeds = 0;
  //Species 1
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[0], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds) < 0.01);
  //Species 2
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[1], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds) < 0.01);
  //Species 3
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[2], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds) < 0.01);

  //Cell 2, 2
  iX = 2;
  iY = 2;
  fExpectedSeeds = 0;
  //Species 1
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[0], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds) < 0.01);
  //Species 2
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[1], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds) < 0.01);
  //Species 3
  p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[2], &fActualSeeds);
  CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds) < 0.01);

  //Make sure there are no seeds from species 4
  fExpectedSeeds = 0;
  for (iX = 0; iX < iNumXCells; iX++) {
    for (iY = 0; iY < iNumYCells; iY++) {
      p_oSeedGrid->GetValueOfCell(iX, iY, p_iCodes[3], &fActualSeeds);
      CPPUNIT_ASSERT(fabs(fExpectedSeeds - fActualSeeds) < 0.01);
    }
  }
  delete[] p_iCodes;
  delete[] cCode;
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  TestErrorProcessing()
/////////////////////////////////////////////////////////////////////////////
void clTestAnisotropicDisperse::TestErrorProcessing()
{
  clSimManager * p_oSimManager = new clSimManager(3, 3, "");

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile1());
    CppUnit::Asserter::fail("TestAnisotropicDispersal error processing didn't catch error for WriteXMLErrorFile1.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clFuncResponseSeedPredation") == NULL)
    {
      CppUnit::Asserter::fail("TestAnisotropicDispersal error processing caught wrong error for WriteXMLErrorFile1.");
    }
    ;
  }

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile2());
    CppUnit::Asserter::fail("TestAnisotropicDispersal error processing didn't catch error for WriteXMLErrorFile2.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clFuncResponseSeedPredation") == NULL)
    {
      CppUnit::Asserter::fail("TestAnisotropicDispersal error processing caught wrong error for WriteXMLErrorFile2.");
    }
    ;
  }

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile3());
    CppUnit::Asserter::fail("TestAnisotropicDispersal error processing didn't catch error for WriteXMLErrorFile3.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clFuncResponseSeedPredation") == NULL)
    {
      CppUnit::Asserter::fail("TestAnisotropicDispersal error processing caught wrong error for WriteXMLErrorFile3.");
    }
    ;
  }

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile4());
    CppUnit::Asserter::fail("TestAnisotropicDispersal error processing didn't catch error for WriteXMLErrorFile4.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clFuncResponseSeedPredation") == NULL)
    {
      CppUnit::Asserter::fail("TestAnisotropicDispersal error processing caught wrong error for WriteXMLErrorFile4.");
    }
    ;
  }

  //Clean up
  delete p_oSimManager;
}

//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile1()
/////////////////////////////////////////////////////////////////////////////
char * clTestAnisotropicDisperse::WriteXMLFile1()
{
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>1</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>"
       << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>100.0</plot_lenX>"
       << "<plot_lenY>100.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "<tr_species speciesName=\"Species_2\"/>"
       << "<tr_species speciesName=\"Species_3\"/>"
       << "<tr_species speciesName=\"Species_4\"/>"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_2\">39.54</tr_chVal>"
       << "<tr_chVal species=\"Species_3\">40.0</tr_chVal>"
       << "<tr_chVal species=\"Species_4\">39.48</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_canopyRadiusToDBH>"
       << "<tr_crtdVal species=\"Species_1\">0.0549</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_2\">0.0614</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_3\">0.0242</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_4\">0.0549</tr_crtdVal>"
       << "</tr_canopyRadiusToDBH>"
       << "<tr_canopyRadiusExponent>"
       << "<tr_creVal species=\"Species_1\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_2\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_3\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_4\">1.0</tr_creVal>"
       << "</tr_canopyRadiusExponent>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_2\">0.5944</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_3\">0.7059</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_4\">0.5944</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_fractOfHeightInCanopy>"
       << "<tr_fohicVal species=\"Species_1\">0.389</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_2\">0.368</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_3\">0.464</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_4\">0.368</tr_fohicVal>"
       << "</tr_fractOfHeightInCanopy>"
       << "<tr_canopyDepthExponent>"
       << "<tr_cdeVal species=\"Species_1\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_2\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_3\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_4\">1.0</tr_cdeVal>"
       << "</tr_canopyDepthExponent>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_2\">0.0269</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_3\">0.02871</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_4\">0.0269</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_2\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_3\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_4\">10.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_2\">0.0241</tr_soahVal>"
       << "<tr_soahVal species=\"Species_3\">0.0263</tr_soahVal>"
       << "<tr_soahVal species=\"Species_4\">0.0241</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "<tr_treemap>"
       << "<tm_treeSettings sp=\"Species_1\" tp=\"3\">"
       << "<tm_floatCodes>"
       << "<tm_floatCode label=\"X\">0</tm_floatCode>"
       << "<tm_floatCode label=\"Y\">1</tm_floatCode>"
       << "<tm_floatCode label=\"DBH\">2</tm_floatCode>"
       << "</tm_floatCodes>"
       << "</tm_treeSettings>"
       << "<tm_treeSettings sp=\"Species_2\" tp=\"3\">"
       << "<tm_floatCodes>"
       << "<tm_floatCode label=\"X\">0</tm_floatCode>"
       << "<tm_floatCode label=\"Y\">1</tm_floatCode>"
       << "<tm_floatCode label=\"DBH\">2</tm_floatCode>"
       << "</tm_floatCodes>"
       << "</tm_treeSettings>"
       << "<tm_treeSettings sp=\"Species_3\" tp=\"3\">"
       << "<tm_floatCodes>"
       << "<tm_floatCode label=\"X\">0</tm_floatCode>"
       << "<tm_floatCode label=\"Y\">1</tm_floatCode>"
       << "<tm_floatCode label=\"DBH\">2</tm_floatCode>"
       << "</tm_floatCodes>"
       << "</tm_treeSettings>"
       << "<tm_treeSettings sp=\"Species_3\" tp=\"4\">"
       << "<tm_floatCodes>"
       << "<tm_floatCode label=\"X\">0</tm_floatCode>"
       << "<tm_floatCode label=\"Y\">1</tm_floatCode>"
       << "<tm_floatCode label=\"DBH\">2</tm_floatCode>"
       << "</tm_floatCodes>"
       << "</tm_treeSettings>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">25</fl>"
       << "<fl c=\"1\">25</fl>"
       << "<fl c=\"2\">12</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">50</fl>"
       << "<fl c=\"1\">50</fl>"
       << "<fl c=\"2\">34</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">54</fl>"
       << "<fl c=\"1\">54</fl>"
       << "<fl c=\"2\">87</fl>"
       << "</tree>"
       << "<tree sp=\"1\" tp=\"3\">"
       << "<fl c=\"0\">63</fl>"
       << "<fl c=\"1\">45</fl>"
       << "<fl c=\"2\">53</fl>"
       << "</tree>"
       << "<tree sp=\"1\" tp=\"3\">"
       << "<fl c=\"0\">55</fl>"
       << "<fl c=\"1\">45</fl>"
       << "<fl c=\"2\">43</fl>"
       << "</tree>"
       << "<tree sp=\"2\" tp=\"3\">"
       << "<fl c=\"0\">51</fl>"
       << "<fl c=\"1\">61</fl>"
       << "<fl c=\"2\">74</fl>"
       << "</tree>"
       << "<tree sp=\"2\" tp=\"3\">"
       << "<fl c=\"0\">63</fl>"
       << "<fl c=\"1\">61</fl>"
       << "<fl c=\"2\">31</fl>"
       << "</tree>"
       << "<tree sp=\"2\" tp=\"4\">"
       << "<fl c=\"0\">41</fl>"
       << "<fl c=\"1\">52</fl>"
       << "<fl c=\"2\">22</fl>"
       << "</tree>"
       << "</tr_treemap>"
       << "</trees>"
       //Set the default grids
       << "<grid gridName=\"Dispersed Seeds\">"
       << "<ma_intCodes>"
       << "<ma_intCode label=\"seeds_0\">0</ma_intCode>"
       << "<ma_intCode label=\"seeds_1\">1</ma_intCode>"
       << "<ma_intCode label=\"seeds_2\">2</ma_intCode>"
       << "<ma_intCode label=\"seeds_3\">3</ma_intCode>"
       << "<ma_intCode label=\"count\">6</ma_intCode>"
       << "</ma_intCodes>"
       << "<ma_lengthXCells>10</ma_lengthXCells>"
       << "<ma_lengthYCells>10</ma_lengthYCells>"
       << "</grid>"
       << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>isotropic disperse</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_3\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_3\" type=\"Stump\"/>"
       << "</behavior>"
       << "</behaviorList>"
       << "<disperse>"
       << "<di_minDbhForReproduction>"
       << "<di_mdfrVal species=\"Species_1\">15.0</di_mdfrVal>"
       << "<di_mdfrVal species=\"Species_2\">15.0</di_mdfrVal>"
       << "<di_mdfrVal species=\"Species_3\">15.0</di_mdfrVal>"
       << "</di_minDbhForReproduction>"
       << "<di_weibullCanopySTR>"
       << "<di_wcsVal species=\"Species_1\">19.325</di_wcsVal>"
       << "<di_wcsVal species=\"Species_2\">23</di_wcsVal>"
       << "</di_weibullCanopySTR>"
       << "<di_weibullCanopyBeta>"
       << "<di_wcbVal species=\"Species_1\">2.0</di_wcbVal>"
       << "<di_wcbVal species=\"Species_2\">2.1</di_wcbVal>"
       << "</di_weibullCanopyBeta>"
       << "<di_weibullCanopyDispersal>"
       << "<di_wcdVal species=\"Species_1\">1.76E-4</di_wcdVal>"
       << "<di_wcdVal species=\"Species_2\">0.000182</di_wcdVal>"
       << "</di_weibullCanopyDispersal>"
       << "<di_weibullCanopyTheta>"
       << "<di_wctVal species=\"Species_1\">3.0</di_wctVal>"
       << "<di_wctVal species=\"Species_2\">3.1</di_wctVal>"
       << "</di_weibullCanopyTheta>"
       << "<di_lognormalCanopySTR>"
       << "<di_lcsVal species=\"Species_3\">6.628</di_lcsVal>"
       << "</di_lognormalCanopySTR>"
       << "<di_lognormalCanopyBeta>"
       << "<di_lcbVal species=\"Species_3\">2.2</di_lcbVal>"
       << "</di_lognormalCanopyBeta>"
       << "<di_lognormalCanopyX0>"
       << "<di_lcx0Val species=\"Species_3\">1.000175953</di_lcx0Val>"
       << "</di_lognormalCanopyX0>"
       << "<di_lognormalCanopyXb>"
       << "<di_lcxbVal species=\"Species_3\">5.1</di_lcxbVal>"
       << "</di_lognormalCanopyXb>"
       << "<di_suckerSTR>"
       << "<di_ssVal species=\"Species_3\">6.628</di_ssVal>"
       << "</di_suckerSTR>"
       << "<di_suckerBeta>"
       << "<di_sbVal species=\"Species_3\">2.2</di_sbVal>"
       << "</di_suckerBeta>"
       << "<di_canopyFunction>"
       << "<di_cfVal species=\"Species_1\">0</di_cfVal>"
       << "<di_cfVal species=\"Species_2\">0</di_cfVal>"
       << "<di_cfVal species=\"Species_3\">1</di_cfVal>"
       << "</di_canopyFunction>"
       << "<di_maxSearchRadius>20.0</di_maxSearchRadius>"
       << "</disperse>"
       << "</paramFile>";

  oOut.close();

  return cFileString;
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile2()
/////////////////////////////////////////////////////////////////////////////
char * clTestAnisotropicDisperse::WriteXMLFile2()
{
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>1</timesteps>"
       << "<yearsPerTimestep>3</yearsPerTimestep>"
       << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>100.0</plot_lenX>"
       << "<plot_lenY>100.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "<tr_species speciesName=\"Species_2\"/>"
       << "<tr_species speciesName=\"Species_3\"/>"
       << "<tr_species speciesName=\"Species_4\"/>"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_2\">39.54</tr_chVal>"
       << "<tr_chVal species=\"Species_3\">40.0</tr_chVal>"
       << "<tr_chVal species=\"Species_4\">39.48</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_canopyRadiusToDBH>"
       << "<tr_crtdVal species=\"Species_1\">0.0549</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_2\">0.0614</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_3\">0.0242</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_4\">0.0549</tr_crtdVal>"
       << "</tr_canopyRadiusToDBH>"
       << "<tr_canopyRadiusExponent>"
       << "<tr_creVal species=\"Species_1\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_2\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_3\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_4\">1.0</tr_creVal>"
       << "</tr_canopyRadiusExponent>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_2\">0.5944</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_3\">0.7059</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_4\">0.5944</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_fractOfHeightInCanopy>"
       << "<tr_fohicVal species=\"Species_1\">0.389</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_2\">0.368</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_3\">0.464</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_4\">0.368</tr_fohicVal>"
       << "</tr_fractOfHeightInCanopy>"
       << "<tr_canopyDepthExponent>"
       << "<tr_cdeVal species=\"Species_1\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_2\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_3\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_4\">1.0</tr_cdeVal>"
       << "</tr_canopyDepthExponent>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_2\">0.0269</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_3\">0.02871</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_4\">0.0269</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_2\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_3\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_4\">10.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_2\">0.0241</tr_soahVal>"
       << "<tr_soahVal species=\"Species_3\">0.0263</tr_soahVal>"
       << "<tr_soahVal species=\"Species_4\">0.0241</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "<tr_treemap>"
       << "<tm_treeSettings sp=\"Species_1\" tp=\"3\">"
       << "<tm_floatCodes>"
       << "<tm_floatCode label=\"X\">0</tm_floatCode>"
       << "<tm_floatCode label=\"Y\">1</tm_floatCode>"
       << "<tm_floatCode label=\"DBH\">2</tm_floatCode>"
       << "</tm_floatCodes>"
       << "</tm_treeSettings>"
       << "<tm_treeSettings sp=\"Species_2\" tp=\"3\">"
       << "<tm_floatCodes>"
       << "<tm_floatCode label=\"X\">0</tm_floatCode>"
       << "<tm_floatCode label=\"Y\">1</tm_floatCode>"
       << "<tm_floatCode label=\"DBH\">2</tm_floatCode>"
       << "</tm_floatCodes>"
       << "</tm_treeSettings>"
       << "<tm_treeSettings sp=\"Species_3\" tp=\"3\">"
       << "<tm_floatCodes>"
       << "<tm_floatCode label=\"X\">0</tm_floatCode>"
       << "<tm_floatCode label=\"Y\">1</tm_floatCode>"
       << "<tm_floatCode label=\"DBH\">2</tm_floatCode>"
       << "</tm_floatCodes>"
       << "</tm_treeSettings>"
       << "<tm_treeSettings sp=\"Species_3\" tp=\"4\">"
       << "<tm_floatCodes>"
       << "<tm_floatCode label=\"X\">0</tm_floatCode>"
       << "<tm_floatCode label=\"Y\">1</tm_floatCode>"
       << "<tm_floatCode label=\"DBH\">2</tm_floatCode>"
       << "</tm_floatCodes>"
       << "</tm_treeSettings>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">25</fl>"
       << "<fl c=\"1\">25</fl>"
       << "<fl c=\"2\">12</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">50</fl>"
       << "<fl c=\"1\">50</fl>"
       << "<fl c=\"2\">34</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">54</fl>"
       << "<fl c=\"1\">54</fl>"
       << "<fl c=\"2\">87</fl>"
       << "</tree>"
       << "<tree sp=\"1\" tp=\"3\">"
       << "<fl c=\"0\">63</fl>"
       << "<fl c=\"1\">45</fl>"
       << "<fl c=\"2\">53</fl>"
       << "</tree>"
       << "<tree sp=\"1\" tp=\"3\">"
       << "<fl c=\"0\">55</fl>"
       << "<fl c=\"1\">45</fl>"
       << "<fl c=\"2\">43</fl>"
       << "</tree>"
       << "<tree sp=\"2\" tp=\"3\">"
       << "<fl c=\"0\">51</fl>"
       << "<fl c=\"1\">61</fl>"
       << "<fl c=\"2\">74</fl>"
       << "</tree>"
       << "<tree sp=\"2\" tp=\"3\">"
       << "<fl c=\"0\">63</fl>"
       << "<fl c=\"1\">61</fl>"
       << "<fl c=\"2\">31</fl>"
       << "</tree>"
       << "<tree sp=\"2\" tp=\"4\">"
       << "<fl c=\"0\">41</fl>"
       << "<fl c=\"1\">52</fl>"
       << "<fl c=\"2\">22</fl>"
       << "</tree>"
       << "</tr_treemap>"
       << "</trees>"
       //Set the default grids
       << "<grid gridName=\"Dispersed Seeds\">"
       << "<ma_intCodes>"
       << "<ma_intCode label=\"seeds_0\">0</ma_intCode>"
       << "<ma_intCode label=\"seeds_1\">1</ma_intCode>"
       << "<ma_intCode label=\"seeds_2\">2</ma_intCode>"
       << "<ma_intCode label=\"seeds_3\">3</ma_intCode>"
       << "<ma_intCode label=\"count\">6</ma_intCode>"
       << "</ma_intCodes>"
       << "<ma_lengthXCells>10</ma_lengthXCells>"
       << "<ma_lengthYCells>10</ma_lengthYCells>"
       << "</grid>"
       << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>isotropic disperse</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_3\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_3\" type=\"Stump\"/>"
       << "</behavior>"
       << "</behaviorList>"
       << "<disperse>"
       << "<di_minDbhForReproduction>"
       << "<di_mdfrVal species=\"Species_1\">15.0</di_mdfrVal>"
       << "<di_mdfrVal species=\"Species_2\">15.0</di_mdfrVal>"
       << "<di_mdfrVal species=\"Species_3\">15.0</di_mdfrVal>"
       << "</di_minDbhForReproduction>"
       << "<di_weibullCanopySTR>"
       << "<di_wcsVal species=\"Species_1\">19.325</di_wcsVal>"
       << "<di_wcsVal species=\"Species_2\">23</di_wcsVal>"
       << "</di_weibullCanopySTR>"
       << "<di_weibullCanopyBeta>"
       << "<di_wcbVal species=\"Species_1\">2.0</di_wcbVal>"
       << "<di_wcbVal species=\"Species_2\">2.1</di_wcbVal>"
       << "</di_weibullCanopyBeta>"
       << "<di_weibullCanopyDispersal>"
       << "<di_wcdVal species=\"Species_1\">1.76E-4</di_wcdVal>"
       << "<di_wcdVal species=\"Species_2\">0.000182</di_wcdVal>"
       << "</di_weibullCanopyDispersal>"
       << "<di_weibullCanopyTheta>"
       << "<di_wctVal species=\"Species_1\">3.0</di_wctVal>"
       << "<di_wctVal species=\"Species_2\">3.1</di_wctVal>"
       << "</di_weibullCanopyTheta>"
       << "<di_lognormalCanopySTR>"
       << "<di_lcsVal species=\"Species_3\">6.628</di_lcsVal>"
       << "</di_lognormalCanopySTR>"
       << "<di_lognormalCanopyBeta>"
       << "<di_lcbVal species=\"Species_3\">2.2</di_lcbVal>"
       << "</di_lognormalCanopyBeta>"
       << "<di_lognormalCanopyX0>"
       << "<di_lcx0Val species=\"Species_3\">1.000175953</di_lcx0Val>"
       << "</di_lognormalCanopyX0>"
       << "<di_lognormalCanopyXb>"
       << "<di_lcxbVal species=\"Species_3\">5.1</di_lcxbVal>"
       << "</di_lognormalCanopyXb>"
       << "<di_suckerSTR>"
       << "<di_ssVal species=\"Species_3\">6.628</di_ssVal>"
       << "</di_suckerSTR>"
       << "<di_suckerBeta>"
       << "<di_sbVal species=\"Species_3\">2.2</di_sbVal>"
       << "</di_suckerBeta>"
       << "<di_canopyFunction>"
       << "<di_cfVal species=\"Species_1\">0</di_cfVal>"
       << "<di_cfVal species=\"Species_2\">0</di_cfVal>"
       << "<di_cfVal species=\"Species_3\">1</di_cfVal>"
       << "</di_canopyFunction>"
       << "<di_maxSearchRadius>20.0</di_maxSearchRadius>"
       << "</disperse>"
       << "</paramFile>";

  oOut.close();

  return cFileString;
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile1()
/////////////////////////////////////////////////////////////////////////////
char * clTestAnisotropicDisperse::WriteXMLErrorFile1()
{
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>1</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>"
       << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>100.0</plot_lenX>"
       << "<plot_lenY>100.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "<tr_species speciesName=\"Species_2\"/>"
       << "<tr_species speciesName=\"Species_3\"/>"
       << "<tr_species speciesName=\"Species_4\"/>"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_2\">39.54</tr_chVal>"
       << "<tr_chVal species=\"Species_3\">40.0</tr_chVal>"
       << "<tr_chVal species=\"Species_4\">39.48</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_canopyRadiusToDBH>"
       << "<tr_crtdVal species=\"Species_1\">0.0549</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_2\">0.0614</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_3\">0.0242</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_4\">0.0549</tr_crtdVal>"
       << "</tr_canopyRadiusToDBH>"
       << "<tr_canopyRadiusExponent>"
       << "<tr_creVal species=\"Species_1\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_2\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_3\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_4\">1.0</tr_creVal>"
       << "</tr_canopyRadiusExponent>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_2\">0.5944</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_3\">0.7059</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_4\">0.5944</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_fractOfHeightInCanopy>"
       << "<tr_fohicVal species=\"Species_1\">0.389</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_2\">0.368</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_3\">0.464</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_4\">0.368</tr_fohicVal>"
       << "</tr_fractOfHeightInCanopy>"
       << "<tr_canopyDepthExponent>"
       << "<tr_cdeVal species=\"Species_1\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_2\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_3\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_4\">1.0</tr_cdeVal>"
       << "</tr_canopyDepthExponent>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_2\">0.0269</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_3\">0.02871</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_4\">0.0269</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_2\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_3\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_4\">10.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_2\">0.0241</tr_soahVal>"
       << "<tr_soahVal species=\"Species_3\">0.0263</tr_soahVal>"
       << "<tr_soahVal species=\"Species_4\">0.0241</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "</trees>"
       << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>anisotropic disperse</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_3\" type=\"Adult\"/>"
       << "</behavior>"
       << "</behaviorList>"
       << "<disperse>"
       << "<di_minDbhForReproduction>"
       << "<di_mdfrVal species=\"Species_1\">15.0</di_mdfrVal>"
       << "<di_mdfrVal species=\"Species_2\">15.0</di_mdfrVal>"
       << "<di_mdfrVal species=\"Species_3\">15.0</di_mdfrVal>"
       << "</di_minDbhForReproduction>"
       << "<di_weibullCanopySTR>"
       << "<di_wcsVal species=\"Species_1\">19.325</di_wcsVal>"
       << "<di_wcsVal species=\"Species_2\">23</di_wcsVal>"
       << "</di_weibullCanopySTR>"
       << "<di_weibullCanopyBeta>"
       << "<di_wcbVal species=\"Species_1\">2.0</di_wcbVal>"
       << "<di_wcbVal species=\"Species_2\">2.1</di_wcbVal>"
       << "</di_weibullCanopyBeta>"
       << "<di_weibullCanopyDispersal>"
       << "<di_wcdVal species=\"Species_1\">1.76E-4</di_wcdVal>"
       << "<di_wcdVal species=\"Species_2\">0.000182</di_wcdVal>"
       << "</di_weibullCanopyDispersal>"
       << "<di_weibullCanopyTheta>"
       << "<di_wctVal species=\"Species_1\">3.0</di_wctVal>"
       << "<di_wctVal species=\"Species_2\">3.1</di_wctVal>"
       << "</di_weibullCanopyTheta>"
       << "<di_lognormalCanopySTR>"
       << "<di_lcsVal species=\"Species_3\">6.628</di_lcsVal>"
       << "</di_lognormalCanopySTR>"
       << "<di_lognormalCanopyBeta>"
       << "<di_lcbVal species=\"Species_3\">2.2</di_lcbVal>"
       << "</di_lognormalCanopyBeta>"
       << "<di_lognormalCanopyX0>"
       << "<di_lcx0Val species=\"Species_3\">1.000175953</di_lcx0Val>"
       << "</di_lognormalCanopyX0>"
       << "<di_lognormalCanopyXb>"
       << "<di_lcxbVal species=\"Species_3\">5.1</di_lcxbVal>"
       << "</di_lognormalCanopyXb>"
       << "<di_suckerSTR>"
       << "<di_ssVal species=\"Species_3\">6.628</di_ssVal>"
       << "</di_suckerSTR>"
       << "<di_suckerBeta>"
       << "<di_sbVal species=\"Species_3\">2.2</di_sbVal>"
       << "</di_suckerBeta>"
       << "<di_canopyFunction>"
       << "<di_cfVal species=\"Species_1\">0</di_cfVal>"
       << "<di_cfVal species=\"Species_2\">0</di_cfVal>"
       << "<di_cfVal species=\"Species_3\">1</di_cfVal>"
       << "</di_canopyFunction>"
       << "<di_maxSearchRadius>20.0</di_maxSearchRadius>"
       << "<di_directionMaxDispersal>"
       << "<di_dmdVal species=\"Species_1\">0.352</di_dmdVal>"
       << "<di_dmdVal species=\"Species_2\">-2.70</di_dmdVal>"
       << "<di_dmdVal species=\"Species_3\">1</di_dmdVal>"
       << "</di_directionMaxDispersal>"
       << "<di_anisotropicAmplitude>"
       << "<di_aaVal species=\"Species_1\">0.352</di_aaVal>"
       << "<di_aaVal species=\"Species_2\">8.70</di_aaVal>"
       << "<di_aaVal species=\"Species_3\">1</di_aaVal>"
       << "</di_anisotropicAmplitude>"
       << "</disperse>"
       << "</paramFile>";

  oOut.close();

  return cFileString;
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile2()
/////////////////////////////////////////////////////////////////////////////
char * clTestAnisotropicDisperse::WriteXMLErrorFile2()
{
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>1</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>"
       << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>100.0</plot_lenX>"
       << "<plot_lenY>100.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "<tr_species speciesName=\"Species_2\"/>"
       << "<tr_species speciesName=\"Species_3\"/>"
       << "<tr_species speciesName=\"Species_4\"/>"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_2\">39.54</tr_chVal>"
       << "<tr_chVal species=\"Species_3\">40.0</tr_chVal>"
       << "<tr_chVal species=\"Species_4\">39.48</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_canopyRadiusToDBH>"
       << "<tr_crtdVal species=\"Species_1\">0.0549</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_2\">0.0614</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_3\">0.0242</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_4\">0.0549</tr_crtdVal>"
       << "</tr_canopyRadiusToDBH>"
       << "<tr_canopyRadiusExponent>"
       << "<tr_creVal species=\"Species_1\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_2\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_3\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_4\">1.0</tr_creVal>"
       << "</tr_canopyRadiusExponent>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_2\">0.5944</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_3\">0.7059</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_4\">0.5944</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_fractOfHeightInCanopy>"
       << "<tr_fohicVal species=\"Species_1\">0.389</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_2\">0.368</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_3\">0.464</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_4\">0.368</tr_fohicVal>"
       << "</tr_fractOfHeightInCanopy>"
       << "<tr_canopyDepthExponent>"
       << "<tr_cdeVal species=\"Species_1\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_2\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_3\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_4\">1.0</tr_cdeVal>"
       << "</tr_canopyDepthExponent>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_2\">0.0269</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_3\">0.02871</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_4\">0.0269</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_2\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_3\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_4\">10.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_2\">0.0241</tr_soahVal>"
       << "<tr_soahVal species=\"Species_3\">0.0263</tr_soahVal>"
       << "<tr_soahVal species=\"Species_4\">0.0241</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "</trees>"
       << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>anisotropic disperse</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_3\" type=\"Adult\"/>"
       << "</behavior>"
       << "</behaviorList>"
       << "<disperse>"
       << "<di_minDbhForReproduction>"
       << "<di_mdfrVal species=\"Species_1\">15.0</di_mdfrVal>"
       << "<di_mdfrVal species=\"Species_2\">15.0</di_mdfrVal>"
       << "<di_mdfrVal species=\"Species_3\">15.0</di_mdfrVal>"
       << "</di_minDbhForReproduction>"
       << "<di_weibullCanopySTR>"
       << "<di_wcsVal species=\"Species_1\">19.325</di_wcsVal>"
       << "<di_wcsVal species=\"Species_2\">23</di_wcsVal>"
       << "</di_weibullCanopySTR>"
       << "<di_weibullCanopyBeta>"
       << "<di_wcbVal species=\"Species_1\">2.0</di_wcbVal>"
       << "<di_wcbVal species=\"Species_2\">2.1</di_wcbVal>"
       << "</di_weibullCanopyBeta>"
       << "<di_weibullCanopyDispersal>"
       << "<di_wcdVal species=\"Species_1\">1.76E-4</di_wcdVal>"
       << "<di_wcdVal species=\"Species_2\">0.000182</di_wcdVal>"
       << "</di_weibullCanopyDispersal>"
       << "<di_weibullCanopyTheta>"
       << "<di_wctVal species=\"Species_1\">3.0</di_wctVal>"
       << "<di_wctVal species=\"Species_2\">3.1</di_wctVal>"
       << "</di_weibullCanopyTheta>"
       << "<di_lognormalCanopySTR>"
       << "<di_lcsVal species=\"Species_3\">6.628</di_lcsVal>"
       << "</di_lognormalCanopySTR>"
       << "<di_lognormalCanopyBeta>"
       << "<di_lcbVal species=\"Species_3\">2.2</di_lcbVal>"
       << "</di_lognormalCanopyBeta>"
       << "<di_lognormalCanopyX0>"
       << "<di_lcx0Val species=\"Species_3\">1.000175953</di_lcx0Val>"
       << "</di_lognormalCanopyX0>"
       << "<di_lognormalCanopyXb>"
       << "<di_lcxbVal species=\"Species_3\">5.1</di_lcxbVal>"
       << "</di_lognormalCanopyXb>"
       << "<di_suckerSTR>"
       << "<di_ssVal species=\"Species_3\">6.628</di_ssVal>"
       << "</di_suckerSTR>"
       << "<di_suckerBeta>"
       << "<di_sbVal species=\"Species_3\">2.2</di_sbVal>"
       << "</di_suckerBeta>"
       << "<di_canopyFunction>"
       << "<di_cfVal species=\"Species_1\">0</di_cfVal>"
       << "<di_cfVal species=\"Species_2\">0</di_cfVal>"
       << "<di_cfVal species=\"Species_3\">1</di_cfVal>"
       << "</di_canopyFunction>"
       << "<di_maxSearchRadius>20.0</di_maxSearchRadius>"
       << "<di_directionMaxDispersal>"
       << "<di_dmdVal species=\"Species_1\">10.352</di_dmdVal>"
       << "<di_dmdVal species=\"Species_2\">2.70</di_dmdVal>"
       << "<di_dmdVal species=\"Species_3\">1</di_dmdVal>"
       << "</di_directionMaxDispersal>"
       << "<di_anisotropicAmplitude>"
       << "<di_aaVal species=\"Species_1\">0.352</di_aaVal>"
       << "<di_aaVal species=\"Species_2\">8.70</di_aaVal>"
       << "<di_aaVal species=\"Species_3\">1</di_aaVal>"
       << "</di_anisotropicAmplitude>"
       << "</disperse>"
       << "</paramFile>";

  oOut.close();

  return cFileString;
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile3()
/////////////////////////////////////////////////////////////////////////////
char * clTestAnisotropicDisperse::WriteXMLErrorFile3()
{
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>1</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>"
       << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>100.0</plot_lenX>"
       << "<plot_lenY>100.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "<tr_species speciesName=\"Species_2\"/>"
       << "<tr_species speciesName=\"Species_3\"/>"
       << "<tr_species speciesName=\"Species_4\"/>"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_2\">39.54</tr_chVal>"
       << "<tr_chVal species=\"Species_3\">40.0</tr_chVal>"
       << "<tr_chVal species=\"Species_4\">39.48</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_canopyRadiusToDBH>"
       << "<tr_crtdVal species=\"Species_1\">0.0549</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_2\">0.0614</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_3\">0.0242</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_4\">0.0549</tr_crtdVal>"
       << "</tr_canopyRadiusToDBH>"
       << "<tr_canopyRadiusExponent>"
       << "<tr_creVal species=\"Species_1\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_2\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_3\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_4\">1.0</tr_creVal>"
       << "</tr_canopyRadiusExponent>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_2\">0.5944</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_3\">0.7059</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_4\">0.5944</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_fractOfHeightInCanopy>"
       << "<tr_fohicVal species=\"Species_1\">0.389</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_2\">0.368</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_3\">0.464</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_4\">0.368</tr_fohicVal>"
       << "</tr_fractOfHeightInCanopy>"
       << "<tr_canopyDepthExponent>"
       << "<tr_cdeVal species=\"Species_1\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_2\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_3\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_4\">1.0</tr_cdeVal>"
       << "</tr_canopyDepthExponent>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_2\">0.0269</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_3\">0.02871</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_4\">0.0269</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_2\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_3\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_4\">10.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_2\">0.0241</tr_soahVal>"
       << "<tr_soahVal species=\"Species_3\">0.0263</tr_soahVal>"
       << "<tr_soahVal species=\"Species_4\">0.0241</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "</trees>"
       << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>anisotropic disperse</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_3\" type=\"Adult\"/>"
       << "</behavior>"
       << "</behaviorList>"
       << "<disperse>"
       << "<di_minDbhForReproduction>"
       << "<di_mdfrVal species=\"Species_1\">15.0</di_mdfrVal>"
       << "<di_mdfrVal species=\"Species_2\">15.0</di_mdfrVal>"
       << "<di_mdfrVal species=\"Species_3\">15.0</di_mdfrVal>"
       << "</di_minDbhForReproduction>"
       << "<di_weibullCanopySTR>"
       << "<di_wcsVal species=\"Species_1\">19.325</di_wcsVal>"
       << "<di_wcsVal species=\"Species_2\">23</di_wcsVal>"
       << "</di_weibullCanopySTR>"
       << "<di_weibullCanopyBeta>"
       << "<di_wcbVal species=\"Species_1\">2.0</di_wcbVal>"
       << "<di_wcbVal species=\"Species_2\">2.1</di_wcbVal>"
       << "</di_weibullCanopyBeta>"
       << "<di_weibullCanopyDispersal>"
       << "<di_wcdVal species=\"Species_1\">1.76E-4</di_wcdVal>"
       << "<di_wcdVal species=\"Species_2\">0.000182</di_wcdVal>"
       << "</di_weibullCanopyDispersal>"
       << "<di_weibullCanopyTheta>"
       << "<di_wctVal species=\"Species_1\">3.0</di_wctVal>"
       << "<di_wctVal species=\"Species_2\">3.1</di_wctVal>"
       << "</di_weibullCanopyTheta>"
       << "<di_lognormalCanopySTR>"
       << "<di_lcsVal species=\"Species_3\">6.628</di_lcsVal>"
       << "</di_lognormalCanopySTR>"
       << "<di_lognormalCanopyBeta>"
       << "<di_lcbVal species=\"Species_3\">2.2</di_lcbVal>"
       << "</di_lognormalCanopyBeta>"
       << "<di_lognormalCanopyX0>"
       << "<di_lcx0Val species=\"Species_3\">1.000175953</di_lcx0Val>"
       << "</di_lognormalCanopyX0>"
       << "<di_lognormalCanopyXb>"
       << "<di_lcxbVal species=\"Species_3\">5.1</di_lcxbVal>"
       << "</di_lognormalCanopyXb>"
       << "<di_suckerSTR>"
       << "<di_ssVal species=\"Species_3\">6.628</di_ssVal>"
       << "</di_suckerSTR>"
       << "<di_suckerBeta>"
       << "<di_sbVal species=\"Species_3\">2.2</di_sbVal>"
       << "</di_suckerBeta>"
       << "<di_canopyFunction>"
       << "<di_cfVal species=\"Species_1\">0</di_cfVal>"
       << "<di_cfVal species=\"Species_2\">0</di_cfVal>"
       << "<di_cfVal species=\"Species_3\">1</di_cfVal>"
       << "</di_canopyFunction>"
       << "<di_maxSearchRadius>0</di_maxSearchRadius>"
       << "<di_directionMaxDispersal>"
       << "<di_dmdVal species=\"Species_1\">0.352</di_dmdVal>"
       << "<di_dmdVal species=\"Species_2\">2.70</di_dmdVal>"
       << "<di_dmdVal species=\"Species_3\">1</di_dmdVal>"
       << "</di_directionMaxDispersal>"
       << "<di_anisotropicAmplitude>"
       << "<di_aaVal species=\"Species_1\">0.352</di_aaVal>"
       << "<di_aaVal species=\"Species_2\">8.70</di_aaVal>"
       << "<di_aaVal species=\"Species_3\">1</di_aaVal>"
       << "</di_anisotropicAmplitude>"
       << "</disperse>"
       << "</paramFile>";

  oOut.close();

  return cFileString;
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile4()
/////////////////////////////////////////////////////////////////////////////
char * clTestAnisotropicDisperse::WriteXMLErrorFile4()
{
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>1</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>"
       << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>200.0</plot_lenX>"
       << "<plot_lenY>100.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "<tr_species speciesName=\"Species_2\"/>"
       << "<tr_species speciesName=\"Species_3\"/>"
       << "<tr_species speciesName=\"Species_4\"/>"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_2\">39.54</tr_chVal>"
       << "<tr_chVal species=\"Species_3\">40.0</tr_chVal>"
       << "<tr_chVal species=\"Species_4\">39.48</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_canopyRadiusToDBH>"
       << "<tr_crtdVal species=\"Species_1\">0.0549</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_2\">0.0614</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_3\">0.0242</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_4\">0.0549</tr_crtdVal>"
       << "</tr_canopyRadiusToDBH>"
       << "<tr_canopyRadiusExponent>"
       << "<tr_creVal species=\"Species_1\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_2\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_3\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_4\">1.0</tr_creVal>"
       << "</tr_canopyRadiusExponent>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_2\">0.5944</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_3\">0.7059</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_4\">0.5944</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_fractOfHeightInCanopy>"
       << "<tr_fohicVal species=\"Species_1\">0.389</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_2\">0.368</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_3\">0.464</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_4\">0.368</tr_fohicVal>"
       << "</tr_fractOfHeightInCanopy>"
       << "<tr_canopyDepthExponent>"
       << "<tr_cdeVal species=\"Species_1\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_2\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_3\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_4\">1.0</tr_cdeVal>"
       << "</tr_canopyDepthExponent>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_2\">0.0269</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_3\">0.02871</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_4\">0.0269</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_2\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_3\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_4\">10.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_2\">0.0241</tr_soahVal>"
       << "<tr_soahVal species=\"Species_3\">0.0263</tr_soahVal>"
       << "<tr_soahVal species=\"Species_4\">0.0241</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "</trees>"
       << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>anisotropic disperse</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_3\" type=\"Adult\"/>"
       << "</behavior>"
       << "</behaviorList>"
       << "<disperse>"
       << "<di_minDbhForReproduction>"
       << "<di_mdfrVal species=\"Species_1\">15.0</di_mdfrVal>"
       << "<di_mdfrVal species=\"Species_2\">15.0</di_mdfrVal>"
       << "<di_mdfrVal species=\"Species_3\">15.0</di_mdfrVal>"
       << "</di_minDbhForReproduction>"
       << "<di_weibullCanopySTR>"
       << "<di_wcsVal species=\"Species_1\">19.325</di_wcsVal>"
       << "<di_wcsVal species=\"Species_2\">23</di_wcsVal>"
       << "</di_weibullCanopySTR>"
       << "<di_weibullCanopyBeta>"
       << "<di_wcbVal species=\"Species_1\">2.0</di_wcbVal>"
       << "<di_wcbVal species=\"Species_2\">2.1</di_wcbVal>"
       << "</di_weibullCanopyBeta>"
       << "<di_weibullCanopyDispersal>"
       << "<di_wcdVal species=\"Species_1\">1.76E-4</di_wcdVal>"
       << "<di_wcdVal species=\"Species_2\">0.000182</di_wcdVal>"
       << "</di_weibullCanopyDispersal>"
       << "<di_weibullCanopyTheta>"
       << "<di_wctVal species=\"Species_1\">3.0</di_wctVal>"
       << "<di_wctVal species=\"Species_2\">3.1</di_wctVal>"
       << "</di_weibullCanopyTheta>"
       << "<di_lognormalCanopySTR>"
       << "<di_lcsVal species=\"Species_3\">6.628</di_lcsVal>"
       << "</di_lognormalCanopySTR>"
       << "<di_lognormalCanopyBeta>"
       << "<di_lcbVal species=\"Species_3\">2.2</di_lcbVal>"
       << "</di_lognormalCanopyBeta>"
       << "<di_lognormalCanopyX0>"
       << "<di_lcx0Val species=\"Species_3\">1.000175953</di_lcx0Val>"
       << "</di_lognormalCanopyX0>"
       << "<di_lognormalCanopyXb>"
       << "<di_lcxbVal species=\"Species_3\">5.1</di_lcxbVal>"
       << "</di_lognormalCanopyXb>"
       << "<di_suckerSTR>"
       << "<di_ssVal species=\"Species_3\">6.628</di_ssVal>"
       << "</di_suckerSTR>"
       << "<di_suckerBeta>"
       << "<di_sbVal species=\"Species_3\">2.2</di_sbVal>"
       << "</di_suckerBeta>"
       << "<di_canopyFunction>"
       << "<di_cfVal species=\"Species_1\">0</di_cfVal>"
       << "<di_cfVal species=\"Species_2\">0</di_cfVal>"
       << "<di_cfVal species=\"Species_3\">1</di_cfVal>"
       << "</di_canopyFunction>"
       << "<di_maxSearchRadius>51.0</di_maxSearchRadius>"
       << "<di_directionMaxDispersal>"
       << "<di_dmdVal species=\"Species_1\">0.352</di_dmdVal>"
       << "<di_dmdVal species=\"Species_2\">2.70</di_dmdVal>"
       << "<di_dmdVal species=\"Species_3\">1</di_dmdVal>"
       << "</di_directionMaxDispersal>"
       << "<di_anisotropicAmplitude>"
       << "<di_aaVal species=\"Species_1\">0.352</di_aaVal>"
       << "<di_aaVal species=\"Species_2\">8.70</di_aaVal>"
       << "<di_aaVal species=\"Species_3\">1</di_aaVal>"
       << "</di_anisotropicAmplitude>"
       << "</disperse>"
       << "</paramFile>";

  oOut.close();

  return cFileString;
}
//---------------------------------------------------------------------------



