//---------------------------------------------------------------------------
// TestDensitySelfThinning.h
//---------------------------------------------------------------------------
#if !defined(TestDensitySelfThinning_H)
  #define TestDensitySelfThinning_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>

/**
* Tests DensitySelfThinning.
* @authors Marissa LeBlanc and Rasmus Astrup
*/

class clSimManager;

class clTestDensitySelfThinning : public CppUnit::TestFixture
{
  public:

  /**
  * Tests the Density Self Thinning behavior.  Tests both normal and error processing.
  */
  void TestDensitySelfThinning();

  protected:

  /**
  * Tests normal processing using WriteXMLFile1()
  */
  void TestNormalProcessing (clSimManager *p_oSimManager);

  /**
  * Tests error processing by using each of the Error XML files and then checking to see if the correct error is triggered.
  */
  void TestErrorProcessing (clSimManager *p_oSimManager);

  /**
  * Writes a parameter file with legal parameter values for the behavior.
  */
  char * WriteXMLFile1();

  /**
  * Writes a parameter file with a timestep greater than 1.
  */
  char * WriteXMLErrorFile1();

  /**
  * Writes a parameter file that passes adults to the behavior.
  */
  char * WriteXMLErrorFile2();

  /**
  * Writes a parameter file with a negative SelfThinRadius parameter.
  */
  char * WriteXMLErrorFile3();

  /**
  * Writes a parameter file with a negative MinDensityForMort parameter.
  */
  char * WriteXMLErrorFile4();

  /**
  * Writes a parameter file with a negative SelfThinAsymptote parameter.
  */
  char * WriteXMLErrorFile5();

  /**
  * Writes a parameter file with a negative SelfThinDensityEffect parameter.
  */
  char * WriteXMLErrorFile6();



};
//---------------------------------------------------------------------------
#endif // TestDensitySelfThinning_H

