//---------------------------------------------------------------------------
// TestEstablishment.cpp
//---------------------------------------------------------------------------
#include "TestEstablishment.h"
#include "GridBase.h"
#include "SimManager.h"
#include "BehaviorBase.h"
#include "TreePopulation.h"
#include <stdio.h>
#include <fstream.h>
#include <math.h>

/////////////////////////////////////////////////////////////////////////////
//  TestEstablishment1()
/////////////////////////////////////////////////////////////////////////////
void clTestEstablishment::TestEstablishment1() {

   clSimManager *p_oSimManager = new clSimManager(3, 3, "");

   //Feed our file to the sim manager
   p_oSimManager->ReadFile(WriteXMLFile1());

   //Verify the seed map
   clGridBase *p_oSeeds = p_oSimManager->GetGridObject("Dispersed Seeds");
   float fTemp;
   int iX, iY, iNumXCells = p_oSeeds->GetNumberXCells(), iNumYCells = p_oSeeds->GetNumberYCells();
   for (iX = 0; iX < iNumXCells; iX++) {
     for (iY = 0; iY < iNumYCells; iY++) {
       p_oSeeds->GetValueOfCell(iX, iY, 0, &fTemp);
       CPPUNIT_ASSERT((iX+iY)==fTemp);
       p_oSeeds->GetValueOfCell(iX, iY, 1, &fTemp);
       CPPUNIT_ASSERT((2*(iX+iY))==fTemp);
       p_oSeeds->GetValueOfCell(iX, iY, 2, &fTemp);
       CPPUNIT_ASSERT((3*(iX+iY))==fTemp);
     }
   }

   //Run the disperse and establishment behaviors
   int i;
   for (i = 2; i < p_oSimManager->GetNumberOfBehaviors(); i++) { //skip light & substrate
     clBehaviorBase *p_oBase = p_oSimManager->GetBehaviorObject(i);
     p_oBase->Action();
   }

   clPopulationBase *p_oTemp = p_oSimManager->GetPopulationObject("treepopulation");

   //Verify file 1
   VerifyXMLFile1(p_oSimManager->GetGridObject("Substrate Favorability"), (clTreePopulation*)p_oTemp);

   //Clean up
   delete p_oSimManager;
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  TestEstablishment2()
/////////////////////////////////////////////////////////////////////////////
void clTestEstablishment::TestEstablishment2() {

  clSimManager *p_oSimManager = new clSimManager(3, 3, "");

  //Feed our file to the sim manager
  p_oSimManager->ReadFile(WriteXMLFile2());

  //Run the disperse and establishment behaviors
  int i;
  for (i = 2; i < p_oSimManager->GetNumberOfBehaviors(); i++) { //skip light and substrate
    clBehaviorBase *p_oBase = p_oSimManager->GetBehaviorObject(i);
    p_oBase->Action();
  }

  clPopulationBase *p_oTemp = p_oSimManager->GetPopulationObject("treepopulation");

  //Verify file 2
  VerifyXMLFile2(p_oSimManager->GetGridObject("Substrate Favorability"), (clTreePopulation*)p_oTemp);

  //Clean up
  delete p_oSimManager;
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  VerifyXMLFile1
/////////////////////////////////////////////////////////////////////////////
void clTestEstablishment::VerifyXMLFile1(clGridBase * p_oSubstrateFavGrid, clTreePopulation *p_oPop) {
  clTreeSearch *p_oSeedlings;
  clTree *p_oTree;
  float fExpectedFav, fActualFav,
  fGapMossCan2 = 0.129,
  fGapMossCan3 = 0.48,
  fOrganicsCan2 = 0.983,
  fOrganicsCan3 = 0.391,
  fFreshLogCan2 = 0.353,
  fFreshLogCan3 = 0.424,
  fMossyLogCan2 = 0.058,
  fMossyLogCan3 = 0.024,
  fCanMossCan2 = 0.009,
  fCanMossCan3 = 0.01,
  fGapMossGap3 = 0.213,
  fOrganicsGap3 = 0.179,
  fFreshLogGap3 = 0.319,
  fMossyLogGap3 = 0.048,
  fCanMossGap3 = 0.001,
  fGapMoss, fOrganic, fFreshLog, fMossyLog, fCanMoss;

  int iNumXCells, iNumYCells,
      iX, iY,
      iSpecies2Code = p_oSubstrateFavGrid->GetFloatDataCode("Favorability Index1"),
      iSpecies3Code = p_oSubstrateFavGrid->GetFloatDataCode("Favorability Index2"),
      iNumSeedlings = 0;

  //First check favorability maps.

  //Make sure we got the codes
  CPPUNIT_ASSERT(iSpecies2Code > -1);
  CPPUNIT_ASSERT(iSpecies3Code > -1);

  iNumXCells = p_oSubstrateFavGrid->GetNumberXCells();
  iNumYCells = p_oSubstrateFavGrid->GetNumberYCells();

  for (iX = 0; iX < iNumXCells; iX++) {
    for (iY = 0; iY < iNumYCells; iY++) {

      fGapMoss = 0.3 - ((iX + iY)*0.01);
      fOrganic = (iX + iY)*0.02;
      fFreshLog = 0.1 + ((iX + iY)*0.01);
      fMossyLog = 0.2 + ((iX + iY)*0.01);
      fCanMoss = 1 - (fGapMoss + fOrganic + fFreshLog + fMossyLog);

      //Species 2
      fExpectedFav = (fGapMossCan2*fGapMoss) +
                     (fOrganicsCan2*fOrganic) +
                     (fFreshLogCan2*fFreshLog) +
                     (fMossyLogCan2*fMossyLog) +
                     (fCanMossCan2*fCanMoss);

      p_oSubstrateFavGrid->GetValueOfCell(iX, iY, iSpecies2Code, &fActualFav);
      CPPUNIT_ASSERT(fabs(fExpectedFav - fActualFav) < 0.001);

      //Species 3
      p_oSubstrateFavGrid->GetValueOfCell(iX, iY, iSpecies3Code, &fActualFav);
      if (iX < 2) {
        fExpectedFav = (fGapMossCan3*fGapMoss) +
                       (fOrganicsCan3*fOrganic) +
                       (fFreshLogCan3*fFreshLog) +
                       (fMossyLogCan3*fMossyLog) +
                       (fCanMossCan3*fCanMoss);

      } else {
        fExpectedFav = (fGapMossGap3*fGapMoss) +
                       (fOrganicsGap3*fOrganic) +
                       (fFreshLogGap3*fFreshLog) +
                       (fMossyLogGap3*fMossyLog) +
                       (fCanMossGap3*fCanMoss);
      }
      CPPUNIT_ASSERT(fabs(fExpectedFav - fActualFav) < 0.001);
    }
  }

  //Now count all seedlings of species 1 - should have 100
  //Also find out where they are distributed - should match seedlings
  short int **p_iCount;
  float fCellSize = 10, fX, fY;
  p_iCount = new short int*[iNumXCells];
  for (iX = 0; iX < iNumXCells; iX++) {
    p_iCount[iX] = new short int[iNumYCells];
    for (iY = 0; iY < iNumYCells; iY++) {
      p_iCount[iX][iY] = 0;
    }
  }
  p_oSeedlings = p_oPop->Find("type=1::species=0");
  p_oTree = p_oSeedlings->NextTree();
  while (p_oTree) {
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);
    p_iCount[(int)(fX/fCellSize)][(int)(fY/fCellSize)]++;
    iNumSeedlings++;
    p_oTree = p_oSeedlings->NextTree();
  }

  CPPUNIT_ASSERT(100 == iNumSeedlings);

  for (iX = 0; iX < iNumXCells; iX++) {
    for (iY = 0; iY < iNumYCells; iY++) {
      CPPUNIT_ASSERT(p_iCount[iX][iY] == iX+iY);
    }
  }

  for (iX = 0; iX < iNumXCells; iX++) {
    delete[] p_iCount[iX];
  }
  delete[] p_iCount;
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  VerifyXMLFile2()
/////////////////////////////////////////////////////////////////////////////
void clTestEstablishment::VerifyXMLFile2(clGridBase * p_oSubstrateFavGrid, clTreePopulation *p_oPop) {
  clTreeSearch *p_oSeedlings;
  clTree *p_oTree;
  float fExpectedFav, fActualFav;
  int iNumSeedlings = 0,
      iNumXCells = p_oSubstrateFavGrid->GetNumberXCells(),
      iNumYCells = p_oSubstrateFavGrid->GetNumberYCells(),
      iX, iY;

  //Check the favorabilities
  for (iX = 0; iX < iNumXCells; iX++) {
    for (iY = 0; iY < iNumYCells; iY++) {

      fExpectedFav = 0.4301;

      p_oSubstrateFavGrid->GetValueOfCell(iX, iY, 5, &fActualFav);
      CPPUNIT_ASSERT(fabs(fExpectedFav - fActualFav) < 0.001);
    }
  }

  //Count all seedlings of species 6 - should have close to 21
  p_oSeedlings = p_oPop->Find("type=1::species=5");
  p_oTree = p_oSeedlings->NextTree();
  while (p_oTree) {
    iNumSeedlings++;
    p_oTree = p_oSeedlings->NextTree();
  }

  CPPUNIT_ASSERT(19 <= iNumSeedlings && 23 >= iNumSeedlings);
}
//---------------------------------------------------------------------------


/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile2()
/////////////////////////////////////////////////////////////////////////////
char * clTestEstablishment::WriteXMLFile2()
{
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
     << "<paramFile fileCode=\"06010101\">"
     << "<plot>"
     << "<timesteps>1</timesteps>"
     << "<yearsPerTimestep>5</yearsPerTimestep>"
     << "<randomSeed>1</randomSeed>"
     << "<plot_lenX>50.0</plot_lenX>"
     << "<plot_lenY>50.0</plot_lenY>"
     << "<plot_latitude>55.37</plot_latitude>"
     << "</plot>"
     << "<trees>"
     << "<tr_speciesList>"
     << "<tr_species speciesName=\"Species_1\"/>"
     << "<tr_species speciesName=\"Species_2\"/>"
     << "<tr_species speciesName=\"Species_3\"/>"
     << "<tr_species speciesName=\"Species_4\"/>"
     << "<tr_species speciesName=\"Species_5\"/>"
     << "<tr_species speciesName=\"Species_6\"/>"
     << "</tr_speciesList>"
     << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
     << "<tr_canopyHeight>"
     << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
     << "<tr_chVal species=\"Species_2\">39.48</tr_chVal>"
     << "<tr_chVal species=\"Species_3\">39.48</tr_chVal>"
     << "<tr_chVal species=\"Species_4\">39.48</tr_chVal>"
     << "<tr_chVal species=\"Species_5\">39.48</tr_chVal>"
     << "<tr_chVal species=\"Species_6\">39.48</tr_chVal>"
     << "</tr_canopyHeight>"
     << "<tr_canopyRadiusToDBH>"
     << "<tr_crtdVal species=\"Species_1\">0.0549</tr_crtdVal>"
     << "<tr_crtdVal species=\"Species_2\">0.0549</tr_crtdVal>"
     << "<tr_crtdVal species=\"Species_3\">0.0549</tr_crtdVal>"
     << "<tr_crtdVal species=\"Species_4\">0.0549</tr_crtdVal>"
     << "<tr_crtdVal species=\"Species_5\">0.0549</tr_crtdVal>"
     << "<tr_crtdVal species=\"Species_6\">0.0549</tr_crtdVal>"
     << "</tr_canopyRadiusToDBH>"
     << "<tr_canopyRadiusExponent>"
     << "<tr_creVal species=\"Species_1\">1.0</tr_creVal>"
     << "<tr_creVal species=\"Species_2\">1.0</tr_creVal>"
     << "<tr_creVal species=\"Species_3\">1.0</tr_creVal>"
     << "<tr_creVal species=\"Species_4\">1.0</tr_creVal>"
     << "<tr_creVal species=\"Species_5\">1.0</tr_creVal>"
     << "<tr_creVal species=\"Species_6\">1.0</tr_creVal>"
     << "</tr_canopyRadiusExponent>"
     << "<tr_conversionDiam10ToDBH>"
     << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
     << "<tr_cdtdVal species=\"Species_2\">0.8008</tr_cdtdVal>"
     << "<tr_cdtdVal species=\"Species_3\">0.8008</tr_cdtdVal>"
     << "<tr_cdtdVal species=\"Species_4\">0.8008</tr_cdtdVal>"
     << "<tr_cdtdVal species=\"Species_5\">0.8008</tr_cdtdVal>"
     << "<tr_cdtdVal species=\"Species_6\">0.8008</tr_cdtdVal>"
     << "</tr_conversionDiam10ToDBH>"
     << "<tr_fractOfHeightInCanopy>"
     << "<tr_fohicVal species=\"Species_1\">0.389</tr_fohicVal>"
     << "<tr_fohicVal species=\"Species_2\">0.389</tr_fohicVal>"
     << "<tr_fohicVal species=\"Species_3\">0.389</tr_fohicVal>"
     << "<tr_fohicVal species=\"Species_4\">0.389</tr_fohicVal>"
     << "<tr_fohicVal species=\"Species_5\">0.389</tr_fohicVal>"
     << "<tr_fohicVal species=\"Species_6\">0.389</tr_fohicVal>"
     << "</tr_fractOfHeightInCanopy>"
     << "<tr_canopyDepthExponent>"
     << "<tr_cdeVal species=\"Species_1\">1.0</tr_cdeVal>"
     << "<tr_cdeVal species=\"Species_2\">1.0</tr_cdeVal>"
     << "<tr_cdeVal species=\"Species_3\">1.0</tr_cdeVal>"
     << "<tr_cdeVal species=\"Species_4\">1.0</tr_cdeVal>"
     << "<tr_cdeVal species=\"Species_5\">1.0</tr_cdeVal>"
     << "<tr_cdeVal species=\"Species_6\">1.0</tr_cdeVal>"
     << "</tr_canopyDepthExponent>"
     << "<tr_slopeOfHeight-Diam10>"
     << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
     << "<tr_sohdVal species=\"Species_2\">0.03418</tr_sohdVal>"
     << "<tr_sohdVal species=\"Species_3\">0.03418</tr_sohdVal>"
     << "<tr_sohdVal species=\"Species_4\">0.03418</tr_sohdVal>"
     << "<tr_sohdVal species=\"Species_5\">0.03418</tr_sohdVal>"
     << "<tr_sohdVal species=\"Species_6\">0.03418</tr_sohdVal>"
     << "</tr_slopeOfHeight-Diam10>"
     << "<tr_minAdultDBH>"
     << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
     << "<tr_madVal species=\"Species_2\">10.0</tr_madVal>"
     << "<tr_madVal species=\"Species_3\">10.0</tr_madVal>"
     << "<tr_madVal species=\"Species_4\">10.0</tr_madVal>"
     << "<tr_madVal species=\"Species_5\">10.0</tr_madVal>"
     << "<tr_madVal species=\"Species_6\">10.0</tr_madVal>"
     << "</tr_minAdultDBH>"
     << "<tr_slopeOfAsymHeight>"
     << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
     << "<tr_soahVal species=\"Species_2\">0.0299</tr_soahVal>"
     << "<tr_soahVal species=\"Species_3\">0.0299</tr_soahVal>"
     << "<tr_soahVal species=\"Species_4\">0.0299</tr_soahVal>"
     << "<tr_soahVal species=\"Species_5\">0.0299</tr_soahVal>"
     << "<tr_soahVal species=\"Species_6\">0.0299</tr_soahVal>"
     << "</tr_slopeOfAsymHeight>"
     << "</trees>"
     //Generate the substrate proportions map
     << "<grid gridName=\"Substrate\">"
     << "<ma_floatCodes>"
     << "<ma_floatCode label=\"gapmoss\">0</ma_floatCode>"
     << "<ma_floatCode label=\"canmoss\">1</ma_floatCode>"
     << "<ma_floatCode label=\"organic\">2</ma_floatCode>"
     << "<ma_floatCode label=\"freshlog\">3</ma_floatCode>"
     << "<ma_floatCode label=\"mossylog\">4</ma_floatCode>"
     << "</ma_floatCodes>"
     << "<ma_lengthXCells>10</ma_lengthXCells>"
     << "<ma_lengthYCells>10</ma_lengthYCells>";

  //Auto-generate grid values
  int iNumX = 5, iNumY = 5, iX, iY;
  float fGapMoss = 0.3, fOrganic = 0.35, fFreshLog = 0.1, fMossyLog = 0.2;
  for (iX = 0; iX < iNumX; iX++) {
    for (iY = 0; iY < iNumY; iY++) {
      oOut << "<ma_v x=\"" << iX << "\" y=\"" << iY << "\">"
           << "<fl c=\"0\">" << fGapMoss << "</fl>"
           << "<fl c=\"1\">" << 1 - (fGapMoss+fOrganic+fFreshLog+fMossyLog) << "</fl>"
           << "<fl c=\"2\">" << fOrganic << "</fl>"
           << "<fl c=\"3\">" << fFreshLog << "</fl>"
           << "<fl c=\"4\">" << fMossyLog << "</fl>"
           << "</ma_v>";
    }
  }
  oOut << "</grid>";

  //Auto-generate seed grid values
  oOut << "<grid gridName=\"Dispersed Seeds\">"
       << "<ma_floatCodes>"
       << "<ma_floatCode label=\"seeds_0\">0</ma_floatCode>"
       << "<ma_floatCode label=\"seeds_1\">1</ma_floatCode>"
       << "<ma_floatCode label=\"seeds_2\">2</ma_floatCode>"
       << "<ma_floatCode label=\"seeds_3\">3</ma_floatCode>"
       << "<ma_floatCode label=\"seeds_4\">4</ma_floatCode>"
       << "<ma_floatCode label=\"seeds_5\">5</ma_floatCode>"
       << "</ma_floatCodes>"
       << "<ma_intCodes>"
       << "<ma_intCode label=\"count\">1</ma_intCode>"
       << "</ma_intCodes>"
       << "<ma_boolCodes>"
       << "<ma_boolCode label=\"Is Gap\">0</ma_boolCode>"
       << "</ma_boolCodes>"
       << "<ma_lengthXCells>10</ma_lengthXCells>"
       << "<ma_lengthYCells>10</ma_lengthYCells>";

  for (iX = 0; iX < iNumX; iX++) {
    for (iY = 0; iY < iNumY; iY++) {
      oOut << "<ma_v x=\"" << iX << "\" y=\"" << iY << "\">"
           << "<fl c=\"5\">" << 2 << "</fl>"
           << "</ma_v>";
    }
  }
  oOut << "</grid>"

     << "<behaviorList>"
     << "<behavior>"
     << "<behaviorName>adultstochasticmort</behaviorName>"
     << "<version>1</version>"
     << "<applyTo species=\"Species_6\" type=\"Adult\"/>"
     << "</behavior>"
     << "<behavior>"
     << "<behaviorName>substrate</behaviorName>"
     << "<version>1</version>"
     << "<applyTo species=\"Species_6\" type=\"Adult\"/>"
     << "</behavior>"
     << "<behavior>"
     << "<behaviorName>gap disperse</behaviorName>"
     << "<version>1</version>"
     << "<applyTo species=\"Species_6\" type=\"Adult\"/>"
     << "</behavior>"
     << "<behavior>"
     << "<behaviorName>no gap substrate establishment</behaviorName>"
     << "<version>1</version>"
     << "<applyTo species=\"Species_6\" type=\"Seed\"/>"
     << "</behavior>"
     << "</behaviorList>"
     << "<substrate>"
     << "<su_gapmossDecayAlpha>-5.1E-4</su_gapmossDecayAlpha>"
     << "<su_gapmossDecayBeta>4.4</su_gapmossDecayBeta>"
     << "<su_organicDecayAlpha>-7.0E-4</su_organicDecayAlpha>"
     << "<su_organicDecayBeta>4.3</su_organicDecayBeta>"
     << "<su_freshLogDecayAlpha>-0.05</su_freshLogDecayAlpha>"
     << "<su_freshLogDecayBeta>6.5</su_freshLogDecayBeta>"
     << "<su_mossyLogToCanMoss>0.55</su_mossyLogToCanMoss>"
     << "<su_freshLogFromCanMoss>0.0080</su_freshLogFromCanMoss>"
     << "<su_maxNumberDecayYears>10</su_maxNumberDecayYears>"
     << "<su_initialGapMoss>0.0</su_initialGapMoss>"
     << "<su_initialOrganic>0.0</su_initialOrganic>"
     << "<su_initialFreshLog>0.01</su_initialFreshLog>"
     << "<su_initialMossyLog>0.11</su_initialMossyLog>"
     << "<su_partialCutGapMoss>0.17</su_partialCutGapMoss>"
     << "<su_partialCutOrganic>0.15</su_partialCutOrganic>"
     << "<su_partialCutFreshLog>0.06</su_partialCutFreshLog>"
     << "<su_partialCutMossyLog>0.04</su_partialCutMossyLog>"
     << "<su_gapCutGapMoss>0.45</su_gapCutGapMoss>"
     << "<su_gapCutOrganic>0.34</su_gapCutOrganic>"
     << "<su_gapCutFreshLog>0.09</su_gapCutFreshLog>"
     << "<su_gapCutMossyLog>0.0</su_gapCutMossyLog>"
     << "<su_clearCutGapMoss>0.36</su_clearCutGapMoss>"
     << "<su_clearCutOrganic>0.34</su_clearCutOrganic>"
     << "<su_clearCutFreshLog>0.17</su_clearCutFreshLog>"
     << "<su_clearCutMossyLog>0.01</su_clearCutMossyLog>"
     << "<su_propOfDeadFall>"
     << "<su_podfVal species=\"Species_6\">0.7</su_podfVal>"
     << "</su_propOfDeadFall>"
     << "<su_propOfFallUproot>"
     << "<su_pofuVal species=\"Species_6\">0.5</su_pofuVal>"
     << "</su_propOfFallUproot>"
     << "<su_rootTipupFactor>3.0</su_rootTipupFactor>"
     << "</substrate>"
     << "<disperse>"
     << "<di_minDbhForReproduction>"
     << "<di_mdfrVal species=\"Species_6\">15.0</di_mdfrVal>"
     << "</di_minDbhForReproduction>"
     << "<di_weibullCanopySTR>"
     << "<di_wcsVal species=\"Species_6\">19.325</di_wcsVal>"
     << "</di_weibullCanopySTR>"
     << "<di_weibullCanopyBeta>"
     << "<di_wcbVal species=\"Species_6\">2.0</di_wcbVal>"
     << "</di_weibullCanopyBeta>"
     << "<di_weibullCanopyDispersal>"
     << "<di_wcdVal species=\"Species_6\">1.76E-4</di_wcdVal>"
     << "</di_weibullCanopyDispersal>"
     << "<di_weibullCanopyTheta>"
     << "<di_wctVal species=\"Species_6\">3.0</di_wctVal>"
     << "</di_weibullCanopyTheta>"
     << "<di_weibullGapSTR>"
     << "<di_wgsVal species=\"Species_6\">14.894</di_wgsVal>"
     << "</di_weibullGapSTR>"
     << "<di_weibullGapBeta>"
     << "<di_wgbVal species=\"Species_6\">2.3</di_wgbVal>"
     << "</di_weibullGapBeta>"
     << "<di_weibullGapDispersal>"
     << "<di_wgdVal species=\"Species_6\">1.32E-4</di_wgdVal>"
     << "</di_weibullGapDispersal>"
     << "<di_weibullGapTheta>"
     << "<di_wgtVal species=\"Species_6\">3.0</di_wgtVal>"
     << "</di_weibullGapTheta>"
     << "<di_maxGapDensity>0</di_maxGapDensity>"
     << "<di_gapFunction>"
     << "<di_gfVal species=\"Species_6\">0</di_gfVal>"
     << "</di_gapFunction>"
     << "<di_canopyFunction>"
     << "<di_cfVal species=\"Species_6\">0</di_cfVal>"
     << "</di_canopyFunction>"
     << "<di_seedDistributionMethod>0</di_seedDistributionMethod>"
     << "</disperse>"
     << "<mortality>"
     << "<mo_nonSizeDepMort>"
     << "<mo_nsdmVal species=\"Species_6\">0.0</mo_nsdmVal>"
     << "</mo_nonSizeDepMort>"
     << "</mortality>"
     << "<establishment>"
     << "<es_gapMossCanopyFav>"
     << "<es_gmcfVal species=\"Species_6\">0.129</es_gmcfVal>"
     << "</es_gapMossCanopyFav>"
     << "<es_organicsCanopyFav>"
     << "<es_ocfVal species=\"Species_6\">0.983</es_ocfVal>"
     << "</es_organicsCanopyFav>"
     << "<es_freshLogCanopyFav>"
     << "<es_flcfVal species=\"Species_6\">0.353</es_flcfVal>"
     << "</es_freshLogCanopyFav>"
     << "<es_mossyLogCanopyFav>"
     << "<es_mlcfVal species=\"Species_6\">0.058</es_mlcfVal>"
     << "</es_mossyLogCanopyFav>"
     << "<es_canopyMossCanopyFav>"
     << "<es_cmcfVal species=\"Species_6\">0.009</es_cmcfVal>"
     << "</es_canopyMossCanopyFav>"
     << "<es_maxGapDensity>0</es_maxGapDensity>"
     << "<es_useSeedlingEfficiencyRoutine>0</es_useSeedlingEfficiencyRoutine>"
     << "</establishment>"
     << "</paramFile>";

  oOut.close();

  return cFileString;
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile1()
/////////////////////////////////////////////////////////////////////////////
char * clTestEstablishment::WriteXMLFile1()
{
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>1</timesteps>"
       << "<yearsPerTimestep>5</yearsPerTimestep>"
       << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>50.0</plot_lenX>"
       << "<plot_lenY>50.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "<tr_species speciesName=\"Species_2\"/>"
       << "<tr_species speciesName=\"Species_3\"/>"
       << "<tr_species speciesName=\"Species_4\"/>"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_2\">39.54</tr_chVal>"
       << "<tr_chVal species=\"Species_3\">40.0</tr_chVal>"
       << "<tr_chVal species=\"Species_4\">39.48</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_canopyRadiusToDBH>"
       << "<tr_crtdVal species=\"Species_1\">0.0549</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_2\">0.0614</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_3\">0.0242</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_4\">0.0549</tr_crtdVal>"
       << "</tr_canopyRadiusToDBH>"
       << "<tr_canopyRadiusExponent>"
       << "<tr_creVal species=\"Species_1\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_2\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_3\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_4\">1.0</tr_creVal>"
       << "</tr_canopyRadiusExponent>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_2\">0.5944</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_3\">0.7059</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_4\">0.5944</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_fractOfHeightInCanopy>"
       << "<tr_fohicVal species=\"Species_1\">0.389</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_2\">0.368</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_3\">0.464</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_4\">0.368</tr_fohicVal>"
       << "</tr_fractOfHeightInCanopy>"
       << "<tr_canopyDepthExponent>"
       << "<tr_cdeVal species=\"Species_1\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_2\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_3\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_4\">1.0</tr_cdeVal>"
       << "</tr_canopyDepthExponent>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_2\">0.0269</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_3\">0.02871</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_4\">0.0269</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_2\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_3\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_4\">10.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_2\">0.0241</tr_soahVal>"
       << "<tr_soahVal species=\"Species_3\">0.0263</tr_soahVal>"
       << "<tr_soahVal species=\"Species_4\">0.0241</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "<tr_treemap>"
       << "<tm_treeSettings sp=\"Species_4\" tp=\"3\">"
       << "<tm_floatCodes>"
       << "<tm_floatCode label=\"X\">0</tm_floatCode>"
       << "<tm_floatCode label=\"Y\">1</tm_floatCode>"
       << "<tm_floatCode label=\"DBH\">2</tm_floatCode>"
       << "</tm_floatCodes>"
       << "</tm_treeSettings>"
       << "<tree sp=\"3\" tp=\"3\">"
       << "<fl c=\"0\">1</fl>"
       << "<fl c=\"1\">1</fl>"
       << "<fl c=\"2\">20</fl>"
       << "</tree>"
       << "<tree sp=\"3\" tp=\"3\">"
       << "<fl c=\"0\">1</fl>"
       << "<fl c=\"1\">11</fl>"
       << "<fl c=\"2\">20</fl>"
       << "</tree>"
       << "<tree sp=\"3\" tp=\"3\">"
       << "<fl c=\"0\">1</fl>"
       << "<fl c=\"1\">21</fl>"
       << "<fl c=\"2\">20</fl>"
       << "</tree>"
       << "<tree sp=\"3\" tp=\"3\">"
       << "<fl c=\"0\">1</fl>"
       << "<fl c=\"1\">31</fl>"
       << "<fl c=\"2\">20</fl>"
       << "</tree>"
       << "<tree sp=\"3\" tp=\"3\">"
       << "<fl c=\"0\">1</fl>"
       << "<fl c=\"1\">41</fl>"
       << "<fl c=\"2\">20</fl>"
       << "</tree>"
       << "<tree sp=\"3\" tp=\"3\">"
       << "<fl c=\"0\">11</fl>"
       << "<fl c=\"1\">1</fl>"
       << "<fl c=\"2\">20</fl>"
       << "</tree>"
       << "<tree sp=\"3\" tp=\"3\">"
       << "<fl c=\"0\">11</fl>"
       << "<fl c=\"1\">11</fl>"
       << "<fl c=\"2\">20</fl>"
       << "</tree>"
       << "<tree sp=\"3\" tp=\"3\">"
       << "<fl c=\"0\">11</fl>"
       << "<fl c=\"1\">21</fl>"
       << "<fl c=\"2\">20</fl>"
       << "</tree>"
       << "<tree sp=\"3\" tp=\"3\">"
       << "<fl c=\"0\">11</fl>"
       << "<fl c=\"1\">31</fl>"
       << "<fl c=\"2\">20</fl>"
       << "</tree>"
       << "<tree sp=\"3\" tp=\"3\">"
       << "<fl c=\"0\">11</fl>"
       << "<fl c=\"1\">41</fl>"
       << "<fl c=\"2\">20</fl>"
       << "</tree>"
       << "</tr_treemap>"
       << "</trees>"
       //Generate the substrate proportions map
       << "<grid gridName=\"Substrate\">"
       << "<ma_floatCodes>"
       << "<ma_floatCode label=\"gapmoss\">0</ma_floatCode>"
       << "<ma_floatCode label=\"canmoss\">1</ma_floatCode>"
       << "<ma_floatCode label=\"organic\">2</ma_floatCode>"
       << "<ma_floatCode label=\"freshlog\">3</ma_floatCode>"
       << "<ma_floatCode label=\"mossylog\">4</ma_floatCode>"
       << "</ma_floatCodes>"
       << "<ma_lengthXCells>10</ma_lengthXCells>"
       << "<ma_lengthYCells>10</ma_lengthYCells>";

  //Auto-generate grid values
  int iNumX = 5, iNumY = 5, iX, iY;
  float fGapMoss, fOrganic, fFreshLog, fMossyLog;
  for (iX = 0; iX < iNumX; iX++) {
    for (iY = 0; iY < iNumY; iY++) {
      fGapMoss = 0.3 - ((iX + iY)*0.01);
      fOrganic = (iX + iY)*0.02;
      fFreshLog = 0.1 + ((iX + iY)*0.01);
      fMossyLog = 0.2 + ((iX + iY)*0.01);
      oOut << "<ma_v x=\"" << iX << "\" y=\"" << iY << "\">"
           << "<fl c=\"0\">" << fGapMoss << "</fl>"
           << "<fl c=\"1\">" << 1 - (fGapMoss+fOrganic+fFreshLog+fMossyLog) << "</fl>"
           << "<fl c=\"2\">" << fOrganic << "</fl>"
           << "<fl c=\"3\">" << fFreshLog << "</fl>"
           << "<fl c=\"4\">" << fMossyLog << "</fl>"
           << "</ma_v>";
    }
  }
  oOut << "</grid>";

  //Auto-generate seed grid values
  oOut << "<grid gridName=\"Dispersed Seeds\">"
       << "<ma_floatCodes>"
       << "<ma_floatCode label=\"seeds_0\">0</ma_floatCode>"
       << "<ma_floatCode label=\"seeds_1\">1</ma_floatCode>"
       << "<ma_floatCode label=\"seeds_2\">2</ma_floatCode>"
       << "<ma_floatCode label=\"seeds_3\">3</ma_floatCode>"
       << "</ma_floatCodes>"
       << "<ma_intCodes>"
       << "<ma_intCode label=\"count\">0</ma_intCode>"
       << "</ma_intCodes>"
       << "<ma_boolCodes>"
       << "<ma_boolCode label=\"Is Gap\">0</ma_boolCode>"
       << "</ma_boolCodes>"
       << "<ma_lengthXCells>10</ma_lengthXCells>"
       << "<ma_lengthYCells>10</ma_lengthYCells>";

  for (iX = 0; iX < iNumX; iX++) {
    for (iY = 0; iY < iNumY; iY++) {
      oOut << "<ma_v x=\"" << iX << "\" y=\"" << iY << "\">"
           << "<fl c=\"0\">" << (iX + iY) << "</fl>"
           << "<fl c=\"1\">" << 2*(iX + iY) << "</fl>"
           << "<fl c=\"2\">" << 3*(iX + iY) << "</fl>"
           << "</ma_v>";
    }
  }
  oOut << "</grid>"

       << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>adultstochasticmort</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_3\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_4\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>substrate</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_3\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_4\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>gap disperse</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_3\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>no gap no substrate establishment</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Seed\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>no gap substrate establishment</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_2\" type=\"Seed\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>gap and substrate establishment</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_3\" type=\"Seed\"/>"
       << "</behavior>"
       << "</behaviorList>"
       << "<substrate>"
       << "<su_gapmossDecayAlpha>-5.1E-4</su_gapmossDecayAlpha>"
       << "<su_gapmossDecayBeta>4.4</su_gapmossDecayBeta>"
       << "<su_organicDecayAlpha>-7.0E-4</su_organicDecayAlpha>"
       << "<su_organicDecayBeta>4.3</su_organicDecayBeta>"
       << "<su_freshLogDecayAlpha>-0.05</su_freshLogDecayAlpha>"
       << "<su_freshLogDecayBeta>6.5</su_freshLogDecayBeta>"
       << "<su_mossyLogToCanMoss>0.55</su_mossyLogToCanMoss>"
       << "<su_freshLogFromCanMoss>0.0080</su_freshLogFromCanMoss>"
       << "<su_maxNumberDecayYears>10</su_maxNumberDecayYears>"
       << "<su_initialGapMoss>0.0</su_initialGapMoss>"
       << "<su_initialOrganic>0.0</su_initialOrganic>"
       << "<su_initialFreshLog>0.01</su_initialFreshLog>"
       << "<su_initialMossyLog>0.11</su_initialMossyLog>"
       << "<su_partialCutGapMoss>0.17</su_partialCutGapMoss>"
       << "<su_partialCutOrganic>0.15</su_partialCutOrganic>"
       << "<su_partialCutFreshLog>0.06</su_partialCutFreshLog>"
       << "<su_partialCutMossyLog>0.04</su_partialCutMossyLog>"
       << "<su_gapCutGapMoss>0.45</su_gapCutGapMoss>"
       << "<su_gapCutOrganic>0.34</su_gapCutOrganic>"
       << "<su_gapCutFreshLog>0.09</su_gapCutFreshLog>"
       << "<su_gapCutMossyLog>0.0</su_gapCutMossyLog>"
       << "<su_clearCutGapMoss>0.36</su_clearCutGapMoss>"
       << "<su_clearCutOrganic>0.34</su_clearCutOrganic>"
       << "<su_clearCutFreshLog>0.17</su_clearCutFreshLog>"
       << "<su_clearCutMossyLog>0.01</su_clearCutMossyLog>"
       << "<su_propOfDeadFall>"
       << "<su_podfVal species=\"Species_1\">0.7</su_podfVal>"
       << "<su_podfVal species=\"Species_2\">0.7</su_podfVal>"
       << "<su_podfVal species=\"Species_3\">0.7</su_podfVal>"
       << "<su_podfVal species=\"Species_4\">0.1</su_podfVal>"
       << "</su_propOfDeadFall>"
       << "<su_propOfFallUproot>"
       << "<su_pofuVal species=\"Species_1\">0.5</su_pofuVal>"
       << "<su_pofuVal species=\"Species_2\">0.5</su_pofuVal>"
       << "<su_pofuVal species=\"Species_3\">0.5</su_pofuVal>"
       << "<su_pofuVal species=\"Species_4\">0.5</su_pofuVal>"
       << "</su_propOfFallUproot>"
       << "<su_areaOrgExposed>3.0</su_areaOrgExposed>"
       << "<su_rootTipupFactor>3.0</su_rootTipupFactor>"
       << "</substrate>"
       << "<disperse>"
       << "<di_minDbhForReproduction>"
       << "<di_mdfrVal species=\"Species_1\">15.0</di_mdfrVal>"
       << "<di_mdfrVal species=\"Species_2\">15.0</di_mdfrVal>"
       << "<di_mdfrVal species=\"Species_3\">15.0</di_mdfrVal>"
       << "</di_minDbhForReproduction>"
       << "<di_weibullCanopySTR>"
       << "<di_wcsVal species=\"Species_1\">19.325</di_wcsVal>"
       << "<di_wcsVal species=\"Species_2\">19.325</di_wcsVal>"
       << "<di_wcsVal species=\"Species_3\">12.257</di_wcsVal>"
       << "</di_weibullCanopySTR>"
       << "<di_weibullCanopyBeta>"
       << "<di_wcbVal species=\"Species_1\">2.0</di_wcbVal>"
       << "<di_wcbVal species=\"Species_2\">2.0</di_wcbVal>"
       << "<di_wcbVal species=\"Species_3\">2.2</di_wcbVal>"
       << "</di_weibullCanopyBeta>"
       << "<di_weibullCanopyDispersal>"
       << "<di_wcdVal species=\"Species_1\">1.76E-4</di_wcdVal>"
       << "<di_wcdVal species=\"Species_2\">1.76E-4</di_wcdVal>"
       << "<di_wcdVal species=\"Species_3\">9.61E-5</di_wcdVal>"
       << "</di_weibullCanopyDispersal>"
       << "<di_weibullCanopyTheta>"
       << "<di_wctVal species=\"Species_1\">3.0</di_wctVal>"
       << "<di_wctVal species=\"Species_2\">3.0</di_wctVal>"
       << "<di_wctVal species=\"Species_3\">3.0</di_wctVal>"
       << "</di_weibullCanopyTheta>"
       << "<di_weibullGapSTR>"
       << "<di_wgsVal species=\"Species_1\">14.894</di_wgsVal>"
       << "<di_wgsVal species=\"Species_2\">6.628</di_wgsVal>"
       << "<di_wgsVal species=\"Species_3\">6.628</di_wgsVal>"
       << "</di_weibullGapSTR>"
       << "<di_weibullGapBeta>"
       << "<di_wgbVal species=\"Species_1\">2.3</di_wgbVal>"
       << "<di_wgbVal species=\"Species_2\">2.2</di_wgbVal>"
       << "<di_wgbVal species=\"Species_3\">2.2</di_wgbVal>"
       << "</di_weibullGapBeta>"
       << "<di_weibullGapDispersal>"
       << "<di_wgdVal species=\"Species_1\">1.32E-4</di_wgdVal>"
       << "<di_wgdVal species=\"Species_2\">6.93E-6</di_wgdVal>"
       << "<di_wgdVal species=\"Species_3\">6.93E-6</di_wgdVal>"
       << "</di_weibullGapDispersal>"
       << "<di_weibullGapTheta>"
       << "<di_wgtVal species=\"Species_1\">3.0</di_wgtVal>"
       << "<di_wgtVal species=\"Species_2\">3.0</di_wgtVal>"
       << "<di_wgtVal species=\"Species_3\">3.0</di_wgtVal>"
       << "</di_weibullGapTheta>"
       << "<di_maxGapDensity>0</di_maxGapDensity>"
       << "<di_gapFunction>"
       << "<di_gfVal species=\"Species_1\">0</di_gfVal>"
       << "<di_gfVal species=\"Species_2\">0</di_gfVal>"
       << "<di_gfVal species=\"Species_3\">0</di_gfVal>"
       << "</di_gapFunction>"
       << "<di_canopyFunction>"
       << "<di_cfVal species=\"Species_1\">0</di_cfVal>"
       << "<di_cfVal species=\"Species_2\">0</di_cfVal>"
       << "<di_cfVal species=\"Species_3\">0</di_cfVal>"
       << "</di_canopyFunction>"
       << "<di_seedDistributionMethod>0</di_seedDistributionMethod>"
       << "</disperse>"
       << "<mortality>"
       << "<mo_nonSizeDepMort>"
       << "<mo_nsdmVal species=\"Species_1\">0.0</mo_nsdmVal>"
       << "<mo_nsdmVal species=\"Species_2\">0.0</mo_nsdmVal>"
       << "<mo_nsdmVal species=\"Species_3\">0.0</mo_nsdmVal>"
       << "<mo_nsdmVal species=\"Species_4\">0.0</mo_nsdmVal>"
       << "</mo_nonSizeDepMort>"
       << "</mortality>"
       << "<establishment>"
       << "<es_gapMossCanopyFav>"
       << "<es_gmcfVal species=\"Species_2\">0.129</es_gmcfVal>"
       << "<es_gmcfVal species=\"Species_3\">0.48</es_gmcfVal>"
       << "</es_gapMossCanopyFav>"
       << "<es_organicsCanopyFav>"
       << "<es_ocfVal species=\"Species_2\">0.983</es_ocfVal>"
       << "<es_ocfVal species=\"Species_3\">0.391</es_ocfVal>"
       << "</es_organicsCanopyFav>"
       << "<es_freshLogCanopyFav>"
       << "<es_flcfVal species=\"Species_2\">0.353</es_flcfVal>"
       << "<es_flcfVal species=\"Species_3\">0.424</es_flcfVal>"
       << "</es_freshLogCanopyFav>"
       << "<es_mossyLogCanopyFav>"
       << "<es_mlcfVal species=\"Species_2\">0.058</es_mlcfVal>"
       << "<es_mlcfVal species=\"Species_3\">0.024</es_mlcfVal>"
       << "</es_mossyLogCanopyFav>"
       << "<es_canopyMossCanopyFav>"
       << "<es_cmcfVal species=\"Species_2\">0.009</es_cmcfVal>"
       << "<es_cmcfVal species=\"Species_3\">0.01</es_cmcfVal>"
       << "</es_canopyMossCanopyFav>"
       << "<es_gapMossGapFav>"
       << "<es_gmgfVal species=\"Species_3\">0.213</es_gmgfVal>"
       << "</es_gapMossGapFav>"
       << "<es_organicsGapFav>"
       << "<es_ogfVal species=\"Species_3\">0.179</es_ogfVal>"
       << "</es_organicsGapFav>"
       << "<es_freshLogGapFav>"
       << "<es_flgfVal species=\"Species_3\">0.319</es_flgfVal>"
       << "</es_freshLogGapFav>"
       << "<es_mossyLogGapFav>"
       << "<es_mlgfVal species=\"Species_3\">0.048</es_mlgfVal>"
       << "</es_mossyLogGapFav>"
       << "<es_canopyMossGapFav>"
       << "<es_cmgfVal species=\"Species_3\">0.001</es_cmgfVal>"
       << "</es_canopyMossGapFav>"
       << "<es_maxGapDensity>0</es_maxGapDensity>"
       << "<es_useSeedlingEfficiencyRoutine>0</es_useSeedlingEfficiencyRoutine>"
       << "</establishment>"
       << "</paramFile>";

  oOut.close();

  return cFileString;
}
//---------------------------------------------------------------------------
