//---------------------------------------------------------------------------

#ifndef TestGrowthOrgH
#define TestGrowthOrgH

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>
#include <cppunit/TestSuite.h>

class clSimManager;

/**
* Tests the clGrowthOrg object.  This specifically makes sure that auto-update
* diameter growth and separate diameter-height growth updating work correctly.
* The individual growth behaviors are tested elsewhere.
*/
class clTestGrowthOrg : public CppUnit::TestFixture {

public:

  /**
  * Combines the testing into one behavior.
  */
  void TestGrowthOrg();


  /**
    Tests growth assignments.  It will cause a parameter file to be written,
    and then passed to a clSimManager object.  This object will read it.
    The trees will be retrieved and assigned GLIs, and then growth will be r
    run and the data validated.
  */
//  void TestGrowthAssignments();



  private:

  /**
  * Tests normal processing.  WriteXMLFile1() is used to create the file for
  * this run.  Then six trees are added - a seedling, sapling, and species of
  * each of Species 1 and Species 2.  Both the diameter and height of each
  * tree are set to known values.  Then growth is run.  The diameter and height
  * for each tree are checked against expected values.
  * @param p_oSimManager Sim manager object for doing the run.
  */
  void TestNormalProcessing(clSimManager *p_oSimManager);

  /**
  * Tests error processing.  WriteXMLErrorFile1() and WriteXMLErrorFile2() are
  * used to introduce runs with error conditions in growth setup.
  * @param p_oSimManager Sim manager object for doing the runs.
  */
  void TestErrorProcessing(clSimManager *p_oSimManager);

  /**
  * Writes a parameter file to direct testing.  This contains two species, with
  * either constant basal area or constant radial growth assigned, with both
  * auto-update and diameter and height increment setting done separately.
  *
  * @return Filename written.
  */
  char* WriteXMLFile1();

  /**
  * Writes a parameter file to direct testing for an intentionally triggered
  * error.  This file contains a tree species/type combo that has a diameter-
  * only growth behavior without a corresponding height-only growth behavior.
  *
  * @return Filename written.
  */
  char* WriteXMLErrorFile1();

  /**
  * Writes a parameter file to direct testing for an intentionally triggered
  * error.  This file contains a tree species/type combo that has a height-
  * only growth behavior without a corresponding diameter-only growth behavior.
  *
  * @return Filename written.
  */
  char* WriteXMLErrorFile2();



  /**
    Writes a parameter file to direct testing.  This contains all tree
    information and growth information, and a behavior list that includes all
    growth behaviors being tested.  It also has a tree map so we'll be dealing
    with a known list of trees.

    @return Filename written.
  */
//  char* WriteXMLFile1();

  /**
    Writes a parameter file to direct testing.  The only difference between
    this file and the one in WriteXMLFile1 is that it has a different number
    of years per timestep.

    @return Filename written.
  */
//  char* WriteXMLFile2();

  /**
    Verifies the results in the file written in WriteXMLFile1.

    @param oPop Tree population object which contains the trees.
  */
//  void VerifyXMLFile1(clTreePopulation *p_oPop);

  /**
    Verifies the results in the file written in WriteXMLFile2.

    @param oPop Tree population object which contains the trees.
  */
//  void VerifyXMLFile2(clTreePopulation *p_oPop);

  /**
    Assigns GLIs to the trees written in the tree map of WriteXMLFile1.
  */
//  void AssignGLIs(clTreePopulation *p_oPop);

};
//---------------------------------------------------------------------------
#endif
