//---------------------------------------------------------------------------
// TestLogisticGrowth.cpp
//---------------------------------------------------------------------------
#include "TestLogisticGrowth.h"
#include <fstream.h>
#include <math.h>
#include "TreePopulation.h"
#include "SimManager.h"
#include "BehaviorBase.h"

////////////////////////////////////////////////////////////////////////////
//  TestRelativeGrowth()
////////////////////////////////////////////////////////////////////////////
void clTestLogisticGrowth::TestLogisticGrowth() {
  clSimManager *p_oSimManager = new clSimManager(4, 2, "");

  Test1YearPerTimestep(p_oSimManager);
  Test3YearsPerTimestep(p_oSimManager);

  delete p_oSimManager;
}

////////////////////////////////////////////////////////////////////////////
//  Test1YearPerTimestep()
////////////////////////////////////////////////////////////////////////////
void clTestLogisticGrowth::Test1YearPerTimestep(clSimManager *p_oSimManager) {
  clPopulationBase * p_oTemp;
  clTreePopulation * p_oPop;
  clTree *p_oTree1, *p_oTree2, *p_oTree3, *p_oTree4, *p_oTree5, *p_oTree6,
     *p_oTree7, *p_oTree8;
  float fDiam, fHeight, fExpectedDiam, fExpectedHeight, fLight, fGrowth, fExpectedGrowth;

  //Load our test file and get our tree population pointers
  p_oSimManager->ReadFile( WriteXMLFile1() );
  p_oTemp = p_oSimManager->GetPopulationObject( "treepopulation" );
  p_oPop = ( clTreePopulation * ) p_oTemp;

  //Create our eight trees and set diameter, height, and GLI for each
  //Seedling, Species 1
  fDiam = 0.7;
  fHeight = 0.8;
  fLight = 10;
  p_oTree1 = p_oPop->CreateTree(1, 1, 0, clTreePopulation::seedling, fDiam);
  p_oTree1->SetValue(p_oPop->GetHeightCode(p_oTree1->GetSpecies(), p_oTree1->GetType()), fHeight, false, false);
  p_oTree1->SetValue(p_oPop->GetFloatDataCode("Light", p_oTree1->GetSpecies(), p_oTree1->GetType()), fLight, false, false);

  //Seedling, Species 1
  fDiam = 0.1;
  fHeight = 0.2;
  fLight = 90;
  p_oTree2 = p_oPop->CreateTree(2, 2, 0, clTreePopulation::seedling, fDiam);
  p_oTree2->SetValue(p_oPop->GetHeightCode(p_oTree2->GetSpecies(), p_oTree2->GetType()), fHeight, false, false);
  p_oTree2->SetValue(p_oPop->GetFloatDataCode("Light", p_oTree2->GetSpecies(), p_oTree2->GetType()), fLight, false, false);

  //Sapling, Species 1
  fDiam = 3.2032;
  fHeight = 5;
  fLight = 12;
  p_oTree3 = p_oPop->CreateTree(3, 3, 0, clTreePopulation::sapling, fDiam);
  p_oTree3->SetValue(p_oPop->GetHeightCode(p_oTree3->GetSpecies(), p_oTree3->GetType()), fHeight, false, false);
  p_oTree3->SetValue(p_oPop->GetFloatDataCode("Light", p_oTree3->GetSpecies(), p_oTree3->GetType()), fLight, false, false);

  //Sapling, Species 1
  fDiam = 6.4064;
  fHeight = 3;
  fLight = 100;
  p_oTree4 = p_oPop->CreateTree(4, 4, 0, clTreePopulation::sapling, fDiam);
  p_oTree4->SetValue(p_oPop->GetHeightCode(p_oTree4->GetSpecies(), p_oTree4->GetType()), fHeight, false, false);
  p_oTree4->SetValue(p_oPop->GetFloatDataCode("Light", p_oTree4->GetSpecies(), p_oTree4->GetType()), fLight, false, false);

  //Seedling, Species 2
  fDiam = 0.2;
  fHeight = 0.8;
  fLight = 0;
  p_oTree5 = p_oPop->CreateTree(5, 5, 1, clTreePopulation::seedling, fDiam);
  p_oTree5->SetValue(p_oPop->GetHeightCode(p_oTree5->GetSpecies(), p_oTree5->GetType()), fHeight, false, false);
  p_oTree5->SetValue(p_oPop->GetFloatDataCode("Light", p_oTree5->GetSpecies(), p_oTree5->GetType()), fLight, false, false);

  //Seedling, Species 2
  fDiam = 1.4;
  fHeight = 0.65;
  fLight = 95;
  p_oTree6 = p_oPop->CreateTree(6, 6, 1, clTreePopulation::seedling, fDiam);
  p_oTree6->SetValue(p_oPop->GetHeightCode(p_oTree6->GetSpecies(), p_oTree6->GetType()), fHeight, false, false);
  p_oTree6->SetValue(p_oPop->GetFloatDataCode("Light", p_oTree6->GetSpecies(), p_oTree6->GetType()), fLight, false, false);

  //Sapling, Species 2
  fDiam = 1.7832;
  fHeight = 4;
  fLight = 7;
  p_oTree7 = p_oPop->CreateTree(7, 7, 1, clTreePopulation::sapling, fDiam);
  p_oTree7->SetValue(p_oPop->GetHeightCode(p_oTree7->GetSpecies(), p_oTree7->GetType()), fHeight, false, false);
  p_oTree7->SetValue(p_oPop->GetFloatDataCode("Light", p_oTree7->GetSpecies(), p_oTree7->GetType()), fLight, false, false);

  //Sapling, Species 2
  fDiam = 4.1608;
  fHeight = 6.3;
  fLight = 100;
  p_oTree8 = p_oPop->CreateTree(8, 8, 1, clTreePopulation::sapling, fDiam);
  p_oTree8->SetValue(p_oPop->GetHeightCode(p_oTree8->GetSpecies(), p_oTree8->GetType()), fHeight, false, false);
  p_oTree8->SetValue(p_oPop->GetFloatDataCode("Light", p_oTree8->GetSpecies(), p_oTree8->GetType()), fLight, false, false);

  //Now run growth, but skip light
  int i;
  for (i = 1; i < p_oSimManager->GetNumberOfBehaviors(); i++) {
    clBehaviorBase *p_oBase = p_oSimManager->GetBehaviorObject(i);
    p_oBase->Action();
  }

  //Confirm all trees
  //Seedling, Species 1
  fExpectedDiam = 1.139522479;
  fExpectedHeight = 1.236742427;
  fExpectedGrowth = 2.197612397;
  p_oTree1->GetValue(p_oPop->GetDiam10Code(p_oTree1->GetSpecies(), p_oTree1->GetType()), &fDiam);
  p_oTree1->GetValue(p_oPop->GetHeightCode(p_oTree1->GetSpecies(), p_oTree1->GetType()), &fHeight);
  p_oTree1->GetValue(p_oPop->GetFloatDataCode("Growth", p_oTree1->GetSpecies(), p_oTree1->GetType()), &fGrowth);
  CPPUNIT_ASSERT(fabs(fExpectedDiam - fDiam) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedHeight - fHeight) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedGrowth - fGrowth) < 0.001);

  //Seedling, Species 1 (now sapling)
  fExpectedDiam = 1.609986707;
  fExpectedHeight = 1.703913847;
  fExpectedGrowth = 7.549933536;
  p_oTree2->GetValue(p_oPop->GetDiam10Code(p_oTree2->GetSpecies(), p_oTree2->GetType()), &fDiam);
  p_oTree2->GetValue(p_oPop->GetHeightCode(p_oTree2->GetSpecies(), p_oTree2->GetType()), &fHeight);
  p_oTree2->GetValue(p_oPop->GetFloatDataCode("Growth", p_oTree2->GetSpecies(), p_oTree2->GetType()), &fGrowth);
  CPPUNIT_ASSERT(fabs(fExpectedDiam - fDiam) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedHeight - fHeight) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedGrowth - fGrowth) < 0.001);

  //Sapling, Species 1
  fExpectedDiam = 4.26;
  fExpectedHeight = 5.323898437;
  fExpectedGrowth = 1.3;
  p_oTree3->GetValue(p_oPop->GetDiam10Code(p_oTree3->GetSpecies(), p_oTree3->GetType()), &fDiam);
  p_oTree3->GetValue(p_oPop->GetHeightCode(p_oTree3->GetSpecies(), p_oTree3->GetType()), &fHeight);
  p_oTree3->GetValue(p_oPop->GetFloatDataCode("Growth", p_oTree3->GetSpecies(), p_oTree3->GetType()), &fGrowth);
  CPPUNIT_ASSERT(fabs(fExpectedDiam - fDiam) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedHeight - fHeight) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedGrowth - fGrowth) < 0.001);

  //Sapling, Species 1
  fExpectedDiam = 8.26;
  fExpectedHeight = 3.950128344;
  fExpectedGrowth = 1.3;
  p_oTree4->GetValue(p_oPop->GetDiam10Code(p_oTree4->GetSpecies(), p_oTree4->GetType()), &fDiam);
  p_oTree4->GetValue(p_oPop->GetHeightCode(p_oTree4->GetSpecies(), p_oTree4->GetType()), &fHeight);
  p_oTree4->GetValue(p_oPop->GetFloatDataCode("Growth", p_oTree4->GetSpecies(), p_oTree4->GetType()), &fGrowth);
  CPPUNIT_ASSERT(fabs(fExpectedDiam - fDiam) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedHeight - fHeight) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedGrowth - fGrowth) < 0.001);

  //Seedling, Species 2
  fExpectedDiam = 0.436731602;
  fExpectedHeight = 1.121568468;
  fExpectedGrowth = 1.18365801;
  p_oTree5->GetValue(p_oPop->GetDiam10Code(p_oTree5->GetSpecies(), p_oTree5->GetType()), &fDiam);
  p_oTree5->GetValue(p_oPop->GetHeightCode(p_oTree5->GetSpecies(), p_oTree5->GetType()), &fHeight);
  p_oTree5->GetValue(p_oPop->GetFloatDataCode("Growth", p_oTree5->GetSpecies(), p_oTree5->GetType()), &fGrowth);
  CPPUNIT_ASSERT(fabs(fExpectedDiam - fDiam) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedHeight - fHeight) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedGrowth - fGrowth) < 0.001);

  //Seedling, Species 2 (now sapling)
  fExpectedDiam = 2.594315968;
  fExpectedHeight = 2.566189352;
  fExpectedGrowth = 5.97157984;
  p_oTree6->GetValue(p_oPop->GetDiam10Code(p_oTree6->GetSpecies(), p_oTree6->GetType()), &fDiam);
  p_oTree6->GetValue(p_oPop->GetHeightCode(p_oTree6->GetSpecies(), p_oTree6->GetType()), &fHeight);
  p_oTree6->GetValue(p_oPop->GetFloatDataCode("Growth", p_oTree6->GetSpecies(), p_oTree6->GetType()), &fGrowth);
  CPPUNIT_ASSERT(fabs(fExpectedDiam - fDiam) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedHeight - fHeight) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedGrowth - fGrowth) < 0.001);

  //Sapling, Species 2
  fExpectedDiam = 3.294582708;
  fExpectedHeight = 3.110507789;
  fExpectedGrowth = 1.472913539;
  p_oTree7->GetValue(p_oPop->GetDiam10Code(p_oTree7->GetSpecies(), p_oTree7->GetType()), &fDiam);
  p_oTree7->GetValue(p_oPop->GetHeightCode(p_oTree7->GetSpecies(), p_oTree7->GetType()), &fHeight);
  p_oTree7->GetValue(p_oPop->GetFloatDataCode("Growth", p_oTree7->GetSpecies(), p_oTree7->GetType()), &fGrowth);
  CPPUNIT_ASSERT(fabs(fExpectedDiam - fDiam) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedHeight - fHeight) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedGrowth - fGrowth) < 0.001);

  //Sapling, Species 2
  fExpectedDiam = 8.216299445;
  fExpectedHeight = 5.590474311;
  fExpectedGrowth = 6.081497223;
  p_oTree8->GetValue(p_oPop->GetDiam10Code(p_oTree8->GetSpecies(), p_oTree8->GetType()), &fDiam);
  p_oTree8->GetValue(p_oPop->GetHeightCode(p_oTree8->GetSpecies(), p_oTree8->GetType()), &fHeight);
  p_oTree8->GetValue(p_oPop->GetFloatDataCode("Growth", p_oTree8->GetSpecies(), p_oTree8->GetType()), &fGrowth);
  CPPUNIT_ASSERT(fabs(fExpectedDiam - fDiam) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedHeight - fHeight) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedGrowth - fGrowth) < 0.001);
}

////////////////////////////////////////////////////////////////////////////
//  Test3YearsPerTimestep()
////////////////////////////////////////////////////////////////////////////
void clTestLogisticGrowth::Test3YearsPerTimestep(clSimManager *p_oSimManager) {
  clPopulationBase * p_oTemp;
  clTreePopulation * p_oPop;
  clTree *p_oTree1, *p_oTree2, *p_oTree3, *p_oTree4, *p_oTree5, *p_oTree6,
     *p_oTree7, *p_oTree8;
  float fDiam, fHeight, fExpectedDiam, fExpectedHeight, fLight, fGrowth, fExpectedGrowth;

  //Load our test file and get our tree population pointers
  p_oSimManager->ReadFile( WriteXMLFile2() );
  p_oTemp = p_oSimManager->GetPopulationObject( "treepopulation" );
  p_oPop = ( clTreePopulation * ) p_oTemp;

  //Create our eight trees and set diameter, height, and GLI for each
  //Seedling, Species 1
  fDiam = 0.7;
  fHeight = 0.8;
  fLight = 10;
  p_oTree1 = p_oPop->CreateTree(1, 1, 0, clTreePopulation::seedling, fDiam);
  p_oTree1->SetValue(p_oPop->GetHeightCode(p_oTree1->GetSpecies(), p_oTree1->GetType()), fHeight, false, false);
  p_oTree1->SetValue(p_oPop->GetFloatDataCode("Light", p_oTree1->GetSpecies(), p_oTree1->GetType()), fLight, false, false);

  //Seedling, Species 1
  fDiam = 0.1;
  fHeight = 0.2;
  fLight = 90;
  p_oTree2 = p_oPop->CreateTree(2, 2, 0, clTreePopulation::seedling, fDiam);
  p_oTree2->SetValue(p_oPop->GetHeightCode(p_oTree2->GetSpecies(), p_oTree2->GetType()), fHeight, false, false);
  p_oTree2->SetValue(p_oPop->GetFloatDataCode("Light", p_oTree2->GetSpecies(), p_oTree2->GetType()), fLight, false, false);

  //Sapling, Species 1
  fDiam = 3.2032;
  fHeight = 5;
  fLight = 12;
  p_oTree3 = p_oPop->CreateTree(3, 3, 0, clTreePopulation::sapling, fDiam);
  p_oTree3->SetValue(p_oPop->GetHeightCode(p_oTree3->GetSpecies(), p_oTree3->GetType()), fHeight, false, false);
  p_oTree3->SetValue(p_oPop->GetFloatDataCode("Light", p_oTree3->GetSpecies(), p_oTree3->GetType()), fLight, false, false);

  //Sapling, Species 1
  fDiam = 6.4064;
  fHeight = 3;
  fLight = 100;
  p_oTree4 = p_oPop->CreateTree(4, 4, 0, clTreePopulation::sapling, fDiam);
  p_oTree4->SetValue(p_oPop->GetHeightCode(p_oTree4->GetSpecies(), p_oTree4->GetType()), fHeight, false, false);
  p_oTree4->SetValue(p_oPop->GetFloatDataCode("Light", p_oTree4->GetSpecies(), p_oTree4->GetType()), fLight, false, false);

  //Seedling, Species 2
  fDiam = 0.2;
  fHeight = 0.8;
  fLight = 0;
  p_oTree5 = p_oPop->CreateTree(5, 5, 1, clTreePopulation::seedling, fDiam);
  p_oTree5->SetValue(p_oPop->GetHeightCode(p_oTree5->GetSpecies(), p_oTree5->GetType()), fHeight, false, false);
  p_oTree5->SetValue(p_oPop->GetFloatDataCode("Light", p_oTree5->GetSpecies(), p_oTree5->GetType()), fLight, false, false);

  //Seedling, Species 2
  fDiam = 1.4;
  fHeight = 0.65;
  fLight = 95;
  p_oTree6 = p_oPop->CreateTree(6, 6, 1, clTreePopulation::seedling, fDiam);
  p_oTree6->SetValue(p_oPop->GetHeightCode(p_oTree6->GetSpecies(), p_oTree6->GetType()), fHeight, false, false);
  p_oTree6->SetValue(p_oPop->GetFloatDataCode("Light", p_oTree6->GetSpecies(), p_oTree6->GetType()), fLight, false, false);

  //Sapling, Species 2
  fDiam = 1.7832;
  fHeight = 4;
  fLight = 7;
  p_oTree7 = p_oPop->CreateTree(7, 7, 1, clTreePopulation::sapling, fDiam);
  p_oTree7->SetValue(p_oPop->GetHeightCode(p_oTree7->GetSpecies(), p_oTree7->GetType()), fHeight, false, false);
  p_oTree7->SetValue(p_oPop->GetFloatDataCode("Light", p_oTree7->GetSpecies(), p_oTree7->GetType()), fLight, false, false);

  //Sapling, Species 2
  fDiam = 4.1608;
  fHeight = 6.3;
  fLight = 100;
  p_oTree8 = p_oPop->CreateTree(8, 8, 1, clTreePopulation::sapling, fDiam);
  p_oTree8->SetValue(p_oPop->GetHeightCode(p_oTree8->GetSpecies(), p_oTree8->GetType()), fHeight, false, false);
  p_oTree8->SetValue(p_oPop->GetFloatDataCode("Light", p_oTree8->GetSpecies(), p_oTree8->GetType()), fLight, false, false);

  //Now run growth, but skip light
  int i;
  for (i = 1; i < p_oSimManager->GetNumberOfBehaviors(); i++) {
    clBehaviorBase *p_oBase = p_oSimManager->GetBehaviorObject(i);
    p_oBase->Action();
  }

  //Confirm all trees
  //Seedling, Species 1 (now sapling)
  fExpectedDiam = 2.018567438;
  fExpectedHeight = 2.090788107;
  fExpectedGrowth = 2.197612397;
  p_oTree1->GetValue(p_oPop->GetDiam10Code(p_oTree1->GetSpecies(), p_oTree1->GetType()), &fDiam);
  p_oTree1->GetValue(p_oPop->GetHeightCode(p_oTree1->GetSpecies(), p_oTree1->GetType()), &fHeight);
  p_oTree1->GetValue(p_oPop->GetFloatDataCode("Growth", p_oTree1->GetSpecies(), p_oTree1->GetType()), &fGrowth);
  CPPUNIT_ASSERT(fabs(fExpectedDiam - fDiam) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedHeight - fHeight) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedGrowth - fGrowth) < 0.001);

  //Seedling, Species 1 (now sapling)
  fExpectedDiam = 4.629960122;
  fExpectedHeight = 4.48859682;
  fExpectedGrowth = 7.549933536;
  p_oTree2->GetValue(p_oPop->GetDiam10Code(p_oTree2->GetSpecies(), p_oTree2->GetType()), &fDiam);
  p_oTree2->GetValue(p_oPop->GetHeightCode(p_oTree2->GetSpecies(), p_oTree2->GetType()), &fHeight);
  p_oTree2->GetValue(p_oPop->GetFloatDataCode("Growth", p_oTree2->GetSpecies(), p_oTree2->GetType()), &fGrowth);
  CPPUNIT_ASSERT(fabs(fExpectedDiam - fDiam) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedHeight - fHeight) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedGrowth - fGrowth) < 0.001);

  //Sapling, Species 1
  fExpectedDiam = 4.78;
  fExpectedHeight = 5.971695311;
  fExpectedGrowth = 1.3;
  p_oTree3->GetValue(p_oPop->GetDiam10Code(p_oTree3->GetSpecies(), p_oTree3->GetType()), &fDiam);
  p_oTree3->GetValue(p_oPop->GetHeightCode(p_oTree3->GetSpecies(), p_oTree3->GetType()), &fHeight);
  p_oTree3->GetValue(p_oPop->GetFloatDataCode("Growth", p_oTree3->GetSpecies(), p_oTree3->GetType()), &fGrowth);
  CPPUNIT_ASSERT(fabs(fExpectedDiam - fDiam) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedHeight - fHeight) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedGrowth - fGrowth) < 0.001);

  //Sapling, Species 1
  fExpectedDiam = 8.78;
  fExpectedHeight = 5.850385032;
  fExpectedGrowth = 1.3;
  p_oTree4->GetValue(p_oPop->GetDiam10Code(p_oTree4->GetSpecies(), p_oTree4->GetType()), &fDiam);
  p_oTree4->GetValue(p_oPop->GetHeightCode(p_oTree4->GetSpecies(), p_oTree4->GetType()), &fHeight);
  p_oTree4->GetValue(p_oPop->GetFloatDataCode("Growth", p_oTree4->GetSpecies(), p_oTree4->GetType()), &fGrowth);
  CPPUNIT_ASSERT(fabs(fExpectedDiam - fDiam) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedHeight - fHeight) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedGrowth - fGrowth) < 0.001);

  //Seedling, Species 2
  fExpectedDiam = 0.910194806;
  fExpectedHeight = 1.764705404;
  fExpectedGrowth = 1.18365801;
  p_oTree5->GetValue(p_oPop->GetDiam10Code(p_oTree5->GetSpecies(), p_oTree5->GetType()), &fDiam);
  p_oTree5->GetValue(p_oPop->GetHeightCode(p_oTree5->GetSpecies(), p_oTree5->GetType()), &fHeight);
  p_oTree5->GetValue(p_oPop->GetFloatDataCode("Growth", p_oTree5->GetSpecies(), p_oTree5->GetType()), &fGrowth);
  CPPUNIT_ASSERT(fabs(fExpectedDiam - fDiam) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedHeight - fHeight) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedGrowth - fGrowth) < 0.001);

  //Seedling, Species 2
  fExpectedDiam = 4.982947904;
  fExpectedHeight = 6.398568056;
  fExpectedGrowth = 5.97157984;
  p_oTree6->GetValue(p_oPop->GetDiam10Code(p_oTree6->GetSpecies(), p_oTree6->GetType()), &fDiam);
  p_oTree6->GetValue(p_oPop->GetHeightCode(p_oTree6->GetSpecies(), p_oTree6->GetType()), &fHeight);
  p_oTree6->GetValue(p_oPop->GetFloatDataCode("Growth", p_oTree6->GetSpecies(), p_oTree6->GetType()), &fGrowth);
  CPPUNIT_ASSERT(fabs(fExpectedDiam - fDiam) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedHeight - fHeight) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedGrowth - fGrowth) < 0.001);

  //Sapling, Species 2
  fExpectedDiam = 3.883748124;
  fExpectedHeight = 3.416672275;
  fExpectedGrowth = 1.472913539;
  p_oTree7->GetValue(p_oPop->GetDiam10Code(p_oTree7->GetSpecies(), p_oTree7->GetType()), &fDiam);
  p_oTree7->GetValue(p_oPop->GetHeightCode(p_oTree7->GetSpecies(), p_oTree7->GetType()), &fHeight);
  p_oTree7->GetValue(p_oPop->GetFloatDataCode("Growth", p_oTree7->GetSpecies(), p_oTree7->GetType()), &fGrowth);
  CPPUNIT_ASSERT(fabs(fExpectedDiam - fDiam) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedHeight - fHeight) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedGrowth - fGrowth) < 0.001);

  //Sapling, Species 2
  fExpectedDiam = 10.64889833;
  fExpectedHeight = 6.753140614;
  fExpectedGrowth = 6.081497223;
  p_oTree8->GetValue(p_oPop->GetDiam10Code(p_oTree8->GetSpecies(), p_oTree8->GetType()), &fDiam);
  p_oTree8->GetValue(p_oPop->GetHeightCode(p_oTree8->GetSpecies(), p_oTree8->GetType()), &fHeight);
  p_oTree8->GetValue(p_oPop->GetFloatDataCode("Growth", p_oTree8->GetSpecies(), p_oTree8->GetType()), &fGrowth);
  CPPUNIT_ASSERT(fabs(fExpectedDiam - fDiam) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedHeight - fHeight) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedGrowth - fGrowth) < 0.001);
}


////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile1()
////////////////////////////////////////////////////////////////////////////
char* clTestLogisticGrowth::WriteXMLFile1() {
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  //Write file
  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>1</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>"
       << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>200.0</plot_lenX>"
       << "<plot_lenY>200.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\" />"
       << "<tr_species speciesName=\"Species_2\" />"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_2\">39.54</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_canopyRadiusToDBH>"
       << "<tr_crtdVal species=\"Species_1\">0.0549</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_2\">0.0614</tr_crtdVal>"
       << "</tr_canopyRadiusToDBH>"
       << "<tr_canopyRadiusExponent>"
       << "<tr_creVal species=\"Species_1\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_2\">1.0</tr_creVal>"
       << "</tr_canopyRadiusExponent>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_2\">0.5944</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_fractOfHeightInCanopy>"
       << "<tr_fohicVal species=\"Species_1\">0.389</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_2\">0.368</tr_fohicVal>"
       << "</tr_fractOfHeightInCanopy>"
       << "<tr_canopyDepthExponent>"
       << "<tr_cdeVal species=\"Species_1\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_2\">1.0</tr_cdeVal>"
       << "</tr_canopyDepthExponent>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_2\">0.0269</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_2\">10.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_2\">0.0241</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "</trees>"
       << "<behaviorList>"
       << "<behavior>"
       //Leave in one light so that lgm will be registerd
       << "<behaviorName>radiallight</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_2\" type=\"Seedling\"/>"
       << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>logistic growth diam only</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_2\" type=\"Seedling\"/>"
       << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>logistic growth height only</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_2\" type=\"Seedling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>constradialgrowth diam only</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>height incrementer</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>logistic growth</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
       << "</behavior>"
       << "</behaviorList>"
       << "<growth>"
       << "<gr_adultConstRadialInc>"
       << "<gr_acriVal species=\"Species_1\">1.3</gr_acriVal>"
       << "<gr_acriVal species=\"Species_2\">5.92</gr_acriVal>"
       << "</gr_adultConstRadialInc>"
       << "<gr_diamAsympGrowthAtFullLight>"
       << "<gr_dagaflVal species=\"Species_1\">7.88</gr_dagaflVal>"
       << "<gr_dagaflVal species=\"Species_2\">6.65</gr_dagaflVal>"
       << "</gr_diamAsympGrowthAtFullLight>"
       << "<gr_heightAsympGrowthAtFullLight>"
       << "<gr_hagaflVal species=\"Species_1\">321.8</gr_hagaflVal>"
       << "<gr_hagaflVal species=\"Species_2\">240.8</gr_hagaflVal>"
       << "</gr_heightAsympGrowthAtFullLight>"
       << "<gr_diamShape1b>"
       << "<gr_ds1bVal species=\"Species_1\">1.46</gr_ds1bVal>"
       << "<gr_ds1bVal species=\"Species_2\">1.53</gr_ds1bVal>"
       << "</gr_diamShape1b>"
       << "<gr_heightShape1b>"
       << "<gr_hs1bVal species=\"Species_1\">2.37</gr_hs1bVal>"
       << "<gr_hs1bVal species=\"Species_2\">1.87</gr_hs1bVal>"
       << "</gr_heightShape1b>"
       << "<gr_diamShape2c>"
       << "<gr_ds2cVal species=\"Species_1\">0.051</gr_ds2cVal>"
       << "<gr_ds2cVal species=\"Species_2\">0.039</gr_ds2cVal>"
       << "</gr_diamShape2c>"
       << "<gr_heightShape2c>"
       << "<gr_hs2cVal species=\"Species_1\">0.015</gr_hs2cVal>"
       << "<gr_hs2cVal species=\"Species_2\">0.034</gr_hs2cVal>"
       << "</gr_heightShape2c>"
       << "</growth>"
       << "</paramFile>";

  oOut.close();

  return cFileString;
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile2()
////////////////////////////////////////////////////////////////////////////
char* clTestLogisticGrowth::WriteXMLFile2() {
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  //Write file
  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
         << "<paramFile fileCode=\"06010101\">"
         << "<plot>"
         << "<timesteps>1</timesteps>"
         << "<yearsPerTimestep>3</yearsPerTimestep>"
         << "<randomSeed>1</randomSeed>"
         << "<plot_lenX>200.0</plot_lenX>"
         << "<plot_lenY>200.0</plot_lenY>"
         << "<plot_latitude>55.37</plot_latitude>"
         << "</plot>"
         << "<trees>"
         << "<tr_speciesList>"
         << "<tr_species speciesName=\"Species_1\" />"
         << "<tr_species speciesName=\"Species_2\" />"
         << "</tr_speciesList>"
         << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
         << "<tr_canopyHeight>"
         << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
         << "<tr_chVal species=\"Species_2\">39.54</tr_chVal>"
         << "</tr_canopyHeight>"
         << "<tr_canopyRadiusToDBH>"
         << "<tr_crtdVal species=\"Species_1\">0.0549</tr_crtdVal>"
         << "<tr_crtdVal species=\"Species_2\">0.0614</tr_crtdVal>"
         << "</tr_canopyRadiusToDBH>"
         << "<tr_canopyRadiusExponent>"
         << "<tr_creVal species=\"Species_1\">1.0</tr_creVal>"
         << "<tr_creVal species=\"Species_2\">1.0</tr_creVal>"
         << "</tr_canopyRadiusExponent>"
         << "<tr_conversionDiam10ToDBH>"
         << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
         << "<tr_cdtdVal species=\"Species_2\">0.5944</tr_cdtdVal>"
         << "</tr_conversionDiam10ToDBH>"
         << "<tr_fractOfHeightInCanopy>"
         << "<tr_fohicVal species=\"Species_1\">0.389</tr_fohicVal>"
         << "<tr_fohicVal species=\"Species_2\">0.368</tr_fohicVal>"
         << "</tr_fractOfHeightInCanopy>"
         << "<tr_canopyDepthExponent>"
         << "<tr_cdeVal species=\"Species_1\">1.0</tr_cdeVal>"
         << "<tr_cdeVal species=\"Species_2\">1.0</tr_cdeVal>"
         << "</tr_canopyDepthExponent>"
         << "<tr_slopeOfHeight-Diam10>"
         << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
         << "<tr_sohdVal species=\"Species_2\">0.0269</tr_sohdVal>"
         << "</tr_slopeOfHeight-Diam10>"
         << "<tr_minAdultDBH>"
         << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
         << "<tr_madVal species=\"Species_2\">10.0</tr_madVal>"
         << "</tr_minAdultDBH>"
         << "<tr_slopeOfAsymHeight>"
         << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
         << "<tr_soahVal species=\"Species_2\">0.0241</tr_soahVal>"
         << "</tr_slopeOfAsymHeight>"
         << "</trees>"
         << "<behaviorList>"
         << "<behavior>"
         //Leave in one light so that lgm will be registerd
         << "<behaviorName>radiallight</behaviorName>"
         << "<version>1</version>"
         << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
         << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
         << "<applyTo species=\"Species_2\" type=\"Seedling\"/>"
         << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
         << "</behavior>"
         << "<behavior>"
         << "<behaviorName>logistic growth diam only</behaviorName>"
         << "<version>1</version>"
         << "<applyTo species=\"Species_2\" type=\"Seedling\"/>"
         << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
         << "</behavior>"
         << "<behavior>"
         << "<behaviorName>logistic growth height only</behaviorName>"
         << "<version>1</version>"
         << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
         << "<applyTo species=\"Species_2\" type=\"Seedling\"/>"
         << "</behavior>"
         << "<behavior>"
         << "<behaviorName>constradialgrowth diam only</behaviorName>"
         << "<version>1</version>"
         << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
         << "</behavior>"
         << "<behavior>"
         << "<behaviorName>height incrementer</behaviorName>"
         << "<version>1</version>"
         << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
         << "</behavior>"
         << "<behavior>"
         << "<behaviorName>logistic growth</behaviorName>"
         << "<version>1</version>"
         << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
         << "</behavior>"
         << "</behaviorList>"
         << "<growth>"
         << "<gr_adultConstRadialInc>"
         << "<gr_acriVal species=\"Species_1\">1.3</gr_acriVal>"
         << "<gr_acriVal species=\"Species_2\">5.92</gr_acriVal>"
         << "</gr_adultConstRadialInc>"
         << "<gr_diamAsympGrowthAtFullLight>"
         << "<gr_dagaflVal species=\"Species_1\">7.88</gr_dagaflVal>"
         << "<gr_dagaflVal species=\"Species_2\">6.65</gr_dagaflVal>"
         << "</gr_diamAsympGrowthAtFullLight>"
         << "<gr_heightAsympGrowthAtFullLight>"
         << "<gr_hagaflVal species=\"Species_1\">321.8</gr_hagaflVal>"
         << "<gr_hagaflVal species=\"Species_2\">240.8</gr_hagaflVal>"
         << "</gr_heightAsympGrowthAtFullLight>"
         << "<gr_diamShape1b>"
         << "<gr_ds1bVal species=\"Species_1\">1.46</gr_ds1bVal>"
         << "<gr_ds1bVal species=\"Species_2\">1.53</gr_ds1bVal>"
         << "</gr_diamShape1b>"
         << "<gr_heightShape1b>"
         << "<gr_hs1bVal species=\"Species_1\">2.37</gr_hs1bVal>"
         << "<gr_hs1bVal species=\"Species_2\">1.87</gr_hs1bVal>"
         << "</gr_heightShape1b>"
         << "<gr_diamShape2c>"
         << "<gr_ds2cVal species=\"Species_1\">0.051</gr_ds2cVal>"
         << "<gr_ds2cVal species=\"Species_2\">0.039</gr_ds2cVal>"
         << "</gr_diamShape2c>"
         << "<gr_heightShape2c>"
         << "<gr_hs2cVal species=\"Species_1\">0.015</gr_hs2cVal>"
         << "<gr_hs2cVal species=\"Species_2\">0.034</gr_hs2cVal>"
         << "</gr_heightShape2c>"
         << "</growth>"
         << "</paramFile>";

  oOut.close();

  return cFileString;
}
