//---------------------------------------------------------------------------
// TestMortalityOrg.cpp
//---------------------------------------------------------------------------
#include "TestMortalityOrg.h"
#include "SimManager.h"
#include "TreePopulation.h"
#include <fstream.h>

/////////////////////////////////////////////////////////////////////////////
//  TestMortalityOrg()
/////////////////////////////////////////////////////////////////////////////
void clTestMortalityOrg::TestMortalityOrg() {
  clPopulationBase * p_oTemp;
  clTreePopulation * p_oPop;
  clTreeSearch *p_oAllTrees;
  clTree *p_oTree;
  int iDeadCode, i, j, iNumSpecies, iNumTypes;
  bool bDeadVal;

  //Create the sim manager and load our file
  clSimManager *p_oSimManager = new clSimManager(3, 2, "");
  p_oSimManager->ReadFile( WriteXMLFile1() );
  p_oTemp = p_oSimManager->GetPopulationObject( "treepopulation" );
  p_oPop = ( clTreePopulation * ) p_oTemp;
  iNumSpecies = p_oPop->GetNumberOfSpecies();
  iNumTypes = p_oPop->GetNumberOfTypes();

  //Verify that saplings and adults of species 2 have a "dead" data member
  //and no one else
  for (i = 0; i < iNumSpecies; i++) {
    for (j = 0; j < iNumTypes; j++) {
      iDeadCode = p_oPop->GetBoolDataCode("dead", i, j);
      if (1 == i && (clTreePopulation::sapling == j || clTreePopulation::adult == j)) {
        CPPUNIT_ASSERT(iDeadCode > -1);
      } else {
        CPPUNIT_ASSERT(iDeadCode == -1);
      }
    }
  }

  //Now make sure that no extra bools got thrown to the tree population
  //Try to register a bool with every species and type - we should get
  //an error every time
  for (i = 0; i < iNumSpecies; i++) {
    for (j = 0; j < iNumTypes; j++) {
      try {
        p_oPop->RegisterBool("test", i, j);
        CppUnit::Asserter::fail("TestMortalityOrg failed - extra bool data members were given to clTreePopulation.");
      } catch (modelErr err) {
        ;
      }
    }
  }

  //Put in 10 trees for saplings and adults of species 1 and 2
  for (i = 0; i < 10; i++) {
    p_oPop->CreateTree(i, i, 0, clTreePopulation::sapling, 9);
  }

  for (i = 0; i < 10; i++) {
    p_oPop->CreateTree(i, i, 0, clTreePopulation::adult, 19);
  }

  for (i = 0; i < 10; i++) {
    p_oPop->CreateTree(i, i, 1, clTreePopulation::sapling, 9);
  }

  for (i = 0; i < 10; i++) {
    p_oPop->CreateTree(i, i, 1, clTreePopulation::adult, 19);
  }

  //Run the model
  p_oSimManager->RunSim(1);

  //Count the trees that are left
  int ** p_iTreeCount = new int*[iNumSpecies];
  for (i = 0; i < iNumSpecies; i++) {
    p_iTreeCount[i] = new int[iNumTypes];
    for (j = 0; j < iNumTypes; j++) {
      p_iTreeCount[i][j] = 0;
    }
  }


  p_oAllTrees = p_oPop->Find("all");
  p_oTree = p_oAllTrees->NextTree();
  while (p_oTree) {
    p_iTreeCount[p_oTree->GetSpecies()][p_oTree->GetType()]++;
    p_oTree = p_oAllTrees->NextTree();
  }

  //There should be 10 of species 1 saplings and adults, and species 2 saplings
  for (i = 0; i < iNumSpecies; i++) {
    for (j = 0; j < iNumTypes; j++) {
      if ((1 == i && clTreePopulation::sapling == j) ||
         (0 == i && (clTreePopulation::sapling == j || clTreePopulation::adult == j))) {
        CPPUNIT_ASSERT(10 ==  p_iTreeCount[i][j]);
      } else {
        CPPUNIT_ASSERT(0 == p_iTreeCount[i][j]);
      }
    }
  }


  for (i = 0; i < iNumSpecies; i++) delete[] p_iTreeCount[i];
  delete[] p_iTreeCount;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile1()
/////////////////////////////////////////////////////////////////////////////
char * clTestMortalityOrg::WriteXMLFile1()
{
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">" << "<plot>"
       << "<timesteps>2</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>"
       << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>2000</plot_lenX>"
       << "<plot_lenY>200.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "<tr_species speciesName=\"Species_2\"/>"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_2\">39.48</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_canopyRadiusToDBH>"
       << "<tr_crtdVal species=\"Species_1\">0.0549</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_2\">0.0549</tr_crtdVal>"
       << "</tr_canopyRadiusToDBH>"
       << "<tr_canopyRadiusExponent>"
       << "<tr_creVal species=\"Species_1\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_2\">1.0</tr_creVal>"
       << "</tr_canopyRadiusExponent>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_2\">0.8008</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_fractOfHeightInCanopy>"
       << "<tr_fohicVal species=\"Species_1\">0.389</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_2\">0.389</tr_fohicVal>"
       << "</tr_fractOfHeightInCanopy>"
       << "<tr_canopyDepthExponent>"
       << "<tr_cdeVal species=\"Species_1\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_2\">1.0</tr_cdeVal>"
       << "</tr_canopyDepthExponent>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_2\">0.03418</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_2\">10.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_2\">0.0299</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "</trees>"
       << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>adultstochasticmort</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>juvstochasticmort</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>juvselfthin</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>removedead</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
       << "</behavior>"
       << "</behaviorList>"
       << "<mortality>"
       << "<mo_juvSelfThinSlope>"
       << "<mo_jstsVal species=\"Species_1\">0</mo_jstsVal>"
       << "<mo_jstsVal species=\"Species_2\">0</mo_jstsVal>"
       << "</mo_juvSelfThinSlope>"
       << "<mo_juvSelfThinIntercept>"
       << "<mo_jstiVal species=\"Species_1\">0</mo_jstiVal>"
       << "<mo_jstiVal species=\"Species_2\">0</mo_jstiVal>"
       << "</mo_juvSelfThinIntercept>"
       << "<mo_randomJuvenileMortality>"
       << "<mo_rjmVal species=\"Species_2\">0.0</mo_rjmVal>"
       << "</mo_randomJuvenileMortality>"
       << "<mo_nonSizeDepMort>"
       << "<mo_nsdmVal species=\"Species_2\">1.0</mo_nsdmVal>"
       << "</mo_nonSizeDepMort>"
       << "</mortality>"
       << "</paramFile>";

  oOut.close();

  return cFileString;
}

