//---------------------------------------------------------------------------
// TestNZSubstrate.cpp
//---------------------------------------------------------------------------
#include "TestNZSubstrate.h"
#include "GridBase.h"
#include "SimManager.h"
#include "BehaviorBase.h"
#include "TreePopulation.h"
#include <math.h>
#include <fstream.h>

/////////////////////////////////////////////////////////////////////////////
//  TestNZSubstrate()
/////////////////////////////////////////////////////////////////////////////
void clTestNZSubstrate::TestNZSubstrate()
{
  clSimManager * p_oSimManager = new clSimManager(3, 3, "");
  TestSubstrateGridSetup(p_oSimManager);
  TestSubstrateProcessing(p_oSimManager);
  TestTreeFall(p_oSimManager);
  TestSnagSubstrate(p_oSimManager);
  delete p_oSimManager;
}

/////////////////////////////////////////////////////////////////////////////
//  TestSnagSubstrate()
/////////////////////////////////////////////////////////////////////////////
void clTestNZSubstrate::TestSnagSubstrate(clSimManager *p_oSimManager)
{
  clPopulationBase *p_oTemp;
  clTreePopulation *p_oPop;
  clGridBase *p_oSubstrateGrid;
  clTree *p_oTree;
  float fExpectedFreshLog, fActual;
  int iX, iY, i;
  bool bDead = true;

  //Create and feed in the file
  p_oSimManager->ReadFile(WriteXMLFile10());
  //Grab our pointers
  p_oTemp = p_oSimManager->GetPopulationObject("treepopulation");
  p_oPop = (clTreePopulation*) p_oTemp;
  p_oSubstrateGrid = p_oSimManager->GetGridObject("NZ Substrate");

  //********************************
  //Timestep 1:  Just run.
  //********************************
  p_oSimManager->RunSim(1);

  //Cell 1
  iX = 0; iY = 0;
  fExpectedFreshLog = 0.01;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  //Cell 2
  iX = 0; iY = 1;
  fExpectedFreshLog = 0.01;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  //Cell 3
  iX = 0; iY = 1;
  fExpectedFreshLog = 0.01;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  //********************************
  //Timestep 2:
  //********************************
  //Create and then kill certain trees
  //Tree where y = 1
  p_oTree = p_oPop->CreateTree(1, 1, 0, clTreePopulation::snag, 20);
  p_oTree->SetValue(p_oPop->GetBoolDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), bDead);

  //Tree where y = 2
  p_oTree = p_oPop->CreateTree(1, 2, 0, clTreePopulation::snag, 40);
  p_oTree->SetValue(p_oPop->GetBoolDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), bDead);

  //Tree where y = 3
  p_oTree = p_oPop->CreateTree(1, 3, 0, clTreePopulation::snag, 55);
  p_oTree->SetValue(p_oPop->GetBoolDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), bDead);

  //Tree where y = 18
  p_oTree = p_oPop->CreateTree(1, 18, 0, clTreePopulation::snag, 15);
  p_oTree->SetValue(p_oPop->GetBoolDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), bDead);

  //Now run
  p_oSimManager->RunSim(1);

  //Cell 1
  iX = 0; iY = 0;
  fExpectedFreshLog = 0.260798425;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  //Cell 2
  iX = 0; iY = 1;
  fExpectedFreshLog = 0.009512294;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  //Cell 3
  iX = 0; iY = 1;
  fExpectedFreshLog = 0.009796152;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
}

/////////////////////////////////////////////////////////////////////////////
//  TestTreeFall()
/////////////////////////////////////////////////////////////////////////////
void clTestNZSubstrate::TestTreeFall(clSimManager *p_oSimManager)
{
  clPopulationBase *p_oTemp;
  clTreePopulation *p_oPop;
  clTreeSearch *p_oAllTrees;
  clTree *p_oTree;
  int iNumSp1Snags = 0, iNumSp2Snags = 0, iNumSp3Snags = 0, iSpecies, i;

  //Create and feed in the file
  p_oSimManager->ReadFile(WriteXMLFile8());
  //Grab our pointers
  p_oTemp = p_oSimManager->GetPopulationObject("treepopulation");
  p_oPop = (clTreePopulation*) p_oTemp;

  //Create 1000 of each species of tree, all adults
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(1, 1, 0, clTreePopulation::adult, 17);
  }
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(1, 1, 1, clTreePopulation::adult, 17);
  }
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(1, 1, 2, clTreePopulation::adult, 17);
  }

  //Run for 1 timestep
  p_oSimManager->RunSim(1);

  //Count up the trees left over.  They should all be snags.
  p_oAllTrees = p_oPop->Find("all");
  p_oTree = p_oAllTrees->NextTree();
  while (p_oTree) {

    //Make sure it's not alive
    CPPUNIT_ASSERT(clTreePopulation::snag == p_oTree->GetType());

    //Count it
    iSpecies = p_oTree->GetSpecies();
    if (0 == iSpecies) iNumSp1Snags++;
    else if (1 == iSpecies) iNumSp2Snags++;
    else if (2 == iSpecies) iNumSp3Snags++;

    p_oTree = p_oAllTrees->NextTree();
  }

  //Now make sure we have the number we expect left standing
  CPPUNIT_ASSERT(fabs(iNumSp1Snags - 800)/800 < 0.05);
  CPPUNIT_ASSERT(fabs(iNumSp2Snags - 500)/500 < 0.05);
  CPPUNIT_ASSERT(iNumSp3Snags == 1000);

  //Run another timestep and count them again
  iNumSp1Snags = 0;
  iNumSp2Snags = 0;
  iNumSp3Snags = 0;
  p_oSimManager->RunSim(1);
  p_oAllTrees = p_oPop->Find("all");
  p_oTree = p_oAllTrees->NextTree();
  while (p_oTree) {

    //Make sure it's not alive
    CPPUNIT_ASSERT(clTreePopulation::snag == p_oTree->GetType());

    //Count it
    iSpecies = p_oTree->GetSpecies();
    if (0 == iSpecies) iNumSp1Snags++;
    else if (1 == iSpecies) iNumSp2Snags++;
    else if (2 == iSpecies) iNumSp3Snags++;

    p_oTree = p_oAllTrees->NextTree();
  }

  //Now make sure we have the number we expect left standing
  CPPUNIT_ASSERT(fabs(iNumSp1Snags - 800)/800 < 0.05);
  CPPUNIT_ASSERT(fabs(iNumSp2Snags - 500)/500 < 0.05);
  CPPUNIT_ASSERT(iNumSp3Snags == 1000);

  //***************************
  // Repeat with longer timestep
  //***************************
  p_oSimManager->ReadFile(WriteXMLFile9());
  //Grab our pointers
  p_oTemp = p_oSimManager->GetPopulationObject("treepopulation");
  p_oPop = (clTreePopulation*) p_oTemp;

  //Create 1000 of each species of tree, all adults
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(1, 1, 0, clTreePopulation::adult, 17);
  }
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(1, 1, 1, clTreePopulation::adult, 17);
  }
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(1, 1, 2, clTreePopulation::adult, 17);
  }

  //Run for 1 timestep
  p_oSimManager->RunSim(1);

  //Count up the trees left over.  They should all be snags
  iNumSp1Snags = 0;
  iNumSp2Snags = 0;
  iNumSp3Snags = 0;
  p_oAllTrees = p_oPop->Find("all");
  p_oTree = p_oAllTrees->NextTree();
  while (p_oTree) {

    //Make sure it's not alive
    CPPUNIT_ASSERT(clTreePopulation::snag == p_oTree->GetType());

    //Count it
    iSpecies = p_oTree->GetSpecies();
    if (0 == iSpecies) iNumSp1Snags++;
    else if (1 == iSpecies) iNumSp2Snags++;
    else if (2 == iSpecies) iNumSp3Snags++;

    p_oTree = p_oAllTrees->NextTree();
  }


  //Now make sure we have the number we expect left standing
  CPPUNIT_ASSERT(fabs(iNumSp1Snags - 512)/512 < 0.05);
  CPPUNIT_ASSERT(fabs(iNumSp2Snags - 125)/125 < 0.05);
  CPPUNIT_ASSERT(iNumSp3Snags == 1000);

  //Run another timestep and count them again
  iNumSp1Snags = 0;
  iNumSp2Snags = 0;
  iNumSp3Snags = 0;
  p_oSimManager->RunSim(1);
  p_oAllTrees = p_oPop->Find("all");
  p_oTree = p_oAllTrees->NextTree();
  while (p_oTree) {

    //Make sure it's not alive
    CPPUNIT_ASSERT(clTreePopulation::snag == p_oTree->GetType());

    //Count it
    iSpecies = p_oTree->GetSpecies();
    if (0 == iSpecies) iNumSp1Snags++;
    else if (1 == iSpecies) iNumSp2Snags++;
    else if (2 == iSpecies) iNumSp3Snags++;

    p_oTree = p_oAllTrees->NextTree();
  }

  //Now make sure we have the number we expect left standing
  CPPUNIT_ASSERT(fabs(iNumSp1Snags - 512)/512 < 0.05);
  CPPUNIT_ASSERT(fabs(iNumSp2Snags - 125)/125 < 0.05);
  CPPUNIT_ASSERT(iNumSp3Snags == 1000);
}

/////////////////////////////////////////////////////////////////////////////
//  ProduceDecayGraphs()
/////////////////////////////////////////////////////////////////////////////
void clTestNZSubstrate::ProduceDecayGraphs()
{
  clGridBase * p_oSubstrateGrid;
  clSimManager * p_oSimManager = new clSimManager(3, 3, "");
  clPopulationBase *p_oTemp;
  clTreePopulation *p_oPop;
  clTree *p_oTree;
  float *p_fLoDecayFreshLogs,
        *p_fLoDecayMossyLogs,
        *p_fMedDecayFreshLogs,
        *p_fMedDecayMossyLogs,
        *p_fHiDecayFreshLogs,
        *p_fHiDecayMossyLogs,
        fTemp,
        fDead = -1;
  int iNumTimesteps = 10, i, j, iX = 0, iY = 0;
  fstream oOut;

  p_fLoDecayFreshLogs = new float[iNumTimesteps];
  p_fLoDecayMossyLogs = new float[iNumTimesteps];
  p_fMedDecayFreshLogs = new float[iNumTimesteps];
  p_fMedDecayMossyLogs = new float[iNumTimesteps];
  p_fHiDecayFreshLogs = new float[iNumTimesteps];
  p_fHiDecayMossyLogs = new float[iNumTimesteps];

  oOut.open("Substrate.txt", ios::out | ios::trunc);

  //**********************************
  // Mossy log low decay
  //**********************************
  //Create and feed in the file
  p_oSimManager->ReadFile(WriteXMLFile4());
  //Grab our pointers
  p_oTemp = p_oSimManager->GetPopulationObject("treepopulation");
  p_oPop = (clTreePopulation*) p_oTemp;
  p_oSubstrateGrid = p_oSimManager->GetGridObject("NZ Substrate");
  CPPUNIT_ASSERT(NULL != p_oSubstrateGrid);

  //"Run" for 10 timesteps - we have to fake this to set the fresh logs the
  //way we want them
  for (i = 1; i <= iNumTimesteps; i++) {
    p_oSimManager->SetCurrentTimestep(i);

    p_oTree = p_oPop->CreateTree(1, 1, 0, 3, 44);
    p_oTree->SetValue(p_oPop->GetFloatDataCode("lgm", p_oTree->GetSpecies(), p_oTree->GetType()), fDead);

    //Now run the substrate behavior and the tree killer
    for (j = 1; j < p_oSimManager->GetNumberOfBehaviors(); j++) {
      p_oSimManager->GetBehaviorObject(j)->Action();
    }
    //Now timestep cleanup
    for (j = 0; j < p_oSimManager->GetNumberOfBehaviors(); j++) {
      p_oSimManager->GetBehaviorObject(j)->TimestepCleanup();
    }

    //Get the fresh and mossy log values
    p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fTemp);
    p_fLoDecayFreshLogs[i-1] = fTemp;

    p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("mossylog"), & fTemp);
    p_fLoDecayMossyLogs[i-1] = fTemp;
  }

  //**********************************
  // Mossy log medium decay
  //**********************************
  //Create and feed in the file
  p_oSimManager->ReadFile(WriteXMLFile5());
  //Grab our pointers
  p_oTemp = p_oSimManager->GetPopulationObject("treepopulation");
  p_oPop = (clTreePopulation*) p_oTemp;
  p_oSubstrateGrid = p_oSimManager->GetGridObject("NZ Substrate");
  CPPUNIT_ASSERT(NULL != p_oSubstrateGrid);

  //"Run" for 10 timesteps - we have to fake this to set the fresh logs the
  //way we want them
  for (i = 1; i <= iNumTimesteps; i++) {
    p_oSimManager->SetCurrentTimestep(i);

    p_oTree = p_oPop->CreateTree(1, 1, 0, 3, 44);
    p_oTree->SetValue(p_oPop->GetFloatDataCode("lgm", p_oTree->GetSpecies(), p_oTree->GetType()), fDead);

    //Now run the substrate behavior and the tree killer
    for (j = 1; j < p_oSimManager->GetNumberOfBehaviors(); j++) {
      p_oSimManager->GetBehaviorObject(j)->Action();
    }
    //Now timestep cleanup
    for (j = 0; j < p_oSimManager->GetNumberOfBehaviors(); j++) {
      p_oSimManager->GetBehaviorObject(j)->TimestepCleanup();
    }

    //Get the fresh and mossy log values
    p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fTemp);
    p_fMedDecayFreshLogs[i-1] = fTemp;

    p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("mossylog"), & fTemp);
    p_fMedDecayMossyLogs[i-1] = fTemp;
  }

  //**********************************
  // Mossy log hi decay
  //**********************************
  //Create and feed in the file
  p_oSimManager->ReadFile(WriteXMLFile6());
  //Grab our pointers
  p_oTemp = p_oSimManager->GetPopulationObject("treepopulation");
  p_oPop = (clTreePopulation*) p_oTemp;
  p_oSubstrateGrid = p_oSimManager->GetGridObject("NZ Substrate");
  CPPUNIT_ASSERT(NULL != p_oSubstrateGrid);

  //"Run" for 10 timesteps - we have to fake this to set the fresh logs the
  //way we want them
  for (i = 1; i <= iNumTimesteps; i++) {
    p_oSimManager->SetCurrentTimestep(i);

    p_oTree = p_oPop->CreateTree(1, 1, 0, 3, 44);
    p_oTree->SetValue(p_oPop->GetFloatDataCode("lgm", p_oTree->GetSpecies(), p_oTree->GetType()), fDead);

    //Now run the substrate behavior and the tree killer
    for (j = 1; j < p_oSimManager->GetNumberOfBehaviors(); j++) {
      p_oSimManager->GetBehaviorObject(j)->Action();
    }
    //Now timestep cleanup
    for (j = 0; j < p_oSimManager->GetNumberOfBehaviors(); j++) {
      p_oSimManager->GetBehaviorObject(j)->TimestepCleanup();
    }

    //Get the fresh and mossy log values
    p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fTemp);
    p_fHiDecayFreshLogs[i-1] = fTemp;

    p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("mossylog"), & fTemp);
    p_fHiDecayMossyLogs[i-1] = fTemp;
  }

  //Write output
  //Header for our output file
  oOut << "Timestep\tLo Fresh logs\tLo Mossy logs\tMed Fresh logs\tMed Mossy logs\tHi Fresh logs\tHi Mossy logs\n";
  for (i = 0; i < iNumTimesteps; i++) {
    oOut << i+1 << "\t"
         << p_fLoDecayFreshLogs[i] << "\t" << p_fLoDecayMossyLogs[i] << "\t"
         << p_fMedDecayFreshLogs[i] << "\t" << p_fMedDecayMossyLogs[i] << "\t"
         << p_fHiDecayFreshLogs[i] << "\t" << p_fHiDecayMossyLogs[i] << "\n";
  }

  delete[] p_fLoDecayFreshLogs;
  delete[] p_fLoDecayMossyLogs;
  delete[] p_fMedDecayFreshLogs;
  delete[] p_fMedDecayMossyLogs;
  delete[] p_fHiDecayFreshLogs;
  delete[] p_fHiDecayMossyLogs;
}

/////////////////////////////////////////////////////////////////////////////
//  TestSubstrateProcessing()
/////////////////////////////////////////////////////////////////////////////
void clTestNZSubstrate::TestSubstrateProcessing(clSimManager *p_oSimManager)
{
  clGridBase * p_oSubstrateGrid, *p_oCalcsGrid;
  clPackage * p_oCohort;
  clPopulationBase *p_oTemp;
  clTreePopulation *p_oPop;
  clTree *p_oTree;
  float fExpectedLitter, fExpectedMoss, fExpectedMineralSoil, fExpectedMossyLog,
  fExpectedFreshLog, fActual, fY;
  int iX, iY, i;
  bool bDead = true;

  //Create and feed in the file
  p_oSimManager->ReadFile(WriteXMLFile3());
  //Grab our pointers
  p_oTemp = p_oSimManager->GetPopulationObject("treepopulation");
  p_oPop = (clTreePopulation*) p_oTemp;
  p_oSubstrateGrid = p_oSimManager->GetGridObject("NZ Substrate");
  CPPUNIT_ASSERT(p_oSubstrateGrid != NULL);
  p_oCalcsGrid = p_oSimManager->GetGridObject("nzsubstratecalcs");

  //********************************
  //Timestep 1:  Just run.
  //********************************
  p_oSimManager->RunSim(1);

  //Verify the freshlogs array
  VerifyTimestep1FreshLogArray(p_oCalcsGrid);

  //Verify the new mineral soil
  VerifyTimestep1MineralSoil(p_oCalcsGrid);

  //Verify the cohorts
  VerifyTimestep1Cohorts(p_oSubstrateGrid);

  //Verify the substrate proportions
  VerifyTimestep1SubstrateProportions(p_oSubstrateGrid);

  //********************************
  //Timestep 2:
  //********************************

  //We have to fake this timestep.  The light behavior will interfere with our
  //ability to kill trees, so we have to skip it.  Set the timestep to 2
  p_oSimManager->SetCurrentTimestep(2);

  //Now create and then kill certain trees within cell 0, 2
  //Tree where y = 18
  p_oTree = p_oPop->CreateTree(1, 18, 0, 3, 20);
  p_oTree->SetValue(p_oPop->GetBoolDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), bDead);

  //Tree where y = 19
  p_oTree = p_oPop->CreateTree(1, 19, 0, 3, 40);
  p_oTree->SetValue(p_oPop->GetBoolDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), bDead);

  //Now create and then kill certain trees within cell 0, 3
  //Tree where y = 25
  p_oTree = p_oPop->CreateTree(1, 25, 0, 3, 55);
  p_oTree->SetValue(p_oPop->GetBoolDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), bDead);

  //Tree where y = 26
  p_oTree = p_oPop->CreateTree(1, 26, 0, 3, 15);
  p_oTree->SetValue(p_oPop->GetBoolDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), bDead);

  //Tree where y = 27
  p_oTree = p_oPop->CreateTree(1, 27, 0, 2, 8);
  p_oTree->SetValue(p_oPop->GetBoolDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), bDead);


  //Now run the substrate behavior and the tree killer
  for (i = 1; i < p_oSimManager->GetNumberOfBehaviors(); i++) {
    p_oSimManager->GetBehaviorObject(i)->Action();
  }
  //Now timestep cleanup
  for (i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++) {
    p_oSimManager->GetBehaviorObject(i)->TimestepCleanup();
  }

  //Verify the freshlogs array
  VerifyTimestep2FreshLogArray(p_oCalcsGrid);

  //Verify the new mineral soil
  VerifyTimestep2MineralSoil(p_oCalcsGrid);

  //Verify the cohorts
  VerifyTimestep2Cohorts(p_oSubstrateGrid);

  //Verify the substrate proportions
  VerifyTimestep2SubstrateProportions(p_oSubstrateGrid);

  //********************************
  //Timestep 3:
  //********************************
  //We have to fake this timestep.  The light behavior will interfere with our
  //ability to kill trees, so we have to skip it.  Set the timestep to 3
  p_oSimManager->SetCurrentTimestep(3);

  //Create and kill a tree in cell 0, 0 where y = 2
  p_oTree = p_oPop->CreateTree(1, 2, 0, 2, 65);
  p_oTree->SetValue(p_oPop->GetBoolDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), bDead);

  //Create and kill a tree in cell 0, 1 where y = 10
  p_oTree = p_oPop->CreateTree(1, 10, 0, 2, 100);
  p_oTree->SetValue(p_oPop->GetBoolDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), bDead);

  //Create and kill a tree in cell 0, 2 where y = 20
  p_oTree = p_oPop->CreateTree(1, 20, 0, 2, 35);
  p_oTree->SetValue(p_oPop->GetBoolDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), bDead);

  //Create and kill a tree in cell 0, 3 where y = 28
  p_oTree = p_oPop->CreateTree(1, 28, 0, 2, 42);
  p_oTree->SetValue(p_oPop->GetBoolDataCode("dead", p_oTree->GetSpecies(), p_oTree->GetType()), bDead);

  //Now run the substrate behavior and the tree killer
  for (i = 1; i < p_oSimManager->GetNumberOfBehaviors(); i++) {
    p_oSimManager->GetBehaviorObject(i)->Action();
  }
  //Now timestep cleanup
  for (i = 0; i < p_oSimManager->GetNumberOfBehaviors(); i++) {
    p_oSimManager->GetBehaviorObject(i)->TimestepCleanup();
  }

  //Verify the freshlogs array
  VerifyTimestep3FreshLogArray(p_oCalcsGrid);

  //Verify the new mineral soil
  VerifyTimestep3MineralSoil(p_oCalcsGrid);

  //Verify the cohorts
  VerifyTimestep3Cohorts(p_oSubstrateGrid);

  //Verify the substrate proportions
  VerifyTimestep3SubstrateProportions(p_oSubstrateGrid);

  //********************************
  //Timestep 4:  Just run.
  //********************************
  p_oSimManager->RunSim(1);

  //Verify the freshlogs array
  VerifyTimestep4FreshLogArray(p_oCalcsGrid);

  //Verify the new mineral soil
  VerifyTimestep4MineralSoil(p_oCalcsGrid);

  //Verify the cohorts
  VerifyTimestep4Cohorts(p_oSubstrateGrid);

  //Verify the substrate proportions
  VerifyTimestep4SubstrateProportions(p_oSubstrateGrid);
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyTimestep1SubstrateProportions()
/////////////////////////////////////////////////////////////////////////////
void clTestNZSubstrate::VerifyTimestep1SubstrateProportions(clGridBase *p_oSubstrateGrid) {
  float fExpectedMineralSoil, fExpectedFreshLog, fExpectedLitter,
        fExpectedMossyLog, fExpectedMoss, fActual;
  int iX, iY;

  //Cell 1
  iX = 0; iY = 0;
  fExpectedFreshLog = 0.01;
  fExpectedMossyLog = 0.0495;
  fExpectedMineralSoil = 0.2;
  fExpectedLitter = 0.481325;
  fExpectedMoss = 0.259175;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("litter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLitter) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("mossylog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMossyLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("moss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMoss) < 0.001);

  //Cell 2
  iY = 1;
  fExpectedFreshLog = 0.01;
  fExpectedMossyLog = 0.0495;
  fExpectedMineralSoil = 0.2;
  fExpectedLitter = 0.481325;
  fExpectedMoss = 0.259175;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("litter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLitter) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("moss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("mossylog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMossyLog) < 0.001);

  //Cell 3
  iY = 2;
  fExpectedFreshLog = 0.01;
  fExpectedMossyLog = 0.0495;
  fExpectedMineralSoil = 0.2;
  fExpectedLitter = 0.481325;
  fExpectedMoss = 0.259175;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("litter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLitter) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("moss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("mossylog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMossyLog) < 0.001);

  //Cell 4
  iY = 3;
  fExpectedFreshLog = 0.01;
  fExpectedMossyLog = 0.0495;
  fExpectedMineralSoil = 0.2;
  fExpectedLitter = 0.481325;
  fExpectedMoss = 0.259175;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("litter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLitter) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("moss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("mossylog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMossyLog) < 0.001);

  //Cell 5
  iY = 4;
  fExpectedFreshLog = 0.01;
  fExpectedMossyLog = 0.0495;
  fExpectedMineralSoil = 0.2;
  fExpectedLitter = 0.481325;
  fExpectedMoss = 0.259175;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("litter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLitter) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("moss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("mossylog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMossyLog) < 0.001);
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyTimestep2SubstrateProportions()
/////////////////////////////////////////////////////////////////////////////
void clTestNZSubstrate::VerifyTimestep2SubstrateProportions(clGridBase *p_oSubstrateGrid) {
  float fExpectedMineralSoil, fExpectedFreshLog, fExpectedLitter,
        fExpectedMossyLog, fExpectedMoss, fActual;
  int iX, iY;

  //Cell 1
  iX = 0; iY = 0;
  fExpectedFreshLog = 0.009512294;
  fExpectedMossyLog = 0.022275;
  fExpectedMineralSoil = 0.199860049;
  fExpectedLitter = 0.499429227;
  fExpectedMoss = 0.26892343;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("litter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLitter) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("moss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("mossylog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMossyLog) < 0.001);

  //Cell 2
  iY = 1;
  fExpectedFreshLog = 0.009512294;
  fExpectedMossyLog = 0.022275;
  fExpectedMineralSoil = 0.199860049;
  fExpectedLitter = 0.499429227;
  fExpectedMoss = 0.26892343;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("litter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLitter) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("mossylog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMossyLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("moss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMoss) < 0.001);

  //Cell 3
  iY = 2;
  fExpectedFreshLog = 0.124448823;
  fExpectedMossyLog = 0.022275;
  fExpectedMineralSoil = 0.195495441;
  fExpectedLitter = 0.427557478;
  fExpectedMoss = 0.230223257;

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("mossylog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMossyLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("litter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLitter) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("moss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMoss) < 0.001);

  //Cell 4
  iY = 3;
  fExpectedFreshLog = 0.169256747;
  fExpectedMossyLog = 0.022275;
  fExpectedMineralSoil = 0.198221137;
  fExpectedLitter = 0.396660625;
  fExpectedMoss = 0.213586491;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("litter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLitter) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("moss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("mossylog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMossyLog) < 0.001);

  //Cell 5
  iY = 4;
  fExpectedFreshLog = 0.009512294;
  fExpectedMossyLog = 0.021787294;
  fExpectedMineralSoil = 0.199860049;
  fExpectedLitter = 0.499746236;
  fExpectedMoss = 0.269094127;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("litter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLitter) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("moss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("mossylog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMossyLog) < 0.001);
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyTimestep3SubstrateProportions()
/////////////////////////////////////////////////////////////////////////////
void clTestNZSubstrate::VerifyTimestep3SubstrateProportions(clGridBase *p_oSubstrateGrid) {
  float fExpectedMineralSoil, fExpectedFreshLog, fExpectedLitter,
        fExpectedMossyLog, fExpectedMoss, fActual;
  int iX, iY;

  //Cell 1
  iX = 0; iY = 0;
  fExpectedFreshLog = 0.172840545;
  fExpectedMossyLog = 0.019427749;
  fExpectedMineralSoil = 0.203180676;
  fExpectedLitter = 0.39295817;
  fExpectedMoss = 0.211592861;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("litter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLitter) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("moss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("mossylog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMossyLog) < 0.001);

  //Cell 2
  iY = 1;
  fExpectedFreshLog = 0.293521087;
  fExpectedMossyLog = 0.019427749;
  fExpectedMineralSoil = 0.234042378;
  fExpectedLitter = 0.29445571;
  fExpectedMoss = 0.158553075;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("litter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLitter) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("moss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("mossylog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMossyLog) < 0.001);

  //Cell 3
  iY = 2;
  fExpectedFreshLog = 0.172533986;
  fExpectedMossyLog = 0.025098151;
  fExpectedMineralSoil = 0.191112492;
  fExpectedLitter = 0.397315991;
  fExpectedMoss = 0.21393938;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("litter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLitter) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("moss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("mossylog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMossyLog) < 0.001);

  //Cell 4
  iY = 3;
  fExpectedFreshLog = 0.230107105;
  fExpectedMossyLog = 0.027311351;
  fExpectedMineralSoil = 0.194422339;
  fExpectedLitter = 0.356303483;
  fExpectedMoss = 0.191855722;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("litter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLitter) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("moss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("mossylog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMossyLog) < 0.001);

  //Cell 5
  iY = 4;
  fExpectedFreshLog = 0.000108295;
  fExpectedMossyLog = 0.019208282;
  fExpectedMineralSoil = 0.197261163;
  fExpectedLitter = 0.50922447;
  fExpectedMoss = 0.274197791;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("litter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLitter) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("moss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("mossylog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMossyLog) < 0.001);
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyTimestep4SubstrateProportions()
/////////////////////////////////////////////////////////////////////////////
void clTestNZSubstrate::VerifyTimestep4SubstrateProportions(clGridBase *p_oSubstrateGrid) {
  float fExpectedMineralSoil, fExpectedFreshLog, fExpectedLitter,
        fExpectedMossyLog, fExpectedMoss, fActual;
  int iX, iY;

  //Cell 1
  iX = 0; iY = 0;
  fExpectedFreshLog = 0.164330928;
  fExpectedMossyLog = 0.017167914;
  fExpectedMineralSoil = 0.049791591;
  fExpectedLitter = 0.499661219;
  fExpectedMoss = 0.269048349;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("litter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLitter) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("moss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("mossylog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMossyLog) < 0.001);

  //Cell 2
  iY = 1;
  fExpectedFreshLog = 0.27914526;
  fExpectedMossyLog = 0.023054571;
  fExpectedMineralSoil = 0.117849919;
  fExpectedLitter = 0.376967662;
  fExpectedMoss = 0.202982587;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("litter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLitter) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("moss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("mossylog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMossyLog) < 0.001);

  //Cell 3
  iY = 2;
  fExpectedFreshLog = 0.069011889;
  fExpectedMossyLog = 0.124110809;
  fExpectedMineralSoil = 0.035704405;
  fExpectedLitter = 0.501262383;
  fExpectedMoss = 0.269910514;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("litter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLitter) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("moss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("mossylog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMossyLog) < 0.001);

  //Cell 4
  iY = 3;
  fExpectedFreshLog = 0.090875207;
  fExpectedMossyLog = 0.168882443;
  fExpectedMineralSoil = 0.054255551;
  fExpectedLitter = 0.44589142;
  fExpectedMoss = 0.24009538;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("litter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLitter) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("moss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMoss) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("mossylog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMossyLog) < 0.001);

  //Cell 5
  iY = 4;
  fExpectedFreshLog = 0.000108295;
  fExpectedMossyLog = 0.008643727;
  fExpectedMineralSoil = 0.197261163;
  fExpectedLitter = 0.51609143;
  fExpectedMoss = 0.277895386;
  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("litter"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedLitter) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("mossylog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMossyLog) < 0.001);

  p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("moss"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMoss) < 0.001);
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyTimestep1Cohorts()
/////////////////////////////////////////////////////////////////////////////
void clTestNZSubstrate::VerifyTimestep1Cohorts(clGridBase *p_oSubstrateGrid) {
  clPackage * p_oCohort;
  float fExpectedMineralSoil = 0.2, fExpectedFreshLog = 0.01,
        fExpectedAvgDbh = 20.0, fExpectedDecayedFreshLog = 0.01,
        fActual;
  int iX, iY, iAge = 1, iActual;

  //Cell 1
  iX = 0; iY = 0;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("dec_flog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecayedFreshLog) < 0.001);

/*  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("avglogsize"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedAvgDbh) < 0.001);
*/

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 2
  iY = 1;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("dec_flog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecayedFreshLog) < 0.001);

//  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("avglogsize"), & fActual);
//  CPPUNIT_ASSERT(fabs(fActual - fExpectedAvgDbh) < 0.001);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 3
  iY = 2;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("dec_flog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecayedFreshLog) < 0.001);

//  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("avglogsize"), & fActual);
//  CPPUNIT_ASSERT(fabs(fActual - fExpectedAvgDbh) < 0.001);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 4
  iY = 3;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("dec_flog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecayedFreshLog) < 0.001);

//  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("avglogsize"), & fActual);
//  CPPUNIT_ASSERT(fabs(fActual - fExpectedAvgDbh) < 0.001);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 5
  iY = 4;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("dec_flog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecayedFreshLog) < 0.001);

//  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("avglogsize"), & fActual);
//  CPPUNIT_ASSERT(fabs(fActual - fExpectedAvgDbh) < 0.001);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyTimestep2Cohorts()
/////////////////////////////////////////////////////////////////////////////
void clTestNZSubstrate::VerifyTimestep2Cohorts(clGridBase *p_oSubstrateGrid) {
  clPackage * p_oCohort;
  float fExpectedMineralSoil, fExpectedFreshLog, fExpectedAvgDbh,
        fExpectedDecayedFreshLog, fActual;
  int iX, iY, iAge, iActual;

  //Cell 1
  iX = 0; iY = 0;
  fExpectedMineralSoil = 0.2;
  fExpectedFreshLog = 0.01;
  iAge = 2;
  fExpectedDecayedFreshLog = 0.009512294;
  fExpectedAvgDbh = 20;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("dec_flog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecayedFreshLog) < 0.001);

//  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("avglogsize"), & fActual);
//  CPPUNIT_ASSERT(fabs(fActual - fExpectedAvgDbh) < 0.001);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 2
  iY = 1;
  fExpectedMineralSoil = 0.2;
  fExpectedFreshLog = 0.01;
  iAge = 2;
  fExpectedDecayedFreshLog = 0.009512294;
  fExpectedAvgDbh = 20;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("dec_flog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecayedFreshLog) < 0.001);

//  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("avglogsize"), & fActual);
//  CPPUNIT_ASSERT(fabs(fActual - fExpectedAvgDbh) < 0.001);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 3, cohort 1
  iY = 2;
  fExpectedMineralSoil = 0.172029331;
  fExpectedFreshLog = 0.008601467;
  iAge = 2;
  fExpectedDecayedFreshLog = 0.008181968;
  fExpectedAvgDbh = 20;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("dec_flog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecayedFreshLog) < 0.001);

//  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("avglogsize"), & fActual);
//  CPPUNIT_ASSERT(fabs(fActual - fExpectedAvgDbh) < 0.001);

  //Cell 3, cohort 2
  fExpectedMineralSoil = 0.023586489;
  fExpectedFreshLog = 0.116266855;
  iAge = 1;
  fExpectedDecayedFreshLog = 0.116266855;
  fExpectedAvgDbh = 30;
  p_oCohort = p_oCohort->GetNextPackage();
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("dec_flog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecayedFreshLog) < 0.001);

//  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("avglogsize"), & fActual);
//  CPPUNIT_ASSERT(fabs(fActual - fExpectedAvgDbh) < 0.001);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 4, cohort 1
  iY = 3;
  fExpectedMineralSoil = 0.160005057;
  fExpectedFreshLog = 0.008000253;
  iAge = 2;
  fExpectedDecayedFreshLog = 0.007610076;
  fExpectedAvgDbh = 20;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("dec_flog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecayedFreshLog) < 0.001);

//  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("avglogsize"), & fActual);
//  CPPUNIT_ASSERT(fabs(fActual - fExpectedAvgDbh) < 0.001);

  //Cell 4, cohort 2
  fExpectedMineralSoil = 0.038328044;
  fExpectedFreshLog = 0.161646671;
  iAge = 1;
  fExpectedDecayedFreshLog = 0.161646671;
  fExpectedAvgDbh = 26;
  p_oCohort = p_oCohort->GetNextPackage();
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("dec_flog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecayedFreshLog) < 0.001);

//  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("avglogsize"), & fActual);
//  CPPUNIT_ASSERT(fabs(fActual - fExpectedAvgDbh) < 0.001);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 5
  iY = 4;
  fExpectedMineralSoil = 0.2;
  fExpectedFreshLog = 0.01;
  iAge = 2;
  fExpectedDecayedFreshLog = 0.009512294;
  fExpectedAvgDbh = 20;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("dec_flog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecayedFreshLog) < 0.001);

//  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("avglogsize"), & fActual);
//  CPPUNIT_ASSERT(fabs(fActual - fExpectedAvgDbh) < 0.001);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyTimestep3Cohorts()
/////////////////////////////////////////////////////////////////////////////
void clTestNZSubstrate::VerifyTimestep3Cohorts(clGridBase *p_oSubstrateGrid) {
  clPackage * p_oCohort;
  float fExpectedMineralSoil, fExpectedFreshLog, fExpectedAvgDbh,
        fExpectedDecayedFreshLog, fActual;
  int iX, iY, iAge, iActual;

  //Cell 1
  iX = 0; iY = 0;
  fExpectedMineralSoil = 0.049826457;
  fExpectedFreshLog = 0.172756354;
  iAge = 1;
  fExpectedDecayedFreshLog = 0.172756354;
  fExpectedAvgDbh = 65;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("dec_flog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecayedFreshLog) < 0.001);

//  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("avglogsize"), & fActual);
//  CPPUNIT_ASSERT(fabs(fActual - fExpectedAvgDbh) < 0.001);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 2
  iY = 1;
  fExpectedMineralSoil = 0.117932443;
  fExpectedFreshLog = 0.293457344;
  iAge = 1;
  fExpectedDecayedFreshLog = 0.293457344;
  fExpectedAvgDbh = 100;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("dec_flog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecayedFreshLog) < 0.001);

//  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("avglogsize"), & fActual);
//  CPPUNIT_ASSERT(fabs(fActual - fExpectedAvgDbh) < 0.001);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 3, cohort 1
  iY = 2;
  fExpectedMineralSoil = 0.021563079;
  fExpectedFreshLog = 0.106292692;
  iAge = 2;
  fExpectedDecayedFreshLog = 0.101108736;
  fExpectedAvgDbh = 30;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("dec_flog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecayedFreshLog) < 0.001);

//  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("avglogsize"), & fActual);
//  CPPUNIT_ASSERT(fabs(fActual - fExpectedAvgDbh) < 0.001);

  //Cell 3, cohort 2
  fExpectedMineralSoil = 0.014446724;
  fExpectedFreshLog = 0.071340092;
  iAge = 1;
  fExpectedDecayedFreshLog = 0.071340092;
  fExpectedAvgDbh = 35;
  p_oCohort = p_oCohort->GetNextPackage();
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("dec_flog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecayedFreshLog) < 0.001);

//  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("avglogsize"), & fActual);
//  CPPUNIT_ASSERT(fabs(fActual - fExpectedAvgDbh) < 0.001);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 4, cohort 1
  iY = 3;
  fExpectedMineralSoil = 0.033931489;
  fExpectedFreshLog = 0.143104415;
  iAge = 2;
  fExpectedDecayedFreshLog = 0.13612513;
  fExpectedAvgDbh = 26;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("dec_flog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecayedFreshLog) < 0.001);

//  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("avglogsize"), & fActual);
//  CPPUNIT_ASSERT(fabs(fActual - fExpectedAvgDbh) < 0.001);

  //Cell 4, cohort 2
  fExpectedMineralSoil = 0.020803283;
  fExpectedFreshLog = 0.093905275;
  iAge = 1;
  fExpectedDecayedFreshLog = 0.093905275;
  fExpectedAvgDbh = 42;
  p_oCohort = p_oCohort->GetNextPackage();
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("dec_flog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecayedFreshLog) < 0.001);

//  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("avglogsize"), & fActual);
//  CPPUNIT_ASSERT(fabs(fActual - fExpectedAvgDbh) < 0.001);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 5
  iY = 4;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL == p_oCohort);
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyTimestep4Cohorts()
/////////////////////////////////////////////////////////////////////////////
void clTestNZSubstrate::VerifyTimestep4Cohorts(clGridBase *p_oSubstrateGrid) {
  clPackage * p_oCohort;
  float fExpectedMineralSoil, fExpectedFreshLog, fExpectedAvgDbh,
        fExpectedDecayedFreshLog, fActual;
  int iX, iY, iAge, iActual;

  //Cell 1
  iX = 0; iY = 0;
  fExpectedMineralSoil = 0.049826457;
  fExpectedFreshLog = 0.172756354;
  iAge = 2;
  fExpectedDecayedFreshLog = 0.164330928;
  fExpectedAvgDbh = 65;

  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("dec_flog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecayedFreshLog) < 0.001);

//  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("avglogsize"), & fActual);
//  CPPUNIT_ASSERT(fabs(fActual - fExpectedAvgDbh) < 0.001);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 2, cohort 1
  iY = 1;
  fExpectedMineralSoil = 0.117932443;
  fExpectedFreshLog = 0.293457344;
  iAge = 2;
  fExpectedDecayedFreshLog = 0.27914526;
  fExpectedAvgDbh = 100;

  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("dec_flog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecayedFreshLog) < 0.001);

//  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("avglogsize"), & fActual);
//  CPPUNIT_ASSERT(fabs(fActual - fExpectedAvgDbh) < 0.001);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 3, cohort 1
  iY = 2;
  fExpectedMineralSoil = 0.014446724;
  fExpectedFreshLog = 0.071340092;
  iAge = 2;
  fExpectedDecayedFreshLog = 0.067860795;
  fExpectedAvgDbh = 35;

  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("dec_flog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecayedFreshLog) < 0.001);

//  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("avglogsize"), & fActual);
//  CPPUNIT_ASSERT(fabs(fActual - fExpectedAvgDbh) < 0.001);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 4, cohort 1
  iY = 3;
  fExpectedMineralSoil = 0.020803283;
  fExpectedFreshLog = 0.093905275;
  iAge = 2;
  fExpectedDecayedFreshLog = 0.08932546;
  fExpectedAvgDbh = 42;

  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL != p_oCohort);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("minsoil"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageIntDataCode("age"), & iActual);
  CPPUNIT_ASSERT(iActual == iAge);

  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("dec_flog"), & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedDecayedFreshLog) < 0.001);

//  p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("avglogsize"), & fActual);
//  CPPUNIT_ASSERT(fabs(fActual - fExpectedAvgDbh) < 0.001);

  CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());

  //Cell 5
  iY = 4;
  p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);
  CPPUNIT_ASSERT(NULL == p_oCohort);
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyTimestep1MineralSoil()
/////////////////////////////////////////////////////////////////////////////
void clTestNZSubstrate::VerifyTimestep1MineralSoil(clGridBase *p_oCalcsGrid) {

  float fExpectedMineralSoils = 0.2, fActual;
  int iCode = p_oCalcsGrid->GetFloatDataCode("newminsoil"),
      iX, iY;

  //Cell 1
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoils) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoils) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoils) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoils) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoils) < 0.001);
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyTimestep2MineralSoil()
/////////////////////////////////////////////////////////////////////////////
void clTestNZSubstrate::VerifyTimestep2MineralSoil(clGridBase *p_oCalcsGrid) {

  float fExpectedMineralSoils, fActual;
  int iCode = p_oCalcsGrid->GetFloatDataCode("newminsoil"),
      iX, iY;

  //Cell 1
  iX = 0; iY = 0;
  fExpectedMineralSoils = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoils) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedMineralSoils = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoils) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedMineralSoils = 0.023586489;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoils) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedMineralSoils = 0.038328044;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoils) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedMineralSoils = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoils) < 0.001);
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyTimestep3MineralSoil()
/////////////////////////////////////////////////////////////////////////////
void clTestNZSubstrate::VerifyTimestep3MineralSoil(clGridBase *p_oCalcsGrid) {

  float fExpectedMineralSoils, fActual;
  int iCode = p_oCalcsGrid->GetFloatDataCode("newminsoil"),
      iX, iY;

  //Cell 1
  iX = 0; iY = 0;
  fExpectedMineralSoils = 0.049826457;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoils) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedMineralSoils = 0.117932443;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoils) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedMineralSoils = 0.014446724;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoils) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedMineralSoils = 0.020803283;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoils) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedMineralSoils = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoils) < 0.001);
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyTimestep4MineralSoil()
/////////////////////////////////////////////////////////////////////////////
void clTestNZSubstrate::VerifyTimestep4MineralSoil(clGridBase *p_oCalcsGrid) {

  float fExpectedMineralSoils = 0.0, fActual;
  int iCode = p_oCalcsGrid->GetFloatDataCode("newminsoil"),
      iX, iY;

  //Cell 1
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoils) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoils) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoils) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoils) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, iCode, & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoils) < 0.001);
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyTimestep1FreshLogArray()
/////////////////////////////////////////////////////////////////////////////
void clTestNZSubstrate::VerifyTimestep1FreshLogArray(clGridBase *p_oCalcsGrid) {
  char cLabel[20];
  float fExpectedFreshLog = 0.01, //all values are the same
        fActual;
  int *p_iFreshLogCodes = new int[3],
       iX, iY, i;

  for ( i = 0; i < 3; i++ )
  {
    sprintf( cLabel, "%s%d", "freshlog_", i );
    p_iFreshLogCodes[i] = p_oCalcsGrid->GetFloatDataCode( cLabel );
    CPPUNIT_ASSERT(-1 < p_iFreshLogCodes[i]);
  }

  //Current timestep addition
  //Cell 1
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  //Last timestep
  fExpectedFreshLog = 0.0;
  //Cell 1
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  //Timestep - 2
  //Cell 1
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  delete[] p_iFreshLogCodes;
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyTimestep2FreshLogArray()
/////////////////////////////////////////////////////////////////////////////
void clTestNZSubstrate::VerifyTimestep2FreshLogArray(clGridBase *p_oCalcsGrid) {
  char cLabel[20];
  float fExpectedFreshLog, //all values are the same
        fActual;
  int *p_iFreshLogCodes = new int[3],
       iX, iY, i;

  for ( i = 0; i < 3; i++ )
  {
    sprintf( cLabel, "%s%d", "freshlog_", i );
    p_iFreshLogCodes[i] = p_oCalcsGrid->GetFloatDataCode( cLabel );
    CPPUNIT_ASSERT(-1 < p_iFreshLogCodes[i]);
  }

  //Current timestep addition - total across cells
  //Cell 1
  iX = 0; iY = 0;
  fExpectedFreshLog = 0.0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedFreshLog = 0.0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedFreshLog = 0.116266855;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedFreshLog = 0.161646671;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedFreshLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  //Last timestep
  //Cell 1
  iX = 0; iY = 0;
  fExpectedFreshLog = 0.01;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedFreshLog = 0.01;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedFreshLog = 0.01;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedFreshLog = 0.01;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedFreshLog = 0.01;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  //Timestep - 2
  //Cell 1
  fExpectedFreshLog = 0.0;
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  delete[] p_iFreshLogCodes;
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyTimestep3FreshLogArray()
/////////////////////////////////////////////////////////////////////////////
void clTestNZSubstrate::VerifyTimestep3FreshLogArray(clGridBase *p_oCalcsGrid) {
  char cLabel[20];
  float fExpectedFreshLog, //all values are the same
        fTimestepLog, //total across grids
        fActual;
  int *p_iFreshLogCodes = new int[3],
       iX, iY, i;

  for ( i = 0; i < 3; i++ )
  {
    sprintf( cLabel, "%s%d", "freshlog_", i );
    p_iFreshLogCodes[i] = p_oCalcsGrid->GetFloatDataCode( cLabel );
    CPPUNIT_ASSERT(-1 < p_iFreshLogCodes[i]);
  }

  //Current timestep addition
  //Cell 1
  iX = 0; iY = 0;
  fExpectedFreshLog = 0.172756354;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedFreshLog = 0.293457344;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedFreshLog = 0.071340092;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedFreshLog = 0.093905275;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedFreshLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  //Last timestep
  //Cell 1
  iX = 0; iY = 0;
  fExpectedFreshLog = 0.0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedFreshLog = 0.0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedFreshLog = 0.116266855;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedFreshLog = 0.161646671;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedFreshLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  //Timestep - 2
  //Cell 1
  fExpectedFreshLog = 0.01;
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  delete[] p_iFreshLogCodes;
}

/////////////////////////////////////////////////////////////////////////////
//  VerifyTimestep4FreshLogArray()
/////////////////////////////////////////////////////////////////////////////
void clTestNZSubstrate::VerifyTimestep4FreshLogArray(clGridBase *p_oCalcsGrid) {
  char cLabel[20];
  float fExpectedFreshLog, //all values are the same
        fTimestepLog, //total across grids
        fActual;
  int *p_iFreshLogCodes = new int[3],
       iX, iY, i;

  for ( i = 0; i < 3; i++ )
  {
    sprintf( cLabel, "%s%d", "freshlog_", i );
    p_iFreshLogCodes[i] = p_oCalcsGrid->GetFloatDataCode( cLabel );
    CPPUNIT_ASSERT(-1 < p_iFreshLogCodes[i]);
  }

  //Current timestep addition
  //Cell 1
  fExpectedFreshLog = 0.0;
  iX = 0; iY = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 2
  iY = 1;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 3
  iY = 2;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 4
  iY = 3;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 5
  iY = 4;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[0], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  //Last timestep
  //Cell 1
  iX = 0; iY = 0;
  fExpectedFreshLog = 0.172756354;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedFreshLog = 0.293457344;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedFreshLog = 0.071340092;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedFreshLog = 0.093905275;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedFreshLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[1], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  //Timestep - 2
  //Cell 1
  iX = 0; iY = 0;
  fExpectedFreshLog = 0.0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 2
  iY = 1;
  fExpectedFreshLog = 0.0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 3
  iY = 2;
  fExpectedFreshLog = 0.116266855;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 4
  iY = 3;
  fExpectedFreshLog = 0.161646671;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);
  //Cell 5
  iY = 4;
  fExpectedFreshLog = 0;
  p_oCalcsGrid->GetValueOfCell(iX, iY, p_iFreshLogCodes[2], & fActual);
  CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

  delete[] p_iFreshLogCodes;
}

/////////////////////////////////////////////////////////////////////////////
//  TestSubstrateGridSetup()
/////////////////////////////////////////////////////////////////////////////
void clTestNZSubstrate::TestSubstrateGridSetup(clSimManager *p_oSimManager)
{
  clGridBase * p_oSubstrateGrid, *p_oSubstrateCalcsGrid;
  float fVal;

  //Feed file # 1 and make sure grid resolutions are right
  p_oSimManager->ReadFile(WriteXMLFile1());
  p_oSubstrateGrid = p_oSimManager->GetGridObject("NZ Substrate");
  CPPUNIT_ASSERT(fabs(p_oSubstrateGrid->GetLengthXCells() - 3) < 0.001);
  CPPUNIT_ASSERT(fabs(p_oSubstrateGrid->GetLengthYCells() - 2) < 0.001);
  //Make sure the substrate calcs resolutions are right and the values got read
  p_oSubstrateCalcsGrid = p_oSimManager->GetGridObject("nzsubstratecalcs");
  CPPUNIT_ASSERT(fabs(p_oSubstrateCalcsGrid->GetLengthXCells() - 3) < 0.001);
  CPPUNIT_ASSERT(fabs(p_oSubstrateCalcsGrid->GetLengthYCells() - 2) < 0.001);
  p_oSubstrateCalcsGrid->GetValueOfCell(0, 0, p_oSubstrateCalcsGrid->GetFloatDataCode("newminsoil"), &fVal);
  CPPUNIT_ASSERT(fabs(fVal - 0.1) < 0.001);
  p_oSubstrateCalcsGrid->GetValueOfCell(0, 0, p_oSubstrateCalcsGrid->GetFloatDataCode("freshlog_0"), &fVal);
  CPPUNIT_ASSERT(fabs(fVal - 0.01) < 0.001);
  p_oSubstrateCalcsGrid->GetValueOfCell(0, 0, p_oSubstrateCalcsGrid->GetFloatDataCode("freshlog_1"), &fVal);
  CPPUNIT_ASSERT(fabs(fVal - 0.02) < 0.001);
  p_oSubstrateCalcsGrid->GetValueOfCell(0, 0, p_oSubstrateCalcsGrid->GetFloatDataCode("freshlog_2"), &fVal);
  CPPUNIT_ASSERT(fabs(fVal - 0.03) < 0.001);
  p_oSubstrateCalcsGrid->GetValueOfCell(0, 0, p_oSubstrateCalcsGrid->GetFloatDataCode("freshlog_3"), &fVal);
  CPPUNIT_ASSERT(fabs(fVal - 0.04) < 0.001);
  p_oSubstrateCalcsGrid->GetValueOfCell(0, 0, p_oSubstrateCalcsGrid->GetFloatDataCode("freshlog_4"), &fVal);
  CPPUNIT_ASSERT(fabs(fVal - 0.05) < 0.001);
  p_oSubstrateCalcsGrid->GetValueOfCell(0, 0, p_oSubstrateCalcsGrid->GetFloatDataCode("freshlog_5"), &fVal);
  CPPUNIT_ASSERT(fabs(fVal - 0.06) < 0.001);
  p_oSubstrateCalcsGrid->GetValueOfCell(0, 0, p_oSubstrateCalcsGrid->GetFloatDataCode("freshlog_6"), &fVal);
  CPPUNIT_ASSERT(fabs(fVal - 0.07) < 0.001);
  p_oSubstrateCalcsGrid->GetValueOfCell(0, 0, p_oSubstrateCalcsGrid->GetFloatDataCode("freshlog_7"), &fVal);
  CPPUNIT_ASSERT(fabs(fVal - 0.08) < 0.001);
  p_oSubstrateCalcsGrid->GetValueOfCell(0, 0, p_oSubstrateCalcsGrid->GetFloatDataCode("freshlog_8"), &fVal);
  CPPUNIT_ASSERT(fabs(fVal - 0.09) < 0.001);
  p_oSubstrateCalcsGrid->GetValueOfCell(0, 0, p_oSubstrateCalcsGrid->GetFloatDataCode("freshlog_9"), &fVal);
  CPPUNIT_ASSERT(fabs(fVal - 0.11) < 0.001);


  //Feed file # 2 and make sure grid resolutions are right
  p_oSimManager->ReadFile(WriteXMLFile2());
  p_oSubstrateGrid = p_oSimManager->GetGridObject("NZ Substrate");
  CPPUNIT_ASSERT(fabs(p_oSubstrateGrid->GetLengthXCells() - 8) < 0.001);
  CPPUNIT_ASSERT(fabs(p_oSubstrateGrid->GetLengthYCells() - 8) < 0.001);

  //Now check that initial conditions were correctly populated
  clPackage * p_oCohort;
  float fExpectedLitter = 0.338, fExpectedMoss = 0.182, fExpectedMineralSoil = 0.2, fExpectedMossyLog = 0.11,
       fExpectedFreshLog = 0.17, fActual;
  int iNumXCells = p_oSubstrateGrid->GetNumberXCells(), iNumYCells = p_oSubstrateGrid->GetNumberYCells(), iX, iY;

  for (iX = 0; iX < iNumXCells; iX++)
  {
    for (iY = 0; iY < iNumYCells; iY++)
    {

      //The values all should match the parameter file initial conditions
      p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("litter"), & fActual);
      CPPUNIT_ASSERT(fabs(fActual - fExpectedLitter) < 0.001);

      p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("moss"), & fActual);
      CPPUNIT_ASSERT(fabs(fActual - fExpectedMoss) < 0.001);

      p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("minsoil"), & fActual);
      CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

      p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("freshlog"), & fActual);
      CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

      p_oSubstrateGrid->GetValueOfCell(iX, iY, p_oSubstrateGrid->GetFloatDataCode("mossylog"), & fActual);
      CPPUNIT_ASSERT(fabs(fActual - fExpectedMossyLog) < 0.001);

      //There should be one cohort per cell which also matches the initial conditions
      p_oCohort = p_oSubstrateGrid->GetFirstPackageOfCell(iX, iY);

      p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("minsoil"), & fActual);
      CPPUNIT_ASSERT(fabs(fActual - fExpectedMineralSoil) < 0.001);

      p_oCohort->GetValue(p_oSubstrateGrid->GetPackageFloatDataCode("freshlog"), & fActual);
      CPPUNIT_ASSERT(fabs(fActual - fExpectedFreshLog) < 0.001);

      CPPUNIT_ASSERT(NULL == p_oCohort->GetNextPackage());
    }
  }
}

//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile1()
/////////////////////////////////////////////////////////////////////////////
char * clTestNZSubstrate::WriteXMLFile1()
{
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>" << "<paramFile fileCode=\"06010101\">" << "<plot>"
       << "<timesteps>1</timesteps>" << "<yearsPerTimestep>1</yearsPerTimestep>" << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>200.0</plot_lenX>" << "<plot_lenY>200.0</plot_lenY>" << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>" << "<trees>"
       << "<tr_speciesList>" << "<tr_species speciesName=\"Species_1\"/>" << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>" << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>" << "</tr_canopyHeight>" << "<tr_canopyRadiusToDBH>"
       << "<tr_crtdVal species=\"Species_1\">0.0549</tr_crtdVal>" << "</tr_canopyRadiusToDBH>" << "<tr_canopyRadiusExponent>"
       << "<tr_creVal species=\"Species_1\">1.0</tr_creVal>" << "</tr_canopyRadiusExponent>" << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>" << "</tr_conversionDiam10ToDBH>"
       << "<tr_fractOfHeightInCanopy>" << "<tr_fohicVal species=\"Species_1\">0.389</tr_fohicVal>"
       << "</tr_fractOfHeightInCanopy>" << "<tr_canopyDepthExponent>" << "<tr_cdeVal species=\"Species_1\">1.0</tr_cdeVal>"
       << "</tr_canopyDepthExponent>" << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>" << "</tr_slopeOfHeight-Diam10>" << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>" << "</tr_minAdultDBH>" << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>" << "</tr_slopeOfAsymHeight>" << "</trees>"
       << "<grid gridName=\"NZ Substrate\">"
       << "<ma_floatCodes>"
       << "<ma_floatCode label=\"litter\">0</ma_floatCode>"
       << "<ma_floatCode label=\"moss\">1</ma_floatCode>"
       << "<ma_floatCode label=\"minsoil\">2</ma_floatCode>"
       << "<ma_floatCode label=\"freshlog\">3</ma_floatCode>"
       << "<ma_floatCode label=\"mossylog\">4</ma_floatCode>"
       << "</ma_floatCodes>"
       << "<ma_lengthXCells>3</ma_lengthXCells>"
       << "<ma_lengthYCells>2</ma_lengthYCells>"
       << "</grid>"
       << "<grid gridName=\"nzsubstratecalcs\">"
       << "<ma_floatCodes>"
       << "<ma_floatCode label=\"newminsoil\">0</ma_floatCode>"
       << "<ma_floatCode label=\"freshlog_0\">1</ma_floatCode>"
       << "<ma_floatCode label=\"freshlog_1\">2</ma_floatCode>"
       << "<ma_floatCode label=\"freshlog_2\">3</ma_floatCode>"
       << "<ma_floatCode label=\"freshlog_3\">4</ma_floatCode>"
       << "<ma_floatCode label=\"freshlog_4\">5</ma_floatCode>"
       << "<ma_floatCode label=\"freshlog_5\">6</ma_floatCode>"
       << "<ma_floatCode label=\"freshlog_6\">7</ma_floatCode>"
       << "<ma_floatCode label=\"freshlog_7\">8</ma_floatCode>"
       << "<ma_floatCode label=\"freshlog_8\">9</ma_floatCode>"
       << "<ma_floatCode label=\"freshlog_9\">10</ma_floatCode>"
       << "</ma_floatCodes>"
       << "<ma_lengthXCells>3</ma_lengthXCells>"
       << "<ma_lengthYCells>2</ma_lengthYCells>"
       << "<ma_v x=\"0\" y=\"0\">"
       << "<fl c=\"0\">0.1</fl>"
       << "<fl c=\"1\">0.01</fl>"
       << "<fl c=\"2\">0.02</fl>"
       << "<fl c=\"3\">0.03</fl>"
       << "<fl c=\"4\">0.04</fl>"
       << "<fl c=\"5\">0.05</fl>"
       << "<fl c=\"6\">0.06</fl>"
       << "<fl c=\"7\">0.07</fl>"
       << "<fl c=\"8\">0.08</fl>"
       << "<fl c=\"9\">0.09</fl>"
       << "<fl c=\"10\">0.11</fl>"
       << "</ma_v>"
       << "</grid>"
       << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>adultstochasticmort</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>" << "<behaviorName>NZ Substrate</behaviorName>" << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>" << "</behavior>" << "</behaviorList>"
       << "<mortality>"
       << "<mo_nonSizeDepMort>"
       << "<mo_nsdmVal species=\"Species_1\">0.0</mo_nsdmVal>"
       << "</mo_nonSizeDepMort>"
       << "</mortality>"
       << "<substrate>"
       << "<nzs_mineralSoilDecayAlpha>-7.0E-4</nzs_mineralSoilDecayAlpha>"
       << "<nzs_mineralSoilDecayBeta>4.3</nzs_mineralSoilDecayBeta>"
       << "<nzs_freshLogDecayAlpha>-0.05</nzs_freshLogDecayAlpha>"
       << "<nzs_freshLogDecayBeta>6.5</nzs_freshLogDecayBeta>"
       << "<nzs_mossyLogToLitterMoss>0.55</nzs_mossyLogToLitterMoss>"
       << "<nzs_mossProportion>0.35</nzs_mossProportion>"
       << "<nzs_maxNumberDecayYears>10</nzs_maxNumberDecayYears>"
       << "<nzs_initialMineralSoil>0.0</nzs_initialMineralSoil>"
       << "<nzs_initialFreshLog>0.01</nzs_initialFreshLog>"
       << "<nzs_initialMossyLog>0.11</nzs_initialMossyLog>"
       << "<nzs_initialAvgDbhFreshLog>20</nzs_initialAvgDbhFreshLog>"
       << "<nzs_propOfDeadFall>"
       << "<nzs_podfVal species=\"Species_1\">0.7</nzs_podfVal>" << "</nzs_propOfDeadFall>" << "<nzs_propOfFallUproot>"
       << "<nzs_pofuVal species=\"Species_1\">0.5</nzs_pofuVal>" << "</nzs_propOfFallUproot>"
       << "<nzs_rootTipupFactor>3.0</nzs_rootTipupFactor>" << "</substrate>" << "</paramFile>";

  oOut.close();

  return cFileString;
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile2()
/////////////////////////////////////////////////////////////////////////////
char * clTestNZSubstrate::WriteXMLFile2()
{
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>" << "<paramFile fileCode=\"06010101\">" << "<plot>"
       << "<timesteps>1</timesteps>" << "<yearsPerTimestep>1</yearsPerTimestep>" << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>200.0</plot_lenX>" << "<plot_lenY>200.0</plot_lenY>" << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>" << "<trees>"
       << "<tr_speciesList>" << "<tr_species speciesName=\"Species_1\"/>" << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>" << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>" << "</tr_canopyHeight>" << "<tr_canopyRadiusToDBH>"
       << "<tr_crtdVal species=\"Species_1\">0.0549</tr_crtdVal>" << "</tr_canopyRadiusToDBH>" << "<tr_canopyRadiusExponent>"
       << "<tr_creVal species=\"Species_1\">1.0</tr_creVal>" << "</tr_canopyRadiusExponent>" << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>" << "</tr_conversionDiam10ToDBH>"
       << "<tr_fractOfHeightInCanopy>" << "<tr_fohicVal species=\"Species_1\">0.389</tr_fohicVal>"
       << "</tr_fractOfHeightInCanopy>" << "<tr_canopyDepthExponent>" << "<tr_cdeVal species=\"Species_1\">1.0</tr_cdeVal>"
       << "</tr_canopyDepthExponent>" << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>" << "</tr_slopeOfHeight-Diam10>" << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>" << "</tr_minAdultDBH>" << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>" << "</tr_slopeOfAsymHeight>"
       << "</trees>" << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>adultstochasticmort</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>" << "<behaviorName>NZ Substrate</behaviorName>" << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>" << "</behavior>" << "</behaviorList>"
       << "<mortality>"
       << "<mo_nonSizeDepMort>"
       << "<mo_nsdmVal species=\"Species_1\">0.0</mo_nsdmVal>"
       << "</mo_nonSizeDepMort>"
       << "</mortality>"
       << "<substrate>"
       << "<nzs_mineralSoilDecayAlpha>-7.0E-4</nzs_mineralSoilDecayAlpha>" << "<nzs_mineralSoilDecayBeta>4.3</nzs_mineralSoilDecayBeta>"
       << "<nzs_freshLogDecayAlpha>-0.05</nzs_freshLogDecayAlpha>" << "<nzs_freshLogDecayBeta>6.5</nzs_freshLogDecayBeta>"
       << "<nzs_mossyLogToLitterMoss>0.55</nzs_mossyLogToLitterMoss>"
       << "<nzs_mossProportion>0.35</nzs_mossProportion>"
       << "<nzs_maxNumberDecayYears>10</nzs_maxNumberDecayYears>"
       << "<nzs_initialMineralSoil>0.2</nzs_initialMineralSoil>" << "<nzs_initialFreshLog>0.17</nzs_initialFreshLog>"
       << "<nzs_initialMossyLog>0.11</nzs_initialMossyLog>"
       << "<nzs_initialAvgDbhFreshLog>20</nzs_initialAvgDbhFreshLog>"
       << "<nzs_propOfDeadFall>"
       << "<nzs_podfVal species=\"Species_1\">0.7</nzs_podfVal>" << "</nzs_propOfDeadFall>" << "<nzs_propOfFallUproot>"
       << "<nzs_pofuVal species=\"Species_1\">0.5</nzs_pofuVal>" << "</nzs_propOfFallUproot>"
       << "<nzs_rootTipupFactor>3.0</nzs_rootTipupFactor>" << "</substrate>" << "</paramFile>";

  oOut.close();

  return cFileString;
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile3()
/////////////////////////////////////////////////////////////////////////////
char * clTestNZSubstrate::WriteXMLFile3()
{
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>" << "<paramFile fileCode=\"06010101\">" << "<plot>"
       << "<timesteps>4</timesteps>" << "<yearsPerTimestep>1</yearsPerTimestep>" << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>200.0</plot_lenX>" << "<plot_lenY>200.0</plot_lenY>" << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>" << "<trees>"
       << "<tr_speciesList>" << "<tr_species speciesName=\"Species_1\"/>" << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>" << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>" << "</tr_canopyHeight>" << "<tr_canopyRadiusToDBH>"
       << "<tr_crtdVal species=\"Species_1\">0.0549</tr_crtdVal>" << "</tr_canopyRadiusToDBH>" << "<tr_canopyRadiusExponent>"
       << "<tr_creVal species=\"Species_1\">1.0</tr_creVal>" << "</tr_canopyRadiusExponent>" << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>" << "</tr_conversionDiam10ToDBH>"
       << "<tr_fractOfHeightInCanopy>" << "<tr_fohicVal species=\"Species_1\">0.389</tr_fohicVal>"
       << "</tr_fractOfHeightInCanopy>" << "<tr_canopyDepthExponent>" << "<tr_cdeVal species=\"Species_1\">1.0</tr_cdeVal>"
       << "</tr_canopyDepthExponent>" << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>" << "</tr_slopeOfHeight-Diam10>" << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>" << "</tr_minAdultDBH>" << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>" << "</tr_slopeOfAsymHeight>"
       << "</trees>"
       << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>adultstochasticmort</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>NZ Substrate</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "</behavior>"
       //Let the dead remover get rid of the trees we kill
       << "<behavior>"
       << "<behaviorName>removedead</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\" />"
       << "<applyTo species=\"Species_1\" type=\"Sapling\" />"
       << "</behavior>"
       << "</behaviorList>"
       << "<mortality>"
       << "<mo_nonSizeDepMort>"
       << "<mo_nsdmVal species=\"Species_1\">0.0</mo_nsdmVal>"
       << "</mo_nonSizeDepMort>"
       << "</mortality>"
       << "<substrate>"
       << "<nzs_mineralSoilDecayAlpha>-7.0E-4</nzs_mineralSoilDecayAlpha>" << "<nzs_mineralSoilDecayBeta>4.3</nzs_mineralSoilDecayBeta>"
       << "<nzs_freshLogDecayAlpha>-0.05</nzs_freshLogDecayAlpha>" << "<nzs_freshLogDecayBeta>6.5</nzs_freshLogDecayBeta>"
       << "<nzs_mossyLogToLitterMoss>0.55</nzs_mossyLogToLitterMoss>"
       << "<nzs_mossProportion>0.35</nzs_mossProportion>"
       << "<nzs_maxNumberDecayYears>3</nzs_maxNumberDecayYears>"
       << "<nzs_initialMineralSoil>0.2</nzs_initialMineralSoil>" << "<nzs_initialFreshLog>0.01</nzs_initialFreshLog>"
       << "<nzs_initialMossyLog>0.11</nzs_initialMossyLog>"
       << "<nzs_initialAvgDbhFreshLog>20</nzs_initialAvgDbhFreshLog>"
       << "<nzs_propOfDeadFall>"
       << "<nzs_podfVal species=\"Species_1\">1</nzs_podfVal>" << "</nzs_propOfDeadFall>" << "<nzs_propOfFallUproot>"
       << "<nzs_pofuVal species=\"Species_1\">1</nzs_pofuVal>" << "</nzs_propOfFallUproot>"
       << "<nzs_rootTipupFactor>3.1</nzs_rootTipupFactor>" << "</substrate>" << "</paramFile>";

  oOut.close();

  return cFileString;
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile4()
/////////////////////////////////////////////////////////////////////////////
char * clTestNZSubstrate::WriteXMLFile4()
{
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">" << "<plot>"
       << "<timesteps>10</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>"
       << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>200.0</plot_lenX>"
       << "<plot_lenY>200.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>" << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>" << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_canopyRadiusToDBH>"
       << "<tr_crtdVal species=\"Species_1\">0.0549</tr_crtdVal>"
       << "</tr_canopyRadiusToDBH>" << "<tr_canopyRadiusExponent>"
       << "<tr_creVal species=\"Species_1\">1.0</tr_creVal>"
       << "</tr_canopyRadiusExponent>" << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>" << "</tr_conversionDiam10ToDBH>"
       << "<tr_fractOfHeightInCanopy>" << "<tr_fohicVal species=\"Species_1\">0.389</tr_fohicVal>"
       << "</tr_fractOfHeightInCanopy>" << "<tr_canopyDepthExponent>" << "<tr_cdeVal species=\"Species_1\">1.0</tr_cdeVal>"
       << "</tr_canopyDepthExponent>" << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>" << "</tr_slopeOfHeight-Diam10>" << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>" << "</tr_minAdultDBH>" << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>" << "</tr_slopeOfAsymHeight>"
       << "</trees>" << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>adultstochasticmort</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>NZ Substrate</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "</behavior>"
       //Let the dead remover get rid of the trees we kill
       << "<behavior>"
       << "<behaviorName>removedead</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\" />"
       << "<applyTo species=\"Species_1\" type=\"Sapling\" />"
       << "</behavior>"
       << "</behaviorList>"
       << "<mortality>"
       << "<mo_nonSizeDepMort>"
       << "<mo_nsdmVal species=\"Species_1\">0.0</mo_nsdmVal>"
       << "</mo_nonSizeDepMort>"
       << "</mortality>"
       << "<substrate>"
       << "<nzs_mineralSoilDecayAlpha>-7.0E-4</nzs_mineralSoilDecayAlpha>"
       << "<nzs_mineralSoilDecayBeta>4.3</nzs_mineralSoilDecayBeta>"
       << "<nzs_freshLogDecayAlpha>-0.05</nzs_freshLogDecayAlpha>"
       << "<nzs_freshLogDecayBeta>4.4</nzs_freshLogDecayBeta>"
       << "<nzs_mossyLogToLitterMoss>0.225</nzs_mossyLogToLitterMoss>"
       << "<nzs_mossProportion>0.35</nzs_mossProportion>"
       << "<nzs_maxNumberDecayYears>50</nzs_maxNumberDecayYears>"
       << "<nzs_initialMineralSoil>0.2</nzs_initialMineralSoil>"
       << "<nzs_initialFreshLog>0.01</nzs_initialFreshLog>"
       << "<nzs_initialMossyLog>0.01</nzs_initialMossyLog>"
       << "<nzs_initialAvgDbhFreshLog>20</nzs_initialAvgDbhFreshLog>"
       << "<nzs_propOfDeadFall>"
       << "<nzs_podfVal species=\"Species_1\">1</nzs_podfVal>"
       << "</nzs_propOfDeadFall>"
       << "<nzs_propOfFallUproot>"
       << "<nzs_pofuVal species=\"Species_1\">1</nzs_pofuVal>"
       << "</nzs_propOfFallUproot>"
       << "<nzs_rootTipupFactor>3.1</nzs_rootTipupFactor>"
       << "</substrate>" << "</paramFile>";

  oOut.close();

  return cFileString;
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile5()
/////////////////////////////////////////////////////////////////////////////
char * clTestNZSubstrate::WriteXMLFile5()
{
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">" << "<plot>"
       << "<timesteps>10</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>"
       << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>200.0</plot_lenX>"
       << "<plot_lenY>200.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>" << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>" << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_canopyRadiusToDBH>"
       << "<tr_crtdVal species=\"Species_1\">0.0549</tr_crtdVal>"
       << "</tr_canopyRadiusToDBH>" << "<tr_canopyRadiusExponent>"
       << "<tr_creVal species=\"Species_1\">1.0</tr_creVal>"
       << "</tr_canopyRadiusExponent>" << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>" << "</tr_conversionDiam10ToDBH>"
       << "<tr_fractOfHeightInCanopy>" << "<tr_fohicVal species=\"Species_1\">0.389</tr_fohicVal>"
       << "</tr_fractOfHeightInCanopy>" << "<tr_canopyDepthExponent>" << "<tr_cdeVal species=\"Species_1\">1.0</tr_cdeVal>"
       << "</tr_canopyDepthExponent>" << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>" << "</tr_slopeOfHeight-Diam10>" << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>" << "</tr_minAdultDBH>" << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>" << "</tr_slopeOfAsymHeight>"
       << "</trees>" << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>adultstochasticmort</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>NZ Substrate</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "</behavior>"
       //Let the dead remover get rid of the trees we kill
       << "<behavior>"
       << "<behaviorName>removedead</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\" />"
       << "<applyTo species=\"Species_1\" type=\"Sapling\" />"
       << "</behavior>"
       << "</behaviorList>"
       << "<mortality>"
       << "<mo_nonSizeDepMort>"
       << "<mo_nsdmVal species=\"Species_1\">0.0</mo_nsdmVal>"
       << "</mo_nonSizeDepMort>"
       << "</mortality>"
       << "<substrate>"
       << "<nzs_mineralSoilDecayAlpha>-7.0E-4</nzs_mineralSoilDecayAlpha>"
       << "<nzs_mineralSoilDecayBeta>4.3</nzs_mineralSoilDecayBeta>"
       << "<nzs_freshLogDecayAlpha>-0.05</nzs_freshLogDecayAlpha>"
       << "<nzs_freshLogDecayBeta>4.4</nzs_freshLogDecayBeta>"
       << "<nzs_mossyLogToLitterMoss>0.45</nzs_mossyLogToLitterMoss>"
       << "<nzs_mossProportion>0.35</nzs_mossProportion>"
       << "<nzs_maxNumberDecayYears>50</nzs_maxNumberDecayYears>"
       << "<nzs_initialMineralSoil>0.2</nzs_initialMineralSoil>"
       << "<nzs_initialFreshLog>0.01</nzs_initialFreshLog>"
       << "<nzs_initialMossyLog>0.01</nzs_initialMossyLog>"
       << "<nzs_initialAvgDbhFreshLog>20</nzs_initialAvgDbhFreshLog>"
       << "<nzs_propOfDeadFall>"
       << "<nzs_podfVal species=\"Species_1\">1</nzs_podfVal>"
       << "</nzs_propOfDeadFall>"
       << "<nzs_propOfFallUproot>"
       << "<nzs_pofuVal species=\"Species_1\">1</nzs_pofuVal>"
       << "</nzs_propOfFallUproot>"
       << "<nzs_rootTipupFactor>3.1</nzs_rootTipupFactor>"
       << "</substrate>" << "</paramFile>";

  oOut.close();

  return cFileString;
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile6()
/////////////////////////////////////////////////////////////////////////////
char * clTestNZSubstrate::WriteXMLFile6()
{
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">" << "<plot>"
       << "<timesteps>10</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>"
       << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>200.0</plot_lenX>"
       << "<plot_lenY>200.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>" << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>" << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_canopyRadiusToDBH>"
       << "<tr_crtdVal species=\"Species_1\">0.0549</tr_crtdVal>"
       << "</tr_canopyRadiusToDBH>" << "<tr_canopyRadiusExponent>"
       << "<tr_creVal species=\"Species_1\">1.0</tr_creVal>"
       << "</tr_canopyRadiusExponent>" << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>" << "</tr_conversionDiam10ToDBH>"
       << "<tr_fractOfHeightInCanopy>" << "<tr_fohicVal species=\"Species_1\">0.389</tr_fohicVal>"
       << "</tr_fractOfHeightInCanopy>" << "<tr_canopyDepthExponent>" << "<tr_cdeVal species=\"Species_1\">1.0</tr_cdeVal>"
       << "</tr_canopyDepthExponent>" << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>" << "</tr_slopeOfHeight-Diam10>" << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>" << "</tr_minAdultDBH>" << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>" << "</tr_slopeOfAsymHeight>"
       << "</trees>" << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>adultstochasticmort</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>NZ Substrate</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "</behavior>"
       //Let the dead remover get rid of the trees we kill
       << "<behavior>"
       << "<behaviorName>removedead</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\" />"
       << "<applyTo species=\"Species_1\" type=\"Sapling\" />"
       << "</behavior>"
       << "</behaviorList>"
       << "<mortality>"
       << "<mo_nonSizeDepMort>"
       << "<mo_nsdmVal species=\"Species_1\">0.0</mo_nsdmVal>"
       << "</mo_nonSizeDepMort>"
       << "</mortality>"
       << "<nzsubstrate>"
       << "<nzs_mineralSoilDecayAlpha>-7.0E-4</nzs_mineralSoilDecayAlpha>"
       << "<nzs_mineralSoilDecayBeta>4.3</nzs_mineralSoilDecayBeta>"
       << "<nzs_freshLogDecayAlpha>-0.05</nzs_freshLogDecayAlpha>"
       << "<nzs_freshLogDecayBeta>4.4</nzs_freshLogDecayBeta>"
       << "<nzs_mossyLogToLitterMoss>0.9</nzs_mossyLogToLitterMoss>"
       << "<nzs_mossProportion>0.35</nzs_mossProportion>"
       << "<nzs_maxNumberDecayYears>50</nzs_maxNumberDecayYears>"
       << "<nzs_initialMineralSoil>0.2</nzs_initialMineralSoil>"
       << "<nzs_initialFreshLog>0.01</nzs_initialFreshLog>"
       << "<nzs_initialMossyLog>0.01</nzs_initialMossyLog>"
       << "<nzs_initialAvgDbhFreshLog>20</nzs_initialAvgDbhFreshLog>"
       << "<nzs_propOfDeadFall>"
       << "<nzs_podfVal species=\"Species_1\">1</nzs_podfVal>"
       << "</nzs_propOfDeadFall>"
       << "<nzs_propOfFallUproot>"
       << "<nzs_pofuVal species=\"Species_1\">1</nzs_pofuVal>"
       << "</nzs_propOfFallUproot>"
       << "<nzs_rootTipupFactor>3.1</nzs_rootTipupFactor>"
       << "</nzsubstrate>" << "</paramFile>";

  oOut.close();

  return cFileString;
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile8()
/////////////////////////////////////////////////////////////////////////////
char * clTestNZSubstrate::WriteXMLFile8()
{
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>10</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>"
       << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>200.0</plot_lenX>"
       << "<plot_lenY>200.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "<tr_species speciesName=\"Species_2\"/>"
       << "<tr_species speciesName=\"Species_3\"/>"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>" << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_2\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_3\">39.48</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_canopyRadiusToDBH>"
       << "<tr_crtdVal species=\"Species_1\">0.0549</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_2\">0.0549</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_3\">0.0549</tr_crtdVal>"
       << "</tr_canopyRadiusToDBH>"
       << "<tr_canopyRadiusExponent>"
       << "<tr_creVal species=\"Species_1\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_2\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_3\">1.0</tr_creVal>"
       << "</tr_canopyRadiusExponent>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_2\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_3\">0.8008</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_fractOfHeightInCanopy>"
       << "<tr_fohicVal species=\"Species_1\">0.389</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_2\">0.389</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_3\">0.389</tr_fohicVal>"
       << "</tr_fractOfHeightInCanopy>"
       << "<tr_canopyDepthExponent>"
       << "<tr_cdeVal species=\"Species_1\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_2\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_3\">1.0</tr_cdeVal>"
       << "</tr_canopyDepthExponent>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_2\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_3\">0.03418</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_2\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_3\">10.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_2\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_3\">0.0299</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "</trees>" << "<behaviorList>"
       //Register a behavior for snags
       << "<behavior>"
       << "<behaviorName>radiallight</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Snag\"/>"
       << "<applyTo species=\"Species_2\" type=\"Snag\"/>"
       << "<applyTo species=\"Species_3\" type=\"Snag\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>adultstochasticmort</behaviorName>"
       << "<version>1.0</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_3\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>NZ Substrate</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_3\" type=\"Adult\"/>"
       << "</behavior>"
       //Let the dead remover get rid of the trees we kill
       << "<behavior>"
       << "<behaviorName>removedead</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\" />"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_3\" type=\"Adult\"/>"
       << "</behavior>"
       << "</behaviorList>"
       << "<mortality>"
       << "<mo_nonSizeDepMort>"
       << "<mo_nsdmVal species=\"Species_1\">1.0</mo_nsdmVal>"
       << "<mo_nsdmVal species=\"Species_2\">1.0</mo_nsdmVal>"
       << "<mo_nsdmVal species=\"Species_3\">1.0</mo_nsdmVal>"
       << "</mo_nonSizeDepMort>"
       << "</mortality>"
       << "<nzsubstrate>"
       << "<nzs_mineralSoilDecayAlpha>-7.0E-4</nzs_mineralSoilDecayAlpha>"
       << "<nzs_mineralSoilDecayBeta>4.3</nzs_mineralSoilDecayBeta>"
       << "<nzs_freshLogDecayAlpha>-0.05</nzs_freshLogDecayAlpha>"
       << "<nzs_freshLogDecayBeta>4.4</nzs_freshLogDecayBeta>"
       << "<nzs_mossyLogToLitterMoss>0.9</nzs_mossyLogToLitterMoss>"
       << "<nzs_mossProportion>0.35</nzs_mossProportion>"
       << "<nzs_maxNumberDecayYears>50</nzs_maxNumberDecayYears>"
       << "<nzs_initialMineralSoil>0.2</nzs_initialMineralSoil>"
       << "<nzs_initialFreshLog>0.01</nzs_initialFreshLog>"
       << "<nzs_initialMossyLog>0.01</nzs_initialMossyLog>"
       << "<nzs_initialAvgDbhFreshLog>20</nzs_initialAvgDbhFreshLog>"
       << "<nzs_propOfDeadFall>"
       << "<nzs_podfVal species=\"Species_1\">0.2</nzs_podfVal>"
       << "<nzs_podfVal species=\"Species_2\">0.5</nzs_podfVal>"
       << "<nzs_podfVal species=\"Species_3\">0.0</nzs_podfVal>"
       << "</nzs_propOfDeadFall>"
       << "<nzs_propOfFallUproot>"
       << "<nzs_pofuVal species=\"Species_1\">1</nzs_pofuVal>"
       << "<nzs_pofuVal species=\"Species_2\">1</nzs_pofuVal>"
       << "<nzs_pofuVal species=\"Species_3\">1</nzs_pofuVal>"
       << "</nzs_propOfFallUproot>"
       << "<nzs_rootTipupFactor>3.1</nzs_rootTipupFactor>"
       << "</nzsubstrate>"
       << "</paramFile>";

  oOut.close();

  return cFileString;
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile9()
/////////////////////////////////////////////////////////////////////////////
char * clTestNZSubstrate::WriteXMLFile9()
{
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>10</timesteps>"
       << "<yearsPerTimestep>3</yearsPerTimestep>"
       << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>200.0</plot_lenX>"
       << "<plot_lenY>200.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "<tr_species speciesName=\"Species_2\"/>"
       << "<tr_species speciesName=\"Species_3\"/>"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>" << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_2\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_3\">39.48</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_canopyRadiusToDBH>"
       << "<tr_crtdVal species=\"Species_1\">0.0549</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_2\">0.0549</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_3\">0.0549</tr_crtdVal>"
       << "</tr_canopyRadiusToDBH>"
       << "<tr_canopyRadiusExponent>"
       << "<tr_creVal species=\"Species_1\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_2\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_3\">1.0</tr_creVal>"
       << "</tr_canopyRadiusExponent>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_2\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_3\">0.8008</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_fractOfHeightInCanopy>"
       << "<tr_fohicVal species=\"Species_1\">0.389</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_2\">0.389</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_3\">0.389</tr_fohicVal>"
       << "</tr_fractOfHeightInCanopy>"
       << "<tr_canopyDepthExponent>"
       << "<tr_cdeVal species=\"Species_1\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_2\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_3\">1.0</tr_cdeVal>"
       << "</tr_canopyDepthExponent>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_2\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_3\">0.03418</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_2\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_3\">10.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_2\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_3\">0.0299</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "</trees>" << "<behaviorList>"
       //Register a behavior for snags
       << "<behavior>"
       << "<behaviorName>radiallight</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Snag\"/>"
       << "<applyTo species=\"Species_2\" type=\"Snag\"/>"
       << "<applyTo species=\"Species_3\" type=\"Snag\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>adultstochasticmort</behaviorName>"
       << "<version>1.0</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_3\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>NZ Substrate</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_3\" type=\"Adult\"/>"
       << "</behavior>"
       //Let the dead remover get rid of the trees we kill
       << "<behavior>"
       << "<behaviorName>removedead</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\" />"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_3\" type=\"Adult\"/>"
       << "</behavior>"
       << "</behaviorList>"
       << "<mortality>"
       << "<mo_nonSizeDepMort>"
       << "<mo_nsdmVal species=\"Species_1\">1.0</mo_nsdmVal>"
       << "<mo_nsdmVal species=\"Species_2\">1.0</mo_nsdmVal>"
       << "<mo_nsdmVal species=\"Species_3\">1.0</mo_nsdmVal>"
       << "</mo_nonSizeDepMort>"
       << "</mortality>"
       << "<nzsubstrate>"
       << "<nzs_mineralSoilDecayAlpha>-7.0E-4</nzs_mineralSoilDecayAlpha>"
       << "<nzs_mineralSoilDecayBeta>4.3</nzs_mineralSoilDecayBeta>"
       << "<nzs_freshLogDecayAlpha>-0.05</nzs_freshLogDecayAlpha>"
       << "<nzs_freshLogDecayBeta>4.4</nzs_freshLogDecayBeta>"
       << "<nzs_mossyLogToLitterMoss>0.9</nzs_mossyLogToLitterMoss>"
       << "<nzs_mossProportion>0.35</nzs_mossProportion>"
       << "<nzs_maxNumberDecayYears>50</nzs_maxNumberDecayYears>"
       << "<nzs_initialMineralSoil>0.2</nzs_initialMineralSoil>"
       << "<nzs_initialFreshLog>0.01</nzs_initialFreshLog>"
       << "<nzs_initialMossyLog>0.01</nzs_initialMossyLog>"
       << "<nzs_initialAvgDbhFreshLog>20</nzs_initialAvgDbhFreshLog>"
       << "<nzs_propOfDeadFall>"
       << "<nzs_podfVal species=\"Species_1\">0.2</nzs_podfVal>"
       << "<nzs_podfVal species=\"Species_2\">0.5</nzs_podfVal>"
       << "<nzs_podfVal species=\"Species_3\">0.0</nzs_podfVal>"
       << "</nzs_propOfDeadFall>"
       << "<nzs_propOfFallUproot>"
       << "<nzs_pofuVal species=\"Species_1\">1</nzs_pofuVal>"
       << "<nzs_pofuVal species=\"Species_2\">1</nzs_pofuVal>"
       << "<nzs_pofuVal species=\"Species_3\">1</nzs_pofuVal>"
       << "</nzs_propOfFallUproot>"
       << "<nzs_rootTipupFactor>3.1</nzs_rootTipupFactor>"
       << "</nzsubstrate>"
       << "</paramFile>";


  oOut.close();

  return cFileString;
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile10()
/////////////////////////////////////////////////////////////////////////////
char * clTestNZSubstrate::WriteXMLFile10()
{
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>4</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>"
       << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>200.0</plot_lenX>"
       << "<plot_lenY>200.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_canopyRadiusToDBH>"
       << "<tr_crtdVal species=\"Species_1\">0.0549</tr_crtdVal>"
       << "</tr_canopyRadiusToDBH>"
       << "<tr_canopyRadiusExponent>"
       << "<tr_creVal species=\"Species_1\">1.0</tr_creVal>"
       << "</tr_canopyRadiusExponent>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_fractOfHeightInCanopy>"
       << "<tr_fohicVal species=\"Species_1\">0.389</tr_fohicVal>"
       << "</tr_fractOfHeightInCanopy>" << "<tr_canopyDepthExponent>" << "<tr_cdeVal species=\"Species_1\">1.0</tr_cdeVal>"
       << "</tr_canopyDepthExponent>" << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>" << "</tr_slopeOfHeight-Diam10>" << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>" << "</tr_minAdultDBH>" << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>" << "</tr_slopeOfAsymHeight>"
       << "</trees>" << "<behaviorList>"
       //Have to have a mortality behavior so dead is registered
       << "<behavior>"
       << "<behaviorName>adultstochasticmort</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Snag\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>NZ Substrate</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Snag\"/>"
       << "</behavior>"
       //Let the dead remover get rid of the trees we kill
       << "<behavior>"
       << "<behaviorName>removedead</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Snag\" />"
       << "</behavior>"
       << "</behaviorList>"
       << "<mortality>"
       << "<mo_nonSizeDepMort>"
       << "<mo_nsdmVal species=\"Species_1\">0.0</mo_nsdmVal>"
       << "</mo_nonSizeDepMort>"
       << "</mortality>"
       << "<nzsubstrate>"
       << "<nzs_gapmossDecayAlpha>-5.1E-4</nzs_gapmossDecayAlpha>" << "<nzs_gapmossDecayBeta>4.4</nzs_gapmossDecayBeta>"
       << "<nzs_mineralSoilDecayAlpha>-7.0E-4</nzs_mineralSoilDecayAlpha>"
       << "<nzs_mineralSoilDecayBeta>4.3</nzs_mineralSoilDecayBeta>"
       << "<nzs_freshLogDecayAlpha>-0.05</nzs_freshLogDecayAlpha>"
       << "<nzs_mossyLogToLitterMoss>0.9</nzs_mossyLogToLitterMoss>"
       << "<nzs_freshLogDecayBeta>6.5</nzs_freshLogDecayBeta>"
       << "<nzs_mossyLogToCanMoss>0.55</nzs_mossyLogToCanMoss>"
       << "<nzs_maxNumberDecayYears>3</nzs_maxNumberDecayYears>"
       << "<nzs_mossProportion>0.35</nzs_mossProportion>"
       << "<nzs_initialMineralSoil>0.2</nzs_initialMineralSoil>" << "<nzs_initialFreshLog>0.01</nzs_initialFreshLog>"
       << "<nzs_initialMossyLog>0.11</nzs_initialMossyLog>"
       << "<nzs_propOfDeadFall>"
       << "<nzs_podfVal species=\"Species_1\">0</nzs_podfVal>"
       << "</nzs_propOfDeadFall>"
       << "<nzs_propOfFallUproot>"
       << "<nzs_pofuVal species=\"Species_1\">1</nzs_pofuVal>"
       << "</nzs_propOfFallUproot>"
       << "<nzs_rootTipupFactor>3.1</nzs_rootTipupFactor>"
       << "</nzsubstrate>"
       << "</paramFile>";

  oOut.close();

  return cFileString;
}
//---------------------------------------------------------------------------
