//---------------------------------------------------------------------------
// TestStochasticGapGrowth
//---------------------------------------------------------------------------
#if !defined(TestStochasticGapGrowth_H)
  #define TestStochasticGapGrowth_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>

class clSimManager;

/**
* Tests the clStochasticGapGrowth class.
* @author Lora E. Murphy
*/
class clTestStochasticGapGrowth : public CppUnit::TestFixture
{
  public:

  /**
  * Rolls up all testing into one function.
  */
  void TestStochasticGapGrowth();

  protected:

  /**
  * Tests normal processing.  WriteXMLFile1() does setup.  3 adult trees are
  * placed so as to create non-gap conditions in certain cells of the "Gap
  * Light" grid.  A series of juveniles that are potential growers are placed
  * in other cells.  The simulation is run, and then for gap cells containing
  * juveniles, we make sure that there is only one adult and that it has a
  * size equal to the minimum adult DBH.
  * @param p_oSimManager Sim manager to run the test with.
  */
  void TestNormalProcessing(clSimManager *p_oSimManager);

  /**
  * Tests error processing.  This tests the case where the gap light behavior
  * is not called before the stochastic gap growth behavior.
  * WriteXMLErrorFile1() is used to write the error file.
  */
  void TestErrorProcessing(clSimManager *p_oSimManager);

  /**
  * Writes a parameter file for normal processing.  There are three species.
  * The behaviors are gap light and stochastic gap growth.
  * @return Filename written.
  */
  char * WriteXMLFile1();

  /**
  * Writes a parameter file for error processing.  In this case, the stochastic
  * gap growth function is in the behavior list but the gap light behavior
  * is not.
  * @return Filename written.
  */
  char * WriteXMLErrorFile1();

};
//---------------------------------------------------------------------------
#endif // TestStochasticGapGrowth_H
