//---------------------------------------------------------------------------
// TestTreeAgeCalculator.cpp
//---------------------------------------------------------------------------
#include "TestTreeAgeCalculator.h"
#include <fstream.h>
#include <math.h>
#include "TreePopulation.h"
#include "SimManager.h"
#include "BehaviorBase.h"

////////////////////////////////////////////////////////////////////////////
//  TestTreeAgeCalculator()
////////////////////////////////////////////////////////////////////////////
void clTestTreeAgeCalculator::TestTreeAgeCalculator() {
  clSimManager *p_oSimManager = new clSimManager(4, 2, "");

  TestErrorProcessing(p_oSimManager);
  TestNormalProcessing(p_oSimManager);

  delete p_oSimManager;
}

////////////////////////////////////////////////////////////////////////////
//  TestNormalProcessing()
////////////////////////////////////////////////////////////////////////////
void clTestTreeAgeCalculator::TestNormalProcessing(clSimManager *p_oSimManager) {
  clPopulationBase * p_oTemp;
  clTreePopulation * p_oPop;
  clTree *p_oTargetTree1,
         *p_oTargetTree2,
         *p_oTargetTree3;
  float fDiam;
  int iSpecies, iAge, iExpectedAge;

  //Run 1 of 2
  //Load our test file and get our tree population pointers
  p_oSimManager->ReadFile( WriteXMLFile1() );
  p_oTemp = p_oSimManager->GetPopulationObject( "treepopulation" );
  p_oPop = ( clTreePopulation * ) p_oTemp;

  //Create our trees
  iSpecies = 0;
  fDiam = 15.0;
  p_oTargetTree1 = p_oPop->CreateTree(10, 10, iSpecies, clTreePopulation::adult, fDiam);

  iAge = 4;
  p_oTargetTree2 = p_oPop->CreateTree(10, 10, iSpecies, clTreePopulation::adult, fDiam);
  p_oTargetTree2->SetValue(
        p_oPop->GetIntDataCode("Tree Age",
                                 p_oTargetTree2->GetSpecies(),
                                 p_oTargetTree2->GetType()), iAge);

  iAge = -12;
  p_oTargetTree3 = p_oPop->CreateTree(10, 10, iSpecies, clTreePopulation::adult, fDiam);
  p_oTargetTree3->SetValue(
        p_oPop->GetIntDataCode("Tree Age",
                                 p_oTargetTree3->GetSpecies(),
                                 p_oTargetTree3->GetType()), iAge);


  //Now do the run
  p_oSimManager->RunSim(1);

  //Check the age for each tree
  p_oTargetTree1->GetValue(
        p_oPop->GetIntDataCode("Tree Age",
                                 p_oTargetTree1->GetSpecies(),
                                 p_oTargetTree1->GetType()), &iAge);
  iExpectedAge = 1;
  CPPUNIT_ASSERT(iAge == iExpectedAge);

  p_oTargetTree2->GetValue(
        p_oPop->GetIntDataCode("Tree Age",
                                 p_oTargetTree2->GetSpecies(),
                                 p_oTargetTree2->GetType()), &iAge);
  iExpectedAge = 5;
  CPPUNIT_ASSERT(iAge == iExpectedAge);

  p_oTargetTree3->GetValue(
        p_oPop->GetIntDataCode("Tree Age",
                                 p_oTargetTree3->GetSpecies(),
                                 p_oTargetTree3->GetType()), &iAge);
  iExpectedAge = -11;
  CPPUNIT_ASSERT(iAge == iExpectedAge);


  //Run 2 of 2
  //Load our test file and get our tree population pointers
  p_oSimManager->ReadFile( WriteXMLFile2() );
  p_oTemp = p_oSimManager->GetPopulationObject( "treepopulation" );
  p_oPop = ( clTreePopulation * ) p_oTemp;

  //Create our trees
  iSpecies = 0;
  fDiam = 15.0;
  p_oTargetTree1 = p_oPop->CreateTree(10, 10, iSpecies, clTreePopulation::adult, fDiam);

  iAge = 4;
  p_oTargetTree2 = p_oPop->CreateTree(10, 10, iSpecies, clTreePopulation::adult, fDiam);
  p_oTargetTree2->SetValue(
        p_oPop->GetIntDataCode("Tree Age",
                                 p_oTargetTree2->GetSpecies(),
                                 p_oTargetTree2->GetType()), iAge);

  iAge = -12;
  p_oTargetTree3 = p_oPop->CreateTree(10, 10, iSpecies, clTreePopulation::adult, fDiam);
  p_oTargetTree3->SetValue(
        p_oPop->GetIntDataCode("Tree Age",
                                 p_oTargetTree3->GetSpecies(),
                                 p_oTargetTree3->GetType()), iAge);


  //Now do the run
  p_oSimManager->RunSim(1);

  //Check the age for each tree
  p_oTargetTree1->GetValue(
        p_oPop->GetIntDataCode("Tree Age",
                                 p_oTargetTree1->GetSpecies(),
                                 p_oTargetTree1->GetType()), &iAge);
  iExpectedAge = 3;
  CPPUNIT_ASSERT(iAge == iExpectedAge);

  p_oTargetTree2->GetValue(
        p_oPop->GetIntDataCode("Tree Age",
                                 p_oTargetTree2->GetSpecies(),
                                 p_oTargetTree2->GetType()), &iAge);
  iExpectedAge = 7;
  CPPUNIT_ASSERT(iAge == iExpectedAge);

  p_oTargetTree3->GetValue(
        p_oPop->GetIntDataCode("Tree Age",
                                 p_oTargetTree3->GetSpecies(),
                                 p_oTargetTree3->GetType()), &iAge);
  iExpectedAge = -9;
  CPPUNIT_ASSERT(iAge == iExpectedAge);

}

////////////////////////////////////////////////////////////////////////////
//  TestErrorProcessing()
////////////////////////////////////////////////////////////////////////////
void clTestTreeAgeCalculator::TestErrorProcessing(clSimManager *p_oSimManager) {

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile1());
    CppUnit::Asserter::fail("TestTreeAgeCalculator error processing didn't catch error for WriteXMLErrorFile1.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "FillSingleValue(bool)") == NULL)
    {
      CppUnit::Asserter::fail("TestTreeAgeCalculator error processing caught wrong error for WriteXMLErrorFile1.");
    }
    ;
  }
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile1()
////////////////////////////////////////////////////////////////////////////
char* clTestTreeAgeCalculator::WriteXMLFile1() {
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
     << "<paramFile fileCode=\"06010101\">"
     << "<plot>"
     << "<timesteps>2</timesteps>"
     << "<yearsPerTimestep>1</yearsPerTimestep>"
     << "<randomSeed>1</randomSeed>"
     << "<plot_lenX>200.0</plot_lenX>"
     << "<plot_lenY>200.0</plot_lenY>"
     << "<plot_latitude>55.37</plot_latitude>"
     << "</plot>"
     << "<trees>"
     << "<tr_speciesList>"
     << "<tr_species speciesName=\"Species_1\" />"
     << "</tr_speciesList>"
     << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
     << "<tr_canopyHeight>"
     << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
     << "</tr_canopyHeight>"
     << "<tr_canopyRadiusToDBH>"
     << "<tr_crtdVal species=\"Species_1\">0.0549</tr_crtdVal>"
     << "</tr_canopyRadiusToDBH>"
     << "<tr_canopyRadiusExponent>"
     << "<tr_creVal species=\"Species_1\">1.0</tr_creVal>"
     << "</tr_canopyRadiusExponent>"
     << "<tr_conversionDiam10ToDBH>"
     << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
     << "</tr_conversionDiam10ToDBH>"
     << "<tr_fractOfHeightInCanopy>"
     << "<tr_fohicVal species=\"Species_1\">0.389</tr_fohicVal>"
     << "</tr_fractOfHeightInCanopy>"
     << "<tr_canopyDepthExponent>"
     << "<tr_cdeVal species=\"Species_1\">1.0</tr_cdeVal>"
     << "</tr_canopyDepthExponent>"
     << "<tr_slopeOfHeight-Diam10>"
     << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
     << "</tr_slopeOfHeight-Diam10>"
     << "<tr_minAdultDBH>"
     << "<tr_madVal species=\"Species_1\">0.0</tr_madVal>"
     << "</tr_minAdultDBH>"
     << "<tr_slopeOfAsymHeight>"
     << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
     << "</tr_slopeOfAsymHeight>"
     << "</trees>"
     << "<behaviorList>"
     << "<behavior>"
     << "<behaviorName>tree age calculator</behaviorName>"
     << "<version>1</version>"
     << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
     << "</behavior>"
     << "</behaviorList>";

  oOut << "<analysis>"
       << "<an_ignoreAdults>0</an_ignoreAdults>"
       << "</analysis>"
       << "</paramFile>";

  return cFileString;
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile2()
////////////////////////////////////////////////////////////////////////////
char* clTestTreeAgeCalculator::WriteXMLFile2() {
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
     << "<paramFile fileCode=\"06010101\">"
     << "<plot>"
     << "<timesteps>2</timesteps>"
     << "<yearsPerTimestep>3</yearsPerTimestep>"
     << "<randomSeed>1</randomSeed>"
     << "<plot_lenX>200.0</plot_lenX>"
     << "<plot_lenY>200.0</plot_lenY>"
     << "<plot_latitude>55.37</plot_latitude>"
     << "</plot>"
     << "<trees>"
     << "<tr_speciesList>"
     << "<tr_species speciesName=\"Species_1\" />"
     << "</tr_speciesList>"
     << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
     << "<tr_canopyHeight>"
     << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
     << "</tr_canopyHeight>"
     << "<tr_canopyRadiusToDBH>"
     << "<tr_crtdVal species=\"Species_1\">0.0549</tr_crtdVal>"
     << "</tr_canopyRadiusToDBH>"
     << "<tr_canopyRadiusExponent>"
     << "<tr_creVal species=\"Species_1\">1.0</tr_creVal>"
     << "</tr_canopyRadiusExponent>"
     << "<tr_conversionDiam10ToDBH>"
     << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
     << "</tr_conversionDiam10ToDBH>"
     << "<tr_fractOfHeightInCanopy>"
     << "<tr_fohicVal species=\"Species_1\">0.389</tr_fohicVal>"
     << "</tr_fractOfHeightInCanopy>"
     << "<tr_canopyDepthExponent>"
     << "<tr_cdeVal species=\"Species_1\">1.0</tr_cdeVal>"
     << "</tr_canopyDepthExponent>"
     << "<tr_slopeOfHeight-Diam10>"
     << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
     << "</tr_slopeOfHeight-Diam10>"
     << "<tr_minAdultDBH>"
     << "<tr_madVal species=\"Species_1\">0.0</tr_madVal>"
     << "</tr_minAdultDBH>"
     << "<tr_slopeOfAsymHeight>"
     << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
     << "</tr_slopeOfAsymHeight>"
     << "</trees>"
     << "<behaviorList>"
     << "<behavior>"
     << "<behaviorName>tree age calculator</behaviorName>"
     << "<version>1</version>"
     << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
     << "</behavior>"
     << "</behaviorList>";

  oOut << "<analysis>"
       << "<an_ignoreAdults>0</an_ignoreAdults>"
       << "</analysis>"
       << "</paramFile>";

  return cFileString;
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile1()
////////////////////////////////////////////////////////////////////////////
char* clTestTreeAgeCalculator::WriteXMLErrorFile1() {
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
     << "<paramFile fileCode=\"06010101\">"
     << "<plot>"
     << "<timesteps>2</timesteps>"
     << "<yearsPerTimestep>1</yearsPerTimestep>"
     << "<randomSeed>1</randomSeed>"
     << "<plot_lenX>200.0</plot_lenX>"
     << "<plot_lenY>200.0</plot_lenY>"
     << "<plot_latitude>55.37</plot_latitude>"
     << "</plot>"
     << "<trees>"
     << "<tr_speciesList>"
     << "<tr_species speciesName=\"Species_1\" />"
     << "</tr_speciesList>"
     << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
     << "<tr_canopyHeight>"
     << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
     << "</tr_canopyHeight>"
     << "<tr_canopyRadiusToDBH>"
     << "<tr_crtdVal species=\"Species_1\">0.0549</tr_crtdVal>"
     << "</tr_canopyRadiusToDBH>"
     << "<tr_canopyRadiusExponent>"
     << "<tr_creVal species=\"Species_1\">1.0</tr_creVal>"
     << "</tr_canopyRadiusExponent>"
     << "<tr_conversionDiam10ToDBH>"
     << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
     << "</tr_conversionDiam10ToDBH>"
     << "<tr_fractOfHeightInCanopy>"
     << "<tr_fohicVal species=\"Species_1\">0.389</tr_fohicVal>"
     << "</tr_fractOfHeightInCanopy>"
     << "<tr_canopyDepthExponent>"
     << "<tr_cdeVal species=\"Species_1\">1.0</tr_cdeVal>"
     << "</tr_canopyDepthExponent>"
     << "<tr_slopeOfHeight-Diam10>"
     << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
     << "</tr_slopeOfHeight-Diam10>"
     << "<tr_minAdultDBH>"
     << "<tr_madVal species=\"Species_1\">0.0</tr_madVal>"
     << "</tr_minAdultDBH>"
     << "<tr_slopeOfAsymHeight>"
     << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
     << "</tr_slopeOfAsymHeight>"
     << "</trees>"
     << "<behaviorList>"
     << "<behavior>"
     << "<behaviorName>tree age calculator</behaviorName>"
     << "<version>1</version>"
     << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
     << "</behavior>"
     << "</behaviorList>";

  oOut << "<analysis>"
       << "<an_ignoreAdults>bad</an_ignoreAdults>"
       << "</analysis>"
       << "</paramFile>";

  return cFileString;
}
