//---------------------------------------------------------------------------
// TestTreeAgeCalculator
//---------------------------------------------------------------------------
#if !defined(TestTreeAgeCalculator_H)
  #define TestTreeAgeCalculator_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>

class clSimManager;

/**
* Tests the class clTreeAgeCalculator.
*/
class clTestTreeAgeCalculator : public CppUnit::TestFixture {

  public:

  /**
  * Performs all the tests from one function.
  */
  void TestTreeAgeCalculator();

  protected:

  /**
  * Tests normal processing.  WriteXMLFile1() does setup.  3 trees are created
  * and their age checked to make sure it's correct.
  * @param p_oSimManager Sim Manager object to call for runs.
  */
  void TestNormalProcessing(clSimManager *p_oSimManager);

  /**
  * Tests error processing.  This uses WriteXMLErrorFile1() to introduce errors
  * and makes sure that they are caught.
  */
  void TestErrorProcessing(clSimManager *p_oSimManager);

  /**
  * Writes the parameter file for normal processing.  There is 1 species;
  * timestep length is 1 year; there is only one behavior, the tree age
  * calculator behavior.
  * @return Filename written.
  */
  char* WriteXMLFile1();

  /**
  * Writes the parameter file for normal processing.  There is 1 species;
  * timestep length is 3 years; there is only one behavior, the tree age
  * calculator behavior.
  * @return Filename written.
  */
  char* WriteXMLFile2();

  /**
  * Writes an error file where there is a bad value for the "ignore adults"
  * parameter.
  */
  char* WriteXMLErrorFile1();
};
//---------------------------------------------------------------------------
#endif // TestBiomassCalculator_H
