//---------------------------------------------------------------------------
// TestTreeRemover
//---------------------------------------------------------------------------
#if !defined(TestTreeRemover_H)
  #define TestTreeRemover_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>
#include <cppunit/TestSuite.h>

/**
* Tests the clTreeRemover class.
*/
class clTestTreeRemover : public CppUnit::TestFixture
{
  public:

  /**
  * Tests TreeRemover.  Reads in the file created by WriteXMLFile1().  The file
  * has the following characteristics:
  * <ul>
  * <li>3 species - Species 1, Species 2, Species 3.
  * <li>Adult self-thinning, applied to adults and snags of species 1 and 2.
  * <li>Tree remover behavior, applied to adults of Species 1.
  * <li>Tree map.  Each tree's number equals its X coordinate. The map has
  * the following trees:
  * <ul>
  * <li>X = 1 - Adult, Species 1.
  * <li>X = 2 - Adult, Species 1.
  * <li>X = 3 - Adult, Species 2.
  * <li>X = 4 - Adult, Species 2.
  * <li>X = 5 - Adult, Species 3.
  * <li>
  * </ul>
  * </ul>
  * The value of the "dead" data member is set to true for trees 1 and 3.  Then
  * the TreeRemover behavior is run.  Only tree 1 should go away.
  *
  * Once that's finished, error processing occurs with WriteXMLFile2(), which
  * has TreeRemover assigned to a tree species/type combo without mortality.
  */
  void TestTreeRemover();

  protected:

  /**
  * Writes a parameter file with the specifications given for TestTreeRemover().
  * @return Filename written.
  */
  char * WriteXMLFile1();

  /**
  * Writes a parameter file with an error - the TreeRemover behavior is
  * assigned to a tree species/type combo that is not also assigned to a
  * mortality behavior.
  * @return Filename written.
  */
  char * WriteXMLFile2();

};

//---------------------------------------------------------------------------
#endif // TestTreeRemover_H
