//---------------------------------------------------------------------------
// TestVolumeCalculator.cpp
//---------------------------------------------------------------------------
#include "TestVolumeCalculator.h"
#include <fstream.h>
#include <math.h>
#include "TreePopulation.h"
#include "SimManager.h"
#include "BehaviorBase.h"

////////////////////////////////////////////////////////////////////////////
//  TestVolumeCalculator()
////////////////////////////////////////////////////////////////////////////
void clTestVolumeCalculator::TestVolumeCalculator() {
  clSimManager *p_oSimManager = new clSimManager(4, 2, "");

  TestErrorProcessing(p_oSimManager);
  TestNormalProcessing(p_oSimManager);

  delete p_oSimManager;
}

////////////////////////////////////////////////////////////////////////////
//  TestNormalProcessing()
////////////////////////////////////////////////////////////////////////////
void clTestVolumeCalculator::TestNormalProcessing(clSimManager *p_oSimManager) {
  NormalRun1(p_oSimManager);
  NormalRun2(p_oSimManager);
  NormalRun3(p_oSimManager);
  NormalRun4(p_oSimManager);
  NormalRun5(p_oSimManager);
}

////////////////////////////////////////////////////////////////////////////
//  TestErrorProcessing()
////////////////////////////////////////////////////////////////////////////
void clTestVolumeCalculator::TestErrorProcessing(clSimManager *p_oSimManager) {

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile1());
    CppUnit::Asserter::fail("TestVolumeCalculator error processing didn't catch error for WriteXMLErrorFile1.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clVolumeCalculator::GetData") == NULL)
    {
      CppUnit::Asserter::fail("TestVolumeCalculator error processing caught wrong error for WriteXMLErrorFile1.");
    }
    ;
  }

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile2());
    CppUnit::Asserter::fail("TestVolumeCalculator error processing didn't catch error for WriteXMLErrorFile2.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clVolumeCalculator::GetData") == NULL)
    {
      CppUnit::Asserter::fail("TestVolumeCalculator error processing caught wrong error for WriteXMLErrorFile2.");
    }
    ;
  }

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile3());
    CppUnit::Asserter::fail("TestVolumeCalculator error processing didn't catch error for WriteXMLErrorFile3.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clVolumeCalculator::GetData") == NULL)
    {
      CppUnit::Asserter::fail("TestVolumeCalculator error processing caught wrong error for WriteXMLErrorFile3.");
    }
    ;
  }

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile4());
    CppUnit::Asserter::fail("TestVolumeCalculator error processing didn't catch error for WriteXMLErrorFile4.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clVolumeCalculator::RegisterTreeDataMembers") == NULL)
    {
      CppUnit::Asserter::fail("TestVolumeCalculator error processing caught wrong error for WriteXMLErrorFile4.");
    }
    ;
  }
}

////////////////////////////////////////////////////////////////////////////
//  NormalRun1()
////////////////////////////////////////////////////////////////////////////
void clTestVolumeCalculator::NormalRun1(clSimManager *p_oSimManager) {
  clPopulationBase * p_oTemp;
  clTreePopulation * p_oPop;
  clTree *p_oTargetTree1,
         *p_oTargetTree2,
         *p_oTargetTree3,
         *p_oTargetTree4,
         *p_oTargetTree5;
  float fDiam, fVolume, fExpectedVolume;
  int iSpecies;

  //Load our test file and get our tree population pointers
  p_oSimManager->ReadFile( WriteXMLFile1() );
  p_oTemp = p_oSimManager->GetPopulationObject( "treepopulation" );
  p_oPop = ( clTreePopulation * ) p_oTemp;

  //Create our trees
  fDiam = 15.552;
  iSpecies = 2;
  p_oTargetTree1 = p_oPop->CreateTree(10, 10, iSpecies, clTreePopulation::adult, fDiam);

  fDiam = 25.7345;
  iSpecies = 1;
  p_oTargetTree2 = p_oPop->CreateTree(10, 10, iSpecies, clTreePopulation::adult, fDiam);

  fDiam = 4;
  iSpecies = 2;
  p_oTargetTree3 = p_oPop->CreateTree(10, 10, iSpecies, clTreePopulation::adult, fDiam);

  fDiam = 8;
  iSpecies = 1;
  p_oTargetTree4 = p_oPop->CreateTree(10, 10, iSpecies, clTreePopulation::adult, fDiam);

  fDiam = 75;
  iSpecies = 2;
  p_oTargetTree5 = p_oPop->CreateTree(10, 10, iSpecies, clTreePopulation::adult, fDiam);

  //Now do the run
  p_oSimManager->RunSim(1);

  //Check the volumes
  p_oTargetTree1->GetValue(
        p_oPop->GetFloatDataCode("Volume",
                                 p_oTargetTree1->GetSpecies(),
                                 p_oTargetTree1->GetType()), &fVolume);
  fExpectedVolume = 0.064271476;
  CPPUNIT_ASSERT(fabs(fExpectedVolume - fVolume) < 0.001);

  p_oTargetTree2->GetValue(
        p_oPop->GetFloatDataCode("Volume",
                                 p_oTargetTree2->GetSpecies(),
                                 p_oTargetTree2->GetType()), &fVolume);
  fExpectedVolume = 0.469870084;
  CPPUNIT_ASSERT(fabs(fExpectedVolume - fVolume) < 0.001);

  p_oTargetTree3->GetValue(
        p_oPop->GetFloatDataCode("Volume",
                                 p_oTargetTree3->GetSpecies(),
                                 p_oTargetTree3->GetType()), &fVolume);
  fExpectedVolume = 0;
  CPPUNIT_ASSERT(fabs(fExpectedVolume - fVolume) < 0.001);

  p_oTargetTree4->GetValue(
        p_oPop->GetFloatDataCode("Volume",
                                 p_oTargetTree4->GetSpecies(),
                                 p_oTargetTree4->GetType()), &fVolume);
  fExpectedVolume = 0.010241452;
  CPPUNIT_ASSERT(fabs(fExpectedVolume - fVolume) < 0.001);

  p_oTargetTree5->GetValue(
        p_oPop->GetFloatDataCode("Volume",
                                 p_oTargetTree5->GetSpecies(),
                                 p_oTargetTree5->GetType()), &fVolume);
  fExpectedVolume = 2.594042442;
  CPPUNIT_ASSERT(fabs(fExpectedVolume - fVolume) < 0.001);
}

////////////////////////////////////////////////////////////////////////////
//  NormalRun2()
////////////////////////////////////////////////////////////////////////////
void clTestVolumeCalculator::NormalRun2(clSimManager *p_oSimManager) {
  clPopulationBase * p_oTemp;
  clTreePopulation * p_oPop;
  clTree *p_oTargetTree1,
         *p_oTargetTree2;
  float fDiam, fVolume, fExpectedVolume;
  int iSpecies;

  //Load our test file and get our tree population pointers
  p_oSimManager->ReadFile( WriteXMLFile2() );
  p_oTemp = p_oSimManager->GetPopulationObject( "treepopulation" );
  p_oPop = ( clTreePopulation * ) p_oTemp;

  //Create our trees
  fDiam = 0.5;
  iSpecies = 2;
  p_oTargetTree1 = p_oPop->CreateTree(10, 10, iSpecies, clTreePopulation::adult, fDiam);

  fDiam = 0.7125;
  iSpecies = 1;
  p_oTargetTree2 = p_oPop->CreateTree(10, 10, iSpecies, clTreePopulation::adult, fDiam);

  //Now do the run
  p_oSimManager->RunSim(1);

  //Check the volumes
  p_oTargetTree1->GetValue(
        p_oPop->GetFloatDataCode("Volume",
                                 p_oTargetTree1->GetSpecies(),
                                 p_oTargetTree1->GetType()), &fVolume);
  fExpectedVolume = 7.18624E-05;
  CPPUNIT_ASSERT(fabs(fExpectedVolume - fVolume) < 0.0001);

  p_oTargetTree2->GetValue(
        p_oPop->GetFloatDataCode("Volume",
                                 p_oTargetTree2->GetSpecies(),
                                 p_oTargetTree2->GetType()), &fVolume);
  fExpectedVolume = 2.04402E-05;
  CPPUNIT_ASSERT(fabs(fExpectedVolume - fVolume) < 0.0001);
}

////////////////////////////////////////////////////////////////////////////
//  NormalRun3()
////////////////////////////////////////////////////////////////////////////
void clTestVolumeCalculator::NormalRun3(clSimManager *p_oSimManager) {
  clPopulationBase * p_oTemp;
  clTreePopulation * p_oPop;
  clTree *p_oTargetTree1;
  float fDiam, fVolume, fExpectedVolume;
  int iSpecies;

  //Load our test file and get our tree population pointers
  p_oSimManager->ReadFile( WriteXMLFile3() );
  p_oTemp = p_oSimManager->GetPopulationObject( "treepopulation" );
  p_oPop = ( clTreePopulation * ) p_oTemp;

  //Create our trees
  fDiam = 0.5;
  iSpecies = 2;
  p_oTargetTree1 = p_oPop->CreateTree(10, 10, iSpecies, clTreePopulation::adult, fDiam);

  //Now do the run
  p_oSimManager->RunSim(1);

  //Check the volumes
  p_oTargetTree1->GetValue(
        p_oPop->GetFloatDataCode("Volume",
                                 p_oTargetTree1->GetSpecies(),
                                 p_oTargetTree1->GetType()), &fVolume);
  fExpectedVolume = 0;
  CPPUNIT_ASSERT(fabs(fExpectedVolume - fVolume) < 0.001);
}

////////////////////////////////////////////////////////////////////////////
//  NormalRun4()
////////////////////////////////////////////////////////////////////////////
void clTestVolumeCalculator::NormalRun4(clSimManager *p_oSimManager) {
  clPopulationBase * p_oTemp;
  clTreePopulation * p_oPop;
  clTree *p_oTargetTree1,
         *p_oTargetTree2,
         *p_oTargetTree3,
         *p_oTargetTree4;
  float fDiam, fVolume, fExpectedVolume;
  int iSpecies;

  //Load our test file and get our tree population pointers
  p_oSimManager->ReadFile( WriteXMLFile4() );
  p_oTemp = p_oSimManager->GetPopulationObject( "treepopulation" );
  p_oPop = ( clTreePopulation * ) p_oTemp;

  //Create our trees
  fDiam = 15.552;
  iSpecies = 2;
  p_oTargetTree1 = p_oPop->CreateTree(10, 10, iSpecies, clTreePopulation::adult, fDiam);

  fDiam = 25.7345;
  iSpecies = 1;
  p_oTargetTree2 = p_oPop->CreateTree(10, 10, iSpecies, clTreePopulation::adult, fDiam);

  fDiam = 8;
  iSpecies = 1;
  p_oTargetTree3 = p_oPop->CreateTree(10, 10, iSpecies, clTreePopulation::adult, fDiam);

  fDiam = 75;
  iSpecies = 2;
  p_oTargetTree4 = p_oPop->CreateTree(10, 10, iSpecies, clTreePopulation::adult, fDiam);

  //Now do the run
  p_oSimManager->RunSim(1);

  //Check the volumes
  p_oTargetTree1->GetValue(
        p_oPop->GetFloatDataCode("Volume",
                                 p_oTargetTree1->GetSpecies(),
                                 p_oTargetTree1->GetType()), &fVolume);
  fExpectedVolume = 0.066022717;
  CPPUNIT_ASSERT(fabs(fExpectedVolume - fVolume) < 0.001);

  p_oTargetTree2->GetValue(
        p_oPop->GetFloatDataCode("Volume",
                                 p_oTargetTree2->GetSpecies(),
                                 p_oTargetTree2->GetType()), &fVolume);
  fExpectedVolume = 0.452522026;
  CPPUNIT_ASSERT(fabs(fExpectedVolume - fVolume) < 0.001);

  p_oTargetTree3->GetValue(
        p_oPop->GetFloatDataCode("Volume",
                                 p_oTargetTree3->GetSpecies(),
                                 p_oTargetTree3->GetType()), &fVolume);
  fExpectedVolume = 0.014693001;
  CPPUNIT_ASSERT(fabs(fExpectedVolume - fVolume) < 0.0001);

  p_oTargetTree4->GetValue(
        p_oPop->GetFloatDataCode("Volume",
                                 p_oTargetTree4->GetSpecies(),
                                 p_oTargetTree4->GetType()), &fVolume);
  fExpectedVolume = 2.499628902;
  CPPUNIT_ASSERT(fabs(fExpectedVolume - fVolume) < 10);
}

////////////////////////////////////////////////////////////////////////////
//  NormalRun5()
////////////////////////////////////////////////////////////////////////////
void clTestVolumeCalculator::NormalRun5(clSimManager *p_oSimManager) {
  clPopulationBase * p_oTemp;
  clTreePopulation * p_oPop;
  clTree *p_oTargetTree1,
         *p_oTargetTree2;
  float fDiam, fVolume, fExpectedVolume;
  int iSpecies;

  //Load our test file and get our tree population pointers
  p_oSimManager->ReadFile( WriteXMLFile5() );
  p_oTemp = p_oSimManager->GetPopulationObject( "treepopulation" );
  p_oPop = ( clTreePopulation * ) p_oTemp;

  //Create our trees
  fDiam = 0.5;
  iSpecies = 2;
  p_oTargetTree1 = p_oPop->CreateTree(10, 10, iSpecies, clTreePopulation::adult, fDiam);

  fDiam = 0.7125;
  iSpecies = 1;
  p_oTargetTree2 = p_oPop->CreateTree(10, 10, iSpecies, clTreePopulation::adult, fDiam);

  //Now do the run
  p_oSimManager->RunSim(1);

  //Check the volumes
  p_oTargetTree1->GetValue(
        p_oPop->GetFloatDataCode("Volume",
                                 p_oTargetTree1->GetSpecies(),
                                 p_oTargetTree1->GetType()), &fVolume);
  fExpectedVolume = 0;
  CPPUNIT_ASSERT(fabs(fExpectedVolume - fVolume) < 0.001);

  p_oTargetTree2->GetValue(
        p_oPop->GetFloatDataCode("Volume",
                                 p_oTargetTree2->GetSpecies(),
                                 p_oTargetTree2->GetType()), &fVolume);
  fExpectedVolume = 0;
  CPPUNIT_ASSERT(fabs(fExpectedVolume - fVolume) < 0.001);
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile1()
////////////////////////////////////////////////////////////////////////////
char* clTestVolumeCalculator::WriteXMLFile1() {
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  WriteCommonStuff(oOut);

  oOut << "<vo_stumpHeight>0</vo_stumpHeight>"
       << "<vo_minUsableDiam>5</vo_minUsableDiam>"
       << "<vo_segmentLength>1</vo_segmentLength>"
       << "</volume>"
       << "</paramFile>";

  return cFileString;
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile2()
////////////////////////////////////////////////////////////////////////////
char* clTestVolumeCalculator::WriteXMLFile2() {
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  WriteCommonStuff(oOut);

  oOut << "<vo_stumpHeight>0</vo_stumpHeight>"
       << "<vo_minUsableDiam>0</vo_minUsableDiam>"
       << "<vo_segmentLength>2</vo_segmentLength>"
       << "</volume>"
       << "</paramFile>";

  return cFileString;
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile3()
////////////////////////////////////////////////////////////////////////////
char* clTestVolumeCalculator::WriteXMLFile3() {
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  WriteCommonStuff(oOut);

  oOut << "<vo_stumpHeight>200</vo_stumpHeight>"
       << "<vo_minUsableDiam>0</vo_minUsableDiam>"
       << "<vo_segmentLength>3</vo_segmentLength>"
       << "</volume>"
       << "</paramFile>";

  return cFileString;
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile4()
////////////////////////////////////////////////////////////////////////////
char* clTestVolumeCalculator::WriteXMLFile4() {
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  WriteCommonStuff(oOut);

  oOut << "<vo_stumpHeight>13</vo_stumpHeight>"
       << "<vo_minUsableDiam>0</vo_minUsableDiam>"
       << "<vo_segmentLength>0.75</vo_segmentLength>"
       << "</volume>"
       << "</paramFile>";

  return cFileString;
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile5()
////////////////////////////////////////////////////////////////////////////
char* clTestVolumeCalculator::WriteXMLFile5() {
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  WriteCommonStuff(oOut);

  oOut << "<vo_stumpHeight>0</vo_stumpHeight>"
       << "<vo_minUsableDiam>3</vo_minUsableDiam>"
       << "<vo_segmentLength>2</vo_segmentLength>"
       << "</volume>"
       << "</paramFile>";

  return cFileString;
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile1()
////////////////////////////////////////////////////////////////////////////
char* clTestVolumeCalculator::WriteXMLErrorFile1() {
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  WriteCommonStuff(oOut);

  oOut << "<vo_stumpHeight>-10</vo_stumpHeight>"
       << "<vo_minUsableDiam>5</vo_minUsableDiam>"
       << "<vo_segmentLength>1</vo_segmentLength>"
       << "</volume>"
       << "</paramFile>";

  return cFileString;
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile2()
////////////////////////////////////////////////////////////////////////////
char* clTestVolumeCalculator::WriteXMLErrorFile2() {
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  WriteCommonStuff(oOut);

  oOut << "<vo_stumpHeight>0</vo_stumpHeight>"
       << "<vo_minUsableDiam>-5</vo_minUsableDiam>"
       << "<vo_segmentLength>1</vo_segmentLength>"
       << "</volume>"
       << "</paramFile>";

  return cFileString;
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile3()
////////////////////////////////////////////////////////////////////////////
char* clTestVolumeCalculator::WriteXMLErrorFile3() {
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

 WriteCommonStuff(oOut);

  oOut << "<vo_stumpHeight>0</vo_stumpHeight>"
       << "<vo_minUsableDiam>5</vo_minUsableDiam>"
       << "<vo_segmentLength>-1</vo_segmentLength>"
       << "</volume>"
       << "</paramFile>";

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile4()
/////////////////////////////////////////////////////////////////////////////
char* clTestVolumeCalculator::WriteXMLErrorFile4()
{
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>2</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>"
       << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>200.0</plot_lenX>"
       << "<plot_lenY>200.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\" />"
       << "<tr_species speciesName=\"Species_2\" />"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_2\">39.54</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_canopyRadiusToDBH>"
       << "<tr_crtdVal species=\"Species_1\">0.0549</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_2\">0.0549</tr_crtdVal>"
       << "</tr_canopyRadiusToDBH>"
       << "<tr_canopyRadiusExponent>"
       << "<tr_creVal species=\"Species_1\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_2\">1.0</tr_creVal>"
       << "</tr_canopyRadiusExponent>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_2\">0.8008</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_fractOfHeightInCanopy>"
       << "<tr_fohicVal species=\"Species_1\">0.389</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_2\">0.389</tr_fohicVal>"
       << "</tr_fractOfHeightInCanopy>"
       << "<tr_canopyDepthExponent>"
       << "<tr_cdeVal species=\"Species_1\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_2\">1.0</tr_cdeVal>"
       << "</tr_canopyDepthExponent>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_2\">0.03418</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">0.0</tr_madVal>"
       << "<tr_madVal species=\"Species_2\">0.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_2\">0.0241</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "</trees>"
       << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>tree volume calculator</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
       << "</behavior>"
       << "</behaviorList>";

  oOut << "<volume>"
       << "<vo_taperA>"
       << "<vo_taVal species=\"Species_2\">0.843142</vo_taVal>"
       << "<vo_taVal species=\"Species_3\">0.8</vo_taVal>"
       << "</vo_taperA>"
       << "<vo_taperB>"
       << "<vo_tbVal species=\"Species_2\">1.00813</vo_tbVal>"
       << "<vo_tbVal species=\"Species_3\">0.9</vo_tbVal>"
       << "</vo_taperB>"
       << "<vo_taperC>"
       << "<vo_tcVal species=\"Species_2\">0.031402</vo_tcVal>"
       << "<vo_tcVal species=\"Species_3\">0.032</vo_tcVal>"
       << "</vo_taperC>"
       << "<vo_taperD>"
       << "<vo_tdVal species=\"Species_2\">0.319773</vo_tdVal>"
       << "<vo_tdVal species=\"Species_3\">0.321</vo_tdVal>"
       << "</vo_taperD>"
       << "<vo_taperF>"
       << "<vo_tfVal species=\"Species_2\">-0.532732</vo_tfVal>"
       << "<vo_tfVal species=\"Species_3\">-0.51</vo_tfVal>"
       << "</vo_taperF>"
       << "<vo_taperG>"
       << "<vo_tgVal species=\"Species_2\">0.585361</vo_tgVal>"
       << "<vo_tgVal species=\"Species_3\">0.59</vo_tgVal>"
       << "</vo_taperG>"
       << "<vo_taperI>"
       << "<vo_tiVal species=\"Species_2\">1.0315</vo_tiVal>"
       << "<vo_tiVal species=\"Species_3\">1.04</vo_tiVal>"
       << "</vo_taperI>"
       << "<vo_taperJ>"
       << "<vo_tjVal species=\"Species_2\">0.018622</vo_tjVal>"
       << "<vo_tjVal species=\"Species_3\">0.02</vo_tjVal>"
       << "</vo_taperJ>"
       << "<vo_taperK>"
       << "<vo_tkVal species=\"Species_2\">-0.225146</vo_tkVal>"
       << "<vo_tkVal species=\"Species_3\">-0.23</vo_tkVal>"
       << "</vo_taperK>"
       << "<vo_barkA>"
       << "<vo_baVal species=\"Species_2\">0.03</vo_baVal>"
       << "<vo_baVal species=\"Species_3\">0.01</vo_baVal>"
       << "</vo_barkA>"
       << "<vo_barkB>"
       << "<vo_bbVal species=\"Species_2\">0.04</vo_bbVal>"
       << "<vo_bbVal species=\"Species_3\">1</vo_bbVal>"
       << "</vo_barkB>"
       << "<vo_barkC>"
       << "<vo_bcVal species=\"Species_2\">-0.0005</vo_bcVal>"
       << "<vo_bcVal species=\"Species_3\">0.001</vo_bcVal>"
       << "</vo_barkC>";
  oOut << "<vo_stumpHeight>0</vo_stumpHeight>"
       << "<vo_minUsableDiam>5</vo_minUsableDiam>"
       << "<vo_segmentLength>1</vo_segmentLength>"
       << "</volume>"
       << "</paramFile>";

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteCommonStuff()
/////////////////////////////////////////////////////////////////////////////
void clTestVolumeCalculator::WriteCommonStuff( std::fstream &oOut )
{
  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>2</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>"
       << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>200.0</plot_lenX>"
       << "<plot_lenY>200.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\" />"
       << "<tr_species speciesName=\"Species_2\" />"
       << "<tr_species speciesName=\"Species_3\" />"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_2\">39.54</tr_chVal>"
       << "<tr_chVal species=\"Species_3\">45</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_canopyRadiusToDBH>"
       << "<tr_crtdVal species=\"Species_1\">0.0549</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_2\">0.0549</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_3\">0.0549</tr_crtdVal>"
       << "</tr_canopyRadiusToDBH>"
       << "<tr_canopyRadiusExponent>"
       << "<tr_creVal species=\"Species_1\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_2\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_3\">1.0</tr_creVal>"
       << "</tr_canopyRadiusExponent>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_2\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_3\">0.8008</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_fractOfHeightInCanopy>"
       << "<tr_fohicVal species=\"Species_1\">0.389</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_2\">0.389</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_3\">0.389</tr_fohicVal>"
       << "</tr_fractOfHeightInCanopy>"
       << "<tr_canopyDepthExponent>"
       << "<tr_cdeVal species=\"Species_1\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_2\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_3\">1.0</tr_cdeVal>"
       << "</tr_canopyDepthExponent>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_2\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_3\">0.03418</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">0.0</tr_madVal>"
       << "<tr_madVal species=\"Species_2\">0.0</tr_madVal>"
       << "<tr_madVal species=\"Species_3\">0.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_2\">0.0241</tr_soahVal>"
       << "<tr_soahVal species=\"Species_3\">0.0263</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "</trees>"
       << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>tree volume calculator</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_3\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_3\" type=\"Sapling\"/>"
       << "</behavior>"
       << "</behaviorList>";

  oOut << "<volume>"
       << "<vo_taperA>"
       << "<vo_taVal species=\"Species_2\">0.843142</vo_taVal>"
       << "<vo_taVal species=\"Species_3\">0.8</vo_taVal>"
       << "</vo_taperA>"
       << "<vo_taperB>"
       << "<vo_tbVal species=\"Species_2\">1.00813</vo_tbVal>"
       << "<vo_tbVal species=\"Species_3\">0.9</vo_tbVal>"
       << "</vo_taperB>"
       << "<vo_taperC>"
       << "<vo_tcVal species=\"Species_2\">0.031402</vo_tcVal>"
       << "<vo_tcVal species=\"Species_3\">0.032</vo_tcVal>"
       << "</vo_taperC>"
       << "<vo_taperD>"
       << "<vo_tdVal species=\"Species_2\">0.319773</vo_tdVal>"
       << "<vo_tdVal species=\"Species_3\">0.321</vo_tdVal>"
       << "</vo_taperD>"
       << "<vo_taperF>"
       << "<vo_tfVal species=\"Species_2\">-0.532732</vo_tfVal>"
       << "<vo_tfVal species=\"Species_3\">-0.51</vo_tfVal>"
       << "</vo_taperF>"
       << "<vo_taperG>"
       << "<vo_tgVal species=\"Species_2\">0.585361</vo_tgVal>"
       << "<vo_tgVal species=\"Species_3\">0.59</vo_tgVal>"
       << "</vo_taperG>"
       << "<vo_taperI>"
       << "<vo_tiVal species=\"Species_2\">1.0315</vo_tiVal>"
       << "<vo_tiVal species=\"Species_3\">1.04</vo_tiVal>"
       << "</vo_taperI>"
       << "<vo_taperJ>"
       << "<vo_tjVal species=\"Species_2\">0.018622</vo_tjVal>"
       << "<vo_tjVal species=\"Species_3\">0.02</vo_tjVal>"
       << "</vo_taperJ>"
       << "<vo_taperK>"
       << "<vo_tkVal species=\"Species_2\">-0.225146</vo_tkVal>"
       << "<vo_tkVal species=\"Species_3\">-0.23</vo_tkVal>"
       << "</vo_taperK>"
       << "<vo_barkA>"
       << "<vo_baVal species=\"Species_2\">0.2</vo_baVal>"
       << "<vo_baVal species=\"Species_3\">0.01</vo_baVal>"
       << "</vo_barkA>"
       << "<vo_barkB>"
       << "<vo_bbVal species=\"Species_2\">0.8</vo_bbVal>"
       << "<vo_bbVal species=\"Species_3\">1</vo_bbVal>"
       << "</vo_barkB>"
       << "<vo_barkC>"
       << "<vo_bcVal species=\"Species_2\">0.01</vo_bcVal>"
       << "<vo_bcVal species=\"Species_3\">0.001</vo_bcVal>"
       << "</vo_barkC>";
}
