//---------------------------------------------------------------------------
// TestNZEstablishment.cpp
//---------------------------------------------------------------------------
#include "TestNZEstablishment.h"
#include "SimManager.h"
#include "GridBase.h"
#include "TreePopulation.h"
#include <fstream.h>

/////////////////////////////////////////////////////////////////////////////
//  TestNZEstablishment()
/////////////////////////////////////////////////////////////////////////////
void clTestNZEstablishment::TestNZEstablishment() {
  clSimManager * p_oSimManager = new clSimManager(3, 3, "");

  TestGetGLIs(p_oSimManager);

  TestCalculateLightFavorabilities(p_oSimManager);

  TestLightTest(p_oSimManager);

  TestGetSubstrateProportions(p_oSimManager);

  TestGetFreshLogProportions(p_oSimManager);

  TestSetZAndRespite(p_oSimManager);

  TestAction(p_oSimManager);

  TestDoesSeedSurviveSubstrate(p_oSimManager);

  TestValidateData(p_oSimManager);

  TestMoundZDistributions(p_oSimManager);

  TestFreshLogZDistributions(p_oSimManager);

  delete p_oSimManager;
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  TestDoesSeedSurviveSubstrate()
/////////////////////////////////////////////////////////////////////////////
void clTestNZEstablishment::TestDoesSeedSurviveSubstrate(clSimManager *p_oSimManager) {
  clPopulationBase * p_oTemp;
  clTreePopulation * p_oPop;
  clGridBase *p_oSeeds, *p_oSubstrate;
  clTreeSearch *p_oAllSeedlings;
  clTree *p_oSeedling;
  float fValue, fExpectedNumber;
  int *p_iNumSeedlings = new int[10], i;

  for (i = 0; i < 10; i++) {
    p_iNumSeedlings[i] = 0;
  }

  //Set up the run
  p_oSimManager->ReadFile(WriteXMLFile6());
  p_oTemp = p_oSimManager->GetPopulationObject("treepopulation");
  p_oPop = (clTreePopulation *) p_oTemp;

  //In cell 0,0 of the "Disperse Seeds" grid, put 1000 seeds of each species
  p_oSeeds = p_oSimManager->GetGridObject("Dispersed Seeds");
  fValue = 1000;
  p_oSeeds->SetValueOfCell(0, 0, p_oSeeds->GetFloatDataCode("seeds_0"), fValue);
  p_oSeeds->SetValueOfCell(0, 0, p_oSeeds->GetFloatDataCode("seeds_1"), fValue);
  p_oSeeds->SetValueOfCell(0, 0, p_oSeeds->GetFloatDataCode("seeds_2"), fValue);
  p_oSeeds->SetValueOfCell(0, 0, p_oSeeds->GetFloatDataCode("seeds_3"), fValue);
  p_oSeeds->SetValueOfCell(0, 0, p_oSeeds->GetFloatDataCode("seeds_4"), fValue);
  p_oSeeds->SetValueOfCell(0, 0, p_oSeeds->GetFloatDataCode("seeds_5"), fValue);
  p_oSeeds->SetValueOfCell(0, 0, p_oSeeds->GetFloatDataCode("seeds_6"), fValue);
  p_oSeeds->SetValueOfCell(0, 0, p_oSeeds->GetFloatDataCode("seeds_7"), fValue);
  p_oSeeds->SetValueOfCell(0, 0, p_oSeeds->GetFloatDataCode("seeds_8"), fValue);
  p_oSeeds->SetValueOfCell(0, 0, p_oSeeds->GetFloatDataCode("seeds_9"), fValue);

  //In cell 0,0 of the "NZ Substrate" distribute the substrate equally
  p_oSubstrate = p_oSimManager->GetGridObject("NZ Substrate");
  fValue = 0.2;
  p_oSubstrate->SetValueOfCell(0, 0, p_oSubstrate->GetFloatDataCode("mossylog"), fValue);
  p_oSubstrate->SetValueOfCell(0, 0, p_oSubstrate->GetFloatDataCode("freshlog"), fValue);
  p_oSubstrate->SetValueOfCell(0, 0, p_oSubstrate->GetFloatDataCode("minsoil"), fValue);
  p_oSubstrate->SetValueOfCell(0, 0, p_oSubstrate->GetFloatDataCode("litter"), fValue);
  p_oSubstrate->SetValueOfCell(0, 0, p_oSubstrate->GetFloatDataCode("moss"), fValue);

  //Run Establishment
  p_oSimManager->GetBehaviorObject("NZ establishment")->Action();

  //Count the seedlings
  p_oAllSeedlings = p_oPop->Find("type=1");
  //Count them into their different categories
  p_oSeedling = p_oAllSeedlings->NextTree();
  while (p_oSeedling) {
    p_iNumSeedlings[p_oSeedling->GetSpecies()]++;
    p_oSeedling = p_oAllSeedlings->NextTree();
 }

 //Make sure all are within 20% with these small values
 fExpectedNumber = 10;
 CPPUNIT_ASSERT(fabs(fExpectedNumber - p_iNumSeedlings[0])/fExpectedNumber <= 0.2);
 fExpectedNumber = 20;
 CPPUNIT_ASSERT(fabs(fExpectedNumber - p_iNumSeedlings[1])/fExpectedNumber <= 0.2);
 fExpectedNumber = 30;
 CPPUNIT_ASSERT(fabs(fExpectedNumber - p_iNumSeedlings[2])/fExpectedNumber <= 0.2);
 fExpectedNumber = 40;
 CPPUNIT_ASSERT(fabs(fExpectedNumber - p_iNumSeedlings[3])/fExpectedNumber <= 0.2);
 fExpectedNumber = 50;
 CPPUNIT_ASSERT(fabs(fExpectedNumber - p_iNumSeedlings[4])/fExpectedNumber <= 0.2);
 fExpectedNumber = 60;
 CPPUNIT_ASSERT(fabs(fExpectedNumber - p_iNumSeedlings[5])/fExpectedNumber <= 0.2);
 fExpectedNumber = 70;
 CPPUNIT_ASSERT(fabs(fExpectedNumber - p_iNumSeedlings[6])/fExpectedNumber <= 0.2);
 fExpectedNumber = 80;
 CPPUNIT_ASSERT(fabs(fExpectedNumber - p_iNumSeedlings[7])/fExpectedNumber <= 0.2);
 fExpectedNumber = 90;
 CPPUNIT_ASSERT(fabs(fExpectedNumber - p_iNumSeedlings[8])/fExpectedNumber <= 0.2);
 fExpectedNumber = 100;
 CPPUNIT_ASSERT(fabs(fExpectedNumber - p_iNumSeedlings[9])/fExpectedNumber <= 0.2);
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  TestGetSubstrateProportions()
/////////////////////////////////////////////////////////////////////////////
void clTestNZEstablishment::TestGetSubstrateProportions(clSimManager *p_oSimManager) {
  clNZEstablishment *p_oEstablishment;
  clGridBase *p_oSubstrate, *p_oSeeds;
  float *p_fProportions, fExpected, fValue;
  int iX, iY;

  //************************************
  //First run:  mound proportion between 0 and 1
  //and substrate and seed grid cell resolutions are equal
  //************************************
  //Set up the run
  p_oSimManager->ReadFile(WriteXMLFile2());
  p_oSubstrate = p_oSimManager->GetGridObject("NZ Substrate");
  p_oSeeds = p_oSimManager->GetGridObject("Dispersed Seeds");

  //In cell 0,0 of the "NZ Substrate" set the substrate proportions
  iX = 0; iY = 0;
  fValue = 0.12;
  p_oSubstrate->SetValueOfCell(iX, iY, p_oSubstrate->GetFloatDataCode("mossylog"), fValue);
  fValue = 0.23;
  p_oSubstrate->SetValueOfCell(iX, iY, p_oSubstrate->GetFloatDataCode("freshlog"), fValue);
  fValue = 0.08;
  p_oSubstrate->SetValueOfCell(iX, iY, p_oSubstrate->GetFloatDataCode("moss"), fValue);
  fValue = 0.4;
  p_oSubstrate->SetValueOfCell(iX, iY, p_oSubstrate->GetFloatDataCode("litter"), fValue);
  fValue = 0.17;
  p_oSubstrate->SetValueOfCell(iX, iY, p_oSubstrate->GetFloatDataCode("minsoil"), fValue);

  //In cell 0,1 of the "NZ Substrate" set the substrate proportions
  iX = 0; iY = 1;
  fValue = 0.22;
  p_oSubstrate->SetValueOfCell(iX, iY, p_oSubstrate->GetFloatDataCode("mossylog"), fValue);
  fValue = 0;
  p_oSubstrate->SetValueOfCell(iX, iY, p_oSubstrate->GetFloatDataCode("freshlog"), fValue);
  fValue = 0.45;
  p_oSubstrate->SetValueOfCell(iX, iY, p_oSubstrate->GetFloatDataCode("moss"), fValue);
  fValue = 0.002;
  p_oSubstrate->SetValueOfCell(iX, iY, p_oSubstrate->GetFloatDataCode("litter"), fValue);
  fValue = 0.328;
  p_oSubstrate->SetValueOfCell(iX, iY, p_oSubstrate->GetFloatDataCode("minsoil"), fValue);

  //In cell 0,2 of the "NZ Substrate" set the substrate proportions
  iX = 0; iY = 2;
  fValue = 1;
  p_oSubstrate->SetValueOfCell(iX, iY, p_oSubstrate->GetFloatDataCode("mossylog"), fValue);
  fValue = 0;
  p_oSubstrate->SetValueOfCell(iX, iY, p_oSubstrate->GetFloatDataCode("freshlog"), fValue);
  p_oSubstrate->SetValueOfCell(iX, iY, p_oSubstrate->GetFloatDataCode("moss"), fValue);
  p_oSubstrate->SetValueOfCell(iX, iY, p_oSubstrate->GetFloatDataCode("litter"), fValue);
  p_oSubstrate->SetValueOfCell(iX, iY, p_oSubstrate->GetFloatDataCode("minsoil"), fValue);

  //Now run GetSubstrateProportions
  p_oEstablishment = dynamic_cast<clNZEstablishment *>(p_oSimManager->GetBehaviorObject("NZ establishment"));

  iX = 0, iY = 0;
  p_fProportions = p_oEstablishment->GetSubstrateProportions(
    p_oSeeds->GetOriginXOfCell(iX), p_oSeeds->GetOriginYOfCell(iY),
    p_oSeeds->GetEndXOfCell(iX), p_oSeeds->GetEndYOfCell(iY));
  fExpected = 0.09;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[clNZEstablishment::mound_mossylogs]) < 0.001);
  fExpected = 0.1725;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[clNZEstablishment::mound_freshlogs]) < 0.001);
  fExpected = 0.06;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[clNZEstablishment::mound_moss]) < 0.001);
  fExpected = 0.3;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[clNZEstablishment::mound_litter]) < 0.001);
  fExpected = 0.1275;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[clNZEstablishment::mound_mineralsoil]) < 0.001);
  fExpected = 0.03;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[clNZEstablishment::ground_mossylogs]) < 0.001);
  fExpected = 0.0575;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[clNZEstablishment::ground_freshlogs]) < 0.001);
  fExpected = 0.02;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[clNZEstablishment::ground_moss]) < 0.001);
  fExpected = 0.1;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[clNZEstablishment::ground_litter]) < 0.001);
  fExpected = 0.0425;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[clNZEstablishment::ground_mineralsoil]) < 0.001);
  delete[] p_fProportions;

  iX = 0, iY = 1;
  p_fProportions = p_oEstablishment->GetSubstrateProportions(
    p_oSeeds->GetOriginXOfCell(iX), p_oSeeds->GetOriginYOfCell(iY),
    p_oSeeds->GetEndXOfCell(iX), p_oSeeds->GetEndYOfCell(iY));
  fExpected = 0.165;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[clNZEstablishment::mound_mossylogs]) < 0.001);
  fExpected = 0;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[clNZEstablishment::mound_freshlogs]) < 0.001);
  fExpected = 0.3375;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[clNZEstablishment::mound_moss]) < 0.001);
  fExpected = 0.0015;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[clNZEstablishment::mound_litter]) < 0.001);
  fExpected = 0.246;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[clNZEstablishment::mound_mineralsoil]) < 0.001);
  fExpected = 0.055;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[clNZEstablishment::ground_mossylogs]) < 0.001);
  fExpected = 0;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[clNZEstablishment::ground_freshlogs]) < 0.001);
  fExpected = 0.1125;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[clNZEstablishment::ground_moss]) < 0.001);
  fExpected = 0.0005;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[clNZEstablishment::ground_litter]) < 0.001);
  fExpected = 0.082;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[clNZEstablishment::ground_mineralsoil]) < 0.001);
  delete[] p_fProportions;

  iX = 0, iY = 2;
  p_fProportions = p_oEstablishment->GetSubstrateProportions(
    p_oSeeds->GetOriginXOfCell(iX), p_oSeeds->GetOriginYOfCell(iY),
    p_oSeeds->GetEndXOfCell(iX), p_oSeeds->GetEndYOfCell(iY));
  fExpected = 0.75;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[clNZEstablishment::mound_mossylogs]) < 0.001);
  fExpected = 0;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[clNZEstablishment::mound_freshlogs]) < 0.001);
  fExpected = 0;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[clNZEstablishment::mound_moss]) < 0.001);
  fExpected = 0;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[clNZEstablishment::mound_litter]) < 0.001);
  fExpected = 0;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[clNZEstablishment::mound_mineralsoil]) < 0.001);
  fExpected = 0.25;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[clNZEstablishment::ground_mossylogs]) < 0.001);
  fExpected = 0;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[clNZEstablishment::ground_freshlogs]) < 0.001);
  fExpected = 0;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[clNZEstablishment::ground_moss]) < 0.001);
  fExpected = 0;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[clNZEstablishment::ground_litter]) < 0.001);
  fExpected = 0;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[clNZEstablishment::ground_mineralsoil]) < 0.001);
  delete[] p_fProportions;


}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  TestGetFreshLogProportions()
/////////////////////////////////////////////////////////////////////////////
void clTestNZEstablishment::TestGetFreshLogProportions(clSimManager *p_oSimManager) {
  clNZEstablishment *p_oEstablishment;
  clGridBase *p_oSubstrate, *p_oSeeds;
  clPackage *p_oCohort;
  float *p_fProportions, fExpected, fValue;
  int iX, iY, iValue;

  //************************************
  //First run:  Substrate and seed grid cell resolutions are equal
  //************************************
  //Set up the run
  p_oSimManager->ReadFile(WriteXMLFile2());
  p_oSubstrate = p_oSimManager->GetGridObject("NZ Substrate");
  p_oSeeds = p_oSimManager->GetGridObject("Dispersed Seeds");

  //In cell 0,0 of the "NZ Substrate" set the cohorts
  iX = 0; iY = 0;
  p_oCohort = p_oSubstrate->CreatePackageOfCell(iX, iY);
  fValue = 0.777417188;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 3;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.521118325;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);
//  fValue = 20;
//  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("avglogsize"), fValue);

  //In cell 0,1 of the "NZ Substrate" set the cohorts
  iX = 0; iY = 1;
  p_oCohort = p_oSubstrate->CreatePackageOfCell(iX, iY);
  fValue = 0.258861021;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 4;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.067107199;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);
//  fValue = 20;
//  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("avglogsize"), fValue);

  p_oCohort = p_oSubstrate->CreatePackage(p_oCohort);
  fValue = 0.293457344;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 3;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.19671034;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);
//  fValue = 48.7;
//  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("avglogsize"), fValue);

  p_oCohort = p_oSubstrate->CreatePackage(p_oCohort);
  fValue = 0.071340092;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 2;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.067860795;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);
//  fValue = 12.4;
//  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("avglogsize"), fValue);

  p_oCohort = p_oSubstrate->CreatePackage(p_oCohort);
  fValue = 0.13;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 1;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.13;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);
//  fValue = 35;
//  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("avglogsize"), fValue);

  //In cell 0,2 of the "NZ Substrate" set the cohorts
  iX = 0; iY = 2;
  p_oCohort = p_oSubstrate->CreatePackageOfCell(iX, iY);
  fValue = 0.786357413;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 3;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.527111137;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);
//  fValue = 20;
//  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("avglogsize"), fValue);

  p_oCohort = p_oSubstrate->CreatePackage(p_oCohort);
  fValue = 0.106292692;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 2;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.101108736;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  p_oCohort = p_oSubstrate->CreatePackage(p_oCohort);
  fValue = 0.071340092;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 1;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.071340092;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  //Cell 0,3 will have no cohorts

  //Now run GetFreshLogProportions
  p_oEstablishment = dynamic_cast<clNZEstablishment *>(p_oSimManager->GetBehaviorObject("NZ establishment"));

  iX = 0, iY = 0;
  p_fProportions = p_oEstablishment->GetFreshLogProportions(
    p_oSeeds->GetOriginXOfCell(iX), p_oSeeds->GetOriginYOfCell(iY),
    p_oSeeds->GetEndXOfCell(iX), p_oSeeds->GetEndYOfCell(iY));
  fExpected = 0;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[0]) < 0.001);
  fExpected = 0;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[1]) < 0.001);
  fExpected = 1;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[2]) < 0.001);
  fExpected = 1;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[3]) < 0.001);
  delete[] p_fProportions;

  iX = 0, iY = 1;
  p_fProportions = p_oEstablishment->GetFreshLogProportions(
    p_oSeeds->GetOriginXOfCell(iX), p_oSeeds->GetOriginYOfCell(iY),
    p_oSeeds->GetEndXOfCell(iX), p_oSeeds->GetEndYOfCell(iY));
  fExpected = 0.281581332;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[0]) < 0.001);
  fExpected = 0.428568508;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[1]) < 0.001);
  fExpected = 0.85464512;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[2]) < 0.001);
  fExpected = 1;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[3]) < 0.001);
  delete[] p_fProportions;

  iX = 0, iY = 2;
  p_fProportions = p_oEstablishment->GetFreshLogProportions(
    p_oSeeds->GetOriginXOfCell(iX), p_oSeeds->GetOriginYOfCell(iY),
    p_oSeeds->GetEndXOfCell(iX), p_oSeeds->GetEndYOfCell(iY));
  fExpected = 0.101978523;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[0]) < 0.001);
  fExpected = 0.24651043;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[1]) < 0.001);
  fExpected = 1;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[2]) < 0.001);
  fExpected = 1;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[3]) < 0.001);
  delete[] p_fProportions;

   iX = 0, iY = 3;
  p_fProportions = p_oEstablishment->GetFreshLogProportions(
    p_oSeeds->GetOriginXOfCell(iX), p_oSeeds->GetOriginYOfCell(iY),
    p_oSeeds->GetEndXOfCell(iX), p_oSeeds->GetEndYOfCell(iY));
  fExpected = 0;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[0]) < 0.001);
  fExpected = 0;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[1]) < 0.001);
  fExpected = 0;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[2]) < 0.001);
  fExpected = 1;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[3]) < 0.001);
  delete[] p_fProportions;


  //************************************
  //Second run:  Substrate has a bigger cell resolution than seed
  //************************************
  //Set up the run
  p_oSimManager->ReadFile(WriteXMLFile3());
  p_oSubstrate = p_oSimManager->GetGridObject("NZ Substrate");
  p_oSeeds = p_oSimManager->GetGridObject("Dispersed Seeds");

  //In cell 0,0 of the "NZ Substrate" set the cohorts
  iX = 0; iY = 0;
  p_oCohort = p_oSubstrate->CreatePackageOfCell(iX, iY);
  fValue = 0.577417188;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 2;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.54925622;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  //In cell 0,1 of the "NZ Substrate" set the cohorts
  iX = 0; iY = 1;
  p_oCohort = p_oSubstrate->CreatePackageOfCell(iX, iY);
  fValue = 0.258861021;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 4;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.067107199;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  p_oCohort = p_oSubstrate->CreatePackage(p_oCohort);
  fValue = 0.293457344;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 3;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.19671034;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  p_oCohort = p_oSubstrate->CreatePackage(p_oCohort);
  fValue = 0.071340092;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 2;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.067860795;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  p_oCohort = p_oSubstrate->CreatePackage(p_oCohort);
  fValue = 0.13;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 1;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.13;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  //In cell 0,2 of the "NZ Substrate" set the cohorts
  iX = 0; iY = 2;
  p_oCohort = p_oSubstrate->CreatePackageOfCell(iX, iY);
  fValue = 0.786357413;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 3;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.527111137;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  p_oCohort = p_oSubstrate->CreatePackage(p_oCohort);
  fValue = 0.106292692;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 2;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.101108736;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  p_oCohort = p_oSubstrate->CreatePackage(p_oCohort);
  fValue = 0.071340092;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 1;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.071340092;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  //Cell 0,3 will have no cohorts

  //Now run GetSubstrateProportions
  p_oEstablishment = dynamic_cast<clNZEstablishment *>(p_oSimManager->GetBehaviorObject("NZ establishment"));

  iX = 0, iY = 0;
  p_fProportions = p_oEstablishment->GetFreshLogProportions(
    p_oSeeds->GetOriginXOfCell(iX), p_oSeeds->GetOriginYOfCell(iY),
    p_oSeeds->GetEndXOfCell(iX), p_oSeeds->GetEndYOfCell(iY));
  fExpected = 0;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[0]) < 0.001);
  fExpected = 1;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[1]) < 0.001);
  fExpected = 1;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[2]) < 0.001);
  fExpected = 1;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[3]) < 0.001);
  delete[] p_fProportions;

  iX = 0, iY = 1;
  p_fProportions = p_oEstablishment->GetFreshLogProportions(
    p_oSeeds->GetOriginXOfCell(iX), p_oSeeds->GetOriginYOfCell(iY),
    p_oSeeds->GetEndXOfCell(iX), p_oSeeds->GetEndYOfCell(iY));
  fExpected = 0;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[0]) < 0.001);
  fExpected = 1;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[1]) < 0.001);
  fExpected = 1;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[2]) < 0.001);
  fExpected = 1;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[3]) < 0.001);
  delete[] p_fProportions;

  iX = 0, iY = 2;
  p_fProportions = p_oEstablishment->GetFreshLogProportions(
    p_oSeeds->GetOriginXOfCell(iX), p_oSeeds->GetOriginYOfCell(iY),
    p_oSeeds->GetEndXOfCell(iX), p_oSeeds->GetEndYOfCell(iY));
  fExpected = 0.281581332;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[0]) < 0.001);
  fExpected = 0.428568508;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[1]) < 0.001);
  fExpected = 0.85464512;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[2]) < 0.001);
  fExpected = 1;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[3]) < 0.001);
  delete[] p_fProportions;

  iX = 0, iY = 3;
  p_fProportions = p_oEstablishment->GetFreshLogProportions(
    p_oSeeds->GetOriginXOfCell(iX), p_oSeeds->GetOriginYOfCell(iY),
    p_oSeeds->GetEndXOfCell(iX), p_oSeeds->GetEndYOfCell(iY));
  fExpected = 0.281581332;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[0]) < 0.001);
  fExpected = 0.428568508;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[1]) < 0.001);
  fExpected = 0.85464512;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[2]) < 0.001);
  fExpected = 1;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[3]) < 0.001);
  delete[] p_fProportions;

  iX = 0, iY = 4;
  p_fProportions = p_oEstablishment->GetFreshLogProportions(
    p_oSeeds->GetOriginXOfCell(iX), p_oSeeds->GetOriginYOfCell(iY),
    p_oSeeds->GetEndXOfCell(iX), p_oSeeds->GetEndYOfCell(iY));
  fExpected = 0.101978523;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[0]) < 0.001);
  fExpected = 0.24651043;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[1]) < 0.001);
  fExpected = 1;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[2]) < 0.001);
  fExpected = 1;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[3]) < 0.001);
  delete[] p_fProportions;

  iX = 0, iY = 5;
  p_fProportions = p_oEstablishment->GetFreshLogProportions(
    p_oSeeds->GetOriginXOfCell(iX), p_oSeeds->GetOriginYOfCell(iY),
    p_oSeeds->GetEndXOfCell(iX), p_oSeeds->GetEndYOfCell(iY));
  fExpected = 0.101978523;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[0]) < 0.001);
  fExpected = 0.24651043;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[1]) < 0.001);
  fExpected = 1;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[2]) < 0.001);
  fExpected = 1;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[3]) < 0.001);
  delete[] p_fProportions;

  iX = 0, iY = 6;
  p_fProportions = p_oEstablishment->GetFreshLogProportions(
    p_oSeeds->GetOriginXOfCell(iX), p_oSeeds->GetOriginYOfCell(iY),
    p_oSeeds->GetEndXOfCell(iX), p_oSeeds->GetEndYOfCell(iY));
  fExpected = 0;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[0]) < 0.001);
  fExpected = 0;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[1]) < 0.001);
  fExpected = 0;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[2]) < 0.001);
  fExpected = 1;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[3]) < 0.001);
  delete[] p_fProportions;

  iX = 0, iY = 7;
  p_fProportions = p_oEstablishment->GetFreshLogProportions(
    p_oSeeds->GetOriginXOfCell(iX), p_oSeeds->GetOriginYOfCell(iY),
    p_oSeeds->GetEndXOfCell(iX), p_oSeeds->GetEndYOfCell(iY));
  fExpected = 0;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[0]) < 0.001);
  fExpected = 0;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[1]) < 0.001);
  fExpected = 0;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[2]) < 0.001);
  fExpected = 1;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[3]) < 0.001);
  delete[] p_fProportions;

  //************************************
  //Third run:  Substrate has a smaller grid cell resolution than seeds
  //************************************
  //Set up the run
  p_oSimManager->ReadFile(WriteXMLFile5());
  p_oSubstrate = p_oSimManager->GetGridObject("NZ Substrate");
  p_oSeeds = p_oSimManager->GetGridObject("Dispersed Seeds");

  //In cell 0,0 of the "NZ Substrate" set the cohorts
  iX = 0; iY = 0;
  p_oCohort = p_oSubstrate->CreatePackageOfCell(iX, iY);
  fValue = 0.577417188;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 1;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.577417188;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  //In cell 0,1 of the "NZ Substrate" set the cohorts
  iX = 0; iY = 1;
  p_oCohort = p_oSubstrate->CreatePackageOfCell(iX, iY);
  fValue = 0.588610213;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 4;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.152591465;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  p_oCohort = p_oSubstrate->CreatePackage(p_oCohort);
  fValue = 0.293457344;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 3;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.19671034;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  p_oCohort = p_oSubstrate->CreatePackage(p_oCohort);
  fValue = 0.071340092;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 2;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.067860795;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  p_oCohort = p_oSubstrate->CreatePackage(p_oCohort);
  fValue = 0.13;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 1;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.13;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  //In cell 0,2 of the "NZ Substrate" set the cohorts
  iX = 0; iY = 2;
  p_oCohort = p_oSubstrate->CreatePackageOfCell(iX, iY);
  fValue = 0.786357413;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 3;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.527111137;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  p_oCohort = p_oSubstrate->CreatePackage(p_oCohort);
  fValue = 0.106292692;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 2;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.101108736;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  p_oCohort = p_oSubstrate->CreatePackage(p_oCohort);
  fValue = 0.071340092;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 1;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.071340092;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  //No cohorts for cell 0,3

  //In cell 0,4 of the "NZ Substrate" set the cohorts
  iX = 0; iY = 4;
  p_oCohort = p_oSubstrate->CreatePackageOfCell(iX, iY);
  fValue = 0.386357413;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 3;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.258983119;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  p_oCohort = p_oSubstrate->CreatePackage(p_oCohort);
  fValue = 0.106292692;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 2;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.101108736;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  //In cell 0,5 of the "NZ Substrate" set the cohorts
  iX = 0; iY = 5;
  p_oCohort = p_oSubstrate->CreatePackageOfCell(iX, iY);
  fValue = 0.577417188;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 1;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.577417188;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  //In cell 0,6 of the "NZ Substrate" set the cohorts
  iX = 0; iY = 6;
  p_oCohort = p_oSubstrate->CreatePackageOfCell(iX, iY);
  fValue = 0.588610213;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 4;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.152591465;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  p_oCohort = p_oSubstrate->CreatePackage(p_oCohort);
  fValue = 0.293457344;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 3;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.19671034;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  p_oCohort = p_oSubstrate->CreatePackage(p_oCohort);
  fValue = 0.071340092;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 2;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.067860795;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  p_oCohort = p_oSubstrate->CreatePackage(p_oCohort);
  fValue = 0.13;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 1;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.13;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  //In cell 0,7 of the "NZ Substrate" set the cohorts
  iX = 0; iY = 7;
  p_oCohort = p_oSubstrate->CreatePackageOfCell(iX, iY);
  fValue = 0.786357413;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 3;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.527111137;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  p_oCohort = p_oSubstrate->CreatePackage(p_oCohort);
  fValue = 0.106292692;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 2;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.101108736;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  p_oCohort = p_oSubstrate->CreatePackage(p_oCohort);
  fValue = 0.071340092;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 1;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.071340092;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  //No cohorts for cell 0,8

  //In cell 0,9 of the "NZ Substrate" set the cohorts
  iX = 0; iY = 9;
  p_oCohort = p_oSubstrate->CreatePackageOfCell(iX, iY);
  fValue = 0.386357413;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 3;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.258983119;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  p_oCohort = p_oSubstrate->CreatePackage(p_oCohort);
  fValue = 0.106292692;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 2;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.101108736;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  //In cell 0,10 of the "NZ Substrate" set the cohorts
  iX = 0; iY = 10;
  p_oCohort = p_oSubstrate->CreatePackageOfCell(iX, iY);
  fValue = 0.577417188;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 1;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.577417188;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  //In cell 0,11 of the "NZ Substrate" set the cohorts
  iX = 0; iY = 11;
  p_oCohort = p_oSubstrate->CreatePackageOfCell(iX, iY);
  fValue = 0.588610213;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 4;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.152591465;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  p_oCohort = p_oSubstrate->CreatePackage(p_oCohort);
  fValue = 0.293457344;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 3;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.19671034;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  p_oCohort = p_oSubstrate->CreatePackage(p_oCohort);
  fValue = 0.071340092;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 2;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.067860795;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  p_oCohort = p_oSubstrate->CreatePackage(p_oCohort);
  fValue = 0.13;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 1;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.13;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  //In cell 0,12 of the "NZ Substrate" set the cohorts
  iX = 0; iY = 12;
  p_oCohort = p_oSubstrate->CreatePackageOfCell(iX, iY);
  fValue = 0.786357413;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 3;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.527111137;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  p_oCohort = p_oSubstrate->CreatePackage(p_oCohort);
  fValue = 0.106292692;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 2;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.101108736;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  p_oCohort = p_oSubstrate->CreatePackage(p_oCohort);
  fValue = 0.071340092;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 1;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.071340092;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  //No cohorts for cell 0,13

  //In cell 0,14 of the "NZ Substrate" set the cohorts
  iX = 0; iY = 14;
  p_oCohort = p_oSubstrate->CreatePackageOfCell(iX, iY);
  fValue = 0.386357413;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 3;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.258983119;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  p_oCohort = p_oSubstrate->CreatePackage(p_oCohort);
  fValue = 0.106292692;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 2;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.101108736;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  //In cell 0,15 of the "NZ Substrate" set the cohorts
  iX = 0; iY = 15;
  p_oCohort = p_oSubstrate->CreatePackageOfCell(iX, iY);
  fValue = 0.577417188;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 1;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.577417188;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  //Now run GetFreshLogProportions
  p_oEstablishment = dynamic_cast<clNZEstablishment *>(p_oSimManager->GetBehaviorObject("NZ establishment"));

  iX = 0, iY = 0;
  p_fProportions = p_oEstablishment->GetFreshLogProportions(
    p_oSeeds->GetOriginXOfCell(iX), p_oSeeds->GetOriginYOfCell(iY),
    p_oSeeds->GetEndXOfCell(iX), p_oSeeds->GetEndYOfCell(iY));
  fExpected = 0.426917553;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[0]) < 0.001);
  fExpected = 0.51954726;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[1]) < 0.001);
  fExpected = 0.916348808;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[2]) < 0.001);
  fExpected = 1;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[3]) < 0.001);
  delete[] p_fProportions;

  iX = 0, iY = 1;
  p_fProportions = p_oEstablishment->GetFreshLogProportions(
    p_oSeeds->GetOriginXOfCell(iX), p_oSeeds->GetOriginYOfCell(iY),
    p_oSeeds->GetEndXOfCell(iX), p_oSeeds->GetEndYOfCell(iY));
  fExpected = 0.356536036;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[0]) < 0.001);
  fExpected = 0.480185134;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[1]) < 0.001);
  fExpected = 0.930139522;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[2]) < 0.001);
  fExpected = 1;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[3]) < 0.001);
  delete[] p_fProportions;

  iX = 0, iY = 2;
  p_fProportions = p_oEstablishment->GetFreshLogProportions(
    p_oSeeds->GetOriginXOfCell(iX), p_oSeeds->GetOriginYOfCell(iY),
    p_oSeeds->GetEndXOfCell(iX), p_oSeeds->GetEndYOfCell(iY));
  fExpected = 0.476480568;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[0]) < 0.001);
  fExpected = 0.590289929;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[1]) < 0.001);
  fExpected = 0.897222079;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[2]) < 0.001);
  fExpected = 1;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[3]) < 0.001);
  delete[] p_fProportions;

  iX = 0, iY = 3;
  p_fProportions = p_oEstablishment->GetFreshLogProportions(
    p_oSeeds->GetOriginXOfCell(iX), p_oSeeds->GetOriginYOfCell(iY),
    p_oSeeds->GetEndXOfCell(iX), p_oSeeds->GetEndYOfCell(iY));
  fExpected = 0.396291957;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[0]) < 0.001);
  fExpected = 0.519816054;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[1]) < 0.001);
  fExpected = 1;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[2]) < 0.001);
  fExpected = 1;
  CPPUNIT_ASSERT(fabs(fExpected - p_fProportions[3]) < 0.001);
  delete[] p_fProportions;
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  TestLightTest()
/////////////////////////////////////////////////////////////////////////////
void clTestNZEstablishment::TestLightTest(clSimManager *p_oSimManager) {
  clPopulationBase * p_oTemp;
  clTreePopulation * p_oPop;
  clGridBase *p_oSeeds;
  clTreeSearch *p_oAllSeedlings;
  clTree *p_oSeedling;
  float fValue, fExpectedNumber = 822.9253;
  int iNumSeedlings = 0;

  //Set up the run
  p_oSimManager->ReadFile(WriteXMLFile2());
  p_oTemp = p_oSimManager->GetPopulationObject("treepopulation");
  p_oPop = (clTreePopulation *) p_oTemp;

  //In cell 0,0 of the "Disperse Seeds" grid, put 1000 seeds of species 2
  p_oSeeds = p_oSimManager->GetGridObject("Dispersed Seeds");
  fValue = 1000;
  p_oSeeds->SetValueOfCell(0, 0, p_oSeeds->GetFloatDataCode("seeds_1"), fValue);

  //Make there be some substrate - any substrate
  fValue = 1;
  p_oSimManager->GetGridObject("NZ Substrate")->SetValueOfCell(0,0,0,fValue);

  //Run Establishment
  p_oSimManager->GetBehaviorObject("NZ establishment")->Action();

  //Count the seedlings
  p_oAllSeedlings = p_oPop->Find("type=1");
  //Count them into their different categories
  p_oSeedling = p_oAllSeedlings->NextTree();
  while (p_oSeedling) {
    iNumSeedlings++;
    p_oSeedling = p_oAllSeedlings->NextTree();
 }

 //Make sure it's within 2%
 CPPUNIT_ASSERT(fabs(fExpectedNumber - iNumSeedlings)/fExpectedNumber < 0.02);


}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  TestMoundZDistributions()
/////////////////////////////////////////////////////////////////////////////
void clTestNZEstablishment::TestMoundZDistributions(clSimManager *p_oSimManager) {
  fstream oOut;
  clPopulationBase * p_oTemp;
  clTreePopulation * p_oPop;
  clTreeSearch *p_oAllSeedlings;
  clTree *p_oSeedling;
  clGridBase *p_oSeeds, *p_oSubstrate;
  float fValue;
  int iZValue;

  //Set up the run
  p_oSimManager->ReadFile(WriteXMLFile3());
  p_oTemp = p_oSimManager->GetPopulationObject("treepopulation");
  p_oPop = (clTreePopulation *) p_oTemp;

  //In cell 0,0 of the "Disperse Seeds" grid, put 1000 seeds of species 1
  p_oSeeds = p_oSimManager->GetGridObject("Dispersed Seeds");
  fValue = 1000;
  p_oSeeds->SetValueOfCell(0, 0, p_oSeeds->GetFloatDataCode("seeds_0"), fValue);

  //In cell 0,0 of the "NZ Substrate" make the substrate all moss
  p_oSubstrate = p_oSimManager->GetGridObject("NZ Substrate");
  fValue = 0;
  p_oSubstrate->SetValueOfCell(0, 0, p_oSubstrate->GetFloatDataCode("mossylog"), fValue);
  p_oSubstrate->SetValueOfCell(0, 0, p_oSubstrate->GetFloatDataCode("freshlog"), fValue);
  p_oSubstrate->SetValueOfCell(0, 0, p_oSubstrate->GetFloatDataCode("minsoil"), fValue);
  p_oSubstrate->SetValueOfCell(0, 0, p_oSubstrate->GetFloatDataCode("litter"), fValue);
  fValue = 1;
  p_oSubstrate->SetValueOfCell(0, 0, p_oSubstrate->GetFloatDataCode("moss"), fValue);

  //Run establishment
  p_oSimManager->GetBehaviorObject("NZ establishment")->Action();

  //Open file to write to
  oOut.open("MoundZ.txt", ios::out | ios::trunc);
  oOut << "Height in mm\n";

  //Get all the seedlings
  p_oAllSeedlings = p_oPop->Find("type=1");
  //Count them into their different categories
  p_oSeedling = p_oAllSeedlings->NextTree();
  while (p_oSeedling) {
    //Get the Z value
    p_oSeedling->GetValue(p_oPop->GetIntDataCode("z", p_oSeedling->GetSpecies(), p_oSeedling->GetType()), &iZValue);

    oOut << iZValue << "\n";

    p_oSeedling = p_oAllSeedlings->NextTree();
 }

  oOut.close();
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  TestFreshLogZDistributions()
/////////////////////////////////////////////////////////////////////////////
void clTestNZEstablishment::TestFreshLogZDistributions(clSimManager *p_oSimManager) {
  fstream oOut;
  clPopulationBase * p_oTemp;
  clTreePopulation * p_oPop;
  clTreeSearch *p_oAllSeedlings;
  clTree *p_oSeedling;
  clGridBase *p_oSeeds, *p_oSubstrate;
  float fValue;
  int iZValue;

  //Set up the run
  p_oSimManager->ReadFile(WriteXMLFile3());
  p_oTemp = p_oSimManager->GetPopulationObject("treepopulation");
  p_oPop = (clTreePopulation *) p_oTemp;

  //In cell 0,0 of the "Disperse Seeds" grid, put 1000 seeds of species 1
  p_oSeeds = p_oSimManager->GetGridObject("Dispersed Seeds");
  fValue = 1000;
  p_oSeeds->SetValueOfCell(0, 0, p_oSeeds->GetFloatDataCode("seeds_0"), fValue);

  //In cell 0,0 of the "NZ Substrate" make the substrate all fresh log
  p_oSubstrate = p_oSimManager->GetGridObject("NZ Substrate");
  fValue = 0;
  p_oSubstrate->SetValueOfCell(0, 0, p_oSubstrate->GetFloatDataCode("mossylog"), fValue);
  p_oSubstrate->SetValueOfCell(0, 0, p_oSubstrate->GetFloatDataCode("moss"), fValue);
  p_oSubstrate->SetValueOfCell(0, 0, p_oSubstrate->GetFloatDataCode("minsoil"), fValue);
  p_oSubstrate->SetValueOfCell(0, 0, p_oSubstrate->GetFloatDataCode("litter"), fValue);
  fValue = 1;
  p_oSubstrate->SetValueOfCell(0, 0, p_oSubstrate->GetFloatDataCode("freshlog"), fValue);

  //Run establishment
  p_oSimManager->GetBehaviorObject("NZ establishment")->Action();

  //Open file to write to
  oOut.open("FreshLogZ.txt", ios::out | ios::trunc);
  oOut << "Height in mm\n";

  //Get all the seedlings
  p_oAllSeedlings = p_oPop->Find("type=1");
  //Count them into their different categories
  p_oSeedling = p_oAllSeedlings->NextTree();
  while (p_oSeedling) {
    //Get the Z value
    p_oSeedling->GetValue(p_oPop->GetIntDataCode("z", p_oSeedling->GetSpecies(), p_oSeedling->GetType()), &iZValue);

    oOut << iZValue << "\n";

    p_oSeedling = p_oAllSeedlings->NextTree();
 }

  oOut.close();
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  TestAction()
/////////////////////////////////////////////////////////////////////////////
void clTestNZEstablishment::TestAction(clSimManager *p_oSimManager) {
  clPopulationBase * p_oTemp;
  clTreePopulation * p_oPop;
  clTreeSearch *p_oAllSeedlings;
  clTree *p_oSeedling;
  clGridBase *p_oSeeds, *p_oSubstrate;
  float fValue;
  int iZValue,
  iSpecies2MoundSeedlings = 0,
  iSpecies2GroundSeedlings = 0,
  iSpecies3MoundSeedlings = 0,
  iSpecies3GroundSeedlings = 0,
  iSpecies6MoundSeedlings = 0,
  iSpecies6GroundSeedlings = 0,
  iOther = 0;

  //Set up the run
  p_oSimManager->ReadFile(WriteXMLFile4());
  p_oTemp = p_oSimManager->GetPopulationObject("treepopulation");
  p_oPop = (clTreePopulation *) p_oTemp;

  //In cell 0,0 of the "Disperse Seeds" grid, put 40 seeds of species 2, 52
  //seeds of species 3, and 60 seeds of species 6
  p_oSeeds = p_oSimManager->GetGridObject("Dispersed Seeds");
  fValue = 40;
  p_oSeeds->SetValueOfCell(0, 0, p_oSeeds->GetFloatDataCode("seeds_1"), fValue);
  fValue = 52;
  p_oSeeds->SetValueOfCell(0, 0, p_oSeeds->GetFloatDataCode("seeds_2"), fValue);
  fValue = 60;
  p_oSeeds->SetValueOfCell(0, 0, p_oSeeds->GetFloatDataCode("seeds_5"), fValue);

  //In cell 0,0 of the "NZ Substrate" make the substrate all mineral soil
  p_oSubstrate = p_oSimManager->GetGridObject("NZ Substrate");
  fValue = 0;
  p_oSubstrate->SetValueOfCell(0, 0, p_oSubstrate->GetFloatDataCode("mossylog"), fValue);
  p_oSubstrate->SetValueOfCell(0, 0, p_oSubstrate->GetFloatDataCode("freshlog"), fValue);
  p_oSubstrate->SetValueOfCell(0, 0, p_oSubstrate->GetFloatDataCode("moss"), fValue);
  p_oSubstrate->SetValueOfCell(0, 0, p_oSubstrate->GetFloatDataCode("litter"), fValue);
  fValue = 1;
  p_oSubstrate->SetValueOfCell(0, 0, p_oSubstrate->GetFloatDataCode("minsoil"), fValue);

  //Run establishment
  p_oSimManager->GetBehaviorObject("NZ establishment")->Action();

  //Get all the seedlings
  p_oAllSeedlings = p_oPop->Find("type=1");
  //Count them into their different categories
  p_oSeedling = p_oAllSeedlings->NextTree();
  while (p_oSeedling) {
    //Get the Z value
    p_oSeedling->GetValue(p_oPop->GetIntDataCode("z", p_oSeedling->GetSpecies(), p_oSeedling->GetType()), &iZValue);

    if (p_oSeedling->GetSpecies() == 1 && 0 == iZValue) {
      iSpecies2GroundSeedlings++;
    } else if (p_oSeedling->GetSpecies() == 1 && 0 < iZValue) {
      iSpecies2MoundSeedlings++;
    } else if (p_oSeedling->GetSpecies() == 2 && 0 == iZValue) {
      iSpecies3GroundSeedlings++;
    } else if (p_oSeedling->GetSpecies() == 2 && 0 < iZValue) {
      iSpecies3MoundSeedlings++;
    } else if (p_oSeedling->GetSpecies() == 5 && 0 == iZValue) {
      iSpecies6GroundSeedlings++;
    } else if (p_oSeedling->GetSpecies() == 5 && 0 < iZValue) {
      iSpecies6MoundSeedlings++;
    } else {
      iOther++;
    }

    p_oSeedling = p_oAllSeedlings->NextTree();
  }

  //Make sure we got the right values - 75% of the seeds should be on mound
  CPPUNIT_ASSERT(10 == iSpecies2GroundSeedlings);
  CPPUNIT_ASSERT(30 == iSpecies2MoundSeedlings);
  CPPUNIT_ASSERT(13 == iSpecies3GroundSeedlings);
  CPPUNIT_ASSERT(39 == iSpecies3MoundSeedlings);
  CPPUNIT_ASSERT(15 == iSpecies6GroundSeedlings);
  CPPUNIT_ASSERT(45 == iSpecies6MoundSeedlings);

}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  TestSetZAndRespite()
/////////////////////////////////////////////////////////////////////////////
void clTestNZEstablishment::TestSetZAndRespite(clSimManager *p_oSimManager) {
  clNZEstablishment *p_oEstablishment;
  clPopulationBase * p_oTemp;
  clTreePopulation * p_oPop;
  clTreeSearch *p_oAllSeedlings;
  clTree *p_oSeedling;
  clGridBase *p_oSeeds, *p_oSubstrate;
  clPackage *p_oCohort;
  float *p_fFreshLogProportions, fValue;
  int *p_iAges, iValue, i;

  //Set up the run
  p_oSimManager->ReadFile(WriteXMLFile2());

  p_oEstablishment = dynamic_cast<clNZEstablishment *>(p_oSimManager->GetBehaviorObject("NZ establishment"));
  p_oTemp = p_oSimManager->GetPopulationObject("treepopulation");
  p_oPop = (clTreePopulation *) p_oTemp;

  //Create a seedling
  p_oSeedling = p_oPop->CreateTree(2, 2, 2, clTreePopulation::seedling, 0);

  //See what it gets if it's on the ground - both respite counter and z should be 0
  p_oEstablishment->SetGroundZAndRespite(p_oSeedling);
  p_oSeedling->GetValue(p_oPop->GetIntDataCode("z", p_oSeedling->GetSpecies(), p_oSeedling->GetType()), &iValue);
  CPPUNIT_ASSERT(iValue == 0);
  p_oSeedling->GetValue(p_oPop->GetIntDataCode("lf_count", p_oSeedling->GetSpecies(), p_oSeedling->GetType()), &iValue);
  CPPUNIT_ASSERT(iValue == 0);

  //See what it gets if it's on a mound - respite counter should be 0, z greater
  p_oEstablishment->SetMoundZAndRespite(p_oSeedling);
  p_oSeedling->GetValue(p_oPop->GetIntDataCode("z", p_oSeedling->GetSpecies(), p_oSeedling->GetType()), &iValue);
  CPPUNIT_ASSERT(iValue > 0);
  p_oSeedling->GetValue(p_oPop->GetIntDataCode("lf_count", p_oSeedling->GetSpecies(), p_oSeedling->GetType()), &iValue);
  CPPUNIT_ASSERT(iValue == 0);

  //***************************************
  // Fresh logs
  //***************************************
  p_oSeeds = p_oSimManager->GetGridObject("Dispersed Seeds");
  p_oSubstrate = p_oSimManager->GetGridObject("NZ Substrate");

  //In cell 0,0 of the "NZ Substrate" set the cohorts
  p_oCohort = p_oSubstrate->CreatePackageOfCell(0, 0);
  fValue = 0.25;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 4;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.25;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  p_oCohort = p_oSubstrate->CreatePackage(p_oCohort);
  fValue = 0.25;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 3;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.25;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  p_oCohort = p_oSubstrate->CreatePackage(p_oCohort);
  fValue = 0.25;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 2;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.25;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  p_oCohort = p_oSubstrate->CreatePackage(p_oCohort);
  fValue = 0.25;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("freshlog"), fValue);
  iValue = 1;
  p_oCohort->SetValue(p_oSubstrate->GetPackageIntDataCode("age"), iValue);
  fValue = 0.25;
  p_oCohort->SetValue(p_oSubstrate->GetPackageFloatDataCode("dec_flog"), fValue);

  fValue = 0;
  p_oSubstrate->SetValueOfCell(0, 0, p_oSubstrate->GetFloatDataCode("mossylog"), fValue);
  p_oSubstrate->SetValueOfCell(0, 0, p_oSubstrate->GetFloatDataCode("minsoil"), fValue);
  p_oSubstrate->SetValueOfCell(0, 0, p_oSubstrate->GetFloatDataCode("litter"), fValue);
  p_oSubstrate->SetValueOfCell(0, 0, p_oSubstrate->GetFloatDataCode("moss"), fValue);
  fValue = 1;
  p_oSubstrate->SetValueOfCell(0, 0, p_oSubstrate->GetFloatDataCode("freshlog"), fValue);

  //Add 1000 seeds of species 2
  fValue = 1000;
  p_oSeeds->SetValueOfCell(0, 0, p_oSeeds->GetFloatDataCode("seeds_1"), fValue);

  //Now run the behavior
  p_oEstablishment->Action();

  p_iAges = new int[4];

  for (i = 0; i < 4; i++) {
    p_iAges[i] = 0;
  }

  //Make sure the seedlings all have one of the allowed ages - 0, 2, 4, or 6
  p_oAllSeedlings = p_oPop->Find("type=1");
  //Count them into their different categories
  p_oSeedling = p_oAllSeedlings->NextTree();
  while (p_oSeedling) {
    //Get the count value
    p_oSeedling->GetValue(p_oPop->GetIntDataCode("lf_count", p_oSeedling->GetSpecies(), p_oSeedling->GetType()), &iValue);

    if (0 == iValue) {
      p_iAges[0]++;
    } else if (2 == iValue) {
      p_iAges[1]++;
    } else if (4 == iValue) {
      p_iAges[2]++;
    } else if (6 == iValue) {
      p_iAges[3]++;
    } else {
      CppUnit::Asserter::fail("clNZEstablishment::GetFreshLogZAndRespite failed - unallowed counter value.");
    }

    p_oSeedling = p_oAllSeedlings->NextTree();
 }

 //Make sure that the seedlings are in roughly equal groups
 fValue = (float)p_iAges[0]/((float)p_iAges[0]+(float)p_iAges[1]+(float)p_iAges[2]+(float)p_iAges[3]);
 CPPUNIT_ASSERT(fValue >= 0.22 && fValue <= 0.28);
 fValue = (float)p_iAges[1]/((float)p_iAges[0]+(float)p_iAges[1]+(float)p_iAges[2]+(float)p_iAges[3]);
 CPPUNIT_ASSERT(fValue >= 0.22 && fValue <= 0.28);
 fValue = (float)p_iAges[2]/((float)p_iAges[0]+(float)p_iAges[1]+(float)p_iAges[2]+(float)p_iAges[3]);
 CPPUNIT_ASSERT(fValue >= 0.22 && fValue <= 0.28);
 fValue = (float)p_iAges[3]/((float)p_iAges[0]+(float)p_iAges[1]+(float)p_iAges[2]+(float)p_iAges[3]);
 CPPUNIT_ASSERT(fValue >= 0.22 && fValue <= 0.28);

  delete[] p_iAges;
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  TestCalculateLightFavorabilities()
/////////////////////////////////////////////////////////////////////////////
void clTestNZEstablishment::TestCalculateLightFavorabilities(clSimManager *p_oSimManager) {
  clNZEstablishment *p_oEstablishment;
  clGridBase *p_oEstablishmentLight = NULL;
  float fSp2Expected, fSp3Expected, fSp6Expected, fActual;
  short int iSp2Code, iSp3Code, iSp6Code, iX = 0, iY;

  //Set up the run
  p_oSimManager->ReadFile(WriteXMLFile2());

  p_oEstablishmentLight = p_oSimManager->GetGridObject("Establishment Light");
  CPPUNIT_ASSERT(p_oEstablishmentLight);
  iSp2Code = p_oEstablishmentLight->GetFloatDataCode("litefav_1");
  CPPUNIT_ASSERT(-1 < iSp2Code);
  iSp3Code = p_oEstablishmentLight->GetFloatDataCode("litefav_2");
  CPPUNIT_ASSERT(-1 < iSp3Code);
  iSp6Code = p_oEstablishmentLight->GetFloatDataCode("litefav_5");
  CPPUNIT_ASSERT(-1 < iSp6Code);

  //Set up the "GLIs"
  fSp2Expected = 19;
  iY = 0;
  p_oEstablishmentLight->SetValueOfCell(iX, iY, 0, fSp2Expected);
  fSp2Expected = 98;
  iY = 1;
  p_oEstablishmentLight->SetValueOfCell(iX, iY, 0, fSp2Expected);
  fSp2Expected = 35;
  iY = 2;
  p_oEstablishmentLight->SetValueOfCell(iX, iY, 0, fSp2Expected);
  fSp2Expected = 0;
  iY = 3;
  p_oEstablishmentLight->SetValueOfCell(iX, iY, 0, fSp2Expected);
  fSp2Expected = 100;
  iY = 4;
  p_oEstablishmentLight->SetValueOfCell(iX, iY, 0, fSp2Expected);
  fSp2Expected = 23;
  iY = 5;
  p_oEstablishmentLight->SetValueOfCell(iX, iY, 0, fSp2Expected);
  fSp2Expected = 87;
  iY = 6;
  p_oEstablishmentLight->SetValueOfCell(iX, iY, 0, fSp2Expected);
  fSp2Expected = 55.66;
  iY = 7;
  p_oEstablishmentLight->SetValueOfCell(iX, iY, 0, fSp2Expected);

  //Run the CalculateLightFavorabilities() method
  p_oEstablishment = dynamic_cast<clNZEstablishment *>(p_oSimManager->GetBehaviorObject("NZ establishment"));
  p_oEstablishment->CalculateLightFavorabilities();

  //Check results
  iY = 0;
  fSp2Expected = 0.864;
  fSp3Expected = 0.3336;
  fSp6Expected = 0.59674;
  p_oEstablishmentLight->GetValueOfCell(iX, iY, iSp2Code, &fActual);
  CPPUNIT_ASSERT(fabs(fSp2Expected - fActual) < 0.001);
  p_oEstablishmentLight->GetValueOfCell(iX, iY, iSp3Code, &fActual);
  CPPUNIT_ASSERT(fabs(fSp3Expected - fActual) < 0.001);
  p_oEstablishmentLight->GetValueOfCell(iX, iY, iSp6Code, &fActual);
  CPPUNIT_ASSERT(fabs(fSp6Expected - fActual) < 0.001);

  iY = 1;
  fSp2Expected = 0.8275;
  fSp3Expected = 0.8922;
  fSp6Expected = 0.49192;
  p_oEstablishmentLight->GetValueOfCell(iX, iY, iSp2Code, &fActual);
  CPPUNIT_ASSERT(fabs(fSp2Expected - fActual) < 0.001);
  p_oEstablishmentLight->GetValueOfCell(iX, iY, iSp3Code, &fActual);
  CPPUNIT_ASSERT(fabs(fSp3Expected - fActual) < 0.001);
  p_oEstablishmentLight->GetValueOfCell(iX, iY, iSp6Code, &fActual);
  CPPUNIT_ASSERT(fabs(fSp6Expected - fActual) < 0.001);

  iY = 2;
  fSp2Expected = 0.9724;
  fSp3Expected = 0.4904;
  fSp6Expected = 0.77274;
  p_oEstablishmentLight->GetValueOfCell(iX, iY, iSp2Code, &fActual);
  CPPUNIT_ASSERT(fabs(fSp2Expected - fActual) < 0.001);
  p_oEstablishmentLight->GetValueOfCell(iX, iY, iSp3Code, &fActual);
  CPPUNIT_ASSERT(fabs(fSp3Expected - fActual) < 0.001);
  p_oEstablishmentLight->GetValueOfCell(iX, iY, iSp6Code, &fActual);
  CPPUNIT_ASSERT(fabs(fSp6Expected - fActual) < 0.001);

  iY = 3;
  fSp2Expected = 0.218;
  fSp3Expected = 0.1474;
  fSp6Expected = 0.38774;
  p_oEstablishmentLight->GetValueOfCell(iX, iY, iSp2Code, &fActual);
  CPPUNIT_ASSERT(fabs(fSp2Expected - fActual) < 0.001);
  p_oEstablishmentLight->GetValueOfCell(iX, iY, iSp3Code, &fActual);
  CPPUNIT_ASSERT(fabs(fSp3Expected - fActual) < 0.001);
  p_oEstablishmentLight->GetValueOfCell(iX, iY, iSp6Code, &fActual);
  CPPUNIT_ASSERT(fabs(fSp6Expected - fActual) < 0.001);

  iY = 4;
  fSp2Expected = 0.8229;
  fSp3Expected = 0.8726;
  fSp6Expected = 0.46792;
  p_oEstablishmentLight->GetValueOfCell(iX, iY, iSp2Code, &fActual);
  CPPUNIT_ASSERT(fabs(fSp2Expected - fActual) < 0.001);
  p_oEstablishmentLight->GetValueOfCell(iX, iY, iSp3Code, &fActual);
  CPPUNIT_ASSERT(fabs(fSp3Expected - fActual) < 0.001);
  p_oEstablishmentLight->GetValueOfCell(iX, iY, iSp6Code, &fActual);
  CPPUNIT_ASSERT(fabs(fSp6Expected - fActual) < 0.001);

  iY = 5;
  fSp2Expected = 1;
  fSp3Expected = 0.3728;
  fSp6Expected = 0.64074;
  p_oEstablishmentLight->GetValueOfCell(iX, iY, iSp2Code, &fActual);
  CPPUNIT_ASSERT(fabs(fSp2Expected - fActual) < 0.001);
  p_oEstablishmentLight->GetValueOfCell(iX, iY, iSp3Code, &fActual);
  CPPUNIT_ASSERT(fabs(fSp3Expected - fActual) < 0.001);
  p_oEstablishmentLight->GetValueOfCell(iX, iY, iSp6Code, &fActual);
  CPPUNIT_ASSERT(fabs(fSp6Expected - fActual) < 0.001);

  iY = 6;
  fSp2Expected = 0.8528;
  fSp3Expected = 1;
  fSp6Expected = 0.62392;
  p_oEstablishmentLight->GetValueOfCell(iX, iY, iSp2Code, &fActual);
  CPPUNIT_ASSERT(fabs(fSp2Expected - fActual) < 0.001);
  p_oEstablishmentLight->GetValueOfCell(iX, iY, iSp3Code, &fActual);
  CPPUNIT_ASSERT(fabs(fSp3Expected - fActual) < 0.001);
  p_oEstablishmentLight->GetValueOfCell(iX, iY, iSp6Code, &fActual);
  CPPUNIT_ASSERT(fabs(fSp6Expected - fActual) < 0.001);

  iY = 7;
  fSp2Expected = 0.924882;
  fSp3Expected = 0.692868;
  fSp6Expected = 1;
  p_oEstablishmentLight->GetValueOfCell(iX, iY, iSp2Code, &fActual);
  CPPUNIT_ASSERT(fabs(fSp2Expected - fActual) < 0.001);
  p_oEstablishmentLight->GetValueOfCell(iX, iY, iSp3Code, &fActual);
  CPPUNIT_ASSERT(fabs(fSp3Expected - fActual) < 0.001);
  p_oEstablishmentLight->GetValueOfCell(iX, iY, iSp6Code, &fActual);
  CPPUNIT_ASSERT(fabs(fSp6Expected - fActual) < 0.001);
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  TestGetGLIs()
/////////////////////////////////////////////////////////////////////////////
void clTestNZEstablishment::TestGetGLIs(clSimManager *p_oSimManager) {
  clNZEstablishment *p_oEstablishment;
  clPopulationBase * p_oTemp;
  clTreePopulation * p_oPop;
  clGridBase *p_oQuadratLight = NULL, *p_oEstablishmentLight = NULL;
  float fQuadratGLI, fEstablishmentGLI;
  int iNumXCells, iNumYCells, iX, iY;
  short int iQuadratCode, iEstablishmentCode;

  //Set up the run
  p_oSimManager->ReadFile(WriteXMLFile1());

  //Get the quadrat light and establishment light grids
  p_oQuadratLight = p_oSimManager->GetGridObject("Quadrat GLI");
  p_oEstablishmentLight = p_oSimManager->GetGridObject("Establishment Light");
  CPPUNIT_ASSERT(p_oQuadratLight);
  CPPUNIT_ASSERT(p_oEstablishmentLight);
  iNumXCells = p_oQuadratLight->GetNumberXCells();
  iNumYCells = p_oQuadratLight->GetNumberYCells();

  //Add a seedling to every grid to make sure that all GLI quadrats get a GLI
  p_oTemp = p_oSimManager->GetPopulationObject("treepopulation");
  p_oPop = (clTreePopulation *) p_oTemp;
  for (iX = 0; iX < iNumXCells; iX++) {
    for (iY = 0; iY < iNumYCells; iY++) {
      p_oPop->CreateTree(p_oQuadratLight->GetOriginXOfCell(iX),
      p_oQuadratLight->GetOriginYOfCell(iY), 0, clTreePopulation::seedling, 0);
    }
  }

  //Run the quadrat light object
  p_oSimManager->GetBehaviorObject(0)->Action();

  //Get the NZ Establishment behavior and run its GLI calculator
  p_oEstablishment = dynamic_cast<clNZEstablishment *>(p_oSimManager->GetBehaviorObject("NZ establishment"));
  p_oEstablishment->GetGLIs();

  iQuadratCode = p_oQuadratLight->GetFloatDataCode("GLI");
  iEstablishmentCode = p_oEstablishmentLight->GetFloatDataCode("litefav_0");
  CPPUNIT_ASSERT(-1 < iQuadratCode);
  CPPUNIT_ASSERT(-1 < iEstablishmentCode);

  for (iX = 0; iX < iNumXCells; iX++) {
    for (iY = 0; iY < iNumYCells; iY++) {
      p_oQuadratLight->GetValueOfCell(iX, iY, iQuadratCode, &fQuadratGLI);
      p_oEstablishmentLight->GetValueOfCell(iX, iY, iEstablishmentCode, &fEstablishmentGLI);

      //They should be the same
      CPPUNIT_ASSERT(fabs(fQuadratGLI - fEstablishmentGLI) < 0.001);
    }
  }

}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  TestValidateData()
/////////////////////////////////////////////////////////////////////////////
void clTestNZEstablishment::TestValidateData(clSimManager *p_oSimManager) {

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile1());
    CppUnit::Asserter::fail("TestNZEstablishment error processing didn't catch error for WriteXMLErrorFile1.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clNZEstablishment::ValidateData") == NULL)
    {
      CppUnit::Asserter::fail("TestNZEstablishment error processing caught wrong error for WriteXMLErrorFile1.");
    }
    ;
  }

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile2());
    CppUnit::Asserter::fail("TestNZEstablishment error processing didn't catch error for WriteXMLErrorFile2.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clNZEstablishment::ValidateData") == NULL)
    {
      CppUnit::Asserter::fail("TestNZEstablishment error processing caught wrong error for WriteXMLErrorFile2.");
    }
    ;
  }

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile3());
    CppUnit::Asserter::fail("TestNZEstablishment error processing didn't catch error for WriteXMLErrorFile3.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clNZEstablishment::ValidateData") == NULL)
    {
      CppUnit::Asserter::fail("TestNZEstablishment error processing caught wrong error for WriteXMLErrorFile3.");
    }
    ;
  }

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile4());
    CppUnit::Asserter::fail("TestNZEstablishment error processing didn't catch error for WriteXMLErrorFile4.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clNZEstablishment::ValidateData") == NULL)
    {
      CppUnit::Asserter::fail("TestNZEstablishment error processing caught wrong error for WriteXMLErrorFile4.");
    }
    ;
  }
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile1()
/////////////////////////////////////////////////////////////////////////////
char * clTestNZEstablishment::WriteXMLFile1()
{
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>2</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>"
       << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>100.0</plot_lenX>"
       << "<plot_lenY>100.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_canopyRadiusToDBH>"
       << "<tr_crtdVal species=\"Species_1\">0.0549</tr_crtdVal>"
       << "</tr_canopyRadiusToDBH>"
       << "<tr_canopyRadiusExponent>"
       << "<tr_creVal species=\"Species_1\">1.0</tr_creVal>"
       << "</tr_canopyRadiusExponent>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_fractOfHeightInCanopy>"
       << "<tr_fohicVal species=\"Species_1\">0.389</tr_fohicVal>"
       << "</tr_fractOfHeightInCanopy>"
       << "<tr_canopyDepthExponent>"
       << "<tr_cdeVal species=\"Species_1\">1.0</tr_cdeVal>"
       << "</tr_canopyDepthExponent>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "<tr_treemap>"
       << "<tm_treeSettings sp=\"Species_1\" tp=\"3\">"
       << "<tm_floatCodes>"
       << "<tm_floatCode label=\"X\">0</tm_floatCode>"
       << "<tm_floatCode label=\"Y\">1</tm_floatCode>"
       << "<tm_floatCode label=\"DBH\">2</tm_floatCode>"
       << "</tm_floatCodes>"
       << "</tm_treeSettings>"
       << "<tm_treeSettings sp=\"Species_1\" tp=\"2\">"
       << "<tm_floatCodes>"
       << "<tm_floatCode label=\"X\">0</tm_floatCode>"
       << "<tm_floatCode label=\"Y\">1</tm_floatCode>"
       << "<tm_floatCode label=\"DBH\">2</tm_floatCode>"
       << "</tm_floatCodes>"
       << "</tm_treeSettings>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">1.09253</fl>"
       << "<fl c=\"1\">10.4553</fl>"
       << "<fl c=\"2\">202.309</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">1.14136</fl>"
       << "<fl c=\"1\">95.8496</fl>"
       << "<fl c=\"2\">226.929</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">1.59912</fl>"
       << "<fl c=\"1\">84.5276</fl>"
       << "<fl c=\"2\">48.7422</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">2.76489</fl>"
       << "<fl c=\"1\">98.8098</fl>"
       << "<fl c=\"2\">13.1579</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">3.07617</fl>"
       << "<fl c=\"1\">8.73413</fl>"
       << "<fl c=\"2\">49.2903</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">3.96118</fl>"
       << "<fl c=\"1\">5.93872</fl>"
       << "<fl c=\"2\">28.8731</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">4.06494</fl>"
       << "<fl c=\"1\">86.8958</fl>"
       << "<fl c=\"2\">139.409</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">5.23682</fl>"
       << "<fl c=\"1\">75.7263</fl>"
       << "<fl c=\"2\">192.788</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">5.48706</fl>"
       << "<fl c=\"1\">93.4326</fl>"
       << "<fl c=\"2\">125.893</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">5.81665</fl>"
       << "<fl c=\"1\">36.5723</fl>"
       << "<fl c=\"2\">128.19</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">6.38428</fl>"
       << "<fl c=\"1\">74.6094</fl>"
       << "<fl c=\"2\">217.912</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">6.84204</fl>"
       << "<fl c=\"1\">72.1741</fl>"
       << "<fl c=\"2\">131.25</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">7.37915</fl>"
       << "<fl c=\"1\">59.3201</fl>"
       << "<fl c=\"2\">126.826</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">7.47681</fl>"
       << "<fl c=\"1\">27.5635</fl>"
       << "<fl c=\"2\">198.671</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">7.8064</fl>"
       << "<fl c=\"1\">46.8262</fl>"
       << "<fl c=\"2\">215.398</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">8.78296</fl>"
       << "<fl c=\"1\">47.9004</fl>"
       << "<fl c=\"2\">90.8241</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">9.39331</fl>"
       << "<fl c=\"1\">22.9675</fl>"
       << "<fl c=\"2\">206.305</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">9.82666</fl>"
       << "<fl c=\"1\">43.75</fl>"
       << "<fl c=\"2\">145.627</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">9.87549</fl>"
       << "<fl c=\"1\">79.2419</fl>"
       << "<fl c=\"2\">52.1185</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">10.4248</fl>"
       << "<fl c=\"1\">70.9595</fl>"
       << "<fl c=\"2\">34.0311</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">10.5713</fl>"
       << "<fl c=\"1\">94.2017</fl>"
       << "<fl c=\"2\">82.9081</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">10.5957</fl>"
       << "<fl c=\"1\">99.1211</fl>"
       << "<fl c=\"2\">33.6406</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">11.0718</fl>"
       << "<fl c=\"1\">56.2927</fl>"
       << "<fl c=\"2\">114.147</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">11.1877</fl>"
       << "<fl c=\"1\">33.5144</fl>"
       << "<fl c=\"2\">122.085</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">11.792</fl>"
       << "<fl c=\"1\">80.8533</fl>"
       << "<fl c=\"2\">80.0441</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">12.3108</fl>"
       << "<fl c=\"1\">77.7283</fl>"
       << "<fl c=\"2\">214.776</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">12.384</fl>"
       << "<fl c=\"1\">65.979</fl>"
       << "<fl c=\"2\">88.2278</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">12.4817</fl>"
       << "<fl c=\"1\">91.1743</fl>"
       << "<fl c=\"2\">89.9904</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">13.0737</fl>"
       << "<fl c=\"1\">29.9683</fl>"
       << "<fl c=\"2\">178.051</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">13.4583</fl>"
       << "<fl c=\"1\">4.04663</fl>"
       << "<fl c=\"2\">215.439</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">13.5559</fl>"
       << "<fl c=\"1\">99.6216</fl>"
       << "<fl c=\"2\">41.4172</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">13.9465</fl>"
       << "<fl c=\"1\">81.9641</fl>"
       << "<fl c=\"2\">29.9838</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">13.9648</fl>"
       << "<fl c=\"1\">67.3401</fl>"
       << "<fl c=\"2\">221.196</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">16.2659</fl>"
       << "<fl c=\"1\">15.0879</fl>"
       << "<fl c=\"2\">156.526</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">17.5415</fl>"
       << "<fl c=\"1\">39.6179</fl>"
       << "<fl c=\"2\">212.029</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">18.1763</fl>"
       << "<fl c=\"1\">99.7742</fl>"
       << "<fl c=\"2\">53.8654</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">18.3472</fl>"
       << "<fl c=\"1\">80.9265</fl>"
       << "<fl c=\"2\">205.103</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">18.6096</fl>"
       << "<fl c=\"1\">26.8188</fl>"
       << "<fl c=\"2\">213.62</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">18.8782</fl>"
       << "<fl c=\"1\">71.7102</fl>"
       << "<fl c=\"2\">177.816</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">20.1416</fl>"
       << "<fl c=\"1\">5.62744</fl>"
       << "<fl c=\"2\">20.5617</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">20.4346</fl>"
       << "<fl c=\"1\">57.4097</fl>"
       << "<fl c=\"2\">117.784</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">21.0205</fl>"
       << "<fl c=\"1\">87.1338</fl>"
       << "<fl c=\"2\">130.289</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">21.759</fl>"
       << "<fl c=\"1\">70.2759</fl>"
       << "<fl c=\"2\">43.111</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">22.5098</fl>"
       << "<fl c=\"1\">7.17773</fl>"
       << "<fl c=\"2\">217.687</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">23.4253</fl>"
       << "<fl c=\"1\">97.7234</fl>"
       << "<fl c=\"2\">75.0118</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">23.4863</fl>"
       << "<fl c=\"1\">44.4336</fl>"
       << "<fl c=\"2\">197.454</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"2\">"
       << "<fl c=\"0\">24.2859</fl>"
       << "<fl c=\"1\">81.2378</fl>"
       << "<fl c=\"2\">9.79817</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"2\">"
       << "<fl c=\"0\">24.6094</fl>"
       << "<fl c=\"1\">1.40991</fl>"
       << "<fl c=\"2\">7.92186</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">26.4526</fl>"
       << "<fl c=\"1\">41.7236</fl>"
       << "<fl c=\"2\">43.2599</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">26.5442</fl>"
       << "<fl c=\"1\">15.2222</fl>"
       << "<fl c=\"2\">127.795</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">26.7334</fl>"
       << "<fl c=\"1\">5.58472</fl>"
       << "<fl c=\"2\">64.8789</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">26.8066</fl>"
       << "<fl c=\"1\">15.1733</fl>"
       << "<fl c=\"2\">91.1548</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">27.5391</fl>"
       << "<fl c=\"1\">42.0837</fl>"
       << "<fl c=\"2\">207.294</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">27.6917</fl>"
       << "<fl c=\"1\">23.0774</fl>"
       << "<fl c=\"2\">189.394</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">28.2959</fl>"
       << "<fl c=\"1\">8.1665</fl>"
       << "<fl c=\"2\">12.879</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">28.3142</fl>"
       << "<fl c=\"1\">62.7502</fl>"
       << "<fl c=\"2\">190.65</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">28.6316</fl>"
       << "<fl c=\"1\">89.6545</fl>"
       << "<fl c=\"2\">158.228</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">30.2917</fl>"
       << "<fl c=\"1\">65.9912</fl>"
       << "<fl c=\"2\">204.763</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">30.3589</fl>"
       << "<fl c=\"1\">64.4714</fl>"
       << "<fl c=\"2\">128.546</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">30.7434</fl>"
       << "<fl c=\"1\">38.6169</fl>"
       << "<fl c=\"2\">13.2423</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">31.2744</fl>"
       << "<fl c=\"1\">47.0642</fl>"
       << "<fl c=\"2\">72.7101</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">34.4666</fl>"
       << "<fl c=\"1\">47.168</fl>"
       << "<fl c=\"2\">183.004</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">36.4136</fl>"
       << "<fl c=\"1\">17.5354</fl>"
       << "<fl c=\"2\">217.341</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">38.0249</fl>"
       << "<fl c=\"1\">62.5061</fl>"
       << "<fl c=\"2\">124.335</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">38.9832</fl>"
       << "<fl c=\"1\">6.65283</fl>"
       << "<fl c=\"2\">48.2453</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">39.0076</fl>"
       << "<fl c=\"1\">4.43726</fl>"
       << "<fl c=\"2\">56.1992</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">39.5142</fl>"
       << "<fl c=\"1\">23.4131</fl>"
       << "<fl c=\"2\">157.285</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">40.5151</fl>"
       << "<fl c=\"1\">15.3137</fl>"
       << "<fl c=\"2\">140.76</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">42.3767</fl>"
       << "<fl c=\"1\">47.76</fl>"
       << "<fl c=\"2\">88.3115</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">42.8345</fl>"
       << "<fl c=\"1\">47.8149</fl>"
       << "<fl c=\"2\">194.029</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">43.3533</fl>"
       << "<fl c=\"1\">49.1394</fl>"
       << "<fl c=\"2\">130.204</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">43.5059</fl>"
       << "<fl c=\"1\">52.832</fl>"
       << "<fl c=\"2\">177.179</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">43.6523</fl>"
       << "<fl c=\"1\">40.0879</fl>"
       << "<fl c=\"2\">21.2012</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">43.9819</fl>"
       << "<fl c=\"1\">33.0688</fl>"
       << "<fl c=\"2\">140.911</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">45.5078</fl>"
       << "<fl c=\"1\">10.1074</fl>"
       << "<fl c=\"2\">67.6306</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">45.5627</fl>"
       << "<fl c=\"1\">46.3684</fl>"
       << "<fl c=\"2\">185.204</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">45.636</fl>"
       << "<fl c=\"1\">0.415039</fl>"
       << "<fl c=\"2\">17.7023</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">46.4966</fl>"
       << "<fl c=\"1\">35.6689</fl>"
       << "<fl c=\"2\">27.5321</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">48.0469</fl>"
       << "<fl c=\"1\">20.7092</fl>"
       << "<fl c=\"2\">106.914</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">48.3032</fl>"
       << "<fl c=\"1\">51.1841</fl>"
       << "<fl c=\"2\">111.861</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">49.4385</fl>"
       << "<fl c=\"1\">16.449</fl>"
       << "<fl c=\"2\">159.988</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">49.8474</fl>"
       << "<fl c=\"1\">13.7024</fl>"
       << "<fl c=\"2\">31.5347</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">50.824</fl>"
       << "<fl c=\"1\">48.3459</fl>"
       << "<fl c=\"2\">55.1187</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">50.885</fl>"
       << "<fl c=\"1\">46.6614</fl>"
       << "<fl c=\"2\">185.994</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">51.0925</fl>"
       << "<fl c=\"1\">69.8181</fl>"
       << "<fl c=\"2\">203.535</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">51.355</fl>"
       << "<fl c=\"1\">22.345</fl>"
       << "<fl c=\"2\">17.3205</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">51.9897</fl>"
       << "<fl c=\"1\">65.9363</fl>"
       << "<fl c=\"2\">211.112</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">53.1677</fl>"
       << "<fl c=\"1\">11.1511</fl>"
       << "<fl c=\"2\">191.365</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">53.2776</fl>"
       << "<fl c=\"1\">32.5134</fl>"
       << "<fl c=\"2\">172.064</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">54.8523</fl>"
       << "<fl c=\"1\">42.041</fl>"
       << "<fl c=\"2\">144.17</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">55.3406</fl>"
       << "<fl c=\"1\">61.0229</fl>"
       << "<fl c=\"2\">170.291</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">56.5552</fl>"
       << "<fl c=\"1\">12.5244</fl>"
       << "<fl c=\"2\">187.597</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">56.6833</fl>"
       << "<fl c=\"1\">1.97754</fl>"
       << "<fl c=\"2\">12.3927</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">58.5083</fl>"
       << "<fl c=\"1\">31.0547</fl>"
       << "<fl c=\"2\">18.3411</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">59.8999</fl>"
       << "<fl c=\"1\">37.5916</fl>"
       << "<fl c=\"2\">133.946</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">60.3149</fl>"
       << "<fl c=\"1\">37.03</fl>"
       << "<fl c=\"2\">127.326</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">61.8469</fl>"
       << "<fl c=\"1\">16.0339</fl>"
       << "<fl c=\"2\">142.169</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">63.385</fl>"
       << "<fl c=\"1\">24.115</fl>"
       << "<fl c=\"2\">11.514</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">63.4644</fl>"
       << "<fl c=\"1\">53.0823</fl>"
       << "<fl c=\"2\">75.2214</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">63.7024</fl>"
       << "<fl c=\"1\">9.23462</fl>"
       << "<fl c=\"2\">26.0699</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">65.4175</fl>"
       << "<fl c=\"1\">6.38428</fl>"
       << "<fl c=\"2\">228.775</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">66.4001</fl>"
       << "<fl c=\"1\">38.2446</fl>"
       << "<fl c=\"2\">15.8618</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">66.4734</fl>"
       << "<fl c=\"1\">33.5938</fl>"
       << "<fl c=\"2\">64.2841</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">66.7664</fl>"
       << "<fl c=\"1\">25.2319</fl>"
       << "<fl c=\"2\">183.13</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">67.2607</fl>"
       << "<fl c=\"1\">28.9795</fl>"
       << "<fl c=\"2\">78.7292</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">67.3645</fl>"
       << "<fl c=\"1\">37.0911</fl>"
       << "<fl c=\"2\">67.8604</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">67.3767</fl>"
       << "<fl c=\"1\">37.5549</fl>"
       << "<fl c=\"2\">178.72</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">67.4255</fl>"
       << "<fl c=\"1\">51.9409</fl>"
       << "<fl c=\"2\">36.0825</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">67.6331</fl>"
       << "<fl c=\"1\">63.6963</fl>"
       << "<fl c=\"2\">189.704</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">67.6758</fl>"
       << "<fl c=\"1\">31.5735</fl>"
       << "<fl c=\"2\">222.464</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">68.0908</fl>"
       << "<fl c=\"1\">30.6274</fl>"
       << "<fl c=\"2\">114.948</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">68.4326</fl>"
       << "<fl c=\"1\">7.61108</fl>"
       << "<fl c=\"2\">230.139</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">68.5242</fl>"
       << "<fl c=\"1\">5.10254</fl>"
       << "<fl c=\"2\">133.578</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">68.6401</fl>"
       << "<fl c=\"1\">42.2607</fl>"
       << "<fl c=\"2\">137.062</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">68.8416</fl>"
       << "<fl c=\"1\">15.3381</fl>"
       << "<fl c=\"2\">155.981</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">68.8538</fl>"
       << "<fl c=\"1\">41.5161</fl>"
       << "<fl c=\"2\">89.6296</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">69.3848</fl>"
       << "<fl c=\"1\">12.0728</fl>"
       << "<fl c=\"2\">173.231</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">70.4712</fl>"
       << "<fl c=\"1\">53.2349</fl>"
       << "<fl c=\"2\">178.053</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">70.8984</fl>"
       << "<fl c=\"1\">29.8462</fl>"
       << "<fl c=\"2\">134.815</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">70.9412</fl>"
       << "<fl c=\"1\">61.9263</fl>"
       << "<fl c=\"2\">113.192</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">73.3398</fl>"
       << "<fl c=\"1\">12.8418</fl>"
       << "<fl c=\"2\">140.171</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">74.3286</fl>"
       << "<fl c=\"1\">8.81348</fl>"
       << "<fl c=\"2\">89.3607</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">74.6582</fl>"
       << "<fl c=\"1\">33.3069</fl>"
       << "<fl c=\"2\">171.7</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">76.7944</fl>"
       << "<fl c=\"1\">11.908</fl>"
       << "<fl c=\"2\">160.419</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">76.8066</fl>"
       << "<fl c=\"1\">30.0232</fl>"
       << "<fl c=\"2\">227.877</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">77.0142</fl>"
       << "<fl c=\"1\">35.9253</fl>"
       << "<fl c=\"2\">60.4661</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">78.2898</fl>"
       << "<fl c=\"1\">53.8269</fl>"
       << "<fl c=\"2\">62.9131</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">78.8269</fl>"
       << "<fl c=\"1\">41.9678</fl>"
       << "<fl c=\"2\">37.52</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">78.8635</fl>"
       << "<fl c=\"1\">6.00586</fl>"
       << "<fl c=\"2\">29.7635</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">79.0405</fl>"
       << "<fl c=\"1\">37.616</fl>"
       << "<fl c=\"2\">78.0954</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">80.0781</fl>"
       << "<fl c=\"1\">65.7593</fl>"
       << "<fl c=\"2\">103.557</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">80.7678</fl>"
       << "<fl c=\"1\">43.4753</fl>"
       << "<fl c=\"2\">89.7455</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">81.2073</fl>"
       << "<fl c=\"1\">20.0806</fl>"
       << "<fl c=\"2\">74.0133</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">82.1716</fl>"
       << "<fl c=\"1\">7.03125</fl>"
       << "<fl c=\"2\">194.974</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">82.251</fl>"
       << "<fl c=\"1\">60.5835</fl>"
       << "<fl c=\"2\">88.2366</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">82.8491</fl>"
       << "<fl c=\"1\">7.2937</fl>"
       << "<fl c=\"2\">10.5029</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">83.1543</fl>"
       << "<fl c=\"1\">40.9302</fl>"
       << "<fl c=\"2\">168.538</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">83.8196</fl>"
       << "<fl c=\"1\">46.8506</fl>"
       << "<fl c=\"2\">63.1772</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">85.2356</fl>"
       << "<fl c=\"1\">10.675</fl>"
       << "<fl c=\"2\">117.092</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"2\">"
       << "<fl c=\"0\">85.3577</fl>"
       << "<fl c=\"1\">43.8843</fl>"
       << "<fl c=\"2\">2.76791</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">85.4614</fl>"
       << "<fl c=\"1\">20.2576</fl>"
       << "<fl c=\"2\">21.8835</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">85.5774</fl>"
       << "<fl c=\"1\">8.91113</fl>"
       << "<fl c=\"2\">63.688</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">87.2803</fl>"
       << "<fl c=\"1\">9.48486</fl>"
       << "<fl c=\"2\">223.27</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">87.8052</fl>"
       << "<fl c=\"1\">2.948</fl>"
       << "<fl c=\"2\">229.676</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">87.8357</fl>"
       << "<fl c=\"1\">39.2395</fl>"
       << "<fl c=\"2\">162.179</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">88.0066</fl>"
       << "<fl c=\"1\">22.8149</fl>"
       << "<fl c=\"2\">197.339</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"2\">"
       << "<fl c=\"0\">90.0024</fl>"
       << "<fl c=\"1\">28.3752</fl>"
       << "<fl c=\"2\">5.78306</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">90.3137</fl>"
       << "<fl c=\"1\">32.7332</fl>"
       << "<fl c=\"2\">16.9058</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">90.4053</fl>"
       << "<fl c=\"1\">10.0159</fl>"
       << "<fl c=\"2\">52.9818</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">93.1885</fl>"
       << "<fl c=\"1\">20.4834</fl>"
       << "<fl c=\"2\">16.1824</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">93.927</fl>"
       << "<fl c=\"1\">51.0498</fl>"
       << "<fl c=\"2\">70.1263</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">95.8557</fl>"
       << "<fl c=\"1\">21.5393</fl>"
       << "<fl c=\"2\">223.691</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">95.9534</fl>"
       << "<fl c=\"1\">24.1943</fl>"
       << "<fl c=\"2\">220.506</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">97.7112</fl>"
       << "<fl c=\"1\">59.0881</fl>"
       << "<fl c=\"2\">40.6697</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">98.0896</fl>"
       << "<fl c=\"1\">51.8372</fl>"
       << "<fl c=\"2\">78.3419</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">98.9075</fl>"
       << "<fl c=\"1\">10.6812</fl>"
       << "<fl c=\"2\">144.269</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">99.115</fl>"
       << "<fl c=\"1\">45.7336</fl>"
       << "<fl c=\"2\">62.6597</fl>"
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">99.3835</fl>"
       << "<fl c=\"1\">54.8035</fl>"
       << "<fl c=\"2\">80.9017</fl>"
       << "</tree>"
       << "</tr_treemap>"
       << "</trees>"
       //Quadrat's grid
       << "<grid gridName=\"Quadrat GLI\">"
       << "<ma_floatCodes>"
       << "<ma_floatCode label=\"GLI\">0</ma_floatCode>"
       << "</ma_floatCodes>"
       << "<ma_plotLenX>100.0</ma_plotLenX>"
       << "<ma_plotLenY>100.0</ma_plotLenY>"
       << "<ma_lengthXCells>10.0</ma_lengthXCells>"
       << "<ma_lengthYCells>10.0</ma_lengthYCells>"
       << "</grid>"
       << "<grid gridName=\"Dispersed Seeds\">"
       << "<ma_intCodes>"
       << "<ma_intCode label=\"count\">0</ma_intCode>"
       << "</ma_intCodes>"
       << "<ma_floatCodes>"
       << "<ma_floatCode label=\"seeds_0\">0</ma_floatCode>"
       << "</ma_floatCodes>"
       << "<ma_boolCodes>"
       << "<ma_boolCode label=\"Is Gap\">0</ma_boolCode>"
       << "</ma_boolCodes>"
       << "<ma_plotLenX>100.0</ma_plotLenX>"
       << "<ma_plotLenY>100.0</ma_plotLenY>"
       << "<ma_lengthXCells>10.0</ma_lengthXCells>"
       << "<ma_lengthYCells>10.0</ma_lengthYCells>"
       << "</grid>"
       << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>quadratlight</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>light filter</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>adultstochasticmort</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>NZ Substrate</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>non spatial disperse</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>NZ establishment with light</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Seed\"/>"
       << "</behavior>"
       << "</behaviorList>"
       << "<quadratGliLight>"
       << "<li_heightOfFishEyePhoto>0</li_heightOfFishEyePhoto>"
       << "<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>"
       << "<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>"
       << "<li_julianDayGrowthStarts>105</li_julianDayGrowthStarts>"
       << "<li_julianDayGrowthEnds>258</li_julianDayGrowthEnds>"
       << "<li_minSunAngle>0.785</li_minSunAngle>"
       << "<li_numAltGrids>12</li_numAltGrids>"
       << "<li_numAziGrids>18</li_numAziGrids>"
       << "<li_lightExtinctionCoefficient>"
       << "<li_lecVal species=\"Species_1\">0.08</li_lecVal>"
       << "</li_lightExtinctionCoefficient>"
       << "<li_quadratLightHeight>1</li_quadratLightHeight>"
       << "</quadratGliLight>"
       << "<lightFilter>"
       << "<lf_lightExtinctionCoefficient>1.535</lf_lightExtinctionCoefficient>"
       << "<lf_heightOfFilter>1.3</lf_heightOfFilter>"
       << "</lightFilter>"
       << "<mortality>"
       << "<mo_nonSizeDepMort>"
       << "<mo_nsdmVal species=\"Species_1\">0.0</mo_nsdmVal>"
       << "</mo_nonSizeDepMort>"
       << "</mortality>"
       << "<substrate>"
       << "<nzs_mineralSoilDecayAlpha>-7.0E-4</nzs_mineralSoilDecayAlpha>"
       << "<nzs_mineralSoilDecayBeta>4.3</nzs_mineralSoilDecayBeta>"
       << "<nzs_freshLogDecayAlpha>-0.05</nzs_freshLogDecayAlpha>"
       << "<nzs_freshLogDecayBeta>6.5</nzs_freshLogDecayBeta>"
       << "<nzs_mossyLogToLitterMoss>0.55</nzs_mossyLogToLitterMoss>"
       << "<nzs_mossProportion>0.35</nzs_mossProportion>"
       << "<nzs_maxNumberDecayYears>10</nzs_maxNumberDecayYears>"
       << "<nzs_initialMineralSoil>0.0</nzs_initialMineralSoil>"
       << "<nzs_initialFreshLog>0.01</nzs_initialFreshLog>"
       << "<nzs_initialMossyLog>0.11</nzs_initialMossyLog>"
       << "<nzs_initialAvgDbhFreshLog>20</nzs_initialAvgDbhFreshLog>"
       << "<nzs_propOfDeadFall>"
       << "<nzs_podfVal species=\"Species_1\">0.7</nzs_podfVal>" << "</nzs_propOfDeadFall>" << "<nzs_propOfFallUproot>"
       << "<nzs_pofuVal species=\"Species_1\">0.5</nzs_pofuVal>" << "</nzs_propOfFallUproot>"
       << "<nzs_rootTipupFactor>3.0</nzs_rootTipupFactor>" << "</substrate>"
       << "<disperse>"
       << "<di_minDbhForReproduction>"
       << "<di_mdfrVal species=\"Species_1\">15.0</di_mdfrVal>"
       << "</di_minDbhForReproduction>"
       << "<di_nonSpatialSlopeOfLambda>"
       << "<di_nssolVal species=\"Species_1\">0</di_nssolVal>"
       << "</di_nonSpatialSlopeOfLambda>"
       << "<di_nonSpatialInterceptOfLambda>"
       << "<di_nsiolVal species=\"Species_1\">0</di_nsiolVal>"
       << "</di_nonSpatialInterceptOfLambda>"
       << "<di_seedDistributionMethod>0</di_seedDistributionMethod>"
       << "</disperse>"
       << "<establishment>"
       << "<es_mossMoundFav>"
       << "<es_mmfVal species=\"Species_1\">0.129</es_mmfVal>"
       << "</es_mossMoundFav>"
       << "<es_mineralSoilMoundFav>"
       << "<es_msmfVal species=\"Species_1\">0.983</es_msmfVal>"
       << "</es_mineralSoilMoundFav>"
       << "<es_freshLogMoundFav>"
       << "<es_flmfVal species=\"Species_1\">0.353</es_flmfVal>"
       << "</es_freshLogMoundFav>"
       << "<es_mossyLogMoundFav>"
       << "<es_mlmfVal species=\"Species_1\">0.058</es_mlmfVal>"
       << "</es_mossyLogMoundFav>"
       << "<es_litterMoundFav>"
       << "<es_lmfVal species=\"Species_1\">0.009</es_lmfVal>"
       << "</es_litterMoundFav>"
       << "<es_mossGroundFav>"
       << "<es_mgfVal species=\"Species_1\">0.129</es_mgfVal>"
       << "</es_mossGroundFav>"
       << "<es_mineralSoilGroundFav>"
       << "<es_msgfVal species=\"Species_1\">0.983</es_msgfVal>"
       << "</es_mineralSoilGroundFav>"
       << "<es_freshLogGroundFav>"
       << "<es_flgfVal species=\"Species_1\">0.353</es_flgfVal>"
       << "</es_freshLogGroundFav>"
       << "<es_mossyLogGroundFav>"
       << "<es_mlgfVal species=\"Species_1\">0.058</es_mlgfVal>"
       << "</es_mossyLogGroundFav>"
       << "<es_litterGroundFav>"
       << "<es_lgfVal species=\"Species_1\">0.009</es_lgfVal>"
       << "</es_litterGroundFav>"
       << "<es_optimumGLI>"
       << "<es_ogVal species=\"Species_1\">75</es_ogVal>"
       << "</es_optimumGLI>"
       << "<es_lowSlope>"
       << "<es_lsVal species=\"Species_1\">0.87</es_lsVal>"
       << "</es_lowSlope>"
       << "<es_highSlope>"
       << "<es_hsVal species=\"Species_1\">0.87</es_hsVal>"
       << "</es_highSlope>"
       << "<es_lightHeight>1</es_lightHeight>"
       << "<es_moundProportion>0.87</es_moundProportion>"
       << "<es_maxRespite>3</es_maxRespite>"
       << "<es_meanMoundHeight>3</es_meanMoundHeight>"
       << "<es_moundStdDev>0.5</es_moundStdDev>"
       << "<es_meanFreshLogHeight>20</es_meanFreshLogHeight>"
       << "<es_freshLogStdDev>5</es_freshLogStdDev>"
       << "</establishment>"
       << "</paramFile>";

  oOut.close();

  return cFileString;
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile2()
/////////////////////////////////////////////////////////////////////////////
char * clTestNZEstablishment::WriteXMLFile2()
{
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>2</timesteps>"
       << "<yearsPerTimestep>2</yearsPerTimestep>"
       << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>200.0</plot_lenX>"
       << "<plot_lenY>200.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "<tr_species speciesName=\"Species_2\"/>"
       << "<tr_species speciesName=\"Species_3\"/>"
       << "<tr_species speciesName=\"Species_4\"/>"
       << "<tr_species speciesName=\"Species_5\"/>"
       << "<tr_species speciesName=\"Species_6\"/>"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_2\">39.54</tr_chVal>"
       << "<tr_chVal species=\"Species_3\">40.0</tr_chVal>"
       << "<tr_chVal species=\"Species_4\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_5\">40.0</tr_chVal>"
       << "<tr_chVal species=\"Species_6\">39.48</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_canopyRadiusToDBH>"
       << "<tr_crtdVal species=\"Species_1\">0.0549</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_2\">0.0614</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_3\">0.0242</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_4\">0.0549</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_5\">0.0242</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_6\">0.0549</tr_crtdVal>"
       << "</tr_canopyRadiusToDBH>"
       << "<tr_canopyRadiusExponent>"
       << "<tr_creVal species=\"Species_1\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_2\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_3\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_4\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_5\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_6\">1.0</tr_creVal>"
       << "</tr_canopyRadiusExponent>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_2\">0.5944</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_3\">0.7059</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_4\">0.5944</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_5\">0.7059</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_6\">0.5944</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_fractOfHeightInCanopy>"
       << "<tr_fohicVal species=\"Species_1\">0.389</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_2\">0.368</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_3\">0.464</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_4\">0.368</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_5\">0.464</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_6\">0.368</tr_fohicVal>"
       << "</tr_fractOfHeightInCanopy>"
       << "<tr_canopyDepthExponent>"
       << "<tr_cdeVal species=\"Species_1\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_2\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_3\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_4\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_5\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_6\">1.0</tr_cdeVal>"
       << "</tr_canopyDepthExponent>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_2\">0.0269</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_3\">0.02871</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_4\">0.0269</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_5\">0.02871</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_6\">0.0269</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_2\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_3\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_4\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_5\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_6\">10.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_2\">0.0241</tr_soahVal>"
       << "<tr_soahVal species=\"Species_3\">0.0263</tr_soahVal>"
       << "<tr_soahVal species=\"Species_4\">0.0241</tr_soahVal>"
       << "<tr_soahVal species=\"Species_5\">0.0263</tr_soahVal>"
       << "<tr_soahVal species=\"Species_6\">0.0241</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "</trees>"
       //Set both nz substrate and seeds grid to resolution of 6,6
       << "<grid gridName=\"NZ Substrate\">"
       << "<ma_floatCodes>"
       << "<ma_floatCode label=\"litter\">0</ma_floatCode>"
       << "<ma_floatCode label=\"moss\">1</ma_floatCode>"
       << "<ma_floatCode label=\"minsoil\">2</ma_floatCode>"
       << "<ma_floatCode label=\"freshlog\">3</ma_floatCode>"
       << "<ma_floatCode label=\"mossylog\">4</ma_floatCode>"
       << "</ma_floatCodes>"
       << "<ma_plotLenX>200.0</ma_plotLenX>"
       << "<ma_plotLenY>200.0</ma_plotLenY>"
       << "<ma_lengthXCells>6.0</ma_lengthXCells>"
       << "<ma_lengthYCells>6.0</ma_lengthYCells>"
       << "</grid>"
       << "<grid gridName=\"Dispersed Seeds\">"
       << "<ma_intCodes>"
       << "<ma_intCode label=\"count\">0</ma_intCode>"
       << "</ma_intCodes>"
       << "<ma_floatCodes>"
       << "<ma_floatCode label=\"seeds_0\">0</ma_floatCode>"
       << "<ma_floatCode label=\"seeds_1\">1</ma_floatCode>"
       << "<ma_floatCode label=\"seeds_2\">2</ma_floatCode>"
       << "<ma_floatCode label=\"seeds_3\">3</ma_floatCode>"
       << "<ma_floatCode label=\"seeds_4\">4</ma_floatCode>"
       << "<ma_floatCode label=\"seeds_5\">5</ma_floatCode>"
       << "</ma_floatCodes>"
       << "<ma_boolCodes>"
       << "<ma_boolCode label=\"Is Gap\">0</ma_boolCode>"
       << "</ma_boolCodes>"
       << "<ma_plotLenX>200.0</ma_plotLenX>"
       << "<ma_plotLenY>200.0</ma_plotLenY>"
       << "<ma_lengthXCells>6.0</ma_lengthXCells>"
       << "<ma_lengthYCells>6.0</ma_lengthYCells>"
       << "</grid>"
       << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>quadratlight</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_2\" type=\"Seedling\"/>"
       << "<applyTo species=\"Species_3\" type=\"Seedling\"/>"
       << "<applyTo species=\"Species_6\" type=\"Seedling\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_3\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_6\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>light filter</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_2\" type=\"Seedling\"/>"
       << "<applyTo species=\"Species_3\" type=\"Seedling\"/>"
       << "<applyTo species=\"Species_6\" type=\"Seedling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>adultstochasticmort</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_3\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_6\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>NZ Substrate</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_3\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_6\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>non spatial disperse</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>NZ establishment with light</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_6\" type=\"Seed\"/>"
       << "<applyTo species=\"Species_3\" type=\"Seed\"/>"
       << "<applyTo species=\"Species_2\" type=\"Seed\"/>"
       << "</behavior>"
       << "</behaviorList>"
       << "<quadratGliLight>"
       << "<li_heightOfFishEyePhoto>0</li_heightOfFishEyePhoto>"
       << "<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>"
       << "<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>"
       << "<li_julianDayGrowthStarts>105</li_julianDayGrowthStarts>"
       << "<li_julianDayGrowthEnds>258</li_julianDayGrowthEnds>"
       << "<li_minSunAngle>0.785</li_minSunAngle>"
       << "<li_numAltGrids>12</li_numAltGrids>"
       << "<li_numAziGrids>18</li_numAziGrids>"
       << "<li_lightExtinctionCoefficient>"
       << "<li_lecVal species=\"Species_1\">0.08</li_lecVal>"
       << "<li_lecVal species=\"Species_2\">0.08</li_lecVal>"
       << "<li_lecVal species=\"Species_3\">0.08</li_lecVal>"
       << "<li_lecVal species=\"Species_4\">0.08</li_lecVal>"
       << "<li_lecVal species=\"Species_5\">0.08</li_lecVal>"
       << "<li_lecVal species=\"Species_6\">0.08</li_lecVal>"
       << "</li_lightExtinctionCoefficient>"
       << "<li_quadratLightHeight>1</li_quadratLightHeight>"
       << "</quadratGliLight>"
       << "<lightFilter>"
       << "<lf_lightExtinctionCoefficient>1.535</lf_lightExtinctionCoefficient>"
       << "<lf_heightOfFilter>1.3</lf_heightOfFilter>"
       << "</lightFilter>"
       << "<mortality>"
       << "<mo_nonSizeDepMort>"
       << "<mo_nsdmVal species=\"Species_2\">0.0</mo_nsdmVal>"
       << "<mo_nsdmVal species=\"Species_3\">0.0</mo_nsdmVal>"
       << "<mo_nsdmVal species=\"Species_6\">0.0</mo_nsdmVal>"
       << "</mo_nonSizeDepMort>"
       << "</mortality>"
       << "<substrate>"
       << "<nzs_mineralSoilDecayAlpha>-7.0E-4</nzs_mineralSoilDecayAlpha>"
       << "<nzs_mineralSoilDecayBeta>4.3</nzs_mineralSoilDecayBeta>"
       << "<nzs_freshLogDecayAlpha>-0.05</nzs_freshLogDecayAlpha>"
       << "<nzs_freshLogDecayBeta>6.5</nzs_freshLogDecayBeta>"
       << "<nzs_mossyLogToLitterMoss>0.55</nzs_mossyLogToLitterMoss>"
       << "<nzs_mossProportion>0.35</nzs_mossProportion>"
       << "<nzs_maxNumberDecayYears>10</nzs_maxNumberDecayYears>"
       << "<nzs_initialMineralSoil>0.0</nzs_initialMineralSoil>"
       << "<nzs_initialFreshLog>0.01</nzs_initialFreshLog>"
       << "<nzs_initialMossyLog>0.11</nzs_initialMossyLog>"
       << "<nzs_initialAvgDbhFreshLog>20</nzs_initialAvgDbhFreshLog>"
       << "<nzs_propOfDeadFall>"
       << "<nzs_podfVal species=\"Species_2\">0.7</nzs_podfVal>"
       << "<nzs_podfVal species=\"Species_3\">0.7</nzs_podfVal>"
       << "<nzs_podfVal species=\"Species_6\">0.7</nzs_podfVal>"
       << "</nzs_propOfDeadFall>"
       << "<nzs_propOfFallUproot>"
       << "<nzs_pofuVal species=\"Species_2\">0.5</nzs_pofuVal>"
       << "<nzs_pofuVal species=\"Species_3\">0.5</nzs_pofuVal>"
       << "<nzs_pofuVal species=\"Species_6\">0.5</nzs_pofuVal>"
       << "</nzs_propOfFallUproot>"
       << "<nzs_rootTipupFactor>3.0</nzs_rootTipupFactor>" << "</substrate>"
       << "<disperse>"
       << "<di_minDbhForReproduction>"
       << "<di_mdfrVal species=\"Species_1\">15.0</di_mdfrVal>"
       << "</di_minDbhForReproduction>"
       << "<di_nonSpatialSlopeOfLambda>"
       << "<di_nssolVal species=\"Species_1\">0</di_nssolVal>"
       << "</di_nonSpatialSlopeOfLambda>"
       << "<di_nonSpatialInterceptOfLambda>"
       << "<di_nsiolVal species=\"Species_1\">0</di_nsiolVal>"
       << "</di_nonSpatialInterceptOfLambda>"
       << "<di_seedDistributionMethod>0</di_seedDistributionMethod>"
       << "</disperse>"
       << "<establishment>"
       << "<es_mossMoundFav>"
       << "<es_mmfVal species=\"Species_2\">1</es_mmfVal>"
       << "<es_mmfVal species=\"Species_3\">1</es_mmfVal>"
       << "<es_mmfVal species=\"Species_6\">1</es_mmfVal>"
       << "</es_mossMoundFav>"
       << "<es_mineralSoilMoundFav>"
       << "<es_msmfVal species=\"Species_2\">1</es_msmfVal>"
       << "<es_msmfVal species=\"Species_3\">1</es_msmfVal>"
       << "<es_msmfVal species=\"Species_6\">1</es_msmfVal>"
       << "</es_mineralSoilMoundFav>"
       << "<es_freshLogMoundFav>"
       << "<es_flmfVal species=\"Species_2\">1</es_flmfVal>"
       << "<es_flmfVal species=\"Species_3\">1</es_flmfVal>"
       << "<es_flmfVal species=\"Species_6\">1</es_flmfVal>"
       << "</es_freshLogMoundFav>"
       << "<es_mossyLogMoundFav>"
       << "<es_mlmfVal species=\"Species_2\">1</es_mlmfVal>"
       << "<es_mlmfVal species=\"Species_3\">1</es_mlmfVal>"
       << "<es_mlmfVal species=\"Species_6\">1</es_mlmfVal>"
       << "</es_mossyLogMoundFav>"
       << "<es_litterMoundFav>"
       << "<es_lmfVal species=\"Species_2\">1</es_lmfVal>"
       << "<es_lmfVal species=\"Species_3\">1</es_lmfVal>"
       << "<es_lmfVal species=\"Species_6\">1</es_lmfVal>"
       << "</es_litterMoundFav>"
       << "<es_mossGroundFav>"
       << "<es_mgfVal species=\"Species_2\">1</es_mgfVal>"
       << "<es_mgfVal species=\"Species_3\">1</es_mgfVal>"
       << "<es_mgfVal species=\"Species_6\">1</es_mgfVal>"
       << "</es_mossGroundFav>"
       << "<es_mineralSoilGroundFav>"
       << "<es_msgfVal species=\"Species_2\">1</es_msgfVal>"
       << "<es_msgfVal species=\"Species_3\">1</es_msgfVal>"
       << "<es_msgfVal species=\"Species_6\">1</es_msgfVal>"
       << "</es_mineralSoilGroundFav>"
       << "<es_freshLogGroundFav>"
       << "<es_flgfVal species=\"Species_2\">1</es_flgfVal>"
       << "<es_flgfVal species=\"Species_3\">1</es_flgfVal>"
       << "<es_flgfVal species=\"Species_6\">1</es_flgfVal>"
       << "</es_freshLogGroundFav>"
       << "<es_mossyLogGroundFav>"
       << "<es_mlgfVal species=\"Species_2\">1</es_mlgfVal>"
       << "<es_mlgfVal species=\"Species_3\">1</es_mlgfVal>"
       << "<es_mlgfVal species=\"Species_6\">1</es_mlgfVal>"
       << "</es_mossyLogGroundFav>"
       << "<es_litterGroundFav>"
       << "<es_lgfVal species=\"Species_2\">1</es_lgfVal>"
       << "<es_lgfVal species=\"Species_3\">1</es_lgfVal>"
       << "<es_lgfVal species=\"Species_6\">1</es_lgfVal>"
       << "</es_litterGroundFav>"
       << "<es_optimumGLI>"
       << "<es_ogVal species=\"Species_2\">23</es_ogVal>"
       << "<es_ogVal species=\"Species_3\">87</es_ogVal>"
       << "<es_ogVal species=\"Species_6\">55.66</es_ogVal>"
       << "</es_optimumGLI>"
       << "<es_lowSlope>"
       << "<es_lsVal species=\"Species_2\">0.034</es_lsVal>"
       << "<es_lsVal species=\"Species_3\">0.0098</es_lsVal>"
       << "<es_lsVal species=\"Species_6\">0.011</es_lsVal>"
       << "</es_lowSlope>"
       << "<es_highSlope>"
       << "<es_hsVal species=\"Species_2\">0.0023</es_hsVal>"
       << "<es_hsVal species=\"Species_3\">0.0098</es_hsVal>"
       << "<es_hsVal species=\"Species_6\">0.012</es_hsVal>"
       << "</es_highSlope>"
       << "<es_lightHeight>1</es_lightHeight>"
       << "<es_moundProportion>0.75</es_moundProportion>"
       << "<es_maxRespite>6</es_maxRespite>"
       << "<es_meanMoundHeight>3</es_meanMoundHeight>"
       << "<es_moundStdDev>0.5</es_moundStdDev>"
       << "<es_meanFreshLogHeight>20</es_meanFreshLogHeight>"
       << "<es_freshLogStdDev>5</es_freshLogStdDev>"
       << "</establishment>"
       << "</paramFile>";

  oOut.close();

  return cFileString;
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile3()
/////////////////////////////////////////////////////////////////////////////
char * clTestNZEstablishment::WriteXMLFile3()
{
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>2</timesteps>"
       << "<yearsPerTimestep>2</yearsPerTimestep>"
       << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>200.0</plot_lenX>"
       << "<plot_lenY>200.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_canopyRadiusToDBH>"
       << "<tr_crtdVal species=\"Species_1\">0.0549</tr_crtdVal>"
       << "</tr_canopyRadiusToDBH>"
       << "<tr_canopyRadiusExponent>"
       << "<tr_creVal species=\"Species_1\">1.0</tr_creVal>"
       << "</tr_canopyRadiusExponent>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_fractOfHeightInCanopy>"
       << "<tr_fohicVal species=\"Species_1\">0.389</tr_fohicVal>"
       << "</tr_fractOfHeightInCanopy>"
       << "<tr_canopyDepthExponent>"
       << "<tr_cdeVal species=\"Species_1\">1.0</tr_cdeVal>"
       << "</tr_canopyDepthExponent>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "</trees>"
       //Set nz substrate resolution to 10X10 and seeds grid resolution to 5X5
       << "<grid gridName=\"NZ Substrate\">"
       << "<ma_floatCodes>"
       << "<ma_floatCode label=\"litter\">0</ma_floatCode>"
       << "<ma_floatCode label=\"moss\">1</ma_floatCode>"
       << "<ma_floatCode label=\"minsoil\">2</ma_floatCode>"
       << "<ma_floatCode label=\"freshlog\">3</ma_floatCode>"
       << "<ma_floatCode label=\"mossylog\">4</ma_floatCode>"
       << "</ma_floatCodes>"
       << "<ma_plotLenX>200.0</ma_plotLenX>"
       << "<ma_plotLenY>200.0</ma_plotLenY>"
       << "<ma_lengthXCells>10.0</ma_lengthXCells>"
       << "<ma_lengthYCells>10.0</ma_lengthYCells>"
       << "</grid>"
       << "<grid gridName=\"Dispersed Seeds\">"
       << "<ma_intCodes>"
       << "<ma_intCode label=\"count\">0</ma_intCode>"
       << "</ma_intCodes>"
       << "<ma_floatCodes>"
       << "<ma_floatCode label=\"seeds_0\">0</ma_floatCode>"
       << "</ma_floatCodes>"
       << "<ma_boolCodes>"
       << "<ma_boolCode label=\"Is Gap\">0</ma_boolCode>"
       << "</ma_boolCodes>"
       << "<ma_plotLenX>200.0</ma_plotLenX>"
       << "<ma_plotLenY>200.0</ma_plotLenY>"
       << "<ma_lengthXCells>5.0</ma_lengthXCells>"
       << "<ma_lengthYCells>5.0</ma_lengthYCells>"
       << "</grid>"
       << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>quadratlight</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>light filter</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>adultstochasticmort</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>NZ Substrate</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>non spatial disperse</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>NZ establishment without light</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Seed\"/>"
       << "</behavior>"
       << "</behaviorList>"
       << "<quadratGliLight>"
       << "<li_heightOfFishEyePhoto>0</li_heightOfFishEyePhoto>"
       << "<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>"
       << "<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>"
       << "<li_julianDayGrowthStarts>105</li_julianDayGrowthStarts>"
       << "<li_julianDayGrowthEnds>258</li_julianDayGrowthEnds>"
       << "<li_minSunAngle>0.785</li_minSunAngle>"
       << "<li_numAltGrids>12</li_numAltGrids>"
       << "<li_numAziGrids>18</li_numAziGrids>"
       << "<li_lightExtinctionCoefficient>"
       << "<li_lecVal species=\"Species_1\">0.08</li_lecVal>"
       << "</li_lightExtinctionCoefficient>"
       << "<li_quadratLightHeight>1</li_quadratLightHeight>"
       << "</quadratGliLight>"
       << "<lightFilter>"
       << "<lf_lightExtinctionCoefficient>1.535</lf_lightExtinctionCoefficient>"
       << "<lf_heightOfFilter>1.3</lf_heightOfFilter>"
       << "</lightFilter>"
       << "<mortality>"
       << "<mo_nonSizeDepMort>"
       << "<mo_nsdmVal species=\"Species_1\">0.0</mo_nsdmVal>"
       << "</mo_nonSizeDepMort>"
       << "</mortality>"
       << "<substrate>"
       << "<nzs_mineralSoilDecayAlpha>-7.0E-4</nzs_mineralSoilDecayAlpha>"
       << "<nzs_mineralSoilDecayBeta>4.3</nzs_mineralSoilDecayBeta>"
       << "<nzs_freshLogDecayAlpha>-0.05</nzs_freshLogDecayAlpha>"
       << "<nzs_freshLogDecayBeta>6.5</nzs_freshLogDecayBeta>"
       << "<nzs_mossyLogToLitterMoss>0.55</nzs_mossyLogToLitterMoss>"
       << "<nzs_mossProportion>0.35</nzs_mossProportion>"
       << "<nzs_maxNumberDecayYears>10</nzs_maxNumberDecayYears>"
       << "<nzs_initialMineralSoil>0.0</nzs_initialMineralSoil>"
       << "<nzs_initialFreshLog>0.01</nzs_initialFreshLog>"
       << "<nzs_initialMossyLog>0.11</nzs_initialMossyLog>"
       << "<nzs_initialAvgDbhFreshLog>20</nzs_initialAvgDbhFreshLog>"
       << "<nzs_propOfDeadFall>"
       << "<nzs_podfVal species=\"Species_1\">0.7</nzs_podfVal>" << "</nzs_propOfDeadFall>" << "<nzs_propOfFallUproot>"
       << "<nzs_pofuVal species=\"Species_1\">0.5</nzs_pofuVal>" << "</nzs_propOfFallUproot>"
       << "<nzs_rootTipupFactor>3.0</nzs_rootTipupFactor>" << "</substrate>"
       << "<disperse>"
       << "<di_minDbhForReproduction>"
       << "<di_mdfrVal species=\"Species_1\">15.0</di_mdfrVal>"
       << "</di_minDbhForReproduction>"
       << "<di_nonSpatialSlopeOfLambda>"
       << "<di_nssolVal species=\"Species_1\">0</di_nssolVal>"
       << "</di_nonSpatialSlopeOfLambda>"
       << "<di_nonSpatialInterceptOfLambda>"
       << "<di_nsiolVal species=\"Species_1\">0</di_nsiolVal>"
       << "</di_nonSpatialInterceptOfLambda>"
       << "<di_seedDistributionMethod>0</di_seedDistributionMethod>"
       << "</disperse>"
       << "<establishment>"
       << "<es_mossMoundFav>"
       << "<es_mmfVal species=\"Species_1\">1</es_mmfVal>"
       << "</es_mossMoundFav>"
       << "<es_mineralSoilMoundFav>"
       << "<es_msmfVal species=\"Species_1\">1</es_msmfVal>"
       << "</es_mineralSoilMoundFav>"
       << "<es_freshLogMoundFav>"
       << "<es_flmfVal species=\"Species_1\">1</es_flmfVal>"
       << "</es_freshLogMoundFav>"
       << "<es_mossyLogMoundFav>"
       << "<es_mlmfVal species=\"Species_1\">1</es_mlmfVal>"
       << "</es_mossyLogMoundFav>"
       << "<es_litterMoundFav>"
       << "<es_lmfVal species=\"Species_1\">1</es_lmfVal>"
       << "</es_litterMoundFav>"
       << "<es_mossGroundFav>"
       << "<es_mgfVal species=\"Species_1\">1</es_mgfVal>"
       << "</es_mossGroundFav>"
       << "<es_mineralSoilGroundFav>"
       << "<es_msgfVal species=\"Species_1\">1</es_msgfVal>"
       << "</es_mineralSoilGroundFav>"
       << "<es_freshLogGroundFav>"
       << "<es_flgfVal species=\"Species_1\">1</es_flgfVal>"
       << "</es_freshLogGroundFav>"
       << "<es_mossyLogGroundFav>"
       << "<es_mlgfVal species=\"Species_1\">1</es_mlgfVal>"
       << "</es_mossyLogGroundFav>"
       << "<es_litterGroundFav>"
       << "<es_lgfVal species=\"Species_1\">1</es_lgfVal>"
       << "</es_litterGroundFav>"
       << "<es_moundProportion>1</es_moundProportion>"
       << "<es_maxRespite>6</es_maxRespite>"
       << "<es_meanMoundHeight>3</es_meanMoundHeight>"
       << "<es_moundStdDev>0.5</es_moundStdDev>"
       << "<es_meanFreshLogHeight>20</es_meanFreshLogHeight>"
       << "<es_freshLogStdDev>5</es_freshLogStdDev>"
       << "</establishment>"
       << "</paramFile>";

  oOut.close();

  return cFileString;
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile1()
/////////////////////////////////////////////////////////////////////////////
char * clTestNZEstablishment::WriteXMLErrorFile1()
{
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>1</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>"
       << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>50.0</plot_lenX>"
       << "<plot_lenY>50.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_canopyRadiusToDBH>"
       << "<tr_crtdVal species=\"Species_1\">0.0549</tr_crtdVal>"
       << "</tr_canopyRadiusToDBH>"
       << "<tr_canopyRadiusExponent>"
       << "<tr_creVal species=\"Species_1\">1.0</tr_creVal>"
       << "</tr_canopyRadiusExponent>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_fractOfHeightInCanopy>"
       << "<tr_fohicVal species=\"Species_1\">0.389</tr_fohicVal>"
       << "</tr_fractOfHeightInCanopy>"
       << "<tr_canopyDepthExponent>"
       << "<tr_cdeVal species=\"Species_1\">1.0</tr_cdeVal>"
       << "</tr_canopyDepthExponent>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "</trees>"
       << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>quadratlight</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>radiallight</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>light filter</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>adultstochasticmort</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>NZ Substrate</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>non spatial disperse</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>NZ establishment with light</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Seed\"/>"
       << "</behavior>"
       << "</behaviorList>"
       << "<gliLight>"
       << "<li_heightOfFishEyePhoto>0</li_heightOfFishEyePhoto>"
       << "<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>"
       << "<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>"
       << "<li_julianDayGrowthStarts>105</li_julianDayGrowthStarts>"
       << "<li_julianDayGrowthEnds>258</li_julianDayGrowthEnds>"
       << "<li_minSunAngle>0.785</li_minSunAngle>"
       << "<li_numAltGrids>12</li_numAltGrids>"
       << "<li_numAziGrids>18</li_numAziGrids>"
       << "<li_lightExtinctionCoefficient>"
       << "<li_lecVal species=\"Species_1\">0.08</li_lecVal>"
       << "</li_lightExtinctionCoefficient>"
       << "</gliLight>"
       << "<lightFilter>"
       << "<lf_lightExtinctionCoefficient>1.535</lf_lightExtinctionCoefficient>"
       << "<lf_heightOfFilter>1.3</lf_heightOfFilter>"
       << "</lightFilter>"
       << "<mortality>"
       << "<mo_nonSizeDepMort>"
       << "<mo_nsdmVal species=\"Species_1\">0.0</mo_nsdmVal>"
       << "</mo_nonSizeDepMort>"
       << "</mortality>"
       << "<substrate>"
       << "<nzs_mineralSoilDecayAlpha>-7.0E-4</nzs_mineralSoilDecayAlpha>"
       << "<nzs_mineralSoilDecayBeta>4.3</nzs_mineralSoilDecayBeta>"
       << "<nzs_freshLogDecayAlpha>-0.05</nzs_freshLogDecayAlpha>"
       << "<nzs_freshLogDecayBeta>6.5</nzs_freshLogDecayBeta>"
       << "<nzs_mossyLogToLitterMoss>0.55</nzs_mossyLogToLitterMoss>"
       << "<nzs_mossProportion>0.35</nzs_mossProportion>"
       << "<nzs_maxNumberDecayYears>10</nzs_maxNumberDecayYears>"
       << "<nzs_initialMineralSoil>0.0</nzs_initialMineralSoil>"
       << "<nzs_initialFreshLog>0.01</nzs_initialFreshLog>"
       << "<nzs_initialMossyLog>0.11</nzs_initialMossyLog>"
       << "<nzs_initialAvgDbhFreshLog>20</nzs_initialAvgDbhFreshLog>"
       << "<nzs_propOfDeadFall>"
       << "<nzs_podfVal species=\"Species_1\">0.7</nzs_podfVal>" << "</nzs_propOfDeadFall>" << "<nzs_propOfFallUproot>"
       << "<nzs_pofuVal species=\"Species_1\">0.5</nzs_pofuVal>" << "</nzs_propOfFallUproot>"
       << "<nzs_rootTipupFactor>3.0</nzs_rootTipupFactor>" << "</substrate>"
       << "<disperse>"
       << "<di_minDbhForReproduction>"
       << "<di_mdfrVal species=\"Species_1\">15.0</di_mdfrVal>"
       << "</di_minDbhForReproduction>"
       << "<di_nonSpatialSlopeOfLambda>"
       << "<di_nssolVal species=\"Species_1\">0</di_nssolVal>"
       << "</di_nonSpatialSlopeOfLambda>"
       << "<di_nonSpatialInterceptOfLambda>"
       << "<di_nsiolVal species=\"Species_1\">0.3</di_nsiolVal>"
       << "</di_nonSpatialInterceptOfLambda>"
       << "<di_seedDistributionMethod>0</di_seedDistributionMethod>"
       << "</disperse>"
       << "<establishment>"
       << "<es_mossMoundFav>"
       << "<es_mmfVal species=\"Species_1\">0.129</es_mmfVal>"
       << "</es_mossMoundFav>"
       << "<es_mineralSoilMoundFav>"
       << "<es_msmfVal species=\"Species_1\">0.983</es_msmfVal>"
       << "</es_mineralSoilMoundFav>"
       << "<es_freshLogMoundFav>"
       << "<es_flmfVal species=\"Species_1\">0.353</es_flmfVal>"
       << "</es_freshLogMoundFav>"
       << "<es_mossyLogMoundFav>"
       << "<es_mlmfVal species=\"Species_1\">0.058</es_mlmfVal>"
       << "</es_mossyLogMoundFav>"
       << "<es_litterMoundFav>"
       << "<es_lmfVal species=\"Species_1\">1.009</es_lmfVal>"
       << "</es_litterMoundFav>"
       << "<es_mossGroundFav>"
       << "<es_mgfVal species=\"Species_1\">0.129</es_mgfVal>"
       << "</es_mossGroundFav>"
       << "<es_mineralSoilGroundFav>"
       << "<es_msgfVal species=\"Species_1\">0.983</es_msgfVal>"
       << "</es_mineralSoilGroundFav>"
       << "<es_freshLogGroundFav>"
       << "<es_flgfVal species=\"Species_1\">0.353</es_flgfVal>"
       << "</es_freshLogGroundFav>"
       << "<es_mossyLogGroundFav>"
       << "<es_mlgfVal species=\"Species_1\">0.058</es_mlgfVal>"
       << "</es_mossyLogGroundFav>"
       << "<es_litterGroundFav>"
       << "<es_lgfVal species=\"Species_1\">0.009</es_lgfVal>"
       << "</es_litterGroundFav>"
       << "<es_optimumGLI>"
       << "<es_ogVal species=\"Species_1\">75</es_ogVal>"
       << "</es_optimumGLI>"
       << "<es_lowSlope>"
       << "<es_lsVal species=\"Species_1\">0.87</es_lsVal>"
       << "</es_lowSlope>"
       << "<es_highSlope>"
       << "<es_hsVal species=\"Species_1\">0.87</es_hsVal>"
       << "</es_highSlope>"
       << "<es_lightHeight>0.87</es_lightHeight>"
       << "<es_moundProportion>0.87</es_moundProportion>"
       << "<es_maxRespite>3</es_maxRespite>"
       << "<es_meanMoundHeight>3</es_meanMoundHeight>"
       << "<es_moundStdDev>0.5</es_moundStdDev>"
       << "<es_meanFreshLogHeight>20</es_meanFreshLogHeight>"
       << "<es_freshLogStdDev>5</es_freshLogStdDev>"
       << "</establishment>"
       << "</paramFile>";

  oOut.close();

  return cFileString;
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile2()
/////////////////////////////////////////////////////////////////////////////
char * clTestNZEstablishment::WriteXMLErrorFile2()
{
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>1</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>"
       << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>50.0</plot_lenX>"
       << "<plot_lenY>50.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_canopyRadiusToDBH>"
       << "<tr_crtdVal species=\"Species_1\">0.0549</tr_crtdVal>"
       << "</tr_canopyRadiusToDBH>"
       << "<tr_canopyRadiusExponent>"
       << "<tr_creVal species=\"Species_1\">1.0</tr_creVal>"
       << "</tr_canopyRadiusExponent>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_fractOfHeightInCanopy>"
       << "<tr_fohicVal species=\"Species_1\">0.389</tr_fohicVal>"
       << "</tr_fractOfHeightInCanopy>"
       << "<tr_canopyDepthExponent>"
       << "<tr_cdeVal species=\"Species_1\">1.0</tr_cdeVal>"
       << "</tr_canopyDepthExponent>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "</trees>"
       << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>quadratlight</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>radiallight</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>light filter</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>adultstochasticmort</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>NZ Substrate</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>non spatial disperse</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>NZ establishment with light</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Seed\"/>"
       << "</behavior>"
       << "</behaviorList>"
       << "<gliLight>"
       << "<li_heightOfFishEyePhoto>0</li_heightOfFishEyePhoto>"
       << "<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>"
       << "<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>"
       << "<li_julianDayGrowthStarts>105</li_julianDayGrowthStarts>"
       << "<li_julianDayGrowthEnds>258</li_julianDayGrowthEnds>"
       << "<li_minSunAngle>0.785</li_minSunAngle>"
       << "<li_numAltGrids>12</li_numAltGrids>"
       << "<li_numAziGrids>18</li_numAziGrids>"
       << "<li_lightExtinctionCoefficient>"
       << "<li_lecVal species=\"Species_1\">0.08</li_lecVal>"
       << "</li_lightExtinctionCoefficient>"
       << "</gliLight>"
       << "<lightFilter>"
       << "<lf_lightExtinctionCoefficient>1.535</lf_lightExtinctionCoefficient>"
       << "<lf_heightOfFilter>1.3</lf_heightOfFilter>"
       << "</lightFilter>"
       << "<mortality>"
       << "<mo_nonSizeDepMort>"
       << "<mo_nsdmVal species=\"Species_1\">0.0</mo_nsdmVal>"
       << "</mo_nonSizeDepMort>"
       << "</mortality>"
       << "<substrate>"
       << "<nzs_mineralSoilDecayAlpha>-7.0E-4</nzs_mineralSoilDecayAlpha>"
       << "<nzs_mineralSoilDecayBeta>4.3</nzs_mineralSoilDecayBeta>"
       << "<nzs_freshLogDecayAlpha>-0.05</nzs_freshLogDecayAlpha>"
       << "<nzs_freshLogDecayBeta>6.5</nzs_freshLogDecayBeta>"
       << "<nzs_mossyLogToLitterMoss>0.55</nzs_mossyLogToLitterMoss>"
       << "<nzs_mossProportion>0.35</nzs_mossProportion>"
       << "<nzs_maxNumberDecayYears>10</nzs_maxNumberDecayYears>"
       << "<nzs_initialMineralSoil>0.0</nzs_initialMineralSoil>"
       << "<nzs_initialFreshLog>0.01</nzs_initialFreshLog>"
       << "<nzs_initialMossyLog>0.11</nzs_initialMossyLog>"
       << "<nzs_initialAvgDbhFreshLog>20</nzs_initialAvgDbhFreshLog>"
       << "<nzs_propOfDeadFall>"
       << "<nzs_podfVal species=\"Species_1\">0.7</nzs_podfVal>" << "</nzs_propOfDeadFall>" << "<nzs_propOfFallUproot>"
       << "<nzs_pofuVal species=\"Species_1\">0.5</nzs_pofuVal>" << "</nzs_propOfFallUproot>"
       << "<nzs_rootTipupFactor>3.0</nzs_rootTipupFactor>" << "</substrate>"
       << "<disperse>"
       << "<di_minDbhForReproduction>"
       << "<di_mdfrVal species=\"Species_1\">15.0</di_mdfrVal>"
       << "</di_minDbhForReproduction>"
       << "<di_nonSpatialSlopeOfLambda>"
       << "<di_nssolVal species=\"Species_1\">0</di_nssolVal>"
       << "</di_nonSpatialSlopeOfLambda>"
       << "<di_nonSpatialInterceptOfLambda>"
       << "<di_nsiolVal species=\"Species_1\">0.3</di_nsiolVal>"
       << "</di_nonSpatialInterceptOfLambda>"
       << "<di_seedDistributionMethod>0</di_seedDistributionMethod>"
       << "</disperse>"
       << "<establishment>"
       << "<es_mossMoundFav>"
       << "<es_mmfVal species=\"Species_1\">0.129</es_mmfVal>"
       << "</es_mossMoundFav>"
       << "<es_mineralSoilMoundFav>"
       << "<es_msmfVal species=\"Species_1\">0.983</es_msmfVal>"
       << "</es_mineralSoilMoundFav>"
       << "<es_freshLogMoundFav>"
       << "<es_flmfVal species=\"Species_1\">0.353</es_flmfVal>"
       << "</es_freshLogMoundFav>"
       << "<es_mossyLogMoundFav>"
       << "<es_mlmfVal species=\"Species_1\">0.058</es_mlmfVal>"
       << "</es_mossyLogMoundFav>"
       << "<es_litterMoundFav>"
       << "<es_lmfVal species=\"Species_1\">0.009</es_lmfVal>"
       << "</es_litterMoundFav>"
       << "<es_mossGroundFav>"
       << "<es_mgfVal species=\"Species_1\">0.129</es_mgfVal>"
       << "</es_mossGroundFav>"
       << "<es_mineralSoilGroundFav>"
       << "<es_msgfVal species=\"Species_1\">0.983</es_msgfVal>"
       << "</es_mineralSoilGroundFav>"
       << "<es_freshLogGroundFav>"
       << "<es_flgfVal species=\"Species_1\">0.353</es_flgfVal>"
       << "</es_freshLogGroundFav>"
       << "<es_mossyLogGroundFav>"
       << "<es_mlgfVal species=\"Species_1\">0.058</es_mlgfVal>"
       << "</es_mossyLogGroundFav>"
       << "<es_litterGroundFav>"
       << "<es_lgfVal species=\"Species_1\">0.009</es_lgfVal>"
       << "</es_litterGroundFav>"
       << "<es_optimumGLI>"
       << "<es_ogVal species=\"Species_1\">75</es_ogVal>"
       << "</es_optimumGLI>"
       << "<es_lowSlope>"
       << "<es_lsVal species=\"Species_1\">0.87</es_lsVal>"
       << "</es_lowSlope>"
       << "<es_highSlope>"
       << "<es_hsVal species=\"Species_1\">0.87</es_hsVal>"
       << "</es_highSlope>"
       << "<es_lightHeight>0.87</es_lightHeight>"
       << "<es_moundProportion>-0.87</es_moundProportion>"
       << "<es_maxRespite>3</es_maxRespite>"
       << "<es_meanMoundHeight>3</es_meanMoundHeight>"
       << "<es_moundStdDev>0.5</es_moundStdDev>"
       << "<es_meanFreshLogHeight>20</es_meanFreshLogHeight>"
       << "<es_freshLogStdDev>5</es_freshLogStdDev>"
       << "</establishment>"
       << "</paramFile>";

  oOut.close();

  return cFileString;
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile3()
/////////////////////////////////////////////////////////////////////////////
char * clTestNZEstablishment::WriteXMLErrorFile3()
{
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>1</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>"
       << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>50.0</plot_lenX>"
       << "<plot_lenY>50.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_canopyRadiusToDBH>"
       << "<tr_crtdVal species=\"Species_1\">0.0549</tr_crtdVal>"
       << "</tr_canopyRadiusToDBH>"
       << "<tr_canopyRadiusExponent>"
       << "<tr_creVal species=\"Species_1\">1.0</tr_creVal>"
       << "</tr_canopyRadiusExponent>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_fractOfHeightInCanopy>"
       << "<tr_fohicVal species=\"Species_1\">0.389</tr_fohicVal>"
       << "</tr_fractOfHeightInCanopy>"
       << "<tr_canopyDepthExponent>"
       << "<tr_cdeVal species=\"Species_1\">1.0</tr_cdeVal>"
       << "</tr_canopyDepthExponent>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "</trees>"
       << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>quadratlight</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>radiallight</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>light filter</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>adultstochasticmort</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>NZ Substrate</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>non spatial disperse</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>NZ establishment with light</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Seed\"/>"
       << "</behavior>"
       << "</behaviorList>"
       << "<gliLight>"
       << "<li_heightOfFishEyePhoto>0</li_heightOfFishEyePhoto>"
       << "<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>"
       << "<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>"
       << "<li_julianDayGrowthStarts>105</li_julianDayGrowthStarts>"
       << "<li_julianDayGrowthEnds>258</li_julianDayGrowthEnds>"
       << "<li_minSunAngle>0.785</li_minSunAngle>"
       << "<li_numAltGrids>12</li_numAltGrids>"
       << "<li_numAziGrids>18</li_numAziGrids>"
       << "<li_lightExtinctionCoefficient>"
       << "<li_lecVal species=\"Species_1\">0.08</li_lecVal>"
       << "</li_lightExtinctionCoefficient>"
       << "</gliLight>"
       << "<lightFilter>"
       << "<lf_lightExtinctionCoefficient>1.535</lf_lightExtinctionCoefficient>"
       << "<lf_heightOfFilter>1.3</lf_heightOfFilter>"
       << "</lightFilter>"
       << "<mortality>"
       << "<mo_nonSizeDepMort>"
       << "<mo_nsdmVal species=\"Species_1\">0.0</mo_nsdmVal>"
       << "</mo_nonSizeDepMort>"
       << "</mortality>"
       << "<substrate>"
       << "<nzs_mineralSoilDecayAlpha>-7.0E-4</nzs_mineralSoilDecayAlpha>"
       << "<nzs_mineralSoilDecayBeta>4.3</nzs_mineralSoilDecayBeta>"
       << "<nzs_freshLogDecayAlpha>-0.05</nzs_freshLogDecayAlpha>"
       << "<nzs_freshLogDecayBeta>6.5</nzs_freshLogDecayBeta>"
       << "<nzs_mossyLogToLitterMoss>0.55</nzs_mossyLogToLitterMoss>"
       << "<nzs_mossProportion>0.35</nzs_mossProportion>"
       << "<nzs_maxNumberDecayYears>10</nzs_maxNumberDecayYears>"
       << "<nzs_initialMineralSoil>0.0</nzs_initialMineralSoil>"
       << "<nzs_initialFreshLog>0.01</nzs_initialFreshLog>"
       << "<nzs_initialMossyLog>0.11</nzs_initialMossyLog>"
       << "<nzs_initialAvgDbhFreshLog>20</nzs_initialAvgDbhFreshLog>"
       << "<nzs_propOfDeadFall>"
       << "<nzs_podfVal species=\"Species_1\">0.7</nzs_podfVal>" << "</nzs_propOfDeadFall>" << "<nzs_propOfFallUproot>"
       << "<nzs_pofuVal species=\"Species_1\">0.5</nzs_pofuVal>" << "</nzs_propOfFallUproot>"
       << "<nzs_rootTipupFactor>3.0</nzs_rootTipupFactor>" << "</substrate>"
       << "<disperse>"
       << "<di_minDbhForReproduction>"
       << "<di_mdfrVal species=\"Species_1\">15.0</di_mdfrVal>"
       << "</di_minDbhForReproduction>"
       << "<di_nonSpatialSlopeOfLambda>"
       << "<di_nssolVal species=\"Species_1\">0</di_nssolVal>"
       << "</di_nonSpatialSlopeOfLambda>"
       << "<di_nonSpatialInterceptOfLambda>"
       << "<di_nsiolVal species=\"Species_1\">0.3</di_nsiolVal>"
       << "</di_nonSpatialInterceptOfLambda>"
       << "<di_seedDistributionMethod>0</di_seedDistributionMethod>"
       << "</disperse>"
       << "<establishment>"
       << "<es_mossMoundFav>"
       << "<es_mmfVal species=\"Species_1\">0.129</es_mmfVal>"
       << "</es_mossMoundFav>"
       << "<es_mineralSoilMoundFav>"
       << "<es_msmfVal species=\"Species_1\">0.983</es_msmfVal>"
       << "</es_mineralSoilMoundFav>"
       << "<es_freshLogMoundFav>"
       << "<es_flmfVal species=\"Species_1\">0.353</es_flmfVal>"
       << "</es_freshLogMoundFav>"
       << "<es_mossyLogMoundFav>"
       << "<es_mlmfVal species=\"Species_1\">0.058</es_mlmfVal>"
       << "</es_mossyLogMoundFav>"
       << "<es_litterMoundFav>"
       << "<es_lmfVal species=\"Species_1\">0.009</es_lmfVal>"
       << "</es_litterMoundFav>"
       << "<es_mossGroundFav>"
       << "<es_mgfVal species=\"Species_1\">0.129</es_mgfVal>"
       << "</es_mossGroundFav>"
       << "<es_mineralSoilGroundFav>"
       << "<es_msgfVal species=\"Species_1\">0.983</es_msgfVal>"
       << "</es_mineralSoilGroundFav>"
       << "<es_freshLogGroundFav>"
       << "<es_flgfVal species=\"Species_1\">0.353</es_flgfVal>"
       << "</es_freshLogGroundFav>"
       << "<es_mossyLogGroundFav>"
       << "<es_mlgfVal species=\"Species_1\">0.058</es_mlgfVal>"
       << "</es_mossyLogGroundFav>"
       << "<es_litterGroundFav>"
       << "<es_lgfVal species=\"Species_1\">0.009</es_lgfVal>"
       << "</es_litterGroundFav>"
       << "<es_optimumGLI>"
       << "<es_ogVal species=\"Species_1\">75</es_ogVal>"
       << "</es_optimumGLI>"
       << "<es_lowSlope>"
       << "<es_lsVal species=\"Species_1\">0.87</es_lsVal>"
       << "</es_lowSlope>"
       << "<es_highSlope>"
       << "<es_hsVal species=\"Species_1\">0.87</es_hsVal>"
       << "</es_highSlope>"
       << "<es_lightHeight>-0.87</es_lightHeight>"
       << "<es_moundProportion>0.87</es_moundProportion>"
       << "<es_maxRespite>3</es_maxRespite>"
       << "<es_meanMoundHeight>3</es_meanMoundHeight>"
       << "<es_moundStdDev>0.5</es_moundStdDev>"
       << "<es_meanFreshLogHeight>20</es_meanFreshLogHeight>"
       << "<es_freshLogStdDev>5</es_freshLogStdDev>"
       << "</establishment>"
       << "</paramFile>";

  oOut.close();

  return cFileString;
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile4()
/////////////////////////////////////////////////////////////////////////////
char * clTestNZEstablishment::WriteXMLErrorFile4()
{
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>1</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>"
       << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>50.0</plot_lenX>"
       << "<plot_lenY>50.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_canopyRadiusToDBH>"
       << "<tr_crtdVal species=\"Species_1\">0.0549</tr_crtdVal>"
       << "</tr_canopyRadiusToDBH>"
       << "<tr_canopyRadiusExponent>"
       << "<tr_creVal species=\"Species_1\">1.0</tr_creVal>"
       << "</tr_canopyRadiusExponent>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_fractOfHeightInCanopy>"
       << "<tr_fohicVal species=\"Species_1\">0.389</tr_fohicVal>"
       << "</tr_fractOfHeightInCanopy>"
       << "<tr_canopyDepthExponent>"
       << "<tr_cdeVal species=\"Species_1\">1.0</tr_cdeVal>"
       << "</tr_canopyDepthExponent>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "</trees>"
       << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>quadratlight</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>radiallight</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>light filter</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>adultstochasticmort</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>NZ Substrate</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>non spatial disperse</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>NZ establishment with light</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Seed\"/>"
       << "</behavior>"
       << "</behaviorList>"
       << "<gliLight>"
       << "<li_heightOfFishEyePhoto>0</li_heightOfFishEyePhoto>"
       << "<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>"
       << "<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>"
       << "<li_julianDayGrowthStarts>105</li_julianDayGrowthStarts>"
       << "<li_julianDayGrowthEnds>258</li_julianDayGrowthEnds>"
       << "<li_minSunAngle>0.785</li_minSunAngle>"
       << "<li_numAltGrids>12</li_numAltGrids>"
       << "<li_numAziGrids>18</li_numAziGrids>"
       << "<li_lightExtinctionCoefficient>"
       << "<li_lecVal species=\"Species_1\">0.08</li_lecVal>"
       << "</li_lightExtinctionCoefficient>"
       << "</gliLight>"
       << "<lightFilter>"
       << "<lf_lightExtinctionCoefficient>1.535</lf_lightExtinctionCoefficient>"
       << "<lf_heightOfFilter>1.3</lf_heightOfFilter>"
       << "</lightFilter>"
       << "<mortality>"
       << "<mo_nonSizeDepMort>"
       << "<mo_nsdmVal species=\"Species_1\">0.0</mo_nsdmVal>"
       << "</mo_nonSizeDepMort>"
       << "</mortality>"
       << "<substrate>"
       << "<nzs_mineralSoilDecayAlpha>-7.0E-4</nzs_mineralSoilDecayAlpha>"
       << "<nzs_mineralSoilDecayBeta>4.3</nzs_mineralSoilDecayBeta>"
       << "<nzs_freshLogDecayAlpha>-0.05</nzs_freshLogDecayAlpha>"
       << "<nzs_freshLogDecayBeta>6.5</nzs_freshLogDecayBeta>"
       << "<nzs_mossyLogToLitterMoss>0.55</nzs_mossyLogToLitterMoss>"
       << "<nzs_mossProportion>0.35</nzs_mossProportion>"
       << "<nzs_maxNumberDecayYears>10</nzs_maxNumberDecayYears>"
       << "<nzs_initialMineralSoil>0.0</nzs_initialMineralSoil>"
       << "<nzs_initialFreshLog>0.01</nzs_initialFreshLog>"
       << "<nzs_initialMossyLog>0.11</nzs_initialMossyLog>"
       << "<nzs_initialAvgDbhFreshLog>20</nzs_initialAvgDbhFreshLog>"
       << "<nzs_propOfDeadFall>"
       << "<nzs_podfVal species=\"Species_1\">0.7</nzs_podfVal>" << "</nzs_propOfDeadFall>" << "<nzs_propOfFallUproot>"
       << "<nzs_pofuVal species=\"Species_1\">0.5</nzs_pofuVal>" << "</nzs_propOfFallUproot>"
       << "<nzs_rootTipupFactor>3.0</nzs_rootTipupFactor>" << "</substrate>"
       << "<disperse>"
       << "<di_minDbhForReproduction>"
       << "<di_mdfrVal species=\"Species_1\">15.0</di_mdfrVal>"
       << "</di_minDbhForReproduction>"
       << "<di_nonSpatialSlopeOfLambda>"
       << "<di_nssolVal species=\"Species_1\">0</di_nssolVal>"
       << "</di_nonSpatialSlopeOfLambda>"
       << "<di_nonSpatialInterceptOfLambda>"
       << "<di_nsiolVal species=\"Species_1\">0.3</di_nsiolVal>"
       << "</di_nonSpatialInterceptOfLambda>"
       << "<di_seedDistributionMethod>0</di_seedDistributionMethod>"
       << "</disperse>"
       << "<establishment>"
       << "<es_mossMoundFav>"
       << "<es_mmfVal species=\"Species_1\">0.129</es_mmfVal>"
       << "</es_mossMoundFav>"
       << "<es_mineralSoilMoundFav>"
       << "<es_msmfVal species=\"Species_1\">0.983</es_msmfVal>"
       << "</es_mineralSoilMoundFav>"
       << "<es_freshLogMoundFav>"
       << "<es_flmfVal species=\"Species_1\">0.353</es_flmfVal>"
       << "</es_freshLogMoundFav>"
       << "<es_mossyLogMoundFav>"
       << "<es_mlmfVal species=\"Species_1\">0.058</es_mlmfVal>"
       << "</es_mossyLogMoundFav>"
       << "<es_litterMoundFav>"
       << "<es_lmfVal species=\"Species_1\">0.009</es_lmfVal>"
       << "</es_litterMoundFav>"
       << "<es_mossGroundFav>"
       << "<es_mgfVal species=\"Species_1\">0.129</es_mgfVal>"
       << "</es_mossGroundFav>"
       << "<es_mineralSoilGroundFav>"
       << "<es_msgfVal species=\"Species_1\">0.983</es_msgfVal>"
       << "</es_mineralSoilGroundFav>"
       << "<es_freshLogGroundFav>"
       << "<es_flgfVal species=\"Species_1\">0.353</es_flgfVal>"
       << "</es_freshLogGroundFav>"
       << "<es_mossyLogGroundFav>"
       << "<es_mlgfVal species=\"Species_1\">0.058</es_mlgfVal>"
       << "</es_mossyLogGroundFav>"
       << "<es_litterGroundFav>"
       << "<es_lgfVal species=\"Species_1\">0.009</es_lgfVal>"
       << "</es_litterGroundFav>"
       << "<es_optimumGLI>"
       << "<es_ogVal species=\"Species_1\">175</es_ogVal>"
       << "</es_optimumGLI>"
       << "<es_lowSlope>"
       << "<es_lsVal species=\"Species_1\">0.87</es_lsVal>"
       << "</es_lowSlope>"
       << "<es_highSlope>"
       << "<es_hsVal species=\"Species_1\">0.87</es_hsVal>"
       << "</es_highSlope>"
       << "<es_lightHeight>0.87</es_lightHeight>"
       << "<es_moundProportion>0.87</es_moundProportion>"
       << "<es_maxRespite>3</es_maxRespite>"
       << "<es_meanMoundHeight>3</es_meanMoundHeight>"
       << "<es_moundStdDev>0.5</es_moundStdDev>"
       << "<es_meanFreshLogHeight>20</es_meanFreshLogHeight>"
       << "<es_freshLogStdDev>5</es_freshLogStdDev>"
       << "</establishment>"
       << "</paramFile>";

  oOut.close();

  return cFileString;
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile4()
/////////////////////////////////////////////////////////////////////////////
char * clTestNZEstablishment::WriteXMLFile4()
{
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>2</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>"
       << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>100.0</plot_lenX>"
       << "<plot_lenY>100.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "<tr_species speciesName=\"Species_2\"/>"
       << "<tr_species speciesName=\"Species_3\"/>"
       << "<tr_species speciesName=\"Species_4\"/>"
       << "<tr_species speciesName=\"Species_5\"/>"
       << "<tr_species speciesName=\"Species_6\"/>"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_2\">39.54</tr_chVal>"
       << "<tr_chVal species=\"Species_3\">40.0</tr_chVal>"
       << "<tr_chVal species=\"Species_4\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_5\">40.0</tr_chVal>"
       << "<tr_chVal species=\"Species_6\">39.48</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_canopyRadiusToDBH>"
       << "<tr_crtdVal species=\"Species_1\">0.0549</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_2\">0.0614</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_3\">0.0242</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_4\">0.0549</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_5\">0.0242</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_6\">0.0549</tr_crtdVal>"
       << "</tr_canopyRadiusToDBH>"
       << "<tr_canopyRadiusExponent>"
       << "<tr_creVal species=\"Species_1\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_2\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_3\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_4\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_5\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_6\">1.0</tr_creVal>"
       << "</tr_canopyRadiusExponent>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_2\">0.5944</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_3\">0.7059</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_4\">0.5944</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_5\">0.7059</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_6\">0.5944</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_fractOfHeightInCanopy>"
       << "<tr_fohicVal species=\"Species_1\">0.389</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_2\">0.368</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_3\">0.464</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_4\">0.368</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_5\">0.464</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_6\">0.368</tr_fohicVal>"
       << "</tr_fractOfHeightInCanopy>"
       << "<tr_canopyDepthExponent>"
       << "<tr_cdeVal species=\"Species_1\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_2\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_3\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_4\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_5\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_6\">1.0</tr_cdeVal>"
       << "</tr_canopyDepthExponent>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_2\">0.0269</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_3\">0.02871</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_4\">0.0269</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_5\">0.02871</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_6\">0.0269</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_2\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_3\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_4\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_5\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_6\">10.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_2\">0.0241</tr_soahVal>"
       << "<tr_soahVal species=\"Species_3\">0.0263</tr_soahVal>"
       << "<tr_soahVal species=\"Species_4\">0.0241</tr_soahVal>"
       << "<tr_soahVal species=\"Species_5\">0.0263</tr_soahVal>"
       << "<tr_soahVal species=\"Species_6\">0.0241</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "</trees>"
       << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>radiallight</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_2\" type=\"Seedling\"/>"
       << "<applyTo species=\"Species_3\" type=\"Seedling\"/>"
       << "<applyTo species=\"Species_6\" type=\"Seedling\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_3\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_6\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>light filter</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_2\" type=\"Seedling\"/>"
       << "<applyTo species=\"Species_3\" type=\"Seedling\"/>"
       << "<applyTo species=\"Species_6\" type=\"Seedling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>adultstochasticmort</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_3\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_6\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>NZ Substrate</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_3\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_6\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>non spatial disperse</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>NZ establishment without light</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_6\" type=\"Seed\"/>"
       << "<applyTo species=\"Species_3\" type=\"Seed\"/>"
       << "<applyTo species=\"Species_2\" type=\"Seed\"/>"
       << "</behavior>"
       << "</behaviorList>"
       << "<mortality>"
       << "<mo_nonSizeDepMort>"
       << "<mo_nsdmVal species=\"Species_2\">0.0</mo_nsdmVal>"
       << "<mo_nsdmVal species=\"Species_3\">0.0</mo_nsdmVal>"
       << "<mo_nsdmVal species=\"Species_6\">0.0</mo_nsdmVal>"
       << "</mo_nonSizeDepMort>"
       << "</mortality>"
       << "<substrate>"
       << "<nzs_mineralSoilDecayAlpha>-7.0E-4</nzs_mineralSoilDecayAlpha>"
       << "<nzs_mineralSoilDecayBeta>4.3</nzs_mineralSoilDecayBeta>"
       << "<nzs_freshLogDecayAlpha>-0.05</nzs_freshLogDecayAlpha>"
       << "<nzs_freshLogDecayBeta>6.5</nzs_freshLogDecayBeta>"
       << "<nzs_mossyLogToLitterMoss>0.55</nzs_mossyLogToLitterMoss>"
       << "<nzs_mossProportion>0.35</nzs_mossProportion>"
       << "<nzs_maxNumberDecayYears>10</nzs_maxNumberDecayYears>"
       << "<nzs_initialMineralSoil>0.0</nzs_initialMineralSoil>"
       << "<nzs_initialFreshLog>0.01</nzs_initialFreshLog>"
       << "<nzs_initialMossyLog>0.11</nzs_initialMossyLog>"
       << "<nzs_initialAvgDbhFreshLog>20</nzs_initialAvgDbhFreshLog>"
       << "<nzs_propOfDeadFall>"
       << "<nzs_podfVal species=\"Species_2\">0.7</nzs_podfVal>"
       << "<nzs_podfVal species=\"Species_3\">0.7</nzs_podfVal>"
       << "<nzs_podfVal species=\"Species_6\">0.7</nzs_podfVal>"
       << "</nzs_propOfDeadFall>"
       << "<nzs_propOfFallUproot>"
       << "<nzs_pofuVal species=\"Species_2\">0.5</nzs_pofuVal>"
       << "<nzs_pofuVal species=\"Species_3\">0.5</nzs_pofuVal>"
       << "<nzs_pofuVal species=\"Species_6\">0.5</nzs_pofuVal>"
       << "</nzs_propOfFallUproot>"
       << "<nzs_rootTipupFactor>3.0</nzs_rootTipupFactor>"
       << "</substrate>"
       << "<lightFilter>"
       << "<lf_lightExtinctionCoefficient>1.535</lf_lightExtinctionCoefficient>"
       << "<lf_heightOfFilter>1.3</lf_heightOfFilter>"
       << "</lightFilter>"
       << "<disperse>"
       << "<di_minDbhForReproduction>"
       << "<di_mdfrVal species=\"Species_1\">15.0</di_mdfrVal>"
       << "</di_minDbhForReproduction>"
       << "<di_nonSpatialSlopeOfLambda>"
       << "<di_nssolVal species=\"Species_1\">0</di_nssolVal>"
       << "</di_nonSpatialSlopeOfLambda>"
       << "<di_nonSpatialInterceptOfLambda>"
       << "<di_nsiolVal species=\"Species_1\">0</di_nsiolVal>"
       << "</di_nonSpatialInterceptOfLambda>"
       << "<di_seedDistributionMethod>0</di_seedDistributionMethod>"
       << "</disperse>"
       << "<establishment>"
       << "<es_mossMoundFav>"
       << "<es_mmfVal species=\"Species_2\">1</es_mmfVal>"
       << "<es_mmfVal species=\"Species_3\">1</es_mmfVal>"
       << "<es_mmfVal species=\"Species_6\">1</es_mmfVal>"
       << "</es_mossMoundFav>"
       << "<es_mineralSoilMoundFav>"
       << "<es_msmfVal species=\"Species_2\">1</es_msmfVal>"
       << "<es_msmfVal species=\"Species_3\">1</es_msmfVal>"
       << "<es_msmfVal species=\"Species_6\">1</es_msmfVal>"
       << "</es_mineralSoilMoundFav>"
       << "<es_freshLogMoundFav>"
       << "<es_flmfVal species=\"Species_2\">1</es_flmfVal>"
       << "<es_flmfVal species=\"Species_3\">1</es_flmfVal>"
       << "<es_flmfVal species=\"Species_6\">1</es_flmfVal>"
       << "</es_freshLogMoundFav>"
       << "<es_mossyLogMoundFav>"
       << "<es_mlmfVal species=\"Species_2\">1</es_mlmfVal>"
       << "<es_mlmfVal species=\"Species_3\">1</es_mlmfVal>"
       << "<es_mlmfVal species=\"Species_6\">1</es_mlmfVal>"
       << "</es_mossyLogMoundFav>"
       << "<es_litterMoundFav>"
       << "<es_lmfVal species=\"Species_2\">1</es_lmfVal>"
       << "<es_lmfVal species=\"Species_3\">1</es_lmfVal>"
       << "<es_lmfVal species=\"Species_6\">1</es_lmfVal>"
       << "</es_litterMoundFav>"
       << "<es_mossGroundFav>"
       << "<es_mgfVal species=\"Species_2\">1</es_mgfVal>"
       << "<es_mgfVal species=\"Species_3\">1</es_mgfVal>"
       << "<es_mgfVal species=\"Species_6\">1</es_mgfVal>"
       << "</es_mossGroundFav>"
       << "<es_mineralSoilGroundFav>"
       << "<es_msgfVal species=\"Species_2\">1</es_msgfVal>"
       << "<es_msgfVal species=\"Species_3\">1</es_msgfVal>"
       << "<es_msgfVal species=\"Species_6\">1</es_msgfVal>"
       << "</es_mineralSoilGroundFav>"
       << "<es_freshLogGroundFav>"
       << "<es_flgfVal species=\"Species_2\">1</es_flgfVal>"
       << "<es_flgfVal species=\"Species_3\">1</es_flgfVal>"
       << "<es_flgfVal species=\"Species_6\">1</es_flgfVal>"
       << "</es_freshLogGroundFav>"
       << "<es_mossyLogGroundFav>"
       << "<es_mlgfVal species=\"Species_2\">1</es_mlgfVal>"
       << "<es_mlgfVal species=\"Species_3\">1</es_mlgfVal>"
       << "<es_mlgfVal species=\"Species_6\">1</es_mlgfVal>"
       << "</es_mossyLogGroundFav>"
       << "<es_litterGroundFav>"
       << "<es_lgfVal species=\"Species_2\">1</es_lgfVal>"
       << "<es_lgfVal species=\"Species_3\">1</es_lgfVal>"
       << "<es_lgfVal species=\"Species_6\">1</es_lgfVal>"
       << "</es_litterGroundFav>"
       << "<es_moundProportion>0.75</es_moundProportion>"
       << "<es_maxRespite>3</es_maxRespite>"
       << "<es_meanMoundHeight>3</es_meanMoundHeight>"
       << "<es_moundStdDev>0.5</es_moundStdDev>"
       << "<es_meanFreshLogHeight>20</es_meanFreshLogHeight>"
       << "<es_freshLogStdDev>5</es_freshLogStdDev>"
       << "</establishment>"
       << "</paramFile>";

  oOut.close();

  return cFileString;
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile5()
/////////////////////////////////////////////////////////////////////////////
char * clTestNZEstablishment::WriteXMLFile5()
{
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>2</timesteps>"
       << "<yearsPerTimestep>2</yearsPerTimestep>"
       << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>200.0</plot_lenX>"
       << "<plot_lenY>200.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_canopyRadiusToDBH>"
       << "<tr_crtdVal species=\"Species_1\">0.0549</tr_crtdVal>"
       << "</tr_canopyRadiusToDBH>"
       << "<tr_canopyRadiusExponent>"
       << "<tr_creVal species=\"Species_1\">1.0</tr_creVal>"
       << "</tr_canopyRadiusExponent>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_fractOfHeightInCanopy>"
       << "<tr_fohicVal species=\"Species_1\">0.389</tr_fohicVal>"
       << "</tr_fractOfHeightInCanopy>"
       << "<tr_canopyDepthExponent>"
       << "<tr_cdeVal species=\"Species_1\">1.0</tr_cdeVal>"
       << "</tr_canopyDepthExponent>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "</trees>"
       //Set nz substrate resolution to 2X2 and seeds grid resolution to 8X8
       << "<grid gridName=\"NZ Substrate\">"
       << "<ma_floatCodes>"
       << "<ma_floatCode label=\"litter\">0</ma_floatCode>"
       << "<ma_floatCode label=\"moss\">1</ma_floatCode>"
       << "<ma_floatCode label=\"minsoil\">2</ma_floatCode>"
       << "<ma_floatCode label=\"freshlog\">3</ma_floatCode>"
       << "<ma_floatCode label=\"mossylog\">4</ma_floatCode>"
       << "</ma_floatCodes>"
       << "<ma_plotLenX>200.0</ma_plotLenX>"
       << "<ma_plotLenY>200.0</ma_plotLenY>"
       << "<ma_lengthXCells>2.0</ma_lengthXCells>"
       << "<ma_lengthYCells>2.0</ma_lengthYCells>"
       << "</grid>"
       << "<grid gridName=\"Dispersed Seeds\">"
       << "<ma_intCodes>"
       << "<ma_intCode label=\"count\">0</ma_intCode>"
       << "</ma_intCodes>"
       << "<ma_floatCodes>"
       << "<ma_floatCode label=\"seeds_0\">0</ma_floatCode>"
       << "</ma_floatCodes>"
       << "<ma_boolCodes>"
       << "<ma_boolCode label=\"Is Gap\">0</ma_boolCode>"
       << "</ma_boolCodes>"
       << "<ma_plotLenX>200.0</ma_plotLenX>"
       << "<ma_plotLenY>200.0</ma_plotLenY>"
       << "<ma_lengthXCells>8.0</ma_lengthXCells>"
       << "<ma_lengthYCells>8.0</ma_lengthYCells>"
       << "</grid>"
       << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>quadratlight</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>light filter</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>adultstochasticmort</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>NZ Substrate</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>non spatial disperse</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>NZ establishment without light</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Seed\"/>"
       << "</behavior>"
       << "</behaviorList>"
       << "<quadratGliLight>"
       << "<li_heightOfFishEyePhoto>0</li_heightOfFishEyePhoto>"
       << "<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>"
       << "<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>"
       << "<li_julianDayGrowthStarts>105</li_julianDayGrowthStarts>"
       << "<li_julianDayGrowthEnds>258</li_julianDayGrowthEnds>"
       << "<li_minSunAngle>0.785</li_minSunAngle>"
       << "<li_numAltGrids>12</li_numAltGrids>"
       << "<li_numAziGrids>18</li_numAziGrids>"
       << "<li_lightExtinctionCoefficient>"
       << "<li_lecVal species=\"Species_1\">0.08</li_lecVal>"
       << "</li_lightExtinctionCoefficient>"
       << "<li_quadratLightHeight>1</li_quadratLightHeight>"
       << "</quadratGliLight>"
       << "<lightFilter>"
       << "<lf_lightExtinctionCoefficient>1.535</lf_lightExtinctionCoefficient>"
       << "<lf_heightOfFilter>1.3</lf_heightOfFilter>"
       << "</lightFilter>"
       << "<mortality>"
       << "<mo_nonSizeDepMort>"
       << "<mo_nsdmVal species=\"Species_1\">0.0</mo_nsdmVal>"
       << "</mo_nonSizeDepMort>"
       << "</mortality>"
       << "<substrate>"
       << "<nzs_mineralSoilDecayAlpha>-7.0E-4</nzs_mineralSoilDecayAlpha>"
       << "<nzs_mineralSoilDecayBeta>4.3</nzs_mineralSoilDecayBeta>"
       << "<nzs_freshLogDecayAlpha>-0.05</nzs_freshLogDecayAlpha>"
       << "<nzs_freshLogDecayBeta>6.5</nzs_freshLogDecayBeta>"
       << "<nzs_mossyLogToLitterMoss>0.55</nzs_mossyLogToLitterMoss>"
       << "<nzs_mossProportion>0.35</nzs_mossProportion>"
       << "<nzs_maxNumberDecayYears>10</nzs_maxNumberDecayYears>"
       << "<nzs_initialMineralSoil>0.0</nzs_initialMineralSoil>"
       << "<nzs_initialFreshLog>0.01</nzs_initialFreshLog>"
       << "<nzs_initialMossyLog>0.11</nzs_initialMossyLog>"
       << "<nzs_initialAvgDbhFreshLog>20</nzs_initialAvgDbhFreshLog>"
       << "<nzs_propOfDeadFall>"
       << "<nzs_podfVal species=\"Species_1\">0.7</nzs_podfVal>" << "</nzs_propOfDeadFall>" << "<nzs_propOfFallUproot>"
       << "<nzs_pofuVal species=\"Species_1\">0.5</nzs_pofuVal>" << "</nzs_propOfFallUproot>"
       << "<nzs_rootTipupFactor>3.0</nzs_rootTipupFactor>" << "</substrate>"
       << "<disperse>"
       << "<di_minDbhForReproduction>"
       << "<di_mdfrVal species=\"Species_1\">15.0</di_mdfrVal>"
       << "</di_minDbhForReproduction>"
       << "<di_nonSpatialSlopeOfLambda>"
       << "<di_nssolVal species=\"Species_1\">0</di_nssolVal>"
       << "</di_nonSpatialSlopeOfLambda>"
       << "<di_nonSpatialInterceptOfLambda>"
       << "<di_nsiolVal species=\"Species_1\">0</di_nsiolVal>"
       << "</di_nonSpatialInterceptOfLambda>"
       << "<di_seedDistributionMethod>0</di_seedDistributionMethod>"
       << "</disperse>"
       << "<establishment>"
       << "<es_mossMoundFav>"
       << "<es_mmfVal species=\"Species_1\">1</es_mmfVal>"
       << "</es_mossMoundFav>"
       << "<es_mineralSoilMoundFav>"
       << "<es_msmfVal species=\"Species_1\">1</es_msmfVal>"
       << "</es_mineralSoilMoundFav>"
       << "<es_freshLogMoundFav>"
       << "<es_flmfVal species=\"Species_1\">1</es_flmfVal>"
       << "</es_freshLogMoundFav>"
       << "<es_mossyLogMoundFav>"
       << "<es_mlmfVal species=\"Species_1\">1</es_mlmfVal>"
       << "</es_mossyLogMoundFav>"
       << "<es_litterMoundFav>"
       << "<es_lmfVal species=\"Species_1\">1</es_lmfVal>"
       << "</es_litterMoundFav>"
       << "<es_mossGroundFav>"
       << "<es_mgfVal species=\"Species_1\">1</es_mgfVal>"
       << "</es_mossGroundFav>"
       << "<es_mineralSoilGroundFav>"
       << "<es_msgfVal species=\"Species_1\">1</es_msgfVal>"
       << "</es_mineralSoilGroundFav>"
       << "<es_freshLogGroundFav>"
       << "<es_flgfVal species=\"Species_1\">1</es_flgfVal>"
       << "</es_freshLogGroundFav>"
       << "<es_mossyLogGroundFav>"
       << "<es_mlgfVal species=\"Species_1\">1</es_mlgfVal>"
       << "</es_mossyLogGroundFav>"
       << "<es_litterGroundFav>"
       << "<es_lgfVal species=\"Species_1\">1</es_lgfVal>"
       << "</es_litterGroundFav>"
       << "<es_moundProportion>0</es_moundProportion>"
       << "<es_maxRespite>6</es_maxRespite>"
       << "<es_meanMoundHeight>3</es_meanMoundHeight>"
       << "<es_moundStdDev>0.5</es_moundStdDev>"
       << "<es_meanFreshLogHeight>20</es_meanFreshLogHeight>"
       << "<es_freshLogStdDev>5</es_freshLogStdDev>"
       << "</establishment>"
       << "</paramFile>";

  oOut.close();

  return cFileString;
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile6()
/////////////////////////////////////////////////////////////////////////////
char * clTestNZEstablishment::WriteXMLFile6()
{
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>1</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>"
       << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>50.0</plot_lenX>"
       << "<plot_lenY>50.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "<tr_species speciesName=\"Species_2\"/>"
       << "<tr_species speciesName=\"Species_3\"/>"
       << "<tr_species speciesName=\"Species_4\"/>"
       << "<tr_species speciesName=\"Species_5\"/>"
       << "<tr_species speciesName=\"Species_6\"/>"
       << "<tr_species speciesName=\"Species_7\"/>"
       << "<tr_species speciesName=\"Species_8\"/>"
       << "<tr_species speciesName=\"Species_9\"/>"
       << "<tr_species speciesName=\"Species_10\"/>"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_2\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_3\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_4\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_5\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_6\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_7\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_8\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_9\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_10\">39.48</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_canopyRadiusToDBH>"
       << "<tr_crtdVal species=\"Species_1\">0.0549</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_2\">0.0549</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_3\">0.0549</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_4\">0.0549</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_5\">0.0549</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_6\">0.0549</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_7\">0.0549</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_8\">0.0549</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_9\">0.0549</tr_crtdVal>"
       << "<tr_crtdVal species=\"Species_10\">0.0549</tr_crtdVal>"
       << "</tr_canopyRadiusToDBH>"
       << "<tr_canopyRadiusExponent>"
       << "<tr_creVal species=\"Species_1\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_2\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_3\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_4\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_5\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_6\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_7\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_8\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_9\">1.0</tr_creVal>"
       << "<tr_creVal species=\"Species_10\">1.0</tr_creVal>"
       << "</tr_canopyRadiusExponent>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_2\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_3\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_4\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_5\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_6\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_7\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_8\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_9\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_10\">0.8008</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_fractOfHeightInCanopy>"
       << "<tr_fohicVal species=\"Species_1\">0.389</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_2\">0.389</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_3\">0.389</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_4\">0.389</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_5\">0.389</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_6\">0.389</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_7\">0.389</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_8\">0.389</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_9\">0.389</tr_fohicVal>"
       << "<tr_fohicVal species=\"Species_10\">0.389</tr_fohicVal>"
       << "</tr_fractOfHeightInCanopy>"
       << "<tr_canopyDepthExponent>"
       << "<tr_cdeVal species=\"Species_1\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_2\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_3\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_4\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_5\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_6\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_7\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_8\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_9\">1.0</tr_cdeVal>"
       << "<tr_cdeVal species=\"Species_10\">1.0</tr_cdeVal>"
       << "</tr_canopyDepthExponent>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_2\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_3\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_4\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_5\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_6\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_7\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_8\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_9\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_10\">0.03418</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_2\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_3\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_4\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_5\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_6\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_7\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_8\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_9\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_10\">10.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_2\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_3\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_4\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_5\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_6\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_7\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_8\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_9\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_10\">0.0299</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "</trees>"
       << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>radiallight</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
       << "<applyTo species=\"Species_2\" type=\"Seedling\"/>"
       << "<applyTo species=\"Species_3\" type=\"Seedling\"/>"
       << "<applyTo species=\"Species_4\" type=\"Seedling\"/>"
       << "<applyTo species=\"Species_5\" type=\"Seedling\"/>"
       << "<applyTo species=\"Species_6\" type=\"Seedling\"/>"
       << "<applyTo species=\"Species_7\" type=\"Seedling\"/>"
       << "<applyTo species=\"Species_8\" type=\"Seedling\"/>"
       << "<applyTo species=\"Species_9\" type=\"Seedling\"/>"
       << "<applyTo species=\"Species_10\" type=\"Seedling\"/>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>light filter</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
       << "<applyTo species=\"Species_2\" type=\"Seedling\"/>"
       << "<applyTo species=\"Species_3\" type=\"Seedling\"/>"
       << "<applyTo species=\"Species_4\" type=\"Seedling\"/>"
       << "<applyTo species=\"Species_5\" type=\"Seedling\"/>"
       << "<applyTo species=\"Species_6\" type=\"Seedling\"/>"
       << "<applyTo species=\"Species_7\" type=\"Seedling\"/>"
       << "<applyTo species=\"Species_8\" type=\"Seedling\"/>"
       << "<applyTo species=\"Species_9\" type=\"Seedling\"/>"
       << "<applyTo species=\"Species_10\" type=\"Seedling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>adultstochasticmort</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>NZ Substrate</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>non spatial disperse</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>NZ establishment without light</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Seed\"/>"
       << "<applyTo species=\"Species_2\" type=\"Seed\"/>"
       << "<applyTo species=\"Species_3\" type=\"Seed\"/>"
       << "<applyTo species=\"Species_4\" type=\"Seed\"/>"
       << "<applyTo species=\"Species_5\" type=\"Seed\"/>"
       << "<applyTo species=\"Species_6\" type=\"Seed\"/>"
       << "<applyTo species=\"Species_7\" type=\"Seed\"/>"
       << "<applyTo species=\"Species_8\" type=\"Seed\"/>"
       << "<applyTo species=\"Species_9\" type=\"Seed\"/>"
       << "<applyTo species=\"Species_10\" type=\"Seed\"/>"
       << "</behavior>"
       << "</behaviorList>"
       << "<lightFilter>"
       << "<lf_lightExtinctionCoefficient>1.535</lf_lightExtinctionCoefficient>"
       << "<lf_heightOfFilter>1.3</lf_heightOfFilter>"
       << "</lightFilter>"
       << "<mortality>"
       << "<mo_nonSizeDepMort>"
       << "<mo_nsdmVal species=\"Species_1\">0.0</mo_nsdmVal>"
       << "</mo_nonSizeDepMort>"
       << "</mortality>"
       << "<substrate>"
       << "<nzs_mineralSoilDecayAlpha>-7.0E-4</nzs_mineralSoilDecayAlpha>"
       << "<nzs_mineralSoilDecayBeta>4.3</nzs_mineralSoilDecayBeta>"
       << "<nzs_freshLogDecayAlpha>-0.05</nzs_freshLogDecayAlpha>"
       << "<nzs_freshLogDecayBeta>6.5</nzs_freshLogDecayBeta>"
       << "<nzs_mossyLogToLitterMoss>0.55</nzs_mossyLogToLitterMoss>"
       << "<nzs_mossProportion>0.35</nzs_mossProportion>"
       << "<nzs_maxNumberDecayYears>10</nzs_maxNumberDecayYears>"
       << "<nzs_initialMineralSoil>0.0</nzs_initialMineralSoil>"
       << "<nzs_initialFreshLog>0.01</nzs_initialFreshLog>"
       << "<nzs_initialMossyLog>0.11</nzs_initialMossyLog>"
       << "<nzs_initialAvgDbhFreshLog>20</nzs_initialAvgDbhFreshLog>"
       << "<nzs_propOfDeadFall>"
       << "<nzs_podfVal species=\"Species_1\">0.7</nzs_podfVal>" << "</nzs_propOfDeadFall>" << "<nzs_propOfFallUproot>"
       << "<nzs_pofuVal species=\"Species_1\">0.5</nzs_pofuVal>" << "</nzs_propOfFallUproot>"
       << "<nzs_rootTipupFactor>3.0</nzs_rootTipupFactor>" << "</substrate>"
       << "<disperse>"
       << "<di_minDbhForReproduction>"
       << "<di_mdfrVal species=\"Species_1\">15.0</di_mdfrVal>"
       << "</di_minDbhForReproduction>"
       << "<di_nonSpatialSlopeOfLambda>"
       << "<di_nssolVal species=\"Species_1\">0</di_nssolVal>"
       << "</di_nonSpatialSlopeOfLambda>"
       << "<di_nonSpatialInterceptOfLambda>"
       << "<di_nsiolVal species=\"Species_1\">0.3</di_nsiolVal>"
       << "</di_nonSpatialInterceptOfLambda>"
       << "<di_seedDistributionMethod>0</di_seedDistributionMethod>"
       << "</disperse>"
       << "<establishment>"
       << "<es_mossMoundFav>"
       << "<es_mmfVal species=\"Species_1\">0.1</es_mmfVal>"
       << "<es_mmfVal species=\"Species_2\">0</es_mmfVal>"
       << "<es_mmfVal species=\"Species_3\">0</es_mmfVal>"
       << "<es_mmfVal species=\"Species_4\">0</es_mmfVal>"
       << "<es_mmfVal species=\"Species_5\">0</es_mmfVal>"
       << "<es_mmfVal species=\"Species_6\">0</es_mmfVal>"
       << "<es_mmfVal species=\"Species_7\">0</es_mmfVal>"
       << "<es_mmfVal species=\"Species_8\">0</es_mmfVal>"
       << "<es_mmfVal species=\"Species_9\">0</es_mmfVal>"
       << "<es_mmfVal species=\"Species_10\">0</es_mmfVal>"
       << "</es_mossMoundFav>"
       << "<es_mineralSoilMoundFav>"
       << "<es_msmfVal species=\"Species_1\">0</es_msmfVal>"
       << "<es_msmfVal species=\"Species_2\">0.2</es_msmfVal>"
       << "<es_msmfVal species=\"Species_3\">0</es_msmfVal>"
       << "<es_msmfVal species=\"Species_4\">0</es_msmfVal>"
       << "<es_msmfVal species=\"Species_5\">0</es_msmfVal>"
       << "<es_msmfVal species=\"Species_6\">0</es_msmfVal>"
       << "<es_msmfVal species=\"Species_7\">0</es_msmfVal>"
       << "<es_msmfVal species=\"Species_8\">0</es_msmfVal>"
       << "<es_msmfVal species=\"Species_9\">0</es_msmfVal>"
       << "<es_msmfVal species=\"Species_10\">0</es_msmfVal>"
       << "</es_mineralSoilMoundFav>"
       << "<es_freshLogMoundFav>"
       << "<es_flmfVal species=\"Species_1\">0</es_flmfVal>"
       << "<es_flmfVal species=\"Species_2\">0</es_flmfVal>"
       << "<es_flmfVal species=\"Species_3\">0.3</es_flmfVal>"
       << "<es_flmfVal species=\"Species_4\">0</es_flmfVal>"
       << "<es_flmfVal species=\"Species_5\">0</es_flmfVal>"
       << "<es_flmfVal species=\"Species_6\">0</es_flmfVal>"
       << "<es_flmfVal species=\"Species_7\">0</es_flmfVal>"
       << "<es_flmfVal species=\"Species_8\">0</es_flmfVal>"
       << "<es_flmfVal species=\"Species_9\">0</es_flmfVal>"
       << "<es_flmfVal species=\"Species_10\">0</es_flmfVal>"
       << "</es_freshLogMoundFav>"
       << "<es_mossyLogMoundFav>"
       << "<es_mlmfVal species=\"Species_1\">0</es_mlmfVal>"
       << "<es_mlmfVal species=\"Species_2\">0</es_mlmfVal>"
       << "<es_mlmfVal species=\"Species_3\">0</es_mlmfVal>"
       << "<es_mlmfVal species=\"Species_4\">0.4</es_mlmfVal>"
       << "<es_mlmfVal species=\"Species_5\">0</es_mlmfVal>"
       << "<es_mlmfVal species=\"Species_6\">0</es_mlmfVal>"
       << "<es_mlmfVal species=\"Species_7\">0</es_mlmfVal>"
       << "<es_mlmfVal species=\"Species_8\">0</es_mlmfVal>"
       << "<es_mlmfVal species=\"Species_9\">0</es_mlmfVal>"
       << "<es_mlmfVal species=\"Species_10\">0</es_mlmfVal>"
       << "</es_mossyLogMoundFav>"
       << "<es_litterMoundFav>"
       << "<es_lmfVal species=\"Species_1\">0</es_lmfVal>"
       << "<es_lmfVal species=\"Species_2\">0</es_lmfVal>"
       << "<es_lmfVal species=\"Species_3\">0</es_lmfVal>"
       << "<es_lmfVal species=\"Species_4\">0</es_lmfVal>"
       << "<es_lmfVal species=\"Species_5\">0.5</es_lmfVal>"
       << "<es_lmfVal species=\"Species_6\">0</es_lmfVal>"
       << "<es_lmfVal species=\"Species_7\">0</es_lmfVal>"
       << "<es_lmfVal species=\"Species_8\">0</es_lmfVal>"
       << "<es_lmfVal species=\"Species_9\">0</es_lmfVal>"
       << "<es_lmfVal species=\"Species_10\">0</es_lmfVal>"
       << "</es_litterMoundFav>"
       << "<es_mossGroundFav>"
       << "<es_mgfVal species=\"Species_1\">0</es_mgfVal>"
       << "<es_mgfVal species=\"Species_2\">0</es_mgfVal>"
       << "<es_mgfVal species=\"Species_3\">0</es_mgfVal>"
       << "<es_mgfVal species=\"Species_4\">0</es_mgfVal>"
       << "<es_mgfVal species=\"Species_5\">0</es_mgfVal>"
       << "<es_mgfVal species=\"Species_6\">0.6</es_mgfVal>"
       << "<es_mgfVal species=\"Species_7\">0</es_mgfVal>"
       << "<es_mgfVal species=\"Species_8\">0</es_mgfVal>"
       << "<es_mgfVal species=\"Species_9\">0</es_mgfVal>"
       << "<es_mgfVal species=\"Species_10\">0</es_mgfVal>"
       << "</es_mossGroundFav>"
       << "<es_mineralSoilGroundFav>"
       << "<es_msgfVal species=\"Species_1\">0</es_msgfVal>"
       << "<es_msgfVal species=\"Species_2\">0</es_msgfVal>"
       << "<es_msgfVal species=\"Species_3\">0</es_msgfVal>"
       << "<es_msgfVal species=\"Species_4\">0</es_msgfVal>"
       << "<es_msgfVal species=\"Species_5\">0</es_msgfVal>"
       << "<es_msgfVal species=\"Species_6\">0</es_msgfVal>"
       << "<es_msgfVal species=\"Species_7\">0.7</es_msgfVal>"
       << "<es_msgfVal species=\"Species_8\">0</es_msgfVal>"
       << "<es_msgfVal species=\"Species_9\">0</es_msgfVal>"
       << "<es_msgfVal species=\"Species_10\">0</es_msgfVal>"
       << "</es_mineralSoilGroundFav>"
       << "<es_freshLogGroundFav>"
       << "<es_flgfVal species=\"Species_1\">0</es_flgfVal>"
       << "<es_flgfVal species=\"Species_2\">0</es_flgfVal>"
       << "<es_flgfVal species=\"Species_3\">0</es_flgfVal>"
       << "<es_flgfVal species=\"Species_4\">0</es_flgfVal>"
       << "<es_flgfVal species=\"Species_5\">0</es_flgfVal>"
       << "<es_flgfVal species=\"Species_6\">0</es_flgfVal>"
       << "<es_flgfVal species=\"Species_7\">0</es_flgfVal>"
       << "<es_flgfVal species=\"Species_8\">0.8</es_flgfVal>"
       << "<es_flgfVal species=\"Species_9\">0</es_flgfVal>"
       << "<es_flgfVal species=\"Species_10\">0</es_flgfVal>"
       << "</es_freshLogGroundFav>"
       << "<es_mossyLogGroundFav>"
       << "<es_mlgfVal species=\"Species_1\">0</es_mlgfVal>"
       << "<es_mlgfVal species=\"Species_2\">0</es_mlgfVal>"
       << "<es_mlgfVal species=\"Species_3\">0</es_mlgfVal>"
       << "<es_mlgfVal species=\"Species_4\">0</es_mlgfVal>"
       << "<es_mlgfVal species=\"Species_5\">0</es_mlgfVal>"
       << "<es_mlgfVal species=\"Species_6\">0</es_mlgfVal>"
       << "<es_mlgfVal species=\"Species_7\">0</es_mlgfVal>"
       << "<es_mlgfVal species=\"Species_8\">0</es_mlgfVal>"
       << "<es_mlgfVal species=\"Species_9\">0.9</es_mlgfVal>"
       << "<es_mlgfVal species=\"Species_10\">0</es_mlgfVal>"
       << "</es_mossyLogGroundFav>"
       << "<es_litterGroundFav>"
       << "<es_lgfVal species=\"Species_1\">0</es_lgfVal>"
       << "<es_lgfVal species=\"Species_2\">0</es_lgfVal>"
       << "<es_lgfVal species=\"Species_3\">0</es_lgfVal>"
       << "<es_lgfVal species=\"Species_4\">0</es_lgfVal>"
       << "<es_lgfVal species=\"Species_5\">0</es_lgfVal>"
       << "<es_lgfVal species=\"Species_6\">0</es_lgfVal>"
       << "<es_lgfVal species=\"Species_7\">0</es_lgfVal>"
       << "<es_lgfVal species=\"Species_8\">0</es_lgfVal>"
       << "<es_lgfVal species=\"Species_9\">0</es_lgfVal>"
       << "<es_lgfVal species=\"Species_10\">1</es_lgfVal>"
       << "</es_litterGroundFav>"
       << "<es_moundProportion>0.5</es_moundProportion>"
       << "<es_maxRespite>3</es_maxRespite>"
       << "<es_meanMoundHeight>3</es_meanMoundHeight>"
       << "<es_moundStdDev>0.5</es_moundStdDev>"
       << "<es_meanFreshLogHeight>20</es_meanFreshLogHeight>"
       << "<es_freshLogStdDev>5</es_freshLogStdDev>"
       << "</establishment>"
       << "</paramFile>";

  oOut.close();

  return cFileString;
}
//---------------------------------------------------------------------------
